@@ -4,16 +4,15 @@
  *	  index scan strategy manipulation code and index strategy manipulation
  *	  operator code.
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/access/index/istrat.c,v 1.42 2000/04/12 17:14:47 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/access/index/istrat.c,v 1.56 2001/11/05 17:46:24 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
-
 #include "postgres.h"
 
 #include "access/heapam.h"
@@ -24,6 +23,7 @@
 #include "catalog/pg_index.h"
 #include "catalog/pg_operator.h"
 #include "miscadmin.h"
+#include "utils/fmgroids.h"
 #include "utils/syscache.h"
 
 #ifdef USE_ASSERT_CHECKING
@@ -36,7 +36,6 @@ static bool StrategyOperatorIsValid(StrategyOperator operator,
 						StrategyNumber maxStrategy);
 static bool StrategyTermIsValid(StrategyTerm term,
 					StrategyNumber maxStrategy);
-
 #endif
 
 
@@ -121,9 +120,9 @@ StrategyOperatorIsValid(StrategyOperator operator,
 						StrategyNumber maxStrategy)
 {
 	return (bool)
-	(PointerIsValid(operator) &&
-	 StrategyNumberIsInBounds(operator->strategy, maxStrategy) &&
-	 !(operator->flags & ~(SK_NEGATE | SK_COMMUTE)));
+		(PointerIsValid(operator) &&
+		 StrategyNumberIsInBounds(operator->strategy, maxStrategy) &&
+		 !(operator->flags & ~(SK_NEGATE | SK_COMMUTE)));
 }
 
 /* ----------------
@@ -187,25 +186,19 @@ StrategyEvaluationIsValid(StrategyEvaluation evaluation)
 		!StrategyTransformMapIsValid(evaluation->negateTransform) ||
 		!StrategyTransformMapIsValid(evaluation->commuteTransform) ||
 		!StrategyTransformMapIsValid(evaluation->negateCommuteTransform))
-	{
-
 		return false;
-	}
 
 	for (index = 0; index < evaluation->maxStrategy; index += 1)
 	{
 		if (!StrategyExpressionIsValid(evaluation->expression[index],
 									   evaluation->maxStrategy))
-		{
-
 			return false;
-		}
 	}
 	return true;
 }
-
 #endif
 
+#ifdef NOT_USED
 /* ----------------
  *		StrategyTermEvaluate
  * ----------------
@@ -216,15 +209,14 @@ StrategyTermEvaluate(StrategyTerm term,
 					 Datum left,
 					 Datum right)
 {
+	bool		result = false;
 	Index		index;
-	long		tmpres = 0;
-	bool		result = 0;
 	StrategyOperator operator;
-	ScanKey		entry;
 
 	for (index = 0, operator = &term->operatorData[0];
 		 index < term->degree; index += 1, operator += 1)
 	{
+		ScanKey		entry;
 
 		entry = &map->entry[operator->strategy - 1];
 
@@ -233,41 +225,41 @@ StrategyTermEvaluate(StrategyTerm term,
 		switch (operator->flags ^ entry->sk_flags)
 		{
 			case 0x0:
-				tmpres = (long) FMGR_PTR2(&entry->sk_func,
-										  left, right);
+				result = DatumGetBool(FunctionCall2(&entry->sk_func,
+													left, right));
 				break;
 
 			case SK_NEGATE:
-				tmpres = (long) !FMGR_PTR2(&entry->sk_func,
-										   left, right);
+				result = !DatumGetBool(FunctionCall2(&entry->sk_func,
+													 left, right));
 				break;
 
 			case SK_COMMUTE:
-				tmpres = (long) FMGR_PTR2(&entry->sk_func,
-										  right, left);
+				result = DatumGetBool(FunctionCall2(&entry->sk_func,
+													right, left));
 				break;
 
 			case SK_NEGATE | SK_COMMUTE:
-				tmpres = (long) !FMGR_PTR2(&entry->sk_func,
-										   right, left);
+				result = !DatumGetBool(FunctionCall2(&entry->sk_func,
+													 right, left));
 				break;
 
 			default:
-				elog(FATAL, "StrategyTermEvaluate: impossible case %d",
+				elog(ERROR, "StrategyTermEvaluate: impossible case %d",
 					 operator->flags ^ entry->sk_flags);
 		}
-
-		result = (bool) tmpres;
 		if (!result)
 			return result;
 	}
 
 	return result;
 }
-
+#endif
 
 /* ----------------
  *		RelationGetStrategy
+ *
+ * Identify strategy number that describes given procedure, if there is one.
  * ----------------
  */
 StrategyNumber
@@ -286,8 +278,7 @@ RelationGetStrategy(Relation relation,
 	numattrs = RelationGetNumberOfAttributes(relation);
 
 	Assert(relation->rd_rel->relkind == RELKIND_INDEX); /* XXX use accessor */
-	Assert(AttributeNumberIsValid(attributeNumber));
-	Assert((attributeNumber >= 1) && (attributeNumber < 1 + numattrs));
+	Assert((attributeNumber >= 1) && (attributeNumber <= numattrs));
 
 	Assert(StrategyEvaluationIsValid(evaluation));
 	Assert(RegProcedureIsValid(procedure));
@@ -332,7 +323,6 @@ RelationGetStrategy(Relation relation,
 			elog(FATAL, "RelationGetStrategy: impossible case %d", entry->sk_flags);
 	}
 
-
 	if (!StrategyNumberIsInBounds(strategy, evaluation->maxStrategy))
 	{
 		if (!StrategyNumberIsValid(strategy))
@@ -342,6 +332,7 @@ RelationGetStrategy(Relation relation,
 	return strategy;
 }
 
+#ifdef NOT_USED
 /* ----------------
  *		RelationInvokeStrategy
  * ----------------
@@ -390,7 +381,6 @@ RelationInvokeStrategy(Relation relation,
 	newStrategy = evaluation->negateTransform->strategy[strategy - 1];
 	if (newStrategy != strategy && StrategyNumberIsValid(newStrategy))
 	{
-
 		entry = StrategyMapGetScanKeyEntry(strategyMap, newStrategy);
 
 		if (RegProcedureIsValid(entry->sk_procedure))
@@ -405,7 +395,6 @@ RelationInvokeStrategy(Relation relation,
 	newStrategy = evaluation->commuteTransform->strategy[strategy - 1];
 	if (newStrategy != strategy && StrategyNumberIsValid(newStrategy))
 	{
-
 		entry = StrategyMapGetScanKeyEntry(strategyMap, newStrategy);
 
 		if (RegProcedureIsValid(entry->sk_procedure))
@@ -420,7 +409,6 @@ RelationInvokeStrategy(Relation relation,
 	newStrategy = evaluation->negateCommuteTransform->strategy[strategy - 1];
 	if (newStrategy != strategy && StrategyNumberIsValid(newStrategy))
 	{
-
 		entry = StrategyMapGetScanKeyEntry(strategyMap, newStrategy);
 
 		if (RegProcedureIsValid(entry->sk_procedure))
@@ -462,226 +450,162 @@ RelationInvokeStrategy(Relation relation,
 
 	/* not reached, just to make compiler happy */
 	return FALSE;
-
-
 }
+#endif
 
 /* ----------------
- *		OperatorRelationFillScanKeyEntry
+ *		FillScanKeyEntry
+ *
+ * Initialize a ScanKey entry for the given operator OID.
  * ----------------
  */
 static void
-OperatorRelationFillScanKeyEntry(Relation operatorRelation,
-								 Oid operatorObjectId,
-								 ScanKey entry)
+FillScanKeyEntry(Oid operatorObjectId, ScanKey entry)
 {
 	HeapTuple	tuple;
-	HeapScanDesc scan = NULL;
-	bool		cachesearch = (!IsBootstrapProcessingMode()) && IsCacheInitialized();
 
-	if (cachesearch)
-	{
-		tuple = SearchSysCacheTuple(OPEROID,
-									ObjectIdGetDatum(operatorObjectId),
-									0, 0, 0);
-	}
-	else
-	{
-		ScanKeyData scanKeyData;
-
-		ScanKeyEntryInitialize(&scanKeyData, 0,
-							   ObjectIdAttributeNumber,
-							   F_OIDEQ,
-							   ObjectIdGetDatum(operatorObjectId));
-
-		scan = heap_beginscan(operatorRelation, false, SnapshotNow,
-							  1, &scanKeyData);
-
-		tuple = heap_getnext(scan, 0);
-	}
+	tuple = SearchSysCache(OPEROID,
+						   ObjectIdGetDatum(operatorObjectId),
+						   0, 0, 0);
 
 	if (!HeapTupleIsValid(tuple))
-	{
-		if (!cachesearch)
-			heap_endscan(scan);
-		elog(ERROR, "OperatorObjectIdFillScanKeyEntry: unknown operator %u",
+		elog(ERROR, "FillScanKeyEntry: unknown operator %u",
 			 operatorObjectId);
-	}
 
+	MemSet(entry, 0, sizeof(*entry));
 	entry->sk_flags = 0;
 	entry->sk_procedure = ((Form_pg_operator) GETSTRUCT(tuple))->oprcode;
-	fmgr_info(entry->sk_procedure, &entry->sk_func);
-	entry->sk_nargs = entry->sk_func.fn_nargs;
 
-	if (!cachesearch)
-		heap_endscan(scan);
+	ReleaseSysCache(tuple);
 
 	if (!RegProcedureIsValid(entry->sk_procedure))
-	{
-		elog(ERROR,
-		"OperatorObjectIdFillScanKeyEntry: no procedure for operator %u",
+		elog(ERROR, "FillScanKeyEntry: no procedure for operator %u",
 			 operatorObjectId);
-	}
+
+	/*
+	 * Mark entry->sk_func invalid, until and unless someone sets it up.
+	 */
+	entry->sk_func.fn_oid = InvalidOid;
 }
 
 
 /*
  * IndexSupportInitialize
  *		Initializes an index strategy and associated support procedures.
+ *
+ * Data is returned into *indexStrategy, *indexSupport, and *isUnique,
+ * all of which are objects allocated by the caller.
+ *
+ * The primary input keys are indexObjectId and accessMethodObjectId.
+ * The caller also passes maxStrategyNumber, maxSupportNumber, and
+ * maxAttributeNumber, since these indicate the size of the indexStrategy
+ * and indexSupport arrays it has allocated --- but in practice these
+ * numbers must always match those obtainable from the system catalog
+ * entries for the index and access method.
  */
 void
 IndexSupportInitialize(IndexStrategy indexStrategy,
 					   RegProcedure *indexSupport,
+					   bool *isUnique,
 					   Oid indexObjectId,
 					   Oid accessMethodObjectId,
 					   StrategyNumber maxStrategyNumber,
 					   StrategyNumber maxSupportNumber,
 					   AttrNumber maxAttributeNumber)
 {
-	Relation	relation = NULL;
-	HeapScanDesc scan = NULL;
-	ScanKeyData entry[2];
-	Relation	operatorRelation;
 	HeapTuple	tuple;
-	StrategyMap map;
-	AttrNumber	attributeNumber;
-	int			attributeIndex;
+	Form_pg_index iform;
+	int			attIndex;
 	Oid			operatorClassObjectId[INDEX_MAX_KEYS];
-	bool		cachesearch = (!IsBootstrapProcessingMode()) && IsCacheInitialized();
-
-	if (cachesearch)
-	{
-		tuple = SearchSysCacheTuple(INDEXRELID,
-									ObjectIdGetDatum(indexObjectId),
-									0, 0, 0);
-	}
-	else
-	{
-		ScanKeyEntryInitialize(&entry[0], 0, Anum_pg_index_indexrelid,
-							   F_OIDEQ,
-							   ObjectIdGetDatum(indexObjectId));
-
-		relation = heap_openr(IndexRelationName, AccessShareLock);
-		scan = heap_beginscan(relation, false, SnapshotNow, 1, entry);
-		tuple = heap_getnext(scan, 0);
-	}
-
-	if (!HeapTupleIsValid(tuple))
-		elog(ERROR, "IndexSupportInitialize: corrupted catalogs");
 
 	maxStrategyNumber = AMStrategies(maxStrategyNumber);
 
+	tuple = SearchSysCache(INDEXRELID,
+						   ObjectIdGetDatum(indexObjectId),
+						   0, 0, 0);
+	if (!HeapTupleIsValid(tuple))
+		elog(ERROR, "IndexSupportInitialize: no pg_index entry for index %u",
+			 indexObjectId);
+	iform = (Form_pg_index) GETSTRUCT(tuple);
+
+	*isUnique = iform->indisunique;
+
 	/*
 	 * XXX note that the following assumes the INDEX tuple is well formed
 	 * and that the *key and *class are 0 terminated.
 	 */
-	for (attributeIndex = 0; attributeIndex < maxAttributeNumber; attributeIndex++)
+	for (attIndex = 0; attIndex < maxAttributeNumber; attIndex++)
 	{
-		Form_pg_index iform;
-
-		iform = (Form_pg_index) GETSTRUCT(tuple);
-
-		if (!OidIsValid(iform->indkey[attributeIndex]))
-		{
-			if (attributeIndex == InvalidAttrNumber)
-				elog(ERROR, "IndexSupportInitialize: no pg_index tuple");
-			break;
-		}
-
-		operatorClassObjectId[attributeIndex] = iform->indclass[attributeIndex];
+		if (iform->indkey[attIndex] == InvalidAttrNumber ||
+			!OidIsValid(iform->indclass[attIndex]))
+			elog(ERROR, "IndexSupportInitialize: bogus pg_index tuple");
+		operatorClassObjectId[attIndex] = iform->indclass[attIndex];
 	}
 
-	if (!cachesearch)
-	{
-		heap_endscan(scan);
-		heap_close(relation, AccessShareLock);
-	}
+	ReleaseSysCache(tuple);
 
 	/* if support routines exist for this access method, load them */
 	if (maxSupportNumber > 0)
 	{
-		ScanKeyEntryInitialize(&entry[0], 0, Anum_pg_amproc_amid,
-							   F_OIDEQ,
-							   ObjectIdGetDatum(accessMethodObjectId));
-
-		ScanKeyEntryInitialize(&entry[1], 0, Anum_pg_amproc_amopclaid,
-							   F_OIDEQ, 0);
-
-		relation = heap_openr(AccessMethodProcedureRelationName,
-							  AccessShareLock);
-
-		for (attributeNumber = 1; attributeNumber <= maxAttributeNumber;
-			 attributeNumber++)
+		for (attIndex = 0; attIndex < maxAttributeNumber; attIndex++)
 		{
-			int16		support;
-			Form_pg_amproc aform;
+			Oid			opclass = operatorClassObjectId[attIndex];
 			RegProcedure *loc;
+			StrategyNumber support;
 
-			loc = &indexSupport[((attributeNumber - 1) * maxSupportNumber)];
+			loc = &indexSupport[attIndex * maxSupportNumber];
 
 			for (support = 0; support < maxSupportNumber; ++support)
-				loc[support] = InvalidOid;
-
-			entry[1].sk_argument =
-				ObjectIdGetDatum(operatorClassObjectId[attributeNumber - 1]);
-
-			scan = heap_beginscan(relation, false, SnapshotNow, 2, entry);
-
-			while (HeapTupleIsValid(tuple = heap_getnext(scan, 0)))
 			{
-				aform = (Form_pg_amproc) GETSTRUCT(tuple);
-				loc[(aform->amprocnum - 1)] = aform->amproc;
-			}
+				tuple = SearchSysCache(AMPROCNUM,
+									   ObjectIdGetDatum(opclass),
+									   Int16GetDatum(support + 1),
+									   0, 0);
+				if (HeapTupleIsValid(tuple))
+				{
+					Form_pg_amproc amprocform;
 
-			heap_endscan(scan);
+					amprocform = (Form_pg_amproc) GETSTRUCT(tuple);
+					loc[support] = amprocform->amproc;
+					ReleaseSysCache(tuple);
+				}
+				else
+					loc[support] = InvalidOid;
+			}
 		}
-		heap_close(relation, AccessShareLock);
 	}
 
-	ScanKeyEntryInitialize(&entry[0], 0,
-						   Anum_pg_amop_amopid,
-						   F_OIDEQ,
-						   ObjectIdGetDatum(accessMethodObjectId));
-
-	ScanKeyEntryInitialize(&entry[1], 0,
-						   Anum_pg_amop_amopclaid,
-						   F_OIDEQ, 0);
-
-	relation = heap_openr(AccessMethodOperatorRelationName, AccessShareLock);
-	operatorRelation = heap_openr(OperatorRelationName, AccessShareLock);
-
-	for (attributeNumber = maxAttributeNumber; attributeNumber > 0;
-		 attributeNumber--)
+	/* Now load the strategy information for the index operators */
+	for (attIndex = 0; attIndex < maxAttributeNumber; attIndex++)
 	{
+		Oid			opclass = operatorClassObjectId[attIndex];
+		StrategyMap map;
 		StrategyNumber strategy;
 
-		entry[1].sk_argument =
-			ObjectIdGetDatum(operatorClassObjectId[attributeNumber - 1]);
-
 		map = IndexStrategyGetStrategyMap(indexStrategy,
 										  maxStrategyNumber,
-										  attributeNumber);
+										  attIndex + 1);
 
 		for (strategy = 1; strategy <= maxStrategyNumber; strategy++)
-			ScanKeyEntrySetIllegal(StrategyMapGetScanKeyEntry(map, strategy));
-
-		scan = heap_beginscan(relation, false, SnapshotNow, 2, entry);
-
-		while (HeapTupleIsValid(tuple = heap_getnext(scan, 0)))
 		{
-			Form_pg_amop aform;
+			ScanKey		mapentry = StrategyMapGetScanKeyEntry(map, strategy);
 
-			aform = (Form_pg_amop) GETSTRUCT(tuple);
-			OperatorRelationFillScanKeyEntry(operatorRelation,
-											 aform->amopopr,
-				   StrategyMapGetScanKeyEntry(map, aform->amopstrategy));
+			tuple = SearchSysCache(AMOPSTRATEGY,
+								   ObjectIdGetDatum(opclass),
+								   Int16GetDatum(strategy),
+								   0, 0);
+			if (HeapTupleIsValid(tuple))
+			{
+				Form_pg_amop amopform;
+
+				amopform = (Form_pg_amop) GETSTRUCT(tuple);
+				FillScanKeyEntry(amopform->amopopr, mapentry);
+				ReleaseSysCache(tuple);
+			}
+			else
+				ScanKeyEntrySetIllegal(mapentry);
 		}
-
-		heap_endscan(scan);
 	}
-
-	heap_close(operatorRelation, AccessShareLock);
-	heap_close(relation, AccessShareLock);
 }
 
 /* ----------------
@@ -701,7 +625,6 @@ IndexStrategyDisplay(IndexStrategy indexStrategy,
 	for (attributeNumber = 1; attributeNumber <= numberOfAttributes;
 		 attributeNumber += 1)
 	{
-
 		strategyMap = IndexStrategyGetStrategyMap(indexStrategy,
 												  numberOfStrategies,
 												  attributeNumber);
@@ -710,7 +633,6 @@ IndexStrategyDisplay(IndexStrategy indexStrategy,
 			 strategyNumber <= AMStrategies(numberOfStrategies);
 			 strategyNumber += 1)
 		{
-
 			printf(":att %d\t:str %d\t:opr 0x%x(%d)\n",
 				   attributeNumber, strategyNumber,
 				   strategyMap->entry[strategyNumber - 1].sk_procedure,
@@ -719,4 +641,4 @@ IndexStrategyDisplay(IndexStrategy indexStrategy,
 	}
 }
 
-#endif	 /* defined(ISTRATDEBUG) */
+#endif   /* defined(ISTRATDEBUG) */
