@@ -3,12 +3,12 @@
  * rtstrat.c
  *	  strategy map data for rtrees.
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/access/rtree/rtstrat.c,v 1.15 2000/01/26 05:56:00 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/access/rtree/rtstrat.c,v 1.19 2001/10/28 06:25:42 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -96,7 +96,7 @@ static StrategyNumber RTNegateCommute[RTNStrategies] = {
  *	Early in the development of the POSTGRES access methods, it was believed
  *	that writing functions was harder than writing arrays.	This is wrong;
  *	TermData is hard to understand and hard to get right.  In general, when
- *	someone populates a new operator class, the populate it completely.  If
+ *	someone populates a new operator class, they populate it completely.  If
  *	Mike Hirohama had forced Cimarron Taylor to populate the strategy map
  *	for btree int2_ops completely in 1988, you wouldn't have to deal with
  *	all this now.  Too bad for you.
@@ -119,9 +119,9 @@ static StrategyNumber RTNegateCommute[RTNStrategies] = {
 /* if you only have "contained-by", how do you determine equality? */
 static uint16 RTContainedByTermData[] = {
 	2,							/* make two comparisons */
-	RTContainedByStrategyNumber,/* use "a contained-by b" */
+	RTContainedByStrategyNumber,	/* use "a contained-by b" */
 	0x0,						/* without any magic */
-	RTContainedByStrategyNumber,/* then use contained-by, */
+	RTContainedByStrategyNumber,	/* then use contained-by, */
 	SK_COMMUTE					/* swapping a and b */
 };
 
@@ -157,24 +157,23 @@ static StrategyTerm RTEqualExpressionData[] = {
  *	in the access methods just isn't worth the trouble, though.
  */
 
+static StrategyExpression RTEvaluationExpressions[RTNStrategies] = {
+	NULL,						/* express left */
+	NULL,						/* express overleft */
+	NULL,						/* express overlap */
+	NULL,						/* express overright */
+	NULL,						/* express right */
+	(StrategyExpression) RTEqualExpressionData, /* express same */
+	NULL,						/* express contains */
+	NULL						/* express contained-by */
+};
+
 static StrategyEvaluationData RTEvaluationData = {
 	RTNStrategies,				/* # of strategies */
 	(StrategyTransformMap) RTNegate,	/* how to do (not qual) */
 	(StrategyTransformMap) RTCommute,	/* how to swap operands */
 	(StrategyTransformMap) RTNegateCommute,		/* how to do both */
-	{
-		NULL,					/* express left */
-		NULL,					/* express overleft */
-		NULL,					/* express over */
-		NULL,					/* express overright */
-		NULL,					/* express right */
-		(StrategyExpression) RTEqualExpressionData,		/* express same */
-		NULL,					/* express contains */
-		NULL,					/* express contained-by */
-		NULL,
-		NULL,
-		NULL
-	}
+	RTEvaluationExpressions
 };
 
 /*
@@ -222,7 +221,6 @@ RelationInvokeRTStrategy(Relation r,
 	return (RelationInvokeStrategy(r, &RTEvaluationData, attnum, s,
 								   left, right));
 }
-
 #endif
 
 RegProcedure
