@@ -4,88 +4,47 @@
  *	  routines to support running postgres in 'bootstrap' mode
  *	bootstrap mode is used to create the initial template database
  *
- * Copyright (c) 1994, Regents of the University of California
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/bootstrap/bootstrap.c,v 1.60 1999/05/25 16:07:56 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/bootstrap/bootstrap.c,v 1.120.2.1 2002/03/15 19:20:44 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
-#include <unistd.h>				/* For getopt() */
+#include "postgres.h"
+
+#include <unistd.h>
 #include <time.h>
-#include <stdio.h>
 #include <signal.h>
 #include <setjmp.h>
-#include <string.h>
-#ifdef __CYGWIN32__
+#ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
 #define BOOTSTRAP_INCLUDE		/* mask out stuff in tcop/tcopprot.h */
 
-#include "postgres.h"
-
-#include "miscadmin.h"
-#include "fmgr.h"
-
-#include "access/attnum.h"
-#include "access/funcindex.h"
 #include "access/genam.h"
 #include "access/heapam.h"
-#include "access/htup.h"
-#include "access/itup.h"
-#include "access/relscan.h"
-#include "access/sdir.h"
-#include "access/skey.h"
-#include "access/strat.h"
-#include "access/tupdesc.h"
-#include "access/xact.h"
+#include "access/xlog.h"
 #include "bootstrap/bootstrap.h"
 #include "catalog/catname.h"
 #include "catalog/index.h"
-#include "catalog/pg_am.h"
-#include "catalog/pg_attribute.h"
-#include "catalog/pg_class.h"
 #include "catalog/pg_type.h"
-#include "executor/execdesc.h"
-#include "executor/hashjoin.h"
-#include "executor/tuptable.h"
 #include "libpq/pqsignal.h"
-#include "nodes/execnodes.h"
-#include "nodes/memnodes.h"
-#include "nodes/nodes.h"
-#include "nodes/params.h"
-#include "nodes/parsenodes.h"
-#include "nodes/plannodes.h"
-#include "nodes/pg_list.h"
-#include "nodes/primnodes.h"
-#include "rewrite/prs2lock.h"
-#include "storage/block.h"
-#include "storage/buf.h"
-#include "storage/fd.h"
-#include "storage/ipc.h"
-#include "storage/itemptr.h"
-#include "storage/lock.h"
-#include "storage/off.h"
-#include "storage/shmem.h"
-#include "storage/spin.h"
-#include "tcop/dest.h"
+#include "miscadmin.h"
+#include "storage/proc.h"
 #include "tcop/tcopprot.h"
 #include "utils/builtins.h"
-#include "utils/geo_decls.h"
-#include "utils/hsearch.h"
+#include "utils/exc.h"
+#include "utils/fmgroids.h"
+#include "utils/guc.h"
 #include "utils/lsyscache.h"
-#include "utils/mcxt.h"
-#include "utils/nabstime.h"
-#include "utils/portal.h"
-#include "utils/rel.h"
+#include "utils/relcache.h"
 
-#ifndef HAVE_MEMMOVE
-#include "regex/utils.h"
-#endif
 
-#define ALLOC(t, c)		(t *)calloc((unsigned)(c), sizeof(t))
-#define FIRST_TYPE_OID 16		/* OID of the first type */
+#define ALLOC(t, c)		((t *) calloc((unsigned)(c), sizeof(t)))
+
 
 extern int	Int_yyparse(void);
 static hashnode *AddStr(char *str, int strlength, int mderef);
@@ -142,24 +101,22 @@ struct typinfo
 };
 
 static struct typinfo Procid[] = {
-	{"bool", 16, 0, 1, F_BOOLIN, F_BOOLOUT},
-	{"bytea", 17, 0, -1, F_BYTEAIN, F_BYTEAOUT},
-	{"char", 18, 0, 1, F_CHARIN, F_CHAROUT},
-	{"name", 19, 0, NAMEDATALEN, F_NAMEIN, F_NAMEOUT},
-	{"dummy", 20, 0, 16, 0, 0},
-/*	  { "dt",		  20,	 0,  4, F_DTIN,			F_DTOUT}, */
-	{"int2", 21, 0, 2, F_INT2IN, F_INT2OUT},
-	{"int28", 22, 0, 16, F_INT28IN, F_INT28OUT},
-	{"int4", 23, 0, 4, F_INT4IN, F_INT4OUT},
-	{"regproc", 24, 0, 4, F_REGPROCIN, F_REGPROCOUT},
-	{"text", 25, 0, -1, F_TEXTIN, F_TEXTOUT},
-	{"oid", 26, 0, 4, F_INT4IN, F_INT4OUT},
-	{"tid", 27, 0, 6, F_TIDIN, F_TIDOUT},
-	{"xid", 28, 0, 5, F_XIDIN, F_XIDOUT},
-	{"iid", 29, 0, 1, F_CIDIN, F_CIDOUT},
-	{"oid8", 30, 0, 32, F_OID8IN, F_OID8OUT},
+	{"bool", BOOLOID, 0, 1, F_BOOLIN, F_BOOLOUT},
+	{"bytea", BYTEAOID, 0, -1, F_BYTEAIN, F_BYTEAOUT},
+	{"char", CHAROID, 0, 1, F_CHARIN, F_CHAROUT},
+	{"name", NAMEOID, 0, NAMEDATALEN, F_NAMEIN, F_NAMEOUT},
+	{"int2", INT2OID, 0, 2, F_INT2IN, F_INT2OUT},
+	{"int2vector", INT2VECTOROID, 0, INDEX_MAX_KEYS * 2, F_INT2VECTORIN, F_INT2VECTOROUT},
+	{"int4", INT4OID, 0, 4, F_INT4IN, F_INT4OUT},
+	{"regproc", REGPROCOID, 0, 4, F_REGPROCIN, F_REGPROCOUT},
+	{"text", TEXTOID, 0, -1, F_TEXTIN, F_TEXTOUT},
+	{"oid", OIDOID, 0, 4, F_OIDIN, F_OIDOUT},
+	{"tid", TIDOID, 0, 6, F_TIDIN, F_TIDOUT},
+	{"xid", XIDOID, 0, 4, F_XIDIN, F_XIDOUT},
+	{"cid", CIDOID, 0, 4, F_CIDIN, F_CIDOUT},
+	{"oidvector", 30, 0, INDEX_MAX_KEYS * 4, F_OIDVECTORIN, F_OIDVECTOROUT},
 	{"smgr", 210, 0, 2, F_SMGRIN, F_SMGROUT},
-	{"_int4", 1007, 23, -1, F_ARRAY_IN, F_ARRAY_OUT},
+	{"_int4", 1007, INT4OID, -1, F_ARRAY_IN, F_ARRAY_OUT},
 	{"_aclitem", 1034, 1033, -1, F_ARRAY_IN, F_ARRAY_OUT}
 };
 
@@ -180,13 +137,12 @@ static char Blanks[MAXATTR];
 static char *relname;			/* current relation name */
 
 Form_pg_attribute attrtypes[MAXATTR];	/* points to attribute info */
-static char *values[MAXATTR];	/* cooresponding attribute values */
+static Datum values[MAXATTR];	/* corresponding attribute values */
 int			numattr;			/* number of attributes for cur. rel */
-extern bool disableFsync;		/* do not fsync the database */
 
 int			DebugMode;
-static GlobalMemory nogc = (GlobalMemory) NULL; /* special no-gc mem
-												 * context */
+
+static MemoryContext nogc = NULL;		/* special no-gc mem context */
 
 extern int	optind;
 extern char *optarg;
@@ -201,21 +157,12 @@ typedef struct _IndexList
 {
 	char	   *il_heap;
 	char	   *il_ind;
-	int			il_natts;
-	AttrNumber *il_attnos;
-	uint16		il_nparams;
-	Datum	   *il_params;
-	FuncIndexInfo *il_finfo;
-	PredInfo   *il_predInfo;
+	IndexInfo  *il_info;
 	struct _IndexList *il_next;
 } IndexList;
 
 static IndexList *ILHead = (IndexList *) NULL;
 
-typedef void (*sig_func) ();
-
-
-
 
 /* ----------------------------------------------------------------
  *						misc functions
@@ -239,13 +186,14 @@ err_out(void)
 static void
 usage(void)
 {
-	fprintf(stderr, "Usage: postgres -boot [-d] [-C] [-F] [-O] [-Q] ");
-	fprintf(stderr, "[-P portno] [dbName]\n");
-	fprintf(stderr, "     d: debug mode\n");
-	fprintf(stderr, "     C: disable version checking\n");
-	fprintf(stderr, "     F: turn off fsync\n");
-	fprintf(stderr, "     O: set BootstrapProcessing mode\n");
-	fprintf(stderr, "     P portno: specify port number\n");
+	fprintf(stderr,
+			gettext("Usage:\n"
+	"  postgres -boot [-d] [-D datadir] [-F] [-o file] [-x num] dbname\n"
+					"  -d               debug mode\n"
+					"  -D datadir       data directory\n"
+					"  -F               turn off fsync\n"
+					"  -o file          send debug output to file\n"
+					"  -x num           internal use\n"));
 
 	proc_exit(1);
 }
@@ -267,66 +215,66 @@ BootstrapMain(int argc, char *argv[])
  */
 {
 	int			i;
-	int			portFd = -1;
 	char	   *dbName;
 	int			flag;
-	int			override = 1;	/* use BootstrapProcessing or
-								 * InitProcessing mode */
+	int			xlogop = BS_XLOG_NOP;
+	char	   *potential_DataDir = NULL;
 
-	extern int	optind;
-	extern char *optarg;
-
-	/* ----------------
-	 *	initialize signal handlers
-	 * ----------------
-	 */
-	pqsignal(SIGINT, (sig_func) die);
-	pqsignal(SIGHUP, (sig_func) die);
-	pqsignal(SIGTERM, (sig_func) die);
-
-	/* --------------------
-	 *	initialize globals
-	 * -------------------
+	/*
+	 * initialize globals
 	 */
 
 	MyProcPid = getpid();
 
-	/* ----------------
-	 *	process command arguments
-	 * ----------------
+	/*
+	 * Fire up essential subsystems: error and memory management
+	 *
+	 * If we are running under the postmaster, this is done already.
+	 */
+	if (!IsUnderPostmaster)
+	{
+		EnableExceptionHandling(true);
+		MemoryContextInit();
+	}
+
+	/*
+	 * process command arguments
 	 */
 
 	/* Set defaults, to be overriden by explicit options below */
-	Quiet = false;
-	Noversion = false;
 	dbName = NULL;
-	DataDir = getenv("PGDATA"); /* Null if no PGDATA variable */
+	if (!IsUnderPostmaster)
+	{
+		ResetAllOptions(true);
+		potential_DataDir = getenv("PGDATA");	/* Null if no PGDATA
+												 * variable */
+	}
 
-	while ((flag = getopt(argc, argv, "D:dCOQP:F")) != EOF)
+	while ((flag = getopt(argc, argv, "B:dD:Fo:px:")) != -1)
 	{
 		switch (flag)
 		{
 			case 'D':
-				DataDir = optarg;
+				potential_DataDir = optarg;
 				break;
 			case 'd':
 				DebugMode = true;		/* print out debugging info while
 										 * parsing */
 				break;
-			case 'C':
-				Noversion = true;
-				break;
 			case 'F':
-				disableFsync = true;
+				SetConfigOption("fsync", "false", PGC_POSTMASTER, true);
 				break;
-			case 'O':
-				override = true;
+			case 'o':
+				StrNCpy(OutputFileName, optarg, MAXPGPATH);
 				break;
-			case 'Q':
-				Quiet = true;
+			case 'x':
+				xlogop = atoi(optarg);
 				break;
-			case 'P':			/* specify port */
-				portFd = atoi(optarg);
+			case 'p':
+				/* indicates fork from postmaster */
+				break;
+			case 'B':
+				SetConfigOption("shared_buffers", optarg, PGC_POSTMASTER, true);
 				break;
 			default:
 				usage();
@@ -334,49 +282,126 @@ BootstrapMain(int argc, char *argv[])
 		}
 	}							/* while */
 
-	if (argc - optind > 1)
+	if (argc - optind != 1)
 		usage();
-	else if (argc - optind == 1)
-		dbName = argv[optind];
 
-	if (!DataDir)
-	{
-		fprintf(stderr, "%s does not know where to find the database system "
-				"data.  You must specify the directory that contains the "
-				"database system either by specifying the -D invocation "
-			 "option or by setting the PGDATA environment variable.\n\n",
-				argv[0]);
-		proc_exit(1);
-	}
+	dbName = argv[optind];
 
-	if (dbName == NULL)
+	Assert(dbName);
+
+	if (!IsUnderPostmaster)
 	{
-		dbName = getenv("USER");
-		if (dbName == NULL)
+		if (!potential_DataDir)
 		{
-			fputs("bootstrap backend: failed, no db name specified\n", stderr);
-			fputs("          and no USER enviroment variable\n", stderr);
+			fprintf(stderr,
+					gettext("%s does not know where to find the database system data.\n"
+							"You must specify the directory that contains the database system\n"
+							"either by specifying the -D invocation option or by setting the\n"
+							"PGDATA environment variable.\n\n"),
+					argv[0]);
 			proc_exit(1);
 		}
+		SetDataDir(potential_DataDir);
 	}
 
-	/* ----------------
-	 *	initialize input fd
-	 * ----------------
-	 */
-	if (IsUnderPostmaster && portFd < 0)
+	/* Validate we have been given a reasonable-looking DataDir */
+	Assert(DataDir);
+	ValidatePgVersion(DataDir);
+
+	if (IsUnderPostmaster)
 	{
-		fputs("backend: failed, no -P option with -postmaster opt.\n", stderr);
-		proc_exit(1);
+		/*
+		 * Properly accept or ignore signals the postmaster might send us
+		 */
+		pqsignal(SIGHUP, SIG_IGN);
+		pqsignal(SIGINT, SIG_IGN);		/* ignore query-cancel */
+		pqsignal(SIGTERM, die);
+		pqsignal(SIGQUIT, quickdie);
+		pqsignal(SIGALRM, SIG_IGN);
+		pqsignal(SIGPIPE, SIG_IGN);
+		pqsignal(SIGUSR1, SIG_IGN);
+		pqsignal(SIGUSR2, SIG_IGN);
+
+		/*
+		 * Reset some signals that are accepted by postmaster but not here
+		 */
+		pqsignal(SIGCHLD, SIG_DFL);
+		pqsignal(SIGTTIN, SIG_DFL);
+		pqsignal(SIGTTOU, SIG_DFL);
+		pqsignal(SIGCONT, SIG_DFL);
+		pqsignal(SIGWINCH, SIG_DFL);
+
+		/*
+		 * Unblock signals (they were blocked when the postmaster forked
+		 * us)
+		 */
+		PG_SETMASK(&UnBlockSig);
+	}
+	else
+	{
+		/* Set up appropriately for interactive use */
+		pqsignal(SIGHUP, die);
+		pqsignal(SIGINT, die);
+		pqsignal(SIGTERM, die);
+		pqsignal(SIGQUIT, die);
+
+		/*
+		 * Create lockfile for data directory.
+		 */
+		if (!CreateDataDirLockFile(DataDir, false))
+			proc_exit(1);
 	}
 
-	/* ----------------
-	 *	backend initialization
-	 * ----------------
+	SetProcessingMode(BootstrapProcessing);
+	IgnoreSystemIndexes(true);
+
+	XLOGPathInit();
+
+	BaseInit();
+
+	/*
+	 * XLOG operations
 	 */
-	SetProcessingMode((override) ? BootstrapProcessing : InitProcessing);
-	InitPostgres(dbName);
-	LockDisable(true);
+	SetProcessingMode(NormalProcessing);
+
+	switch (xlogop)
+	{
+		case BS_XLOG_NOP:
+			StartupXLOG();
+			break;
+
+		case BS_XLOG_BOOTSTRAP:
+			BootStrapXLOG();
+			StartupXLOG();
+			break;
+
+		case BS_XLOG_CHECKPOINT:
+			if (IsUnderPostmaster)
+				InitDummyProcess();		/* needed to get LWLocks */
+			CreateDummyCaches();
+			CreateCheckPoint(false);
+			SetSavedRedoRecPtr(); /* pass redo ptr back to postmaster */
+			proc_exit(0);		/* done */
+
+		case BS_XLOG_STARTUP:
+			StartupXLOG();
+			proc_exit(0);		/* done */
+
+		case BS_XLOG_SHUTDOWN:
+			ShutdownXLOG();
+			proc_exit(0);		/* done */
+
+		default:
+			elog(STOP, "Unsupported XLOG op %d", xlogop);
+			proc_exit(0);
+	}
+
+	SetProcessingMode(BootstrapProcessing);
+
+	/*
+	 * backend initialization
+	 */
+	InitPostgres(dbName, NULL);
 
 	for (i = 0; i < MAXATTR; i++)
 	{
@@ -388,21 +413,17 @@ BootstrapMain(int argc, char *argv[])
 	for (i = 0; i < HASHTABLESIZE; ++i)
 		hashtable[i] = NULL;
 
-	/* ----------------
-	 *	abort processing resumes here
-	 * ----------------
+	/*
+	 * abort processing resumes here
 	 */
-	pqsignal(SIGHUP, handle_warn);
-
 	if (sigsetjmp(Warn_restart, 1) != 0)
 	{
 		Warnings++;
 		AbortCurrentTransaction();
 	}
 
-	/* ----------------
-	 *	process input.
-	 * ----------------
+	/*
+	 * process input.
 	 */
 
 	/*
@@ -411,6 +432,10 @@ BootstrapMain(int argc, char *argv[])
 	 */
 	Int_yyparse();
 
+	SetProcessingMode(NormalProcessing);
+	CreateCheckPoint(true);
+	SetProcessingMode(BootstrapProcessing);
+
 	/* clean up processing */
 	StartTransactionCommand();
 	cleanup();
@@ -443,8 +468,7 @@ boot_openrel(char *relname)
 
 	if (Typ == (struct typmap **) NULL)
 	{
-		StartPortalAllocMode(DefaultAllocMode, 0);
-		rel = heap_openr(TypeRelationName);
+		rel = heap_openr(TypeRelationName, NoLock);
 		scan = heap_beginscan(rel, 0, SnapshotNow, 0, (ScanKey) NULL);
 		i = 0;
 		while (HeapTupleIsValid(tup = heap_getnext(scan, 0)))
@@ -459,24 +483,23 @@ boot_openrel(char *relname)
 		while (HeapTupleIsValid(tup = heap_getnext(scan, 0)))
 		{
 			(*app)->am_oid = tup->t_data->t_oid;
-			memmove((char *) &(*app++)->am_typ,
-					(char *) GETSTRUCT(tup),
-					sizeof((*app)->am_typ));
+			memcpy((char *) &(*app)->am_typ,
+				   (char *) GETSTRUCT(tup),
+				   sizeof((*app)->am_typ));
+			app++;
 		}
 		heap_endscan(scan);
-		heap_close(rel);
-		EndPortalAllocMode();
+		heap_close(rel, NoLock);
 	}
 
 	if (reldesc != NULL)
 		closerel(NULL);
 
-	if (!Quiet)
-		printf("Amopen: relation %s. attrsize %d\n", relname ? relname : "(null)",
-			   (int) ATTRIBUTE_TUPLE_SIZE);
+	if (DebugMode)
+		elog(DEBUG, "open relation %s, attrsize %d", relname ? relname : "(null)",
+			 (int) ATTRIBUTE_TUPLE_SIZE);
 
-	reldesc = heap_openr(relname);
-	Assert(reldesc);
+	reldesc = heap_openr(relname, NoLock);
 	numattr = reldesc->rd_rel->relnatts;
 	for (i = 0; i < numattr; i++)
 	{
@@ -494,7 +517,7 @@ boot_openrel(char *relname)
 		 */
 		if (namestrcmp(&attrtypes[i]->attname, "attisset") == 0)
 			attrtypes[i]->attisset = get_attisset(RelationGetRelid(reldesc),
-											 attrtypes[i]->attname.data);
+										 NameStr(attrtypes[i]->attname));
 		else
 			attrtypes[i]->attisset = false;
 
@@ -502,11 +525,10 @@ boot_openrel(char *relname)
 		{
 			Form_pg_attribute at = attrtypes[i];
 
-			printf("create attribute %d name %s len %d num %d type %d\n",
-				   i, at->attname.data, at->attlen, at->attnum,
-				   at->atttypid
+			elog(DEBUG, "create attribute %d name %s len %d num %d type %u",
+				 i, NameStr(at->attname), at->attlen, at->attnum,
+				 at->atttypid
 				);
-			fflush(stdout);
 		}
 	}
 }
@@ -522,7 +544,7 @@ closerel(char *name)
 	{
 		if (reldesc)
 		{
-			if (namestrcmp(RelationGetRelationName(reldesc), name) != 0)
+			if (strcmp(RelationGetRelationName(reldesc), name) != 0)
 				elog(ERROR, "closerel: close of '%s' when '%s' was expected",
 					 name, relname ? relname : "(null)");
 		}
@@ -533,12 +555,12 @@ closerel(char *name)
 	}
 
 	if (reldesc == NULL)
-		elog(ERROR, "Warning: no opened relation to close.\n");
+		elog(ERROR, "no open relation to close");
 	else
 	{
-		if (!Quiet)
-			printf("Amclose: relation %s.\n", relname ? relname : "(null)");
-		heap_close(reldesc);
+		if (DebugMode)
+			elog(DEBUG, "close relation %s", relname ? relname : "(null)");
+		heap_close(reldesc, NoLock);
 		reldesc = (Relation) NULL;
 	}
 }
@@ -561,7 +583,7 @@ DefineAttr(char *name, char *type, int attnum)
 
 	if (reldesc != NULL)
 	{
-		fputs("Warning: no open relations allowed with 't' command.\n", stderr);
+		elog(DEBUG, "warning: no open relations allowed with 'create' command");
 		closerel(relname);
 	}
 
@@ -572,21 +594,23 @@ DefineAttr(char *name, char *type, int attnum)
 	{
 		attrtypes[attnum]->atttypid = Ap->am_oid;
 		namestrcpy(&attrtypes[attnum]->attname, name);
-		if (!Quiet)
-			printf("<%s %s> ", attrtypes[attnum]->attname.data, type);
+		if (DebugMode)
+			elog(DEBUG, "column %s %s", NameStr(attrtypes[attnum]->attname), type);
 		attrtypes[attnum]->attnum = 1 + attnum; /* fillatt */
 		attlen = attrtypes[attnum]->attlen = Ap->am_typ.typlen;
 		attrtypes[attnum]->attbyval = Ap->am_typ.typbyval;
+		attrtypes[attnum]->attstorage = Ap->am_typ.typstorage;;
 		attrtypes[attnum]->attalign = Ap->am_typ.typalign;
 	}
 	else
 	{
 		attrtypes[attnum]->atttypid = Procid[typeoid].oid;
 		namestrcpy(&attrtypes[attnum]->attname, name);
-		if (!Quiet)
-			printf("<%s %s> ", attrtypes[attnum]->attname.data, type);
+		if (DebugMode)
+			elog(DEBUG, "column %s %s", NameStr(attrtypes[attnum]->attname), type);
 		attrtypes[attnum]->attnum = 1 + attnum; /* fillatt */
 		attlen = attrtypes[attnum]->attlen = Procid[typeoid].len;
+		attrtypes[attnum]->attstorage = 'p';
 
 		/*
 		 * Cheat like mad to fill in these items from the length only.
@@ -619,7 +643,9 @@ DefineAttr(char *name, char *type, int attnum)
 
 /* ----------------
  *		InsertOneTuple
- *		assumes that 'oid' will not be zero.
+ *
+ * If objectid is not zero, it is a specific OID to assign to the tuple.
+ * Otherwise, an OID will be assigned (if necessary) by heap_insert.
  * ----------------
  */
 void
@@ -627,28 +653,21 @@ InsertOneTuple(Oid objectid)
 {
 	HeapTuple	tuple;
 	TupleDesc	tupDesc;
-
 	int			i;
 
 	if (DebugMode)
-	{
-		printf("InsertOneTuple oid %u, %d attrs\n", objectid, numattr);
-		fflush(stdout);
-	}
+		elog(DEBUG, "inserting row oid %u, %d columns", objectid, numattr);
 
 	tupDesc = CreateTupleDesc(numattr, attrtypes);
-	tuple = heap_formtuple(tupDesc, (Datum *) values, Blanks);
+	tuple = heap_formtuple(tupDesc, values, Blanks);
 	pfree(tupDesc);				/* just free's tupDesc, not the attrtypes */
 
 	if (objectid != (Oid) 0)
 		tuple->t_data->t_oid = objectid;
 	heap_insert(reldesc, tuple);
-	pfree(tuple);
+	heap_freetuple(tuple);
 	if (DebugMode)
-	{
-		printf("End InsertOneTuple, objectid=%u\n", objectid);
-		fflush(stdout);
-	}
+		elog(DEBUG, "row inserted");
 
 	/*
 	 * Reset blanks for next tuple
@@ -662,67 +681,69 @@ InsertOneTuple(Oid objectid)
  * ----------------
  */
 void
-InsertOneValue(Oid objectid, char *value, int i)
+InsertOneValue(char *value, int i)
 {
 	int			typeindex;
 	char	   *prt;
 	struct typmap **app;
 
+	AssertArg(i >= 0 || i < MAXATTR);
+
 	if (DebugMode)
-		printf("Inserting value: '%s'\n", value);
-	if (i < 0 || i >= MAXATTR)
-	{
-		printf("i out of range: %d\n", i);
-		Assert(0);
-	}
+		elog(DEBUG, "inserting column %d value '%s'", i, value);
 
 	if (Typ != (struct typmap **) NULL)
 	{
 		struct typmap *ap;
 
 		if (DebugMode)
-			puts("Typ != NULL");
+			elog(DEBUG, "Typ != NULL");
 		app = Typ;
 		while (*app && (*app)->am_oid != reldesc->rd_att->attrs[i]->atttypid)
 			++app;
 		ap = *app;
 		if (ap == NULL)
 		{
-			printf("Unable to find atttypid in Typ list! %u\n",
-				   reldesc->rd_att->attrs[i]->atttypid
-				);
-			Assert(0);
+			elog(FATAL, "unable to find atttypid %u in Typ list",
+				 reldesc->rd_att->attrs[i]->atttypid);
 		}
-		values[i] = fmgr(ap->am_typ.typinput,
-						 value,
-						 ap->am_typ.typelem,
-						 -1);	/* shouldn't have char() or varchar()
-								 * types during boostrapping but just to
-								 * be safe */
-		prt = fmgr(ap->am_typ.typoutput, values[i],
-				   ap->am_typ.typelem);
-		if (!Quiet)
-			printf("%s ", prt);
+		values[i] = OidFunctionCall3(ap->am_typ.typinput,
+									 CStringGetDatum(value),
+									 ObjectIdGetDatum(ap->am_typ.typelem),
+									 Int32GetDatum(-1));
+		prt = DatumGetCString(OidFunctionCall3(ap->am_typ.typoutput,
+											   values[i],
+									ObjectIdGetDatum(ap->am_typ.typelem),
+											   Int32GetDatum(-1)));
+		if (DebugMode)
+			elog(DEBUG, " -> %s", prt);
 		pfree(prt);
 	}
 	else
 	{
-		typeindex = attrtypes[i]->atttypid - FIRST_TYPE_OID;
+		for (typeindex = 0; typeindex < n_types; typeindex++)
+		{
+			if (Procid[typeindex].oid == attrtypes[i]->atttypid)
+				break;
+		}
+		if (typeindex >= n_types)
+			elog(ERROR, "type oid %u not found", attrtypes[i]->atttypid);
 		if (DebugMode)
-			printf("Typ == NULL, typeindex = %u idx = %d\n", typeindex, i);
-		values[i] = fmgr(Procid[typeindex].inproc, value,
-						 Procid[typeindex].elem, -1);
-		prt = fmgr(Procid[typeindex].outproc, values[i],
-				   Procid[typeindex].elem);
-		if (!Quiet)
-			printf("%s ", prt);
+			elog(DEBUG, "Typ == NULL, typeindex = %u", typeindex);
+		values[i] = OidFunctionCall3(Procid[typeindex].inproc,
+									 CStringGetDatum(value),
+								ObjectIdGetDatum(Procid[typeindex].elem),
+									 Int32GetDatum(-1));
+		prt = DatumGetCString(OidFunctionCall3(Procid[typeindex].outproc,
+											   values[i],
+								ObjectIdGetDatum(Procid[typeindex].elem),
+											   Int32GetDatum(-1)));
+		if (DebugMode)
+			elog(DEBUG, " -> %s", prt);
 		pfree(prt);
 	}
 	if (DebugMode)
-	{
-		puts("End InsertValue");
-		fflush(stdout);
-	}
+		elog(DEBUG, "inserted");
 }
 
 /* ----------------
@@ -733,10 +754,9 @@ void
 InsertOneNull(int i)
 {
 	if (DebugMode)
-		printf("Inserting null\n");
-	if (i < 0 || i >= MAXATTR)
-		elog(FATAL, "i out of range (too many attrs): %d\n", i);
-	values[i] = (char *) NULL;
+		elog(DEBUG, "inserting column %d NULL", i);
+	Assert(i >= 0 || i < MAXATTR);
+	values[i] = PointerGetDatum(NULL);
 	Blanks[i] = 'n';
 }
 
@@ -781,11 +801,11 @@ cleanup()
 		beenhere = 1;
 	else
 	{
-		elog(FATAL, "Memory manager fault: cleanup called twice.\n", stderr);
+		elog(FATAL, "Memory manager fault: cleanup called twice.\n");
 		proc_exit(1);
 	}
 	if (reldesc != (Relation) NULL)
-		heap_close(reldesc);
+		heap_close(reldesc, NoLock);
 	CommitTransactionCommand();
 	proc_exit(Warnings);
 }
@@ -807,7 +827,7 @@ gettype(char *type)
 	{
 		for (app = Typ; *app != (struct typmap *) NULL; app++)
 		{
-			if (strncmp((*app)->am_typ.typname.data, type, NAMEDATALEN) == 0)
+			if (strncmp(NameStr((*app)->am_typ.typname), type, NAMEDATALEN) == 0)
 			{
 				Ap = *app;
 				return (*app)->am_oid;
@@ -822,8 +842,8 @@ gettype(char *type)
 				return i;
 		}
 		if (DebugMode)
-			printf("bootstrap.c: External Type: %s\n", type);
-		rel = heap_openr(TypeRelationName);
+			elog(DEBUG, "external type: %s", type);
+		rel = heap_openr(TypeRelationName, NoLock);
 		scan = heap_beginscan(rel, 0, SnapshotNow, 0, (ScanKey) NULL);
 		i = 0;
 		while (HeapTupleIsValid(tup = heap_getnext(scan, 0)))
@@ -843,7 +863,7 @@ gettype(char *type)
 					sizeof((*app)->am_typ));
 		}
 		heap_endscan(scan);
-		heap_close(rel);
+		heap_close(rel, NoLock);
 		return gettype(type);
 	}
 	elog(ERROR, "Error: unknown type '%s'.\n", type);
@@ -976,7 +996,6 @@ FindStr(char *str, int length, hashnode *mderef)
 	node = hashtable[CompHash(str, length)];
 	while (node != NULL)
 	{
-
 		/*
 		 * We must differentiate between string constants that might have
 		 * the same value as a identifier and the identifier itself.
@@ -1073,16 +1092,9 @@ AddStr(char *str, int strlength, int mderef)
 void
 index_register(char *heap,
 			   char *ind,
-			   int natts,
-			   AttrNumber *attnos,
-			   uint16 nparams,
-			   Datum *params,
-			   FuncIndexInfo *finfo,
-			   PredInfo *predInfo)
+			   IndexInfo *indexInfo)
 {
-	Datum	   *v;
 	IndexList  *newind;
-	int			len;
 	MemoryContext oldcxt;
 
 	/*
@@ -1091,56 +1103,26 @@ index_register(char *heap,
 	 * them later.
 	 */
 
-	if (nogc == (GlobalMemory) NULL)
-		nogc = CreateGlobalMemory("BootstrapNoGC");
+	if (nogc == NULL)
+		nogc = AllocSetContextCreate((MemoryContext) NULL,
+									 "BootstrapNoGC",
+									 ALLOCSET_DEFAULT_MINSIZE,
+									 ALLOCSET_DEFAULT_INITSIZE,
+									 ALLOCSET_DEFAULT_MAXSIZE);
 
-	oldcxt = MemoryContextSwitchTo((MemoryContext) nogc);
+	oldcxt = MemoryContextSwitchTo(nogc);
 
 	newind = (IndexList *) palloc(sizeof(IndexList));
 	newind->il_heap = pstrdup(heap);
 	newind->il_ind = pstrdup(ind);
-	newind->il_natts = natts;
+	newind->il_info = (IndexInfo *) palloc(sizeof(IndexInfo));
 
-	if (PointerIsValid(finfo))
-		len = FIgetnArgs(finfo) * sizeof(AttrNumber);
-	else
-		len = natts * sizeof(AttrNumber);
-
-	newind->il_attnos = (AttrNumber *) palloc(len);
-	memmove(newind->il_attnos, attnos, len);
-
-	if ((newind->il_nparams = nparams) > 0)
-	{
-		v = newind->il_params = (Datum *) palloc(2 * nparams * sizeof(Datum));
-		nparams *= 2;
-		while (nparams-- > 0)
-		{
-			*v = (Datum) palloc(strlen((char *) (*params)) + 1);
-			strcpy((char *) *v++, (char *) *params++);
-		}
-	}
-	else
-		newind->il_params = (Datum *) NULL;
-
-	if (finfo != (FuncIndexInfo *) NULL)
-	{
-		newind->il_finfo = (FuncIndexInfo *) palloc(sizeof(FuncIndexInfo));
-		memmove(newind->il_finfo, finfo, sizeof(FuncIndexInfo));
-	}
-	else
-		newind->il_finfo = (FuncIndexInfo *) NULL;
-
-	if (predInfo != NULL)
-	{
-		newind->il_predInfo = (PredInfo *) palloc(sizeof(PredInfo));
-		newind->il_predInfo->pred = predInfo->pred;
-		newind->il_predInfo->oldPred = predInfo->oldPred;
-	}
-	else
-		newind->il_predInfo = NULL;
+	memcpy(newind->il_info, indexInfo, sizeof(IndexInfo));
+	/* predicate will likely be null, but may as well copy it */
+	newind->il_info->ii_Predicate = (List *)
+		copyObject(indexInfo->ii_Predicate);
 
 	newind->il_next = ILHead;
-
 	ILHead = newind;
 
 	MemoryContextSwitchTo(oldcxt);
@@ -1149,16 +1131,19 @@ index_register(char *heap,
 void
 build_indices()
 {
-	Relation	heap;
-	Relation	ind;
-
 	for (; ILHead != (IndexList *) NULL; ILHead = ILHead->il_next)
 	{
-		heap = heap_openr(ILHead->il_heap);
+		Relation	heap;
+		Relation	ind;
+
+		heap = heap_openr(ILHead->il_heap, NoLock);
 		ind = index_openr(ILHead->il_ind);
-		index_build(heap, ind, ILHead->il_natts, ILHead->il_attnos,
-				 ILHead->il_nparams, ILHead->il_params, ILHead->il_finfo,
-					ILHead->il_predInfo);
+		index_build(heap, ind, ILHead->il_info);
+
+		/*
+		 * In normal processing mode, index_build would close the heap and
+		 * index, but in bootstrap mode it will not.
+		 */
 
 		/*
 		 * All of the rest of this routine is needed only because in
@@ -1168,7 +1153,7 @@ build_indices()
 		 * Unfortunately, there are always two indices defined on each
 		 * catalog causing us to update the same pg_class tuple twice for
 		 * each catalog getting an index during bootstrap resulting in the
-		 * ghost tuple problem (see heap_replace).	To get around this we
+		 * ghost tuple problem (see heap_update).	To get around this we
 		 * change the relhasindex field ourselves in this routine keeping
 		 * track of what catalogs we already changed so that we don't
 		 * modify those tuples twice.  The normal mechanism for updating
@@ -1176,9 +1161,9 @@ build_indices()
 		 *
 		 * -mer
 		 */
-		heap = heap_openr(ILHead->il_heap);
-
 		if (!BootstrapAlreadySeen(RelationGetRelid(heap)))
-			UpdateStats(RelationGetRelid(heap), 0, true);
+			UpdateStats(RelationGetRelid(heap), 0);
+
+		/* XXX Probably we ought to close the heap and index here? */
 	}
 }
