@@ -1,6 +1,7 @@
-#ifndef Y_TAB_H
-# define Y_TAB_H
+#ifndef BISON_Y_TAB_H
+# define BISON_Y_TAB_H
 
+#ifndef YYSTYPE
 typedef union
 {
 	List		*list;
@@ -8,7 +9,9 @@ typedef union
 	char		*str;
 	int			ival;
 	Oid			oidval;
-} YYSTYPE;
+} Int_yystype;
+# define YYSTYPE Int_yystype
+#endif
 # define	CONST	257
 # define	ID	258
 # define	OPEN	259
@@ -38,4 +41,4 @@ typedef union
 
 extern YYSTYPE Int_yylval;
 
-#endif /* not Y_TAB_H */
+#endif /* not BISON_Y_TAB_H */
