@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/executor/execJunk.c,v 1.27 2001/03/22 06:16:12 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/executor/execJunk.c,v 1.29 2001/10/25 05:49:27 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -56,10 +56,12 @@
  * Initialize the Junk filter.
  *
  * The initial targetlist and associated tuple descriptor are passed in.
+ * An optional resultSlot can be passed as well.
  *-------------------------------------------------------------------------
  */
 JunkFilter *
-ExecInitJunkFilter(List *targetList, TupleDesc tupType)
+ExecInitJunkFilter(List *targetList, TupleDesc tupType,
+				   TupleTableSlot *slot)
 {
 	MemoryContext oldContext;
 	MemoryContext junkContext;
@@ -109,7 +111,6 @@ ExecInitJunkFilter(List *targetList, TupleDesc tupType)
 			resjunk = resdom->resjunk;
 			if (!resjunk)
 			{
-
 				/*
 				 * make a copy of the resdom node, changing its resno.
 				 */
@@ -245,6 +246,10 @@ ExecInitJunkFilter(List *targetList, TupleDesc tupType)
 	junkfilter->jf_cleanTupType = cleanTupType;
 	junkfilter->jf_cleanMap = cleanMap;
 	junkfilter->jf_junkContext = junkContext;
+	junkfilter->jf_resultSlot = slot;
+
+	if (slot)
+		ExecSetSlotDescriptor(slot, cleanTupType, false);
 
 	MemoryContextSwitchTo(oldContext);
 
@@ -260,7 +265,6 @@ ExecInitJunkFilter(List *targetList, TupleDesc tupType)
 void
 ExecFreeJunkFilter(JunkFilter *junkfilter)
 {
-
 	/*
 	 * Since the junkfilter is inside its own context, we just have to
 	 * delete the context and we're set.
@@ -336,6 +340,10 @@ ExecGetJunkAttribute(JunkFilter *junkfilter,
  * ExecRemoveJunk
  *
  * Construct and return a tuple with all the junk attributes removed.
+ *
+ * Note: for historical reasons, this does not store the constructed
+ * tuple into the junkfilter's resultSlot.  The caller should do that
+ * if it wants to.
  *-------------------------------------------------------------------------
  */
 HeapTuple
