@@ -3,36 +3,33 @@
  * nodeHash.c
  *	  Routines to hash relations for hashjoin
  *
- * Copyright (c) 1994, Regents of the University of California
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
- *	$Id: nodeHash.c,v 1.36 1999/05/25 16:08:41 momjian Exp $
+ *	$Id: nodeHash.c,v 1.60 2001/10/25 05:49:28 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 /*
  * INTERFACE ROUTINES
  *		ExecHash		- generate an in-memory hash table of the relation
- *		ExecInitHash	- initialize node and subnodes..
+ *		ExecInitHash	- initialize node and subnodes
  *		ExecEndHash		- shutdown node and subnodes
- *
  */
+#include "postgres.h"
 
 #include <sys/types.h>
-#include <stdio.h>
 #include <math.h>
-#include <string.h>
 
-#include "postgres.h"
-#include "miscadmin.h"
 #include "executor/execdebug.h"
-#include "executor/executor.h"
 #include "executor/nodeHash.h"
 #include "executor/nodeHashjoin.h"
-#include "utils/hsearch.h"
-#include "utils/portal.h"
+#include "miscadmin.h"
+#include "parser/parse_expr.h"
+#include "utils/memutils.h"
+#include "utils/lsyscache.h"
 
-extern int	SortMem;
 
 static int	hashFunc(Datum key, int len, bool byVal);
 
@@ -49,16 +46,15 @@ ExecHash(Hash *node)
 	EState	   *estate;
 	HashState  *hashstate;
 	Plan	   *outerNode;
-	Var		   *hashkey;
+	Node	   *hashkey;
 	HashJoinTable hashtable;
 	TupleTableSlot *slot;
 	ExprContext *econtext;
 	int			nbatch;
 	int			i;
 
-	/* ----------------
-	 *	get state info from node
-	 * ----------------
+	/*
+	 * get state info from node
 	 */
 
 	hashstate = node->hashstate;
@@ -73,30 +69,22 @@ ExecHash(Hash *node)
 
 	if (nbatch > 0)
 	{
-		/* ----------------
-		 * Open temp files for inner batches, if needed.
-		 * Note that file buffers are palloc'd in regular executor context.
-		 * ----------------
+		/*
+		 * Open temp files for inner batches, if needed. Note that file
+		 * buffers are palloc'd in regular executor context.
 		 */
 		for (i = 0; i < nbatch; i++)
-		{
-			File		tfile = OpenTemporaryFile();
-
-			Assert(tfile >= 0);
-			hashtable->innerBatchFile[i] = BufFileCreate(tfile);
-		}
+			hashtable->innerBatchFile[i] = BufFileCreateTemp();
 	}
 
-	/* ----------------
-	 *	set expression context
-	 * ----------------
+	/*
+	 * set expression context
 	 */
 	hashkey = node->hashkey;
 	econtext = hashstate->cstate.cs_ExprContext;
 
-	/* ----------------
-	 *	get all inner tuples and insert into the hash table (or temp files)
-	 * ----------------
+	/*
+	 * get all inner tuples and insert into the hash table (or temp files)
 	 */
 	for (;;)
 	{
@@ -108,10 +96,9 @@ ExecHash(Hash *node)
 		ExecClearTuple(slot);
 	}
 
-	/* ---------------------
-	 *	Return the slot so that we have the tuple descriptor
-	 *	when we need to save/restore them.	-Jeff 11 July 1991
-	 * ---------------------
+	/*
+	 * Return the slot so that we have the tuple descriptor when we need
+	 * to save/restore them.  -Jeff 11 July 1991
 	 */
 	return slot;
 }
@@ -131,48 +118,41 @@ ExecInitHash(Hash *node, EState *estate, Plan *parent)
 	SO1_printf("ExecInitHash: %s\n",
 			   "initializing hash node");
 
-	/* ----------------
-	 *	assign the node's execution state
-	 * ----------------
+	/*
+	 * assign the node's execution state
 	 */
 	node->plan.state = estate;
 
-	/* ----------------
+	/*
 	 * create state structure
-	 * ----------------
 	 */
 	hashstate = makeNode(HashState);
 	node->hashstate = hashstate;
 	hashstate->hashtable = NULL;
 
-	/* ----------------
-	 *	Miscellaneous initialization
+	/*
+	 * Miscellaneous initialization
 	 *
-	 *		 +	assign node's base_id
-	 *		 +	assign debugging hooks and
-	 *		 +	create expression context for node
-	 * ----------------
+	 * create expression context for node
 	 */
-	ExecAssignNodeBaseInfo(estate, &hashstate->cstate, parent);
 	ExecAssignExprContext(estate, &hashstate->cstate);
 
-	/* ----------------
+#define HASH_NSLOTS 1
+
+	/*
 	 * initialize our result slot
-	 * ----------------
 	 */
 	ExecInitResultTupleSlot(estate, &hashstate->cstate);
 
-	/* ----------------
+	/*
 	 * initializes child nodes
-	 * ----------------
 	 */
 	outerPlan = outerPlan(node);
 	ExecInitNode(outerPlan, estate, (Plan *) node);
 
-	/* ----------------
-	 *	initialize tuple type. no need to initialize projection
-	 *	info because this node doesn't do projections
-	 * ----------------
+	/*
+	 * initialize tuple type. no need to initialize projection info
+	 * because this node doesn't do projections
 	 */
 	ExecAssignResultTypeFromOuterPlan((Plan *) node, &hashstate->cstate);
 	hashstate->cstate.cs_ProjInfo = NULL;
@@ -183,10 +163,9 @@ ExecInitHash(Hash *node, EState *estate, Plan *parent)
 int
 ExecCountSlotsHash(Hash *node)
 {
-#define HASH_NSLOTS 1
 	return ExecCountSlotsNode(outerPlan(node)) +
-	ExecCountSlotsNode(innerPlan(node)) +
-	HASH_NSLOTS;
+		ExecCountSlotsNode(innerPlan(node)) +
+		HASH_NSLOTS;
 }
 
 /* ---------------------------------------------------------------
@@ -201,22 +180,20 @@ ExecEndHash(Hash *node)
 	HashState  *hashstate;
 	Plan	   *outerPlan;
 
-	/* ----------------
-	 *	get info from the hash state
-	 * ----------------
+	/*
+	 * get info from the hash state
 	 */
 	hashstate = node->hashstate;
 
-	/* ----------------
-	 *	free projection info.  no need to free result type info
-	 *	because that came from the outer plan...
-	 * ----------------
+	/*
+	 * free projection info.  no need to free result type info because
+	 * that came from the outer plan...
 	 */
 	ExecFreeProjectionInfo(&hashstate->cstate);
+	ExecFreeExprContext(&hashstate->cstate);
 
-	/* ----------------
-	 *	shut down the subplan
-	 * ----------------
+	/*
+	 * shut down the subplan
 	 */
 	outerPlan = outerPlan(node);
 	ExecEndNode(outerPlan, (Plan *) node);
@@ -229,47 +206,159 @@ ExecEndHash(Hash *node)
  *		create a hashtable in shared memory for hashjoin.
  * ----------------------------------------------------------------
  */
-#define NTUP_PER_BUCKET			10
-#define FUDGE_FAC				2.0
-
 HashJoinTable
 ExecHashTableCreate(Hash *node)
 {
+	HashJoinTable hashtable;
 	Plan	   *outerNode;
-	int			ntuples;
+	int			totalbuckets;
+	int			nbuckets;
+	int			nbatch;
+	int			i;
+	MemoryContext oldcxt;
+
+	/*
+	 * Get information about the size of the relation to be hashed (it's
+	 * the "outer" subtree of this node, but the inner relation of the
+	 * hashjoin).  Compute the appropriate size of the hash table.
+	 */
+	outerNode = outerPlan(node);
+
+	ExecChooseHashTableSize(outerNode->plan_rows, outerNode->plan_width,
+							&totalbuckets, &nbuckets, &nbatch);
+
+#ifdef HJDEBUG
+	printf("nbatch = %d, totalbuckets = %d, nbuckets = %d\n",
+		   nbatch, totalbuckets, nbuckets);
+#endif
+
+	/*
+	 * Initialize the hash table control block.
+	 *
+	 * The hashtable control block is just palloc'd from the executor's
+	 * per-query memory context.
+	 */
+	hashtable = (HashJoinTable) palloc(sizeof(HashTableData));
+	hashtable->nbuckets = nbuckets;
+	hashtable->totalbuckets = totalbuckets;
+	hashtable->buckets = NULL;
+	hashtable->nbatch = nbatch;
+	hashtable->curbatch = 0;
+	hashtable->innerBatchFile = NULL;
+	hashtable->outerBatchFile = NULL;
+	hashtable->innerBatchSize = NULL;
+	hashtable->outerBatchSize = NULL;
+
+	/*
+	 * Get info about the datatype of the hash key.
+	 */
+	get_typlenbyval(exprType(node->hashkey),
+					&hashtable->typLen,
+					&hashtable->typByVal);
+
+	/*
+	 * Create temporary memory contexts in which to keep the hashtable
+	 * working storage.  See notes in executor/hashjoin.h.
+	 */
+	hashtable->hashCxt = AllocSetContextCreate(CurrentMemoryContext,
+											   "HashTableContext",
+											   ALLOCSET_DEFAULT_MINSIZE,
+											   ALLOCSET_DEFAULT_INITSIZE,
+											   ALLOCSET_DEFAULT_MAXSIZE);
+
+	hashtable->batchCxt = AllocSetContextCreate(hashtable->hashCxt,
+												"HashBatchContext",
+												ALLOCSET_DEFAULT_MINSIZE,
+												ALLOCSET_DEFAULT_INITSIZE,
+												ALLOCSET_DEFAULT_MAXSIZE);
+
+	/* Allocate data that will live for the life of the hashjoin */
+
+	oldcxt = MemoryContextSwitchTo(hashtable->hashCxt);
+
+	if (nbatch > 0)
+	{
+		/*
+		 * allocate and initialize the file arrays in hashCxt
+		 */
+		hashtable->innerBatchFile = (BufFile **)
+			palloc(nbatch * sizeof(BufFile *));
+		hashtable->outerBatchFile = (BufFile **)
+			palloc(nbatch * sizeof(BufFile *));
+		hashtable->innerBatchSize = (long *)
+			palloc(nbatch * sizeof(long));
+		hashtable->outerBatchSize = (long *)
+			palloc(nbatch * sizeof(long));
+		for (i = 0; i < nbatch; i++)
+		{
+			hashtable->innerBatchFile[i] = NULL;
+			hashtable->outerBatchFile[i] = NULL;
+			hashtable->innerBatchSize[i] = 0;
+			hashtable->outerBatchSize[i] = 0;
+		}
+		/* The files will not be opened until later... */
+	}
+
+	/*
+	 * Prepare context for the first-scan space allocations; allocate the
+	 * hashbucket array therein, and set each bucket "empty".
+	 */
+	MemoryContextSwitchTo(hashtable->batchCxt);
+
+	hashtable->buckets = (HashJoinTuple *)
+		palloc(nbuckets * sizeof(HashJoinTuple));
+
+	if (hashtable->buckets == NULL)
+		elog(ERROR, "Insufficient memory for hash table.");
+
+	for (i = 0; i < nbuckets; i++)
+		hashtable->buckets[i] = NULL;
+
+	MemoryContextSwitchTo(oldcxt);
+
+	return hashtable;
+}
+
+
+/*
+ * Compute appropriate size for hashtable given the estimated size of the
+ * relation to be hashed (number of rows and average row width).
+ *
+ * Caution: the input is only the planner's estimates, and so can't be
+ * trusted too far.  Apply a healthy fudge factor.
+ *
+ * This is exported so that the planner's costsize.c can use it.
+ */
+
+/* Target bucket loading (tuples per bucket) */
+#define NTUP_PER_BUCKET			10
+/* Fudge factor to allow for inaccuracy of input estimates */
+#define FUDGE_FAC				2.0
+
+void
+ExecChooseHashTableSize(double ntuples, int tupwidth,
+						int *virtualbuckets,
+						int *physicalbuckets,
+						int *numbatches)
+{
 	int			tupsize;
 	double		inner_rel_bytes;
 	double		hash_table_bytes;
 	int			nbatch;
-	HashJoinTable hashtable;
 	int			nbuckets;
 	int			totalbuckets;
 	int			bucketsize;
-	int			i;
-	Portal		myPortal;
-	char		myPortalName[64];
-	MemoryContext oldcxt;
 
-	/* ----------------
-	 *	Get information about the size of the relation to be hashed
-	 *	(it's the "outer" subtree of this node, but the inner relation of
-	 *	the hashjoin).
-	 *	Caution: this is only the planner's estimates, and so
-	 *	can't be trusted too far.  Apply a healthy fudge factor.
-	 * ----------------
-	 */
-	outerNode = outerPlan(node);
-	ntuples = outerNode->plan_size;
-	if (ntuples <= 0)			/* force a plausible size if no info */
-		ntuples = 1000;
+	/* Force a plausible relation size if no info */
+	if (ntuples <= 0.0)
+		ntuples = 1000.0;
 
 	/*
-	 * estimate tupsize based on footprint of tuple in hashtable... but
+	 * Estimate tupsize based on footprint of tuple in hashtable... but
 	 * what about palloc overhead?
 	 */
-	tupsize = MAXALIGN(outerNode->plan_width) +
-		MAXALIGN(sizeof(HashJoinTupleData));
-	inner_rel_bytes = (double) ntuples *tupsize * FUDGE_FAC;
+	tupsize = MAXALIGN(tupwidth) + MAXALIGN(sizeof(HashJoinTupleData));
+	inner_rel_bytes = ntuples * tupsize * FUDGE_FAC;
 
 	/*
 	 * Target hashtable size is SortMem kilobytes, but not less than
@@ -285,7 +374,7 @@ ExecHashTableCreate(Hash *node)
 	 * for an average bucket load of NTUP_PER_BUCKET (per virtual
 	 * bucket!).
 	 */
-	totalbuckets = (int) ceil((double) ntuples * FUDGE_FAC / NTUP_PER_BUCKET);
+	totalbuckets = (int) ceil(ntuples * FUDGE_FAC / NTUP_PER_BUCKET);
 
 	/*
 	 * Count the number of buckets we think will actually fit in the
@@ -303,7 +392,6 @@ ExecHashTableCreate(Hash *node)
 
 	if (totalbuckets <= nbuckets)
 	{
-
 		/*
 		 * We have enough space, so no batching.  In theory we could even
 		 * reduce nbuckets, but since that could lead to poor behavior if
@@ -315,7 +403,6 @@ ExecHashTableCreate(Hash *node)
 	}
 	else
 	{
-
 		/*
 		 * Need to batch; compute how many batches we want to use. Note
 		 * that nbatch doesn't have to have anything to do with the ratio
@@ -337,95 +424,12 @@ ExecHashTableCreate(Hash *node)
 	 * everything else gets divided into nbatch batches to be processed in
 	 * additional passes.
 	 */
-#ifdef HJDEBUG
-	printf("nbatch = %d, totalbuckets = %d, nbuckets = %d\n",
-		   nbatch, totalbuckets, nbuckets);
-#endif
-
-	/* ----------------
-	 *	Initialize the hash table control block.
-	 *	The hashtable control block is just palloc'd from executor memory.
-	 * ----------------
-	 */
-	hashtable = (HashJoinTable) palloc(sizeof(HashTableData));
-	hashtable->nbuckets = nbuckets;
-	hashtable->totalbuckets = totalbuckets;
-	hashtable->buckets = NULL;
-	hashtable->nbatch = nbatch;
-	hashtable->curbatch = 0;
-	hashtable->innerBatchFile = NULL;
-	hashtable->outerBatchFile = NULL;
-	hashtable->innerBatchSize = NULL;
-	hashtable->outerBatchSize = NULL;
-
-	/* ----------------
-	 *	Create a named portal in which to keep the hashtable working storage.
-	 *	Each hashjoin must have its own portal, so be wary of name conflicts.
-	 * ----------------
-	 */
-	i = 0;
-	do
-	{
-		i++;
-		sprintf(myPortalName, "<hashtable %d>", i);
-		myPortal = GetPortalByName(myPortalName);
-	} while (PortalIsValid(myPortal));
-	myPortal = CreatePortal(myPortalName);
-	Assert(PortalIsValid(myPortal));
-	hashtable->myPortal = (void *) myPortal;	/* kluge for circular
-												 * includes */
-	hashtable->hashCxt = (MemoryContext) PortalGetVariableMemory(myPortal);
-	hashtable->batchCxt = (MemoryContext) PortalGetHeapMemory(myPortal);
-
-	/* Allocate data that will live for the life of the hashjoin */
-
-	oldcxt = MemoryContextSwitchTo(hashtable->hashCxt);
-
-	if (nbatch > 0)
-	{
-		/* ---------------
-		 *	allocate and initialize the file arrays in hashCxt
-		 * ---------------
-		 */
-		hashtable->innerBatchFile = (BufFile **)
-			palloc(nbatch * sizeof(BufFile *));
-		hashtable->outerBatchFile = (BufFile **)
-			palloc(nbatch * sizeof(BufFile *));
-		hashtable->innerBatchSize = (long *)
-			palloc(nbatch * sizeof(long));
-		hashtable->outerBatchSize = (long *)
-			palloc(nbatch * sizeof(long));
-		for (i = 0; i < nbatch; i++)
-		{
-			hashtable->innerBatchFile[i] = NULL;
-			hashtable->outerBatchFile[i] = NULL;
-			hashtable->innerBatchSize[i] = 0;
-			hashtable->outerBatchSize[i] = 0;
-		}
-		/* The files will not be opened until later... */
-	}
-
-	/*
-	 * Prepare portal for the first-scan space allocations; allocate the
-	 * hashbucket array therein, and set each bucket "empty".
-	 */
-	MemoryContextSwitchTo(hashtable->batchCxt);
-	StartPortalAllocMode(DefaultAllocMode, 0);
-
-	hashtable->buckets = (HashJoinTuple *)
-		palloc(nbuckets * sizeof(HashJoinTuple));
-
-	if (hashtable->buckets == NULL)
-		elog(ERROR, "Insufficient memory for hash table.");
-
-	for (i = 0; i < nbuckets; i++)
-		hashtable->buckets[i] = NULL;
-
-	MemoryContextSwitchTo(oldcxt);
-
-	return hashtable;
+	*virtualbuckets = totalbuckets;
+	*physicalbuckets = nbuckets;
+	*numbatches = nbatch;
 }
 
+
 /* ----------------------------------------------------------------
  *		ExecHashTableDestroy
  *
@@ -446,9 +450,8 @@ ExecHashTableDestroy(HashJoinTable hashtable)
 			BufFileClose(hashtable->outerBatchFile[i]);
 	}
 
-	/* Destroy the portal to release all working memory */
-	/* cast here is a kluge for circular includes... */
-	PortalDestroy((Portal *) &hashtable->myPortal);
+	/* Release working memory (batchCxt is a child, so it goes away too) */
+	MemoryContextDelete(hashtable->hashCxt);
 
 	/* And drop the control block */
 	pfree(hashtable);
@@ -464,21 +467,19 @@ ExecHashTableDestroy(HashJoinTable hashtable)
 void
 ExecHashTableInsert(HashJoinTable hashtable,
 					ExprContext *econtext,
-					Var *hashkey)
+					Node *hashkey)
 {
 	int			bucketno = ExecHashGetBucket(hashtable, econtext, hashkey);
 	TupleTableSlot *slot = econtext->ecxt_innertuple;
 	HeapTuple	heapTuple = slot->val;
 
-	/* ----------------
-	 *	decide whether to put the tuple in the hash table or a tmp file
-	 * ----------------
+	/*
+	 * decide whether to put the tuple in the hash table or a tmp file
 	 */
 	if (bucketno < hashtable->nbuckets)
 	{
-		/* ---------------
-		 *	put the tuple in hash table
-		 * ---------------
+		/*
+		 * put the tuple in hash table
 		 */
 		HashJoinTuple hashTuple;
 		int			hashTupleSize;
@@ -491,6 +492,7 @@ ExecHashTableInsert(HashJoinTable hashtable,
 		memcpy((char *) &hashTuple->htup,
 			   (char *) heapTuple,
 			   sizeof(hashTuple->htup));
+		hashTuple->htup.t_datamcxt = hashtable->batchCxt;
 		hashTuple->htup.t_data = (HeapTupleHeader)
 			(((char *) hashTuple) + MAXALIGN(sizeof(*hashTuple)));
 		memcpy((char *) hashTuple->htup.t_data,
@@ -501,9 +503,8 @@ ExecHashTableInsert(HashJoinTable hashtable,
 	}
 	else
 	{
-		/* -----------------
+		/*
 		 * put the tuple into a tmp file for other batches
-		 * -----------------
 		 */
 		int			batchno = (hashtable->nbatch * (bucketno - hashtable->nbuckets)) /
 		(hashtable->totalbuckets - hashtable->nbuckets);
@@ -523,45 +524,48 @@ ExecHashTableInsert(HashJoinTable hashtable,
 int
 ExecHashGetBucket(HashJoinTable hashtable,
 				  ExprContext *econtext,
-				  Var *hashkey)
+				  Node *hashkey)
 {
 	int			bucketno;
 	Datum		keyval;
 	bool		isNull;
-
-	/* ----------------
-	 *	Get the join attribute value of the tuple
-	 *
-	 * ...It's quick hack - use ExecEvalExpr instead of ExecEvalVar:
-	 * hashkey may be T_ArrayRef, not just T_Var.		- vadim 04/22/97
-	 * ----------------
-	 */
-	keyval = ExecEvalExpr((Node *) hashkey, econtext, &isNull, NULL);
+	MemoryContext oldContext;
 
 	/*
-	 * keyval could be null, so we better point it to something valid
-	 * before trying to run hashFunc on it. --djm 8/17/96
+	 * We reset the eval context each time to reclaim any memory leaked in
+	 * the hashkey expression or hashFunc itself.
+	 */
+	ResetExprContext(econtext);
+
+	oldContext = MemoryContextSwitchTo(econtext->ecxt_per_tuple_memory);
+
+	/*
+	 * Get the join attribute value of the tuple
+	 */
+	keyval = ExecEvalExpr(hashkey, econtext, &isNull, NULL);
+
+	/*
+	 * Compute the hash function
 	 */
 	if (isNull)
+		bucketno = 0;
+	else
 	{
-		execConstByVal = 0;
-		execConstLen = 0;
-		keyval = (Datum) "";
+		bucketno = hashFunc(keyval,
+							(int) hashtable->typLen,
+							hashtable->typByVal)
+			% hashtable->totalbuckets;
 	}
 
-	/* ------------------
-	 *	compute the hash function
-	 * ------------------
-	 */
-	bucketno = hashFunc(keyval, execConstLen, execConstByVal) % hashtable->totalbuckets;
-
 #ifdef HJDEBUG
 	if (bucketno >= hashtable->nbuckets)
-		printf("hash(%d) = %d SAVED\n", keyval, bucketno);
+		printf("hash(%ld) = %d SAVED\n", (long) keyval, bucketno);
 	else
-		printf("hash(%d) = %d\n", keyval, bucketno);
+		printf("hash(%ld) = %d\n", (long) keyval, bucketno);
 #endif
 
+	MemoryContextSwitchTo(oldContext);
+
 	return bucketno;
 }
 
@@ -592,7 +596,6 @@ ExecScanHashBucket(HashJoinState *hjstate,
 	{
 		HeapTuple	heapTuple = &hashTuple->htup;
 		TupleTableSlot *inntuple;
-		bool		qualResult;
 
 		/* insert hashtable's tuple into exec slot so ExecQual sees it */
 		inntuple = ExecStoreTuple(heapTuple,	/* tuple to store */
@@ -601,9 +604,10 @@ ExecScanHashBucket(HashJoinState *hjstate,
 								  false);		/* do not pfree this tuple */
 		econtext->ecxt_innertuple = inntuple;
 
-		qualResult = ExecQual(hjclauses, econtext);
+		/* reset temp memory each time to avoid leaks from qual expression */
+		ResetExprContext(econtext);
 
-		if (qualResult)
+		if (ExecQual(hjclauses, econtext, false))
 		{
 			hjstate->hj_CurTuple = hashTuple;
 			return heapTuple;
@@ -612,9 +616,8 @@ ExecScanHashBucket(HashJoinState *hjstate,
 		hashTuple = hashTuple->next;
 	}
 
-	/* ----------------
-	 *	no match
-	 * ----------------
+	/*
+	 * no match
 	 */
 	return NULL;
 }
@@ -623,17 +626,18 @@ ExecScanHashBucket(HashJoinState *hjstate,
  *		hashFunc
  *
  *		the hash function, copied from Margo
+ *
+ *		XXX this probably ought to be replaced with datatype-specific
+ *		hash functions, such as those already implemented for hash indexes.
  * ----------------------------------------------------------------
  */
 static int
 hashFunc(Datum key, int len, bool byVal)
 {
 	unsigned int h = 0;
-	unsigned char *k;
 
 	if (byVal)
 	{
-
 		/*
 		 * If it's a by-value data type, use the 'len' least significant
 		 * bytes of the Datum value.  This should do the right thing on
@@ -648,22 +652,29 @@ hashFunc(Datum key, int len, bool byVal)
 	}
 	else
 	{
-
 		/*
-		 * If this is a variable length type, then 'k' points to a "struct
-		 * varlena" and len == -1. NOTE: VARSIZE returns the "real" data
-		 * length plus the sizeof the "vl_len" attribute of varlena (the
-		 * length information). 'k' points to the beginning of the varlena
-		 * struct, so we have to use "VARDATA" to find the beginning of
-		 * the "real" data.
+		 * If this is a variable length type, then 'key' points to a
+		 * "struct varlena" and len == -1.	NOTE: VARSIZE returns the
+		 * "real" data length plus the sizeof the "vl_len" attribute of
+		 * varlena (the length information). 'key' points to the beginning
+		 * of the varlena struct, so we have to use "VARDATA" to find the
+		 * beginning of the "real" data.  Also, we have to be careful to
+		 * detoast the datum if it's toasted.  (We don't worry about
+		 * freeing the detoasted copy; that happens for free when the
+		 * per-tuple memory context is reset in ExecHashGetBucket.)
 		 */
-		if (len == -1)
+		unsigned char *k;
+
+		if (len < 0)
 		{
-			len = VARSIZE(key) - VARHDRSZ;
-			k = (unsigned char *) VARDATA(key);
+			struct varlena *vkey = PG_DETOAST_DATUM(key);
+
+			len = VARSIZE(vkey) - VARHDRSZ;
+			k = (unsigned char *) VARDATA(vkey);
 		}
 		else
-			k = (unsigned char *) key;
+			k = (unsigned char *) DatumGetPointer(key);
+
 		while (len-- > 0)
 			h = (h * PRIME1) ^ (*k++);
 	}
@@ -689,11 +700,10 @@ ExecHashTableReset(HashJoinTable hashtable, long ntuples)
 
 	/*
 	 * Release all the hash buckets and tuples acquired in the prior pass,
-	 * and reinitialize the portal for a new pass.
+	 * and reinitialize the context for a new pass.
 	 */
+	MemoryContextReset(hashtable->batchCxt);
 	oldcxt = MemoryContextSwitchTo(hashtable->batchCxt);
-	EndPortalAllocMode();
-	StartPortalAllocMode(DefaultAllocMode, 0);
 
 	/*
 	 * We still use the same number of physical buckets as in the first
@@ -723,7 +733,6 @@ ExecHashTableReset(HashJoinTable hashtable, long ntuples)
 void
 ExecReScanHash(Hash *node, ExprContext *exprCtxt, Plan *parent)
 {
-
 	/*
 	 * if chgParam of subnode is not null then plan will be re-scanned by
 	 * first ExecProcNode.
