@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/executor/nodeIndexscan.c,v 1.59 2001/03/22 06:16:13 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/executor/nodeIndexscan.c,v 1.66 2002/02/11 20:10:48 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -173,7 +173,7 @@ IndexNext(IndexScan *node)
 		while ((result = index_getnext(scandesc, direction)) != NULL)
 		{
 			tuple->t_self = result->heap_iptr;
-			heap_fetch(heapRelation, snapshot, tuple, &buffer);
+			heap_fetch(heapRelation, snapshot, tuple, &buffer, scandesc);
 			pfree(result);
 
 			if (tuple->t_data != NULL)
@@ -222,7 +222,7 @@ IndexNext(IndexScan *node)
 					qual = lnext(qual);
 				}
 				if (!prev_matches)
-					return slot;/* OK to return tuple */
+					return slot;	/* OK to return tuple */
 				/* Duplicate tuple, so drop it and loop back for another */
 				ExecClearTuple(slot);
 			}
@@ -301,24 +301,13 @@ ExecIndexReScan(IndexScan *node, ExprContext *exprCtxt, Plan *parent)
 	IndexScanState *indexstate;
 	ExprContext *econtext;
 	ScanDirection direction;
+	int			numIndices;
 	IndexScanDescPtr scanDescs;
 	ScanKey    *scanKeys;
-	IndexScanDesc scan;
-	ScanKey		skey;
-	int			numIndices;
-	int			i;
 	int		  **runtimeKeyInfo;
 	int		   *numScanKeys;
-	List	   *indxqual;
-	List	   *qual;
-	int			n_keys;
-	ScanKey		scan_keys;
-	int		   *run_keys;
+	int			i;
 	int			j;
-	Expr	   *clause;
-	Node	   *scanexpr;
-	Datum		scanvalue;
-	bool		isNull;
 
 	estate = node->scan.plan.state;
 	indexstate = node->indxstate;
@@ -330,20 +319,22 @@ ExecIndexReScan(IndexScan *node, ExprContext *exprCtxt, Plan *parent)
 	scanKeys = indexstate->iss_ScanKeys;
 	runtimeKeyInfo = indexstate->iss_RuntimeKeyInfo;
 	numScanKeys = indexstate->iss_NumScanKeys;
-	if (ScanDirectionIsBackward(node->indxorderdir))
-		indexstate->iss_IndexPtr = numIndices;
-	else
-		indexstate->iss_IndexPtr = -1;
 
 	if (econtext)
 	{
-
 		/*
 		 * If we are being passed an outer tuple, save it for runtime key
-		 * calc
+		 * calc.  We also need to link it into the "regular" per-tuple
+		 * econtext, so it can be used during indexqualorig evaluations.
 		 */
 		if (exprCtxt != NULL)
+		{
+			ExprContext *stdecontext;
+
 			econtext->ecxt_outertuple = exprCtxt->ecxt_outertuple;
+			stdecontext = node->scan.scanstate->cstate.cs_ExprContext;
+			stdecontext->ecxt_outertuple = exprCtxt->ecxt_outertuple;
+		}
 
 		/*
 		 * Reset the runtime-key context so we don't leak memory as each
@@ -353,31 +344,29 @@ ExecIndexReScan(IndexScan *node, ExprContext *exprCtxt, Plan *parent)
 		ResetExprContext(econtext);
 	}
 
-	/* If this is re-scanning of PlanQual ... */
-	if (estate->es_evTuple != NULL &&
-		estate->es_evTuple[node->scan.scanrelid - 1] != NULL)
-	{
-		estate->es_evTupleNull[node->scan.scanrelid - 1] = false;
-		return;
-	}
-
 	/*
-	 * get the index qualifications and recalculate the appropriate values
+	 * If we are doing runtime key calculations (ie, the index keys depend
+	 * on data from an outer scan), compute the new key values
 	 */
-	indxqual = node->indxqual;
-	for (i = 0; i < numIndices; i++)
+	if (runtimeKeyInfo)
 	{
-		qual = lfirst(indxqual);
-		indxqual = lnext(indxqual);
-		n_keys = numScanKeys[i];
-		scan_keys = (ScanKey) scanKeys[i];
+		List	   *indxqual;
 
-		if (runtimeKeyInfo)
+		indxqual = node->indxqual;
+		for (i = 0; i < numIndices; i++)
 		{
+			List	   *qual = lfirst(indxqual);
+			int			n_keys;
+			ScanKey		scan_keys;
+			int		   *run_keys;
+
+			indxqual = lnext(indxqual);
+			n_keys = numScanKeys[i];
+			scan_keys = scanKeys[i];
 			run_keys = runtimeKeyInfo[i];
+
 			for (j = 0; j < n_keys; j++)
 			{
-
 				/*
 				 * If we have a run-time key, then extract the run-time
 				 * expression and evaluate it with respect to the current
@@ -393,7 +382,11 @@ ExecIndexReScan(IndexScan *node, ExprContext *exprCtxt, Plan *parent)
 				 */
 				if (run_keys[j] != NO_OP)
 				{
-					clause = nth(j, qual);
+					Expr	   *clause = nth(j, qual);
+					Node	   *scanexpr;
+					Datum		scanvalue;
+					bool		isNull;
+
 					scanexpr = (run_keys[j] == RIGHT_OP) ?
 						(Node *) get_rightop(clause) :
 						(Node *) get_leftop(clause);
@@ -410,13 +403,31 @@ ExecIndexReScan(IndexScan *node, ExprContext *exprCtxt, Plan *parent)
 				}
 			}
 		}
-		scan = scanDescs[i];
-		skey = scanKeys[i];
-		index_rescan(scan, direction, skey);
+
+		indexstate->iss_RuntimeKeysReady = true;
 	}
 
-	if (runtimeKeyInfo)
-		indexstate->iss_RuntimeKeysReady = true;
+	/* If this is re-scanning of PlanQual ... */
+	if (estate->es_evTuple != NULL &&
+		estate->es_evTuple[node->scan.scanrelid - 1] != NULL)
+	{
+		estate->es_evTupleNull[node->scan.scanrelid - 1] = false;
+		return;
+	}
+
+	/* reset index scans */
+	if (ScanDirectionIsBackward(node->indxorderdir))
+		indexstate->iss_IndexPtr = numIndices;
+	else
+		indexstate->iss_IndexPtr = -1;
+
+	for (i = 0; i < numIndices; i++)
+	{
+		IndexScanDesc scan = scanDescs[i];
+		ScanKey		skey = scanKeys[i];
+
+		index_rescan(scan, direction, skey);
+	}
 }
 
 /* ----------------------------------------------------------------
@@ -615,7 +626,7 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 	 */
 	ExecAssignExprContext(estate, &scanstate->cstate);
 
-#define INDEXSCAN_NSLOTS 3
+#define INDEXSCAN_NSLOTS 2
 
 	/*
 	 * tuple table initialization
@@ -703,10 +714,10 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 			Expr	   *clause; /* one clause of index qual */
 			Oper	   *op;		/* operator used in clause */
 			Node	   *leftop; /* expr on lhs of operator */
-			Node	   *rightop;/* expr on rhs ... */
+			Node	   *rightop;	/* expr on rhs ... */
 			bits16		flags = 0;
 
-			int			scanvar;/* which var identifies varattno */
+			int			scanvar;	/* which var identifies varattno */
 			AttrNumber	varattno = 0;	/* att number used in scan */
 			Oid			opid;	/* operator id used in scan */
 			Datum		scanvalue = 0;	/* value used in scan (if const) */
@@ -772,7 +783,6 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 
 			if (IsA(leftop, Var) &&var_is_rel((Var *) leftop))
 			{
-
 				/*
 				 * if the leftop is a "rel-var", then it means that it is
 				 * a var node which tells us which attribute to use for
@@ -783,7 +793,6 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 			}
 			else if (IsA(leftop, Const))
 			{
-
 				/*
 				 * if the leftop is a const node then it means it
 				 * identifies the value to place in our scan key.
@@ -820,7 +829,6 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 			}
 			else
 			{
-
 				/*
 				 * otherwise, the leftop contains an expression evaluable
 				 * at runtime to figure out the value to place in our scan
@@ -842,7 +850,6 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 
 			if (IsA(rightop, Var) &&var_is_rel((Var *) rightop))
 			{
-
 				/*
 				 * here we make sure only one op identifies the
 				 * scan-attribute...
@@ -861,7 +868,6 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 			}
 			else if (IsA(rightop, Const))
 			{
-
 				/*
 				 * if the rightop is a const node then it means it
 				 * identifies the value to place in our scan key.
@@ -898,7 +904,6 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 			}
 			else
 			{
-
 				/*
 				 * otherwise, the rightop contains an expression evaluable
 				 * at runtime to figure out the value to place in our scan
@@ -993,7 +998,7 @@ ExecInitIndexScan(IndexScan *node, EState *estate, Plan *parent)
 	ExecOpenScanR(reloid,		/* relation */
 				  0,			/* nkeys */
 				  (ScanKey) NULL,		/* scan key */
-				  0,			/* is index */
+				  false,		/* is index */
 				  direction,	/* scan direction */
 				  estate->es_snapshot,	/* */
 				  &currentRelation,		/* return: rel desc */
@@ -1045,5 +1050,5 @@ int
 ExecCountSlotsIndexScan(IndexScan *node)
 {
 	return ExecCountSlotsNode(outerPlan((Plan *) node)) +
-	ExecCountSlotsNode(innerPlan((Plan *) node)) + INDEXSCAN_NSLOTS;
+		ExecCountSlotsNode(innerPlan((Plan *) node)) + INDEXSCAN_NSLOTS;
 }
