@@ -2,56 +2,71 @@
  *
  * dllist.c
  *	  this is a simple doubly linked list implementation
- *	  replaces the old simplelists stuff
  *	  the elements of the lists are void*
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/lib/dllist.c,v 1.17 2000/04/12 17:15:10 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/lib/dllist.c,v 1.24 2001/07/31 02:02:45 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
 
-#include "postgres.h"
-#include "lib/dllist.h"
-
-/* When this file is compiled for inclusion in libpq,
- * it can't use assert checking.  Probably this fix ought to be
- * in c.h or somewhere like that...
- */
+/* can be used in frontend or backend */
 #ifdef FRONTEND
-#undef Assert
+#include "postgres_fe.h"
+/* No assert checks in frontend ... */
 #define Assert(condition)
+#else
+#include "postgres.h"
 #endif
 
+#include "lib/dllist.h"
+
 
 Dllist *
 DLNewList(void)
 {
 	Dllist	   *l;
 
-	l = malloc(sizeof(Dllist));
+	l = (Dllist *) malloc(sizeof(Dllist));
+	if (l == NULL)
+	{
+#ifdef FRONTEND
+		fprintf(stderr, "Memory exhausted in DLNewList\n");
+		exit(1);
+#else
+		elog(ERROR, "Memory exhausted in DLNewList");
+#endif
+	}
 	l->dll_head = 0;
 	l->dll_tail = 0;
 
 	return l;
 }
 
-/* free up a list and all the nodes in it --- but *not* whatever the nodes
+void
+DLInitList(Dllist *list)
+{
+	list->dll_head = 0;
+	list->dll_tail = 0;
+}
+
+/*
+ * free up a list and all the nodes in it --- but *not* whatever the nodes
  * might point to!
  */
 void
-DLFreeList(Dllist *l)
+DLFreeList(Dllist *list)
 {
 	Dlelem	   *curr;
 
-	while ((curr = DLRemHead(l)) != 0)
+	while ((curr = DLRemHead(list)) != 0)
 		free(curr);
 
-	free(l);
+	free(list);
 }
 
 Dlelem *
@@ -59,7 +74,16 @@ DLNewElem(void *val)
 {
 	Dlelem	   *e;
 
-	e = malloc(sizeof(Dlelem));
+	e = (Dlelem *) malloc(sizeof(Dlelem));
+	if (e == NULL)
+	{
+#ifdef FRONTEND
+		fprintf(stderr, "Memory exhausted in DLNewElem\n");
+		exit(1);
+#else
+		elog(ERROR, "Memory exhausted in DLNewElem");
+#endif
+	}
 	e->dle_next = 0;
 	e->dle_prev = 0;
 	e->dle_val = val;
@@ -67,60 +91,21 @@ DLNewElem(void *val)
 	return e;
 }
 
+void
+DLInitElem(Dlelem *e, void *val)
+{
+	e->dle_next = 0;
+	e->dle_prev = 0;
+	e->dle_val = val;
+	e->dle_list = 0;
+}
+
 void
 DLFreeElem(Dlelem *e)
 {
 	free(e);
 }
 
-Dlelem *
-DLGetHead(Dllist *l)
-{
-	return l ? l->dll_head : 0;
-}
-
-/* get the value stored in the first element */
-#ifdef NOT_USED
-void *
-DLGetHeadVal(Dllist *l)
-{
-	Dlelem	   *e = DLGetHead(l);
-
-	return e ? e->dle_val : 0;
-}
-
-#endif
-
-Dlelem *
-DLGetTail(Dllist *l)
-{
-	return l ? l->dll_tail : 0;
-}
-
-/* get the value stored in the last element */
-#ifdef NOT_USED
-void *
-DLGetTailVal(Dllist *l)
-{
-	Dlelem	   *e = DLGetTail(l);
-
-	return e ? e->dle_val : 0;
-}
-
-#endif
-
-Dlelem *
-DLGetPred(Dlelem *e)			/* get predecessor */
-{
-	return e ? e->dle_prev : 0;
-}
-
-Dlelem *
-DLGetSucc(Dlelem *e)			/* get successor */
-{
-	return e ? e->dle_next : 0;
-}
-
 void
 DLRemove(Dlelem *e)
 {
@@ -129,16 +114,16 @@ DLRemove(Dlelem *e)
 	if (e->dle_prev)
 		e->dle_prev->dle_next = e->dle_next;
 	else
-/* must be the head element */
 	{
+		/* must be the head element */
 		Assert(e == l->dll_head);
 		l->dll_head = e->dle_next;
 	}
 	if (e->dle_next)
 		e->dle_next->dle_prev = e->dle_prev;
 	else
-/* must be the tail element */
 	{
+		/* must be the tail element */
 		Assert(e == l->dll_tail);
 		l->dll_tail = e->dle_prev;
 	}
@@ -192,12 +177,12 @@ DLRemHead(Dllist *l)
 
 	l->dll_head = result->dle_next;
 
-	result->dle_next = 0;
-	result->dle_list = 0;
-
 	if (result == l->dll_tail)	/* if the head is also the tail */
 		l->dll_tail = 0;
 
+	result->dle_next = 0;
+	result->dle_list = 0;
+
 	return result;
 }
 
@@ -215,12 +200,12 @@ DLRemTail(Dllist *l)
 
 	l->dll_tail = result->dle_prev;
 
-	result->dle_prev = 0;
-	result->dle_list = 0;
-
 	if (result == l->dll_head)	/* if the tail is also the head */
 		l->dll_head = 0;
 
+	result->dle_prev = 0;
+	result->dle_list = 0;
+
 	return result;
 }
 
@@ -239,8 +224,8 @@ DLMoveToFront(Dlelem *e)
 	if (e->dle_next)
 		e->dle_next->dle_prev = e->dle_prev;
 	else
-/* must be the tail element */
 	{
+		/* must be the tail element */
 		Assert(e == l->dll_tail);
 		l->dll_tail = e->dle_prev;
 	}
