@@ -8,51 +8,59 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/libpq/auth.c,v 1.52 2001/03/22 03:59:30 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/libpq/auth.c,v 1.72.2.1 2002/02/25 20:07:33 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
-/*
- * INTERFACE ROUTINES
- *
- *	   backend (postmaster) routines:
- *		be_recvauth				receive authentication information
- */
-#include <sys/param.h>			/* for MAXHOSTNAMELEN on most */
-#ifndef  MAXHOSTNAMELEN
-#include <netdb.h>				/* for MAXHOSTNAMELEN on some */
-#endif
-#include <pwd.h>
-#include <ctype.h>
-
-#include <sys/types.h>			/* needed by in.h on Ultrix */
-#include <netinet/in.h>
-#include <arpa/inet.h>
 
 #include "postgres.h"
 
+#include <sys/types.h>
+#include <sys/param.h>
+#include <sys/socket.h>
+#if defined(HAVE_STRUCT_CMSGCRED) || defined(HAVE_STRUCT_FCRED) || defined(HAVE_STRUCT_SOCKCRED)
+#include <sys/uio.h>
+#include <sys/ucred.h>
+#include <errno.h>
+#endif
+#include <netinet/in.h>
+#include <arpa/inet.h>
 #include "libpq/auth.h"
 #include "libpq/crypt.h"
 #include "libpq/hba.h"
 #include "libpq/libpq.h"
 #include "libpq/password.h"
+#include "libpq/pqformat.h"
 #include "miscadmin.h"
 
-static void sendAuthRequest(Port *port, AuthRequest areq, PacketDoneProc handler);
-static int	handle_done_auth(void *arg, PacketLen len, void *pkt);
-static int	handle_krb4_auth(void *arg, PacketLen len, void *pkt);
-static int	handle_krb5_auth(void *arg, PacketLen len, void *pkt);
-static int	handle_password_auth(void *arg, PacketLen len, void *pkt);
-static int	readPasswordPacket(void *arg, PacketLen len, void *pkt);
-static int	pg_passwordv0_recvauth(void *arg, PacketLen len, void *pkt);
+static void sendAuthRequest(Port *port, AuthRequest areq);
 static int	checkPassword(Port *port, char *user, char *password);
 static int	old_be_recvauth(Port *port);
 static int	map_old_to_new(Port *port, UserAuth old, int status);
-static void auth_failed(Port *port);
-
+static void auth_failed(Port *port, int status);
+static int	recv_and_check_password_packet(Port *port);
+static int	recv_and_check_passwordv0(Port *port);
 
 char	   *pg_krb_server_keyfile;
 
+#ifdef USE_PAM
+#include <security/pam_appl.h>
+
+#define PGSQL_PAM_SERVICE "postgresql"	/* Service name passed to PAM */
+
+static int	CheckPAMAuth(Port *port, char *user, char *password);
+static int pam_passwd_conv_proc(int num_msg, const struct pam_message ** msg,
+					 struct pam_response ** resp, void *appdata_ptr);
+
+static struct pam_conv pam_passw_conv = {
+	&pam_passwd_conv_proc,
+	NULL
+};
+
+static char *pam_passwd = NULL; /* Workaround for Solaris 2.6 brokenness */
+static Port *pam_port_cludge;	/* Workaround for passing "Port *port"
+								 * into pam_passwd_conv_proc */
+#endif   /* USE_PAM */
 
 #ifdef KRB4
 /*----------------------------------------------------------------
@@ -136,8 +144,7 @@ pg_krb4_recvauth(Port *port)
 
 	return STATUS_ERROR;
 }
-
-#endif	 /* KRB4 */
+#endif   /* KRB4 */
 
 
 #ifdef KRB5
@@ -318,32 +325,40 @@ pg_krb5_recvauth(Port *port)
 
 	return STATUS_ERROR;
 }
-
-#endif	 /* KRB5 */
+#endif   /* KRB5 */
 
 
 /*
  * Handle a v0 password packet.
  */
-
 static int
-pg_passwordv0_recvauth(void *arg, PacketLen len, void *pkt)
+recv_and_check_passwordv0(Port *port)
 {
-	Port	   *port;
+	int32		len;
+	char	   *buf;
 	PasswordPacketV0 *pp;
 	char	   *user,
 			   *password,
 			   *cp,
 			   *start;
+	int			status;
 
-	port = (Port *) arg;
-	pp = (PasswordPacketV0 *) pkt;
+	if (pq_getint(&len, 4) == EOF)
+		return STATUS_EOF;
+	len -= 4;
+	buf = palloc(len);
+	if (pq_getbytes(buf, len) == EOF)
+	{
+		pfree(buf);
+		return STATUS_EOF;
+	}
+
+	pp = (PasswordPacketV0 *) buf;
 
 	/*
 	 * The packet is supposed to comprise the user name and the password
-	 * as C strings.  Be careful the check that this is the case.
+	 * as C strings.  Be careful to check that this is the case.
 	 */
-
 	user = password = NULL;
 
 	len -= sizeof(pp->unused);
@@ -370,12 +385,10 @@ pg_passwordv0_recvauth(void *arg, PacketLen len, void *pkt)
 				 "pg_password_recvauth: badly formed password packet.\n");
 		fputs(PQerrormsg, stderr);
 		pqdebug("%s", PQerrormsg);
-
-		auth_failed(port);
+		status = STATUS_ERROR;
 	}
 	else
 	{
-		int			status;
 		UserAuth	saved;
 
 		/* Check the password. */
@@ -388,12 +401,13 @@ pg_passwordv0_recvauth(void *arg, PacketLen len, void *pkt)
 		port->auth_method = saved;
 
 		/* Adjust the result if necessary. */
-
 		if (map_old_to_new(port, uaPassword, status) != STATUS_OK)
-			auth_failed(port);
+			status = STATUS_ERROR;
 	}
 
-	return STATUS_OK;			/* don't close the connection yet */
+	pfree(buf);
+
+	return status;
 }
 
 
@@ -409,13 +423,24 @@ pg_passwordv0_recvauth(void *arg, PacketLen len, void *pkt)
  * Note that many sorts of failure report additional information in the
  * postmaster log, which we hope is only readable by good guys.
  */
-
 static void
-auth_failed(Port *port)
+auth_failed(Port *port, int status)
 {
-	char		buffer[512];
 	const char *authmethod = "Unknown auth method:";
 
+	/*
+	 * If we failed due to EOF from client, just quit; there's no point in
+	 * trying to send a message to the client, and not much point in
+	 * logging the failure in the postmaster log.  (Logging the failure
+	 * might be desirable, were it not for the fact that libpq closes the
+	 * connection unceremoniously if challenged for a password when it
+	 * hasn't got one to send.  We'll get a useless log entry for every
+	 * psql connection under password auth, even if it's perfectly
+	 * successful, if we log STATUS_EOF events.)
+	 */
+	if (status == STATUS_EOF)
+		proc_exit(0);
+
 	switch (port->auth_method)
 	{
 		case uaReject:
@@ -433,27 +458,32 @@ auth_failed(Port *port)
 		case uaIdent:
 			authmethod = "IDENT";
 			break;
+		case uaMD5:
+		case uaCrypt:
 		case uaPassword:
 			authmethod = "Password";
 			break;
-		case uaCrypt:
-			authmethod = "Password";
+#ifdef USE_PAM
+		case uaPAM:
+			authmethod = "PAM";
 			break;
+#endif   /* USE_PAM */
 	}
 
-	sprintf(buffer, "%s authentication failed for user '%s'",
-			authmethod, port->user);
-
-	PacketSendError(&port->pktInfo, buffer);
+	elog(FATAL, "%s authentication failed for user \"%s\"",
+		 authmethod, port->user);
+	/* doesn't return */
 }
 
 
 /*
- * be_recvauth -- server demux routine for incoming authentication information
+ * Client authentication starts here.  If there is an error, this
+ * function does not return and the backend process is terminated.
  */
 void
-be_recvauth(Port *port)
+ClientAuthentication(Port *port)
 {
+	int			status = STATUS_ERROR;
 
 	/*
 	 * Get the authentication method to use for this frontend/database
@@ -461,235 +491,364 @@ be_recvauth(Port *port)
 	 * hba config file, not with the request.  hba.c should have dropped
 	 * an error message into the postmaster logfile if it failed.
 	 */
-
 	if (hba_getauthmethod(port) != STATUS_OK)
-		PacketSendError(&port->pktInfo,
-						"Missing or erroneous pg_hba.conf file, see postmaster log for details");
+		elog(FATAL, "Missing or erroneous pg_hba.conf file, see postmaster log for details");
 
-	else if (PG_PROTOCOL_MAJOR(port->proto) == 0)
+	/* Handle old style authentication. */
+	if (PG_PROTOCOL_MAJOR(port->proto) == 0)
 	{
-		/* Handle old style authentication. */
-
-		if (old_be_recvauth(port) != STATUS_OK)
-			auth_failed(port);
+		status = old_be_recvauth(port);
+		if (status != STATUS_OK)
+			auth_failed(port, status);
+		return;
 	}
+
+	/* Handle new style authentication. */
+	switch (port->auth_method)
+	{
+		case uaReject:
+
+			/*
+			 * This could have come from an explicit "reject" entry in
+			 * pg_hba.conf, but more likely it means there was no matching
+			 * entry.  Take pity on the poor user and issue a helpful
+			 * error message.  NOTE: this is not a security breach,
+			 * because all the info reported here is known at the frontend
+			 * and must be assumed known to bad guys. We're merely helping
+			 * out the less clueful good guys.
+			 */
+			{
+				const char *hostinfo = "localhost";
+
+				if (port->raddr.sa.sa_family == AF_INET)
+					hostinfo = inet_ntoa(port->raddr.in.sin_addr);
+				elog(FATAL,
+				"No pg_hba.conf entry for host %s, user %s, database %s",
+					 hostinfo, port->user, port->database);
+				break;
+			}
+
+		case uaKrb4:
+			sendAuthRequest(port, AUTH_REQ_KRB4);
+			status = pg_krb4_recvauth(port);
+			break;
+
+		case uaKrb5:
+			sendAuthRequest(port, AUTH_REQ_KRB5);
+			status = pg_krb5_recvauth(port);
+			break;
+
+		case uaIdent:
+#if !defined(SO_PEERCRED) && (defined(HAVE_STRUCT_CMSGCRED) || defined(HAVE_STRUCT_FCRED) || (defined(HAVE_STRUCT_SOCKCRED) && defined(LOCAL_CREDS)))
+
+			/*
+			 * If we are doing ident on unix-domain sockets, use SCM_CREDS
+			 * only if it is defined and SO_PEERCRED isn't.
+			 */
+#if defined(HAVE_STRUCT_FCRED) || defined(HAVE_STRUCT_SOCKCRED)
+
+			/*
+			 * Receive credentials on next message receipt, BSD/OS,
+			 * NetBSD. We need to set this before the client sends the
+			 * next packet.
+			 */
+			{
+				int			on = 1;
+
+				if (setsockopt(port->sock, 0, LOCAL_CREDS, &on, sizeof(on)) < 0)
+					elog(FATAL,
+						 "pg_local_sendauth: can't do setsockopt: %s\n", strerror(errno));
+			}
+#endif
+			if (port->raddr.sa.sa_family == AF_UNIX)
+				sendAuthRequest(port, AUTH_REQ_SCM_CREDS);
+#endif
+			status = authident(port);
+			break;
+
+		case uaMD5:
+			sendAuthRequest(port, AUTH_REQ_MD5);
+			status = recv_and_check_password_packet(port);
+			break;
+
+		case uaCrypt:
+			sendAuthRequest(port, AUTH_REQ_CRYPT);
+			status = recv_and_check_password_packet(port);
+			break;
+
+		case uaPassword:
+			sendAuthRequest(port, AUTH_REQ_PASSWORD);
+			status = recv_and_check_password_packet(port);
+			break;
+
+#ifdef USE_PAM
+		case uaPAM:
+			pam_port_cludge = port;
+			status = CheckPAMAuth(port, port->user, "");
+			break;
+#endif   /* USE_PAM */
+
+		case uaTrust:
+			status = STATUS_OK;
+			break;
+	}
+
+	if (status == STATUS_OK)
+		sendAuthRequest(port, AUTH_REQ_OK);
 	else
-	{
-		/* Handle new style authentication. */
-
-		AuthRequest areq = AUTH_REQ_OK;
-		PacketDoneProc auth_handler = NULL;
-
-		switch (port->auth_method)
-		{
-			case uaReject:
-
-				/*
-				 * This could have come from an explicit "reject" entry in
-				 * pg_hba.conf, but more likely it means there was no
-				 * matching entry.	Take pity on the poor user and issue a
-				 * helpful error message.  NOTE: this is not a security
-				 * breach, because all the info reported here is known at
-				 * the frontend and must be assumed known to bad guys.
-				 * We're merely helping out the less clueful good guys.
-				 * NOTE 2: libpq-be.h defines the maximum error message
-				 * length as 99 characters.  It probably wouldn't hurt
-				 * anything to increase it, but there might be some client
-				 * out there that will fail.  So, be terse.
-				 */
-				{
-					char		buffer[512];
-					const char *hostinfo = "localhost";
-
-					if (port->raddr.sa.sa_family == AF_INET)
-						hostinfo = inet_ntoa(port->raddr.in.sin_addr);
-					sprintf(buffer,
-							"No pg_hba.conf entry for host %s, user %s, database %s",
-							hostinfo, port->user, port->database);
-					PacketSendError(&port->pktInfo, buffer);
-					return;
-				}
-				break;
-
-			case uaKrb4:
-				areq = AUTH_REQ_KRB4;
-				auth_handler = handle_krb4_auth;
-				break;
-
-			case uaKrb5:
-				areq = AUTH_REQ_KRB5;
-				auth_handler = handle_krb5_auth;
-				break;
-
-			case uaTrust:
-				areq = AUTH_REQ_OK;
-				auth_handler = handle_done_auth;
-				break;
-
-			case uaIdent:
-				if (authident(&port->raddr.in, &port->laddr.in,
-							  port->user, port->auth_arg) == STATUS_OK)
-				{
-					areq = AUTH_REQ_OK;
-					auth_handler = handle_done_auth;
-				}
-
-				break;
-
-			case uaPassword:
-				areq = AUTH_REQ_PASSWORD;
-				auth_handler = handle_password_auth;
-				break;
-
-			case uaCrypt:
-				areq = AUTH_REQ_CRYPT;
-				auth_handler = handle_password_auth;
-				break;
-		}
-
-		/* Tell the frontend what we want next. */
-
-		if (auth_handler != NULL)
-			sendAuthRequest(port, areq, auth_handler);
-		else
-			auth_failed(port);
-	}
+		auth_failed(port, status);
 }
 
 
 /*
  * Send an authentication request packet to the frontend.
  */
-
 static void
-sendAuthRequest(Port *port, AuthRequest areq, PacketDoneProc handler)
+sendAuthRequest(Port *port, AuthRequest areq)
 {
-	char	   *dp,
-			   *sp;
-	int			i;
-	uint32		net_areq;
+	StringInfoData buf;
 
-	/* Convert to a byte stream. */
-
-	net_areq = htonl(areq);
-
-	dp = port->pktInfo.pkt.ar.data;
-	sp = (char *) &net_areq;
-
-	*dp++ = 'R';
-
-	for (i = 1; i <= 4; ++i)
-		*dp++ = *sp++;
+	pq_beginmessage(&buf);
+	pq_sendbyte(&buf, 'R');
+	pq_sendint(&buf, (int32) areq, sizeof(int32));
 
 	/* Add the salt for encrypted passwords. */
+	if (areq == AUTH_REQ_MD5)
+		pq_sendbytes(&buf, port->md5Salt, 4);
+	else if (areq == AUTH_REQ_CRYPT)
+		pq_sendbytes(&buf, port->cryptSalt, 2);
 
-	if (areq == AUTH_REQ_CRYPT)
+	pq_endmessage(&buf);
+	pq_flush();
+}
+
+#ifdef USE_PAM
+
+/*
+ * PAM conversation function
+ */
+
+static int
+pam_passwd_conv_proc(int num_msg, const struct pam_message ** msg, struct pam_response ** resp, void *appdata_ptr)
+{
+	StringInfoData buf;
+	int32		len;
+
+	if (num_msg != 1 || msg[0]->msg_style != PAM_PROMPT_ECHO_OFF)
 	{
-		*dp++ = port->salt[0];
-		*dp++ = port->salt[1];
-		i += 2;
+		switch (msg[0]->msg_style)
+		{
+			case PAM_ERROR_MSG:
+				snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+						 "pam_passwd_conv_proc: Error from underlying PAM layer: '%s'\n", msg[0]->msg);
+				fputs(PQerrormsg, stderr);
+				pqdebug("%s", PQerrormsg);
+				return PAM_CONV_ERR;
+			default:
+				snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+						 "pam_passwd_conv_proc: Unexpected PAM conversation %d/'%s'\n",
+						 msg[0]->msg_style, msg[0]->msg);
+				fputs(PQerrormsg, stderr);
+				pqdebug("%s", PQerrormsg);
+				return PAM_CONV_ERR;
+		}
 	}
 
-	PacketSendSetup(&port->pktInfo, i, handler, (void *) port);
-}
-
-
-/*
- * Called when we have told the front end that it is authorised.
- */
-
-static int
-handle_done_auth(void *arg, PacketLen len, void *pkt)
-{
+	if (!appdata_ptr)
+	{
+		/*
+		 * Workaround for Solaris 2.6 where the PAM library is broken and
+		 * does not pass appdata_ptr to the conversation routine
+		 */
+		appdata_ptr = pam_passwd;
+	}
 
 	/*
-	 * Don't generate any more traffic.  This will cause the backend to
-	 * start.
+	 * Password wasn't passed to PAM the first time around - let's go ask
+	 * the client to send a password, which we then stuff into PAM.
 	 */
+	if (strlen(appdata_ptr) == 0)
+	{
+		sendAuthRequest(pam_port_cludge, AUTH_REQ_PASSWORD);
+		if (pq_eof() == EOF || pq_getint(&len, 4) == EOF)
+		{
+			return PAM_CONV_ERR;	/* client didn't want to send password */
+		}
 
-	return STATUS_OK;
+		initStringInfo(&buf);
+		pq_getstr(&buf);
+		if (DebugLvl > 5)
+			fprintf(stderr, "received PAM packet with len=%d, pw=%s\n",
+					len, buf.data);
+
+		if (strlen(buf.data) == 0)
+		{
+			snprintf(PQerrormsg, PQERRORMSG_LENGTH, "pam_passwd_conv_proc: no password\n");
+			fputs(PQerrormsg, stderr);
+			return PAM_CONV_ERR;
+		}
+		appdata_ptr = buf.data;
+	}
+
+	/*
+	 * Explicitly not using palloc here - PAM will free this memory in
+	 * pam_end()
+	 */
+	*resp = calloc(num_msg, sizeof(struct pam_response));
+	if (!*resp)
+	{
+		snprintf(PQerrormsg, PQERRORMSG_LENGTH, "pam_passwd_conv_proc: Out of memory!\n");
+		fputs(PQerrormsg, stderr);
+		pqdebug("%s", PQerrormsg);
+		if (buf.data)
+			pfree(buf.data);
+		return PAM_CONV_ERR;
+	}
+
+	(*resp)[0].resp = strdup((char *) appdata_ptr);
+	(*resp)[0].resp_retcode = 0;
+
+	return ((*resp)[0].resp ? PAM_SUCCESS : PAM_CONV_ERR);
 }
 
 
 /*
- * Called when we have told the front end that it should use Kerberos V4
- * authentication.
+ * Check authentication against PAM.
  */
-
 static int
-handle_krb4_auth(void *arg, PacketLen len, void *pkt)
+CheckPAMAuth(Port *port, char *user, char *password)
 {
-	Port	   *port = (Port *) arg;
+	int			retval;
+	pam_handle_t *pamh = NULL;
 
-	if (pg_krb4_recvauth(port) != STATUS_OK)
-		auth_failed(port);
+	/*
+	 * Apparently, Solaris 2.6 is broken, and needs ugly static variable
+	 * workaround
+	 */
+	pam_passwd = password;
+
+	/*
+	 * Set the application data portion of the conversation struct This is
+	 * later used inside the PAM conversation to pass the password to the
+	 * authentication module.
+	 */
+	pam_passw_conv.appdata_ptr = (char *) password;		/* from password above,
+														 * not allocated */
+
+	/* Optionally, one can set the service name in pg_hba.conf */
+	if (port->auth_arg[0] == '\0')
+		retval = pam_start(PGSQL_PAM_SERVICE, "pgsql@", &pam_passw_conv, &pamh);
 	else
-		sendAuthRequest(port, AUTH_REQ_OK, handle_done_auth);
+		retval = pam_start(port->auth_arg, "pgsql@", &pam_passw_conv, &pamh);
 
-	return STATUS_OK;
-}
+	if (retval != PAM_SUCCESS)
+	{
+		snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+			  "CheckPAMAuth: Failed to create PAM authenticator: '%s'\n",
+				 pam_strerror(pamh, retval));
+		fputs(PQerrormsg, stderr);
+		pqdebug("%s", PQerrormsg);
+		pam_passwd = NULL;		/* Unset pam_passwd */
+		return STATUS_ERROR;
+	}
 
+	retval = pam_set_item(pamh, PAM_USER, user);
 
-/*
- * Called when we have told the front end that it should use Kerberos V5
- * authentication.
- */
+	if (retval != PAM_SUCCESS)
+	{
+		snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+				 "CheckPAMAuth: pam_set_item(PAM_USER) failed: '%s'\n",
+				 pam_strerror(pamh, retval));
+		fputs(PQerrormsg, stderr);
+		pqdebug("%s", PQerrormsg);
+		pam_passwd = NULL;		/* Unset pam_passwd */
+		return STATUS_ERROR;
+	}
 
-static int
-handle_krb5_auth(void *arg, PacketLen len, void *pkt)
-{
-	Port	   *port = (Port *) arg;
+	retval = pam_set_item(pamh, PAM_CONV, &pam_passw_conv);
 
-	if (pg_krb5_recvauth(port) != STATUS_OK)
-		auth_failed(port);
-	else
-		sendAuthRequest(port, AUTH_REQ_OK, handle_done_auth);
+	if (retval != PAM_SUCCESS)
+	{
+		snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+				 "CheckPAMAuth: pam_set_item(PAM_CONV) failed: '%s'\n",
+				 pam_strerror(pamh, retval));
+		fputs(PQerrormsg, stderr);
+		pqdebug("%s", PQerrormsg);
+		pam_passwd = NULL;		/* Unset pam_passwd */
+		return STATUS_ERROR;
+	}
 
-	return STATUS_OK;
-}
+	retval = pam_authenticate(pamh, 0);
 
+	if (retval != PAM_SUCCESS)
+	{
+		snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+				 "CheckPAMAuth: pam_authenticate failed: '%s'\n",
+				 pam_strerror(pamh, retval));
+		fputs(PQerrormsg, stderr);
+		pqdebug("%s", PQerrormsg);
+		pam_passwd = NULL;		/* Unset pam_passwd */
+		return STATUS_ERROR;
+	}
 
-/*
- * Called when we have told the front end that it should use password
- * authentication.
- */
+	retval = pam_acct_mgmt(pamh, 0);
 
-static int
-handle_password_auth(void *arg, PacketLen len, void *pkt)
-{
-	Port	   *port = (Port *) arg;
+	if (retval != PAM_SUCCESS)
+	{
+		snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+				 "CheckPAMAuth: pam_acct_mgmt failed: '%s'\n",
+				 pam_strerror(pamh, retval));
+		fputs(PQerrormsg, stderr);
+		pqdebug("%s", PQerrormsg);
+		pam_passwd = NULL;		/* Unset pam_passwd */
+		return STATUS_ERROR;
+	}
 
-	/* Set up the read of the password packet. */
+	retval = pam_end(pamh, retval);
 
-	PacketReceiveSetup(&port->pktInfo, readPasswordPacket, (void *) port);
+	if (retval != PAM_SUCCESS)
+	{
+		snprintf(PQerrormsg, PQERRORMSG_LENGTH,
+		 "CheckPAMAuth: Failed to release PAM authenticator: '%s'\n",
+				 pam_strerror(pamh, retval));
+		fputs(PQerrormsg, stderr);
+		pqdebug("%s", PQerrormsg);
+	}
 
-	return STATUS_OK;
+	pam_passwd = NULL;		/* Unset pam_passwd */
+
+	return (retval == PAM_SUCCESS ? STATUS_OK : STATUS_ERROR);
 }
+#endif   /* USE_PAM */
 
 
 /*
  * Called when we have received the password packet.
  */
-
 static int
-readPasswordPacket(void *arg, PacketLen len, void *pkt)
+recv_and_check_password_packet(Port *port)
 {
-	char		password[sizeof(PasswordPacket) + 1];
-	Port	   *port = (Port *) arg;
+	StringInfoData buf;
+	int32		len;
+	int			result;
 
-	/* Silently truncate a password that is too big. */
+	if (pq_eof() == EOF || pq_getint(&len, 4) == EOF)
+		return STATUS_EOF;		/* client didn't want to send password */
 
-	if (len > sizeof(PasswordPacket))
-		len = sizeof(PasswordPacket);
+	initStringInfo(&buf);
+	if (pq_getstr(&buf) == EOF) /* receive password */
+	{
+		pfree(buf.data);
+		return STATUS_EOF;
+	}
 
-	StrNCpy(password, ((PasswordPacket *) pkt)->passwd, len);
+	if (DebugLvl > 5)			/* this is probably a BAD idea... */
+		fprintf(stderr, "received password packet with len=%d, pw=%s\n",
+				len, buf.data);
 
-	if (checkPassword(port, port->user, password) != STATUS_OK)
-		auth_failed(port);
-	else
-		sendAuthRequest(port, AUTH_REQ_OK, handle_done_auth);
-
-	return STATUS_OK;			/* don't close the connection yet */
+	result = checkPassword(port, port->user, buf.data);
+	pfree(buf.data);
+	return result;
 }
 
 
@@ -703,7 +862,7 @@ checkPassword(Port *port, char *user, char *password)
 	if (port->auth_arg[0] != '\0')
 		return verify_password(port, user, password);
 
-	return crypt_verify(port, user, password);
+	return md5_crypt_verify(port, user, password);
 }
 
 
@@ -718,7 +877,6 @@ old_be_recvauth(Port *port)
 	MsgType		msgtype = (MsgType) port->proto;
 
 	/* Handle the authentication that's offered. */
-
 	switch (msgtype)
 	{
 		case STARTUP_KRB4_MSG:
@@ -734,15 +892,13 @@ old_be_recvauth(Port *port)
 			break;
 
 		case STARTUP_PASSWORD_MSG:
-			PacketReceiveSetup(&port->pktInfo, pg_passwordv0_recvauth,
-							   (void *) port);
-
-			return STATUS_OK;
+			status = recv_and_check_passwordv0(port);
+			break;
 
 		default:
 			fprintf(stderr, "Invalid startup message type: %u\n", msgtype);
 
-			return STATUS_OK;
+			return STATUS_ERROR;
 	}
 
 	return status;
@@ -754,14 +910,17 @@ old_be_recvauth(Port *port)
  * allow the connection anyway, disallow it anyway, or use the result)
  * depending on what authentication we really want to use.
  */
-
 static int
 map_old_to_new(Port *port, UserAuth old, int status)
 {
 	switch (port->auth_method)
 	{
-			case uaCrypt:
-			case uaReject:
+		case uaMD5:
+		case uaCrypt:
+		case uaReject:
+#ifdef USE_PAM
+		case uaPAM:
+#endif   /* USE_PAM */
 			status = STATUS_ERROR;
 			break;
 
@@ -780,8 +939,7 @@ map_old_to_new(Port *port, UserAuth old, int status)
 			break;
 
 		case uaIdent:
-			status = authident(&port->raddr.in, &port->laddr.in,
-							   port->user, port->auth_arg);
+			status = authident(port);
 			break;
 
 		case uaPassword:
