@@ -9,7 +9,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/optimizer/path/indxpath.c,v 1.104 2001/03/23 04:49:53 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/optimizer/path/indxpath.c,v 1.112 2001/10/25 05:49:32 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -37,6 +37,7 @@
 #include "utils/builtins.h"
 #include "utils/fmgroids.h"
 #include "utils/lsyscache.h"
+#include "utils/selfuncs.h"
 #include "utils/syscache.h"
 
 
@@ -45,16 +46,16 @@
  *
  * Determine whether we should continue matching index keys in a clause.
  * Depends on if there are more to match or if this is a functional index.
- * In the latter case we stop after the first match since the there can
- * be only key (i.e. the function's return value) and the attributes in
+ * In the latter case we stop after the first match since there can
+ * be only 1 key (i.e. the function's return value) and the attributes in
  * keys list represent the arguments to the function.  -mer 3 Oct. 1991
  */
 #define DoneMatchingIndexKeys(indexkeys, index) \
 		(indexkeys[0] == 0 || \
 		 (index->indproc != InvalidOid))
 
-#define is_indexable_operator(clause,opclass,relam,indexkey_on_left) \
-	(indexable_operator(clause,opclass,relam,indexkey_on_left) != InvalidOid)
+#define is_indexable_operator(clause,opclass,indexkey_on_left) \
+	(indexable_operator(clause,opclass,indexkey_on_left) != InvalidOid)
 
 
 static void match_index_orclauses(RelOptInfo *rel, IndexOptInfo *index,
@@ -78,10 +79,10 @@ static bool match_clause_to_indexkey(RelOptInfo *rel, IndexOptInfo *index,
 						 Expr *clause, bool join);
 static bool pred_test(List *predicate_list, List *restrictinfo_list,
 		  List *joininfo_list);
-static bool one_pred_test(Expr *predicate, List *restrictinfo_list);
-static bool one_pred_clause_expr_test(Expr *predicate, Node *clause);
-static bool one_pred_clause_test(Expr *predicate, Node *clause);
-static bool clause_pred_clause_test(Expr *predicate, Node *clause);
+static bool pred_test_restrict_list(Expr *predicate, List *restrictinfo_list);
+static bool pred_test_recurse_clause(Expr *predicate, Node *clause);
+static bool pred_test_recurse_pred(Expr *predicate, Node *clause);
+static bool pred_test_simple_clause(Expr *predicate, Node *clause);
 static void indexable_joinclauses(RelOptInfo *rel, IndexOptInfo *index,
 					  List *joininfo_list, List *restrictinfo_list,
 					  List **clausegroups, List **outerrelids);
@@ -91,10 +92,11 @@ static bool match_index_to_operand(int indexkey, Var *operand,
 					   RelOptInfo *rel, IndexOptInfo *index);
 static bool function_index_operand(Expr *funcOpnd, RelOptInfo *rel,
 					   IndexOptInfo *index);
-static bool match_special_index_operator(Expr *clause, Oid opclass, Oid relam,
+static bool match_special_index_operator(Expr *clause, Oid opclass,
 							 bool indexkey_on_left);
 static List *prefix_quals(Var *leftop, Oid expr_op,
 			 char *prefix, Pattern_Prefix_Status pstatus);
+static List *network_prefix_quals(Var *leftop, Oid expr_op, Datum rightop);
 static Oid	find_operator(const char *opname, Oid datatype);
 static Datum string_to_datum(const char *str, Oid datatype);
 static Const *string_to_const(const char *str, Oid datatype);
@@ -127,18 +129,15 @@ static Const *string_to_const(const char *str, Oid datatype);
  * consideration in nested-loop joins.
  *
  * 'rel' is the relation for which we want to generate index paths
- * 'indices' is a list of available indexes for 'rel'
  */
 void
-create_index_paths(Query *root,
-				   RelOptInfo *rel,
-				   List *indices)
+create_index_paths(Query *root, RelOptInfo *rel)
 {
 	List	   *restrictinfo_list = rel->baserestrictinfo;
 	List	   *joininfo_list = rel->joininfo;
 	List	   *ilist;
 
-	foreach(ilist, indices)
+	foreach(ilist, rel->indexlist)
 	{
 		IndexOptInfo *index = (IndexOptInfo *) lfirst(ilist);
 		List	   *restrictclauses;
@@ -196,8 +195,14 @@ create_index_paths(Query *root,
 		 * 4. Generate an indexscan path if there are relevant restriction
 		 * clauses OR the index ordering is potentially useful for later
 		 * merging or final output ordering.
+		 *
+		 * If there is a predicate, consider it anyway since the index
+		 * predicate has already been found to match the query.  The
+		 * selectivity of the predicate might alone make the index useful.
 		 */
-		if (restrictclauses != NIL || useful_pathkeys != NIL)
+		if (restrictclauses != NIL ||
+			useful_pathkeys != NIL ||
+			index->indpred != NIL)
 			add_path(rel, (Path *)
 					 create_index_path(root, rel, index,
 									   restrictclauses,
@@ -283,7 +288,6 @@ match_index_orclauses(RelOptInfo *rel,
 
 		if (restriction_is_or_clause(restrictinfo))
 		{
-
 			/*
 			 * Add this index to the subclause index list for each
 			 * subclause that it matches.
@@ -400,7 +404,7 @@ match_or_subclause_to_indexkey(RelOptInfo *rel,
 										clause, false);
 }
 
-/*
+/*----------
  * Given an OR subclause that has previously been determined to match
  * the specified index, extract a list of specific opclauses that can be
  * used as indexquals.
@@ -409,10 +413,25 @@ match_or_subclause_to_indexkey(RelOptInfo *rel,
  * given opclause.	However, if the OR subclause is an AND, we have to
  * scan it to find the opclause(s) that match the index.  (There should
  * be at least one, if match_or_subclause_to_indexkey succeeded, but there
- * could be more.)	Also, we apply expand_indexqual_conditions() to convert
- * any special matching opclauses to indexable operators.
+ * could be more.)
+ *
+ * Also, we can look at other restriction clauses of the rel to discover
+ * additional candidate indexquals: for example, consider
+ *			... where (a = 11 or a = 12) and b = 42;
+ * If we are dealing with an index on (a,b) then we can include the clause
+ * b = 42 in the indexqual list generated for each of the OR subclauses.
+ * Essentially, we are making an index-specific transformation from CNF to
+ * DNF.  (NOTE: when we do this, we end up with a slightly inefficient plan
+ * because create_indexscan_plan is not very bright about figuring out which
+ * restriction clauses are implied by the generated indexqual condition.
+ * Currently we'll end up rechecking both the OR clause and the transferred
+ * restriction clause as qpquals.  FIXME someday.)
+ *
+ * Also, we apply expand_indexqual_conditions() to convert any special
+ * matching opclauses to indexable operators.
  *
  * The passed-in clause is not changed.
+ *----------
  */
 List *
 extract_or_indexqual_conditions(RelOptInfo *rel,
@@ -420,54 +439,71 @@ extract_or_indexqual_conditions(RelOptInfo *rel,
 								Expr *orsubclause)
 {
 	List	   *quals = NIL;
+	int		   *indexkeys = index->indexkeys;
+	Oid		   *classes = index->classlist;
 
-	if (and_clause((Node *) orsubclause))
+	/*
+	 * Extract relevant indexclauses in indexkey order.  This is
+	 * essentially just like group_clauses_by_indexkey() except that the
+	 * input and output are lists of bare clauses, not of RestrictInfo
+	 * nodes.
+	 */
+	do
 	{
+		int			curIndxKey = indexkeys[0];
+		Oid			curClass = classes[0];
+		List	   *clausegroup = NIL;
+		List	   *item;
 
-		/*
-		 * Extract relevant sub-subclauses in indexkey order.  This is
-		 * just like group_clauses_by_indexkey() except that the input and
-		 * output are lists of bare clauses, not of RestrictInfo nodes.
-		 */
-		int		   *indexkeys = index->indexkeys;
-		Oid		   *classes = index->classlist;
-
-		do
+		if (and_clause((Node *) orsubclause))
 		{
-			int			curIndxKey = indexkeys[0];
-			Oid			curClass = classes[0];
-			List	   *clausegroup = NIL;
-			List	   *item;
-
 			foreach(item, orsubclause->args)
 			{
+				Expr	   *subsubclause = (Expr *) lfirst(item);
+
 				if (match_clause_to_indexkey(rel, index,
 											 curIndxKey, curClass,
-											 lfirst(item), false))
-					clausegroup = lappend(clausegroup, lfirst(item));
+											 subsubclause, false))
+					clausegroup = lappend(clausegroup, subsubclause);
 			}
+		}
+		else if (match_clause_to_indexkey(rel, index,
+										  curIndxKey, curClass,
+										  orsubclause, false))
+			clausegroup = makeList1(orsubclause);
 
-			/*
-			 * If no clauses match this key, we're done; we don't want to
-			 * look at keys to its right.
-			 */
-			if (clausegroup == NIL)
-				break;
+		/*
+		 * If we found no clauses for this indexkey in the OR subclause
+		 * itself, try looking in the rel's top-level restriction list.
+		 */
+		if (clausegroup == NIL)
+		{
+			foreach(item, rel->baserestrictinfo)
+			{
+				RestrictInfo *rinfo = (RestrictInfo *) lfirst(item);
 
-			quals = nconc(quals, clausegroup);
+				if (match_clause_to_indexkey(rel, index,
+											 curIndxKey, curClass,
+											 rinfo->clause, false))
+					clausegroup = lappend(clausegroup, rinfo->clause);
+			}
+		}
 
-			indexkeys++;
-			classes++;
-		} while (!DoneMatchingIndexKeys(indexkeys, index));
+		/*
+		 * If still no clauses match this key, we're done; we don't want
+		 * to look at keys to its right.
+		 */
+		if (clausegroup == NIL)
+			break;
 
-		if (quals == NIL)
-			elog(ERROR, "extract_or_indexqual_conditions: no matching clause");
-	}
-	else
-	{
-		/* we assume the caller passed a valid indexable qual */
-		quals = makeList1(orsubclause);
-	}
+		quals = nconc(quals, clausegroup);
+
+		indexkeys++;
+		classes++;
+	} while (!DoneMatchingIndexKeys(indexkeys, index));
+
+	if (quals == NIL)
+		elog(ERROR, "extract_or_indexqual_conditions: no matching clause");
 
 	return expand_indexqual_conditions(quals);
 }
@@ -706,7 +742,6 @@ match_clause_to_indexkey(RelOptInfo *rel,
 
 	if (!join)
 	{
-
 		/*
 		 * Not considering joins, so check for clauses of the form:
 		 * (indexkey operator constant) or (constant operator indexkey).
@@ -716,37 +751,34 @@ match_clause_to_indexkey(RelOptInfo *rel,
 		if (match_index_to_operand(indexkey, leftop, rel, index) &&
 			is_pseudo_constant_clause((Node *) rightop))
 		{
-			if (is_indexable_operator(clause, opclass, index->relam, true))
+			if (is_indexable_operator(clause, opclass, true))
 				return true;
 
 			/*
 			 * If we didn't find a member of the index's opclass, see
 			 * whether it is a "special" indexable operator.
 			 */
-			if (match_special_index_operator(clause, opclass, index->relam,
-											 true))
+			if (match_special_index_operator(clause, opclass, true))
 				return true;
 			return false;
 		}
 		if (match_index_to_operand(indexkey, rightop, rel, index) &&
 			is_pseudo_constant_clause((Node *) leftop))
 		{
-			if (is_indexable_operator(clause, opclass, index->relam, false))
+			if (is_indexable_operator(clause, opclass, false))
 				return true;
 
 			/*
 			 * If we didn't find a member of the index's opclass, see
 			 * whether it is a "special" indexable operator.
 			 */
-			if (match_special_index_operator(clause, opclass, index->relam,
-											 false))
+			if (match_special_index_operator(clause, opclass, false))
 				return true;
 			return false;
 		}
 	}
 	else
 	{
-
 		/*
 		 * Check for an indexqual that could be handled by a nestloop
 		 * join. We need the index key to be compared against an
@@ -761,7 +793,7 @@ match_clause_to_indexkey(RelOptInfo *rel,
 			isIndexable =
 				!intMember(lfirsti(rel->relids), othervarnos) &&
 				!contain_noncachable_functions((Node *) rightop) &&
-				is_indexable_operator(clause, opclass, index->relam, true);
+				is_indexable_operator(clause, opclass, true);
 			freeList(othervarnos);
 			return isIndexable;
 		}
@@ -773,7 +805,7 @@ match_clause_to_indexkey(RelOptInfo *rel,
 			isIndexable =
 				!intMember(lfirsti(rel->relids), othervarnos) &&
 				!contain_noncachable_functions((Node *) leftop) &&
-				is_indexable_operator(clause, opclass, index->relam, false);
+				is_indexable_operator(clause, opclass, false);
 			freeList(othervarnos);
 			return isIndexable;
 		}
@@ -784,12 +816,11 @@ match_clause_to_indexkey(RelOptInfo *rel,
 
 /*
  * indexable_operator
- *	  Does a binary opclause contain an operator matching the index's
- *	  access method?
+ *	  Does a binary opclause contain an operator matching the index opclass?
  *
  * If the indexkey is on the right, what we actually want to know
  * is whether the operator has a commutator operator that matches
- * the index's access method.
+ * the index's opclass.
  *
  * We try both the straightforward match and matches that rely on
  * recognizing binary-compatible datatypes.  For example, if we have
@@ -806,8 +837,7 @@ match_clause_to_indexkey(RelOptInfo *rel,
  * OID is *not* commuted; it can be plugged directly into the given clause.
  */
 Oid
-indexable_operator(Expr *clause, Oid opclass, Oid relam,
-				   bool indexkey_on_left)
+indexable_operator(Expr *clause, Oid opclass, bool indexkey_on_left)
 {
 	Oid			expr_op = ((Oper *) clause->oper)->opno;
 	Oid			commuted_op,
@@ -827,8 +857,8 @@ indexable_operator(Expr *clause, Oid opclass, Oid relam,
 	if (commuted_op == InvalidOid)
 		return InvalidOid;
 
-	/* Done if the (commuted) operator is a member of the index's AM */
-	if (op_class(commuted_op, opclass, relam))
+	/* Done if the (commuted) operator is a member of the index's opclass */
+	if (op_in_opclass(commuted_op, opclass))
 		return expr_op;
 
 	/*
@@ -887,7 +917,6 @@ indexable_operator(Expr *clause, Oid opclass, Oid relam,
 	{
 		if (new_op != expr_op)
 		{
-
 			/*
 			 * OK, we found a binary-compatible operator of the same name;
 			 * now does it match the index?
@@ -899,7 +928,7 @@ indexable_operator(Expr *clause, Oid opclass, Oid relam,
 			if (commuted_op == InvalidOid)
 				return InvalidOid;
 
-			if (op_class(commuted_op, opclass, relam))
+			if (op_in_opclass(commuted_op, opclass))
 				return new_op;
 		}
 	}
@@ -922,15 +951,13 @@ indexable_operator(Expr *clause, Oid opclass, Oid relam,
  *	  ANDs in the predicate first, then reduces the qualification
  *	  clauses down to their constituent terms, and iterates over ORs
  *	  in the predicate last.  This order is important to make the test
- *	  succeed whenever possible (assuming the predicate has been
- *	  successfully cnfify()-ed). --Nels, Jan '93
+ *	  succeed whenever possible (assuming the predicate has been converted
+ *	  to CNF format). --Nels, Jan '93
  */
 static bool
 pred_test(List *predicate_list, List *restrictinfo_list, List *joininfo_list)
 {
-	List	   *pred,
-			   *items,
-			   *item;
+	List	   *pred;
 
 	/*
 	 * Note: if Postgres tried to optimize queries by forming equivalence
@@ -940,31 +967,25 @@ pred_test(List *predicate_list, List *restrictinfo_list, List *joininfo_list)
 	 * here with joininfo_list to do more complete tests for the usability
 	 * of a partial index.	For now, the test only uses restriction
 	 * clauses (those in restrictinfo_list). --Nels, Dec '92
+	 *
+	 * XXX as of 7.1, equivalence class info *is* available.  Consider
+	 * improving this code as foreseen by Nels.
 	 */
 
-	if (predicate_list == NULL)
+	if (predicate_list == NIL)
 		return true;			/* no predicate: the index is usable */
-	if (restrictinfo_list == NULL)
+	if (restrictinfo_list == NIL)
 		return false;			/* no restriction clauses: the test must
 								 * fail */
 
 	foreach(pred, predicate_list)
 	{
-
 		/*
 		 * if any clause is not implied, the whole predicate is not
-		 * implied
+		 * implied.  Note we assume that any sub-ANDs have been flattened
+		 * when the predicate was fed through canonicalize_qual().
 		 */
-		if (and_clause(lfirst(pred)))
-		{
-			items = ((Expr *) lfirst(pred))->args;
-			foreach(item, items)
-			{
-				if (!one_pred_test(lfirst(item), restrictinfo_list))
-					return false;
-			}
-		}
-		else if (!one_pred_test(lfirst(pred), restrictinfo_list))
+		if (!pred_test_restrict_list(lfirst(pred), restrictinfo_list))
 			return false;
 	}
 	return true;
@@ -972,22 +993,22 @@ pred_test(List *predicate_list, List *restrictinfo_list, List *joininfo_list)
 
 
 /*
- * one_pred_test
+ * pred_test_restrict_list
  *	  Does the "predicate inclusion test" for one conjunct of a predicate
  *	  expression.
  */
 static bool
-one_pred_test(Expr *predicate, List *restrictinfo_list)
+pred_test_restrict_list(Expr *predicate, List *restrictinfo_list)
 {
-	RestrictInfo *restrictinfo;
 	List	   *item;
 
-	Assert(predicate != NULL);
 	foreach(item, restrictinfo_list)
 	{
-		restrictinfo = (RestrictInfo *) lfirst(item);
+		RestrictInfo *restrictinfo = (RestrictInfo *) lfirst(item);
+
 		/* if any clause implies the predicate, return true */
-		if (one_pred_clause_expr_test(predicate, (Node *) restrictinfo->clause))
+		if (pred_test_recurse_clause(predicate,
+									 (Node *) restrictinfo->clause))
 			return true;
 	}
 	return false;
@@ -995,25 +1016,25 @@ one_pred_test(Expr *predicate, List *restrictinfo_list)
 
 
 /*
- * one_pred_clause_expr_test
+ * pred_test_recurse_clause
  *	  Does the "predicate inclusion test" for a general restriction-clause
- *	  expression.
+ *	  expression.  Here we recursively deal with the possibility that the
+ *	  restriction clause is itself an AND or OR structure.
  */
 static bool
-one_pred_clause_expr_test(Expr *predicate, Node *clause)
+pred_test_recurse_clause(Expr *predicate, Node *clause)
 {
 	List	   *items,
 			   *item;
 
-	if (is_opclause(clause))
-		return one_pred_clause_test(predicate, clause);
-	else if (or_clause(clause))
+	Assert(clause != NULL);
+	if (or_clause(clause))
 	{
 		items = ((Expr *) clause)->args;
 		foreach(item, items)
 		{
 			/* if any OR item doesn't imply the predicate, clause doesn't */
-			if (!one_pred_clause_expr_test(predicate, lfirst(item)))
+			if (!pred_test_recurse_clause(predicate, lfirst(item)))
 				return false;
 		}
 		return true;
@@ -1023,44 +1044,41 @@ one_pred_clause_expr_test(Expr *predicate, Node *clause)
 		items = ((Expr *) clause)->args;
 		foreach(item, items)
 		{
-
 			/*
 			 * if any AND item implies the predicate, the whole clause
 			 * does
 			 */
-			if (one_pred_clause_expr_test(predicate, lfirst(item)))
+			if (pred_test_recurse_clause(predicate, lfirst(item)))
 				return true;
 		}
 		return false;
 	}
 	else
-	{
-		/* unknown clause type never implies the predicate */
-		return false;
-	}
+		return pred_test_recurse_pred(predicate, clause);
 }
 
 
 /*
- * one_pred_clause_test
+ * pred_test_recurse_pred
  *	  Does the "predicate inclusion test" for one conjunct of a predicate
- *	  expression for a simple restriction clause.
+ *	  expression for a simple restriction clause.  Here we recursively deal
+ *	  with the possibility that the predicate conjunct is itself an AND or
+ *	  OR structure.
  */
 static bool
-one_pred_clause_test(Expr *predicate, Node *clause)
+pred_test_recurse_pred(Expr *predicate, Node *clause)
 {
 	List	   *items,
 			   *item;
 
-	if (is_opclause((Node *) predicate))
-		return clause_pred_clause_test(predicate, clause);
-	else if (or_clause((Node *) predicate))
+	Assert(predicate != NULL);
+	if (or_clause((Node *) predicate))
 	{
 		items = predicate->args;
 		foreach(item, items)
 		{
 			/* if any item is implied, the whole predicate is implied */
-			if (one_pred_clause_test(lfirst(item), clause))
+			if (pred_test_recurse_pred(lfirst(item), clause))
 				return true;
 		}
 		return false;
@@ -1070,21 +1088,17 @@ one_pred_clause_test(Expr *predicate, Node *clause)
 		items = predicate->args;
 		foreach(item, items)
 		{
-
 			/*
 			 * if any item is not implied, the whole predicate is not
 			 * implied
 			 */
-			if (!one_pred_clause_test(lfirst(item), clause))
+			if (!pred_test_recurse_pred(lfirst(item), clause))
 				return false;
 		}
 		return true;
 	}
 	else
-	{
-		elog(DEBUG, "Unsupported predicate type, index will not be used");
-		return false;
-	}
+		return pred_test_simple_clause(predicate, clause);
 }
 
 
@@ -1120,17 +1134,26 @@ static const StrategyNumber
 
 
 /*
- * clause_pred_clause_test
- *	  Use operator class info to check whether clause implies predicate.
- *
+ * pred_test_simple_clause
  *	  Does the "predicate inclusion test" for a "simple clause" predicate
- *	  for a single "simple clause" restriction.  Currently, this only handles
- *	  (binary boolean) operators that are in some btree operator class.
+ *	  and a "simple clause" restriction.
+ *
+ *	  We have two strategies for determining whether one simple clause
+ *	  implies another.	A simple and general way is to see if they are
+ *	  equal(); this works for any kind of expression.  (Actually, there
+ *	  is an implied assumption that the functions in the expression are
+ *	  cachable, ie dependent only on their input arguments --- but this
+ *	  was checked for the predicate by CheckPredicate().)
+ *
+ *	  Our other way works only for (binary boolean) operators that are
+ *	  in some btree operator class.  We use the above operator implication
+ *	  table to be able to derive implications between nonidentical clauses.
+ *
  *	  Eventually, rtree operators could also be handled by defining an
  *	  appropriate "RT_implic_table" array.
  */
 static bool
-clause_pred_clause_test(Expr *predicate, Node *clause)
+pred_test_simple_clause(Expr *predicate, Node *clause)
 {
 	Var		   *pred_var,
 			   *clause_var;
@@ -1139,32 +1162,44 @@ clause_pred_clause_test(Expr *predicate, Node *clause)
 	Oid			pred_op,
 				clause_op,
 				test_op;
-	Oid			opclass_id;
-	StrategyNumber pred_strategy,
+	Oid			opclass_id = InvalidOid;
+	StrategyNumber pred_strategy = 0,
 				clause_strategy,
 				test_strategy;
 	Oper	   *test_oper;
 	Expr	   *test_expr;
-	bool		test_result,
-				isNull;
+	Datum		test_result;
+	bool		isNull;
 	Relation	relation;
 	HeapScanDesc scan;
 	HeapTuple	tuple;
-	ScanKeyData entry[3];
+	ScanKeyData entry[1];
 	Form_pg_amop aform;
+	ExprContext *econtext;
 
+	/* First try the equal() test */
+	if (equal((Node *) predicate, clause))
+		return true;
+
+	/*
+	 * Can't do anything more unless they are both binary opclauses with a
+	 * Var on the left and a Const on the right.
+	 */
+	if (!is_opclause((Node *) predicate))
+		return false;
 	pred_var = (Var *) get_leftop(predicate);
 	pred_const = (Const *) get_rightop(predicate);
+
+	if (!is_opclause(clause))
+		return false;
 	clause_var = (Var *) get_leftop((Expr *) clause);
 	clause_const = (Const *) get_rightop((Expr *) clause);
 
-	/* Check the basic form; for now, only allow the simplest case */
-	if (!is_opclause(clause) ||
-		!IsA(clause_var, Var) ||
+	if (!IsA(clause_var, Var) ||
 		clause_const == NULL ||
 		!IsA(clause_const, Const) ||
-		!IsA(predicate->oper, Oper) ||
 		!IsA(pred_var, Var) ||
+		pred_const == NULL ||
 		!IsA(pred_const, Const))
 		return false;
 
@@ -1172,113 +1207,99 @@ clause_pred_clause_test(Expr *predicate, Node *clause)
 	 * The implication can't be determined unless the predicate and the
 	 * clause refer to the same attribute.
 	 */
-	if (clause_var->varattno != pred_var->varattno)
+	if (clause_var->varno != pred_var->varno ||
+		clause_var->varattno != pred_var->varattno)
 		return false;
 
 	/* Get the operators for the two clauses we're comparing */
 	pred_op = ((Oper *) ((Expr *) predicate)->oper)->opno;
 	clause_op = ((Oper *) ((Expr *) clause)->oper)->opno;
 
-
 	/*
 	 * 1. Find a "btree" strategy number for the pred_op
-	 */
-	ScanKeyEntryInitialize(&entry[0], 0,
-						   Anum_pg_amop_amopid,
-						   F_OIDEQ,
-						   ObjectIdGetDatum(BTREE_AM_OID));
-
-	ScanKeyEntryInitialize(&entry[1], 0,
-						   Anum_pg_amop_amopopr,
-						   F_OIDEQ,
-						   ObjectIdGetDatum(pred_op));
-
-	relation = heap_openr(AccessMethodOperatorRelationName, AccessShareLock);
-
-	/*
-	 * The following assumes that any given operator will only be in a
-	 * single btree operator class.  This is true at least for all the
+	 *
+	 * The following assumes that any given operator will only be in a single
+	 * btree operator class.  This is true at least for all the
 	 * pre-defined operator classes.  If it isn't true, then whichever
 	 * operator class happens to be returned first for the given operator
 	 * will be used to find the associated strategy numbers for the test.
 	 * --Nels, Jan '93
 	 */
-	scan = heap_beginscan(relation, false, SnapshotNow, 2, entry);
-	tuple = heap_getnext(scan, 0);
-	if (!HeapTupleIsValid(tuple))
+	ScanKeyEntryInitialize(&entry[0], 0x0,
+						   Anum_pg_amop_amopopr,
+						   F_OIDEQ,
+						   ObjectIdGetDatum(pred_op));
+
+	relation = heap_openr(AccessMethodOperatorRelationName, AccessShareLock);
+	scan = heap_beginscan(relation, false, SnapshotNow, 1, entry);
+
+	while (HeapTupleIsValid(tuple = heap_getnext(scan, 0)))
 	{
-		elog(DEBUG, "clause_pred_clause_test: unknown pred_op");
-		heap_endscan(scan);
-		heap_close(relation, AccessShareLock);
-		return false;
+		aform = (Form_pg_amop) GETSTRUCT(tuple);
+		if (opclass_is_btree(aform->amopclaid))
+		{
+			/* Get the predicate operator's btree strategy number (1 to 5) */
+			pred_strategy = (StrategyNumber) aform->amopstrategy;
+			Assert(pred_strategy >= 1 && pred_strategy <= 5);
+
+			/*
+			 * Remember which operator class this strategy number came
+			 * from
+			 */
+			opclass_id = aform->amopclaid;
+			break;
+		}
 	}
-	aform = (Form_pg_amop) GETSTRUCT(tuple);
-
-	/* Get the predicate operator's strategy number (1 to 5) */
-	pred_strategy = (StrategyNumber) aform->amopstrategy;
-
-	/* Remember which operator class this strategy number came from */
-	opclass_id = aform->amopclaid;
 
 	heap_endscan(scan);
+	heap_close(relation, AccessShareLock);
 
+	if (!OidIsValid(opclass_id))
+	{
+		/* predicate operator isn't btree-indexable */
+		return false;
+	}
 
 	/*
 	 * 2. From the same opclass, find a strategy num for the clause_op
 	 */
-	ScanKeyEntryInitialize(&entry[1], 0,
-						   Anum_pg_amop_amopclaid,
-						   F_OIDEQ,
-						   ObjectIdGetDatum(opclass_id));
-
-	ScanKeyEntryInitialize(&entry[2], 0,
-						   Anum_pg_amop_amopopr,
-						   F_OIDEQ,
-						   ObjectIdGetDatum(clause_op));
-
-	scan = heap_beginscan(relation, false, SnapshotNow, 3, entry);
-	tuple = heap_getnext(scan, 0);
+	tuple = SearchSysCache(AMOPOPID,
+						   ObjectIdGetDatum(opclass_id),
+						   ObjectIdGetDatum(clause_op),
+						   0, 0);
 	if (!HeapTupleIsValid(tuple))
 	{
-		elog(DEBUG, "clause_pred_clause_test: unknown clause_op");
-		heap_endscan(scan);
-		heap_close(relation, AccessShareLock);
+		/* clause operator isn't btree-indexable, or isn't in this opclass */
 		return false;
 	}
 	aform = (Form_pg_amop) GETSTRUCT(tuple);
 
 	/* Get the restriction clause operator's strategy number (1 to 5) */
 	clause_strategy = (StrategyNumber) aform->amopstrategy;
-	heap_endscan(scan);
+	Assert(clause_strategy >= 1 && clause_strategy <= 5);
 
+	ReleaseSysCache(tuple);
 
 	/*
 	 * 3. Look up the "test" strategy number in the implication table
 	 */
-
 	test_strategy = BT_implic_table[clause_strategy - 1][pred_strategy - 1];
 	if (test_strategy == 0)
 	{
-		heap_close(relation, AccessShareLock);
 		return false;			/* the implication cannot be determined */
 	}
 
 	/*
 	 * 4. From the same opclass, find the operator for the test strategy
 	 */
-
-	ScanKeyEntryInitialize(&entry[2], 0,
-						   Anum_pg_amop_amopstrategy,
-						   F_INT2EQ,
-						   Int16GetDatum(test_strategy));
-
-	scan = heap_beginscan(relation, false, SnapshotNow, 3, entry);
-	tuple = heap_getnext(scan, 0);
+	tuple = SearchSysCache(AMOPSTRATEGY,
+						   ObjectIdGetDatum(opclass_id),
+						   Int16GetDatum(test_strategy),
+						   0, 0);
 	if (!HeapTupleIsValid(tuple))
 	{
-		elog(DEBUG, "clause_pred_clause_test: unknown test_op");
-		heap_endscan(scan);
-		heap_close(relation, AccessShareLock);
+		/* this probably shouldn't fail? */
+		elog(DEBUG, "pred_test_simple_clause: unknown test_op");
 		return false;
 	}
 	aform = (Form_pg_amop) GETSTRUCT(tuple);
@@ -1286,9 +1307,7 @@ clause_pred_clause_test(Expr *predicate, Node *clause)
 	/* Get the test operator */
 	test_op = aform->amopopr;
 
-	heap_endscan(scan);
-
-	heap_close(relation, AccessShareLock);
+	ReleaseSysCache(tuple);
 
 	/*
 	 * 5. Evaluate the test
@@ -1297,19 +1316,21 @@ clause_pred_clause_test(Expr *predicate, Node *clause)
 						 InvalidOid,	/* opid */
 						 BOOLOID);		/* opresulttype */
 	replace_opid(test_oper);
-
 	test_expr = make_opclause(test_oper,
-							  copyObject(clause_const),
-							  copyObject(pred_const));
+							  (Var *) clause_const,
+							  (Var *) pred_const);
 
-	test_result = ExecEvalExpr((Node *) test_expr, NULL, &isNull, NULL);
+	econtext = MakeExprContext(NULL, TransactionCommandContext);
+	test_result = ExecEvalExprSwitchContext((Node *) test_expr, econtext,
+											&isNull, NULL);
+	FreeExprContext(econtext);
 
 	if (isNull)
 	{
-		elog(DEBUG, "clause_pred_clause_test: null test result");
+		elog(DEBUG, "pred_test_simple_clause: null test result");
 		return false;
 	}
-	return test_result;
+	return DatumGetBool(test_result);
 }
 
 
@@ -1435,10 +1456,10 @@ index_innerjoin(Query *root, RelOptInfo *rel, IndexOptInfo *index,
 
 		/*
 		 * Note that we are making a pathnode for a single-scan indexscan;
-		 * therefore, both indexid and indexqual should be single-element
-		 * lists.
+		 * therefore, both indexinfo and indexqual should be
+		 * single-element lists.
 		 */
-		pathnode->indexid = makeListi1(index->indexoid);
+		pathnode->indexinfo = makeList1(index);
 		pathnode->indexqual = makeList1(indexquals);
 
 		/* We don't actually care what order the index scans in ... */
@@ -1494,7 +1515,6 @@ match_index_to_operand(int indexkey,
 					   RelOptInfo *rel,
 					   IndexOptInfo *index)
 {
-
 	/*
 	 * Ignore any RelabelType node above the indexkey.	This is needed to
 	 * be able to apply indexscanning in binary-compatible-operator cases.
@@ -1506,7 +1526,6 @@ match_index_to_operand(int indexkey,
 
 	if (index->indproc == InvalidOid)
 	{
-
 		/*
 		 * Simple index.
 		 */
@@ -1631,7 +1650,7 @@ function_index_operand(Expr *funcOpnd, RelOptInfo *rel, IndexOptInfo *index)
  * Return 'true' if we can do something with it anyway.
  */
 static bool
-match_special_index_operator(Expr *clause, Oid opclass, Oid relam,
+match_special_index_operator(Expr *clause, Oid opclass,
 							 bool indexkey_on_left)
 {
 	bool		isIndexable = false;
@@ -1731,6 +1750,13 @@ match_special_index_operator(Expr *clause, Oid opclass, Oid relam,
 				pfree(patt);
 			}
 			break;
+
+		case OID_INET_SUB_OP:
+		case OID_INET_SUBEQ_OP:
+		case OID_CIDR_SUB_OP:
+		case OID_CIDR_SUBEQ_OP:
+			isIndexable = true;
+			break;
 	}
 
 	/* done if the expression doesn't look indexable */
@@ -1749,8 +1775,8 @@ match_special_index_operator(Expr *clause, Oid opclass, Oid relam,
 		case OID_TEXT_ICLIKE_OP:
 		case OID_TEXT_REGEXEQ_OP:
 		case OID_TEXT_ICREGEXEQ_OP:
-			if (!op_class(find_operator(">=", TEXTOID), opclass, relam) ||
-				!op_class(find_operator("<", TEXTOID), opclass, relam))
+			if (!op_in_opclass(find_operator(">=", TEXTOID), opclass) ||
+				!op_in_opclass(find_operator("<", TEXTOID), opclass))
 				isIndexable = false;
 			break;
 
@@ -1758,8 +1784,8 @@ match_special_index_operator(Expr *clause, Oid opclass, Oid relam,
 		case OID_BPCHAR_ICLIKE_OP:
 		case OID_BPCHAR_REGEXEQ_OP:
 		case OID_BPCHAR_ICREGEXEQ_OP:
-			if (!op_class(find_operator(">=", BPCHAROID), opclass, relam) ||
-				!op_class(find_operator("<", BPCHAROID), opclass, relam))
+			if (!op_in_opclass(find_operator(">=", BPCHAROID), opclass) ||
+				!op_in_opclass(find_operator("<", BPCHAROID), opclass))
 				isIndexable = false;
 			break;
 
@@ -1767,8 +1793,8 @@ match_special_index_operator(Expr *clause, Oid opclass, Oid relam,
 		case OID_VARCHAR_ICLIKE_OP:
 		case OID_VARCHAR_REGEXEQ_OP:
 		case OID_VARCHAR_ICREGEXEQ_OP:
-			if (!op_class(find_operator(">=", VARCHAROID), opclass, relam) ||
-				!op_class(find_operator("<", VARCHAROID), opclass, relam))
+			if (!op_in_opclass(find_operator(">=", VARCHAROID), opclass) ||
+				!op_in_opclass(find_operator("<", VARCHAROID), opclass))
 				isIndexable = false;
 			break;
 
@@ -1776,8 +1802,24 @@ match_special_index_operator(Expr *clause, Oid opclass, Oid relam,
 		case OID_NAME_ICLIKE_OP:
 		case OID_NAME_REGEXEQ_OP:
 		case OID_NAME_ICREGEXEQ_OP:
-			if (!op_class(find_operator(">=", NAMEOID), opclass, relam) ||
-				!op_class(find_operator("<", NAMEOID), opclass, relam))
+			if (!op_in_opclass(find_operator(">=", NAMEOID), opclass) ||
+				!op_in_opclass(find_operator("<", NAMEOID), opclass))
+				isIndexable = false;
+			break;
+
+		case OID_INET_SUB_OP:
+		case OID_INET_SUBEQ_OP:
+			/* for SUB we actually need ">" not ">=", but this should do */
+			if (!op_in_opclass(find_operator(">=", INETOID), opclass) ||
+				!op_in_opclass(find_operator("<=", INETOID), opclass))
+				isIndexable = false;
+			break;
+
+		case OID_CIDR_SUB_OP:
+		case OID_CIDR_SUBEQ_OP:
+			/* for SUB we actually need ">" not ">=", but this should do */
+			if (!op_in_opclass(find_operator(">=", CIDROID), opclass) ||
+				!op_in_opclass(find_operator("<=", CIDROID), opclass))
 				isIndexable = false;
 			break;
 	}
@@ -1815,7 +1857,6 @@ expand_indexqual_conditions(List *indexquals)
 
 		switch (expr_op)
 		{
-
 				/*
 				 * LIKE and regex operators are not members of any index
 				 * opclass, so if we find one in an indexqual list we can
@@ -1894,6 +1935,16 @@ expand_indexqual_conditions(List *indexquals)
 				pfree(patt);
 				break;
 
+			case OID_INET_SUB_OP:
+			case OID_INET_SUBEQ_OP:
+			case OID_CIDR_SUB_OP:
+			case OID_CIDR_SUBEQ_OP:
+				constvalue = ((Const *) rightop)->constvalue;
+				resultquals = nconc(resultquals,
+									network_prefix_quals(leftop, expr_op,
+														 constvalue));
+				break;
+
 			default:
 				resultquals = lappend(resultquals, clause);
 				break;
@@ -2007,6 +2058,86 @@ prefix_quals(Var *leftop, Oid expr_op,
 	return result;
 }
 
+/*
+ * Given a leftop and a rightop, and a inet-class sup/sub operator,
+ * generate suitable indexqual condition(s).  expr_op is the original
+ * operator.
+ */
+static List *
+network_prefix_quals(Var *leftop, Oid expr_op, Datum rightop)
+{
+	bool		is_eq;
+	char	   *opr1name;
+	Datum		opr1right;
+	Datum		opr2right;
+	Oid			opr1oid;
+	Oid			opr2oid;
+	List	   *result;
+	Oid			datatype;
+	Oper	   *op;
+	Expr	   *expr;
+
+	switch (expr_op)
+	{
+		case OID_INET_SUB_OP:
+			datatype = INETOID;
+			is_eq = false;
+			break;
+		case OID_INET_SUBEQ_OP:
+			datatype = INETOID;
+			is_eq = true;
+			break;
+		case OID_CIDR_SUB_OP:
+			datatype = CIDROID;
+			is_eq = false;
+			break;
+		case OID_CIDR_SUBEQ_OP:
+			datatype = CIDROID;
+			is_eq = true;
+			break;
+		default:
+			elog(ERROR, "network_prefix_quals: unexpected operator %u",
+				 expr_op);
+			return NIL;
+	}
+
+	/*
+	 * create clause "key >= network_scan_first( rightop )", or ">" if the
+	 * operator disallows equality.
+	 */
+
+	opr1name = is_eq ? ">=" : ">";
+	opr1oid = find_operator(opr1name, datatype);
+	if (opr1oid == InvalidOid)
+		elog(ERROR, "network_prefix_quals: no %s operator for type %u",
+			 opr1name, datatype);
+
+	opr1right = network_scan_first(rightop);
+
+	op = makeOper(opr1oid, InvalidOid, BOOLOID);
+	expr = make_opclause(op, leftop,
+						 (Var *) makeConst(datatype, -1, opr1right,
+										   false, false, false, false));
+	result = makeList1(expr);
+
+	/* create clause "key <= network_scan_last( rightop )" */
+
+	opr2oid = find_operator("<=", datatype);
+	if (opr2oid == InvalidOid)
+		elog(ERROR, "network_prefix_quals: no <= operator for type %u",
+			 datatype);
+
+	opr2right = network_scan_last(rightop);
+
+	op = makeOper(opr2oid, InvalidOid, BOOLOID);
+	expr = make_opclause(op, leftop,
+						 (Var *) makeConst(datatype, -1, opr2right,
+										   false, false, false, false));
+	result = lappend(result, expr);
+
+	return result;
+}
+
 /*
  * Handy subroutines for match_special_index_operator() and friends.
  */
@@ -2030,7 +2161,6 @@ find_operator(const char *opname, Oid datatype)
 static Datum
 string_to_datum(const char *str, Oid datatype)
 {
-
 	/*
 	 * We cheat a little by assuming that textin() will do for bpchar and
 	 * varchar constants too...
