@@ -10,15 +10,14 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/optimizer/plan/createplan.c,v 1.104 2001/03/22 03:59:36 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/optimizer/plan/createplan.c,v 1.111 2001/10/28 06:25:44 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
-#include <sys/types.h>
-
 #include "postgres.h"
 
-#include "catalog/pg_index.h"
+#include <sys/types.h>
+
 #include "nodes/makefuncs.h"
 #include "nodes/nodeFuncs.h"
 #include "optimizer/clauses.h"
@@ -27,6 +26,7 @@
 #include "optimizer/planmain.h"
 #include "optimizer/restrictinfo.h"
 #include "optimizer/tlist.h"
+#include "optimizer/var.h"
 #include "parser/parse_expr.h"
 #include "utils/lsyscache.h"
 #include "utils/syscache.h"
@@ -47,7 +47,8 @@ static NestLoop *create_nestloop_plan(NestPath *best_path, List *tlist,
 					 List *joinclauses, List *otherclauses,
 					 Plan *outer_plan, List *outer_tlist,
 					 Plan *inner_plan, List *inner_tlist);
-static MergeJoin *create_mergejoin_plan(MergePath *best_path, List *tlist,
+static MergeJoin *create_mergejoin_plan(Query *root,
+					  MergePath *best_path, List *tlist,
 					  List *joinclauses, List *otherclauses,
 					  Plan *outer_plan, List *outer_tlist,
 					  Plan *inner_plan, List *inner_tlist);
@@ -55,11 +56,14 @@ static HashJoin *create_hashjoin_plan(HashPath *best_path, List *tlist,
 					 List *joinclauses, List *otherclauses,
 					 Plan *outer_plan, List *outer_tlist,
 					 Plan *inner_plan, List *inner_tlist);
-static List *fix_indxqual_references(List *indexquals, IndexPath *index_path);
-static List *fix_indxqual_sublist(List *indexqual, int baserelid, Oid relam,
-					 Form_pg_index index);
+static void fix_indxqual_references(List *indexquals, IndexPath *index_path,
+						List **fixed_indexquals,
+						List **recheck_indexquals);
+static void fix_indxqual_sublist(List *indexqual, int baserelid,
+					 IndexOptInfo *index,
+					 List **fixed_quals, List **recheck_quals);
 static Node *fix_indxqual_operand(Node *node, int baserelid,
-					 Form_pg_index index,
+					 IndexOptInfo *index,
 					 Oid *opclass);
 static List *switch_outer(List *clauses);
 static void copy_path_costsize(Plan *dest, Path *src);
@@ -244,7 +248,8 @@ create_join_plan(Query *root, JoinPath *best_path)
 	switch (best_path->path.pathtype)
 	{
 		case T_MergeJoin:
-			plan = (Join *) create_mergejoin_plan((MergePath *) best_path,
+			plan = (Join *) create_mergejoin_plan(root,
+												  (MergePath *) best_path,
 												  join_tlist,
 												  joinclauses,
 												  otherclauses,
@@ -365,7 +370,7 @@ create_seqscan_plan(Path *best_path, List *tlist, List *scan_clauses)
  * The indexqual of the path contains a sublist of implicitly-ANDed qual
  * conditions for each scan of the index(es); if there is more than one
  * scan then the retrieved tuple sets are ORed together.  The indexqual
- * and indexid lists must have the same length, ie, the number of scans
+ * and indexinfo lists must have the same length, ie, the number of scans
  * that will occur.  Note it is possible for a qual condition sublist
  * to be empty --- then no index restrictions will be applied during that
  * scan.
@@ -379,10 +384,12 @@ create_indexscan_plan(Query *root,
 	List	   *indxqual = best_path->indexqual;
 	Index		baserelid;
 	List	   *qpqual;
+	Expr	   *indxqual_or_expr = NULL;
 	List	   *fixed_indxqual;
-	List	   *ixid;
+	List	   *recheck_indxqual;
+	List	   *indexids;
+	List	   *ixinfo;
 	IndexScan  *scan_plan;
-	bool		lossy = false;
 
 	/* there should be exactly one base rel involved... */
 	Assert(length(best_path->path.parent->relids) == 1);
@@ -390,33 +397,24 @@ create_indexscan_plan(Query *root,
 
 	baserelid = lfirsti(best_path->path.parent->relids);
 
-	/* check to see if any of the indices are lossy */
-	foreach(ixid, best_path->indexid)
+	/*
+	 * Build list of index OIDs.
+	 */
+	indexids = NIL;
+	foreach(ixinfo, best_path->indexinfo)
 	{
-		HeapTuple	indexTuple;
-		Form_pg_index index;
+		IndexOptInfo *index = (IndexOptInfo *) lfirst(ixinfo);
 
-		indexTuple = SearchSysCache(INDEXRELID,
-									ObjectIdGetDatum(lfirsti(ixid)),
-									0, 0, 0);
-		if (!HeapTupleIsValid(indexTuple))
-			elog(ERROR, "create_plan: index %u not found", lfirsti(ixid));
-		index = (Form_pg_index) GETSTRUCT(indexTuple);
-		if (index->indislossy)
-		{
-			lossy = true;
-			ReleaseSysCache(indexTuple);
-			break;
-		}
-		ReleaseSysCache(indexTuple);
+		indexids = lappendi(indexids, index->indexoid);
 	}
 
 	/*
 	 * The qpqual list must contain all restrictions not automatically
-	 * handled by the index.  Note that for non-lossy indices, the
-	 * predicates in the indxqual are checked fully by the index, while
-	 * for lossy indices the indxqual predicates need to be double-checked
-	 * after the index fetches the best-guess tuples.
+	 * handled by the index.  Normally the predicates in the indxqual are
+	 * checked fully by the index, but if the index is "lossy" for a
+	 * particular operator (as signaled by the amopreqcheck flag in
+	 * pg_amop), then we need to double-check that predicate in qpqual,
+	 * because the index may return more tuples than match the predicate.
 	 *
 	 * Since the indexquals were generated from the restriction clauses given
 	 * by scan_clauses, there will normally be some duplications between
@@ -424,7 +422,6 @@ create_indexscan_plan(Query *root,
 	 */
 	if (length(indxqual) > 1)
 	{
-
 		/*
 		 * Build an expression representation of the indexqual, expanding
 		 * the implicit OR and AND semantics of the first- and
@@ -432,46 +429,64 @@ create_indexscan_plan(Query *root,
 		 */
 		List	   *orclauses = NIL;
 		List	   *orclause;
-		Expr	   *indxqual_expr;
 
 		foreach(orclause, indxqual)
+		{
 			orclauses = lappend(orclauses,
 								make_ands_explicit(lfirst(orclause)));
-		indxqual_expr = make_orclause(orclauses);
+		}
+		indxqual_or_expr = make_orclause(orclauses);
 
-		qpqual = set_difference(scan_clauses, makeList1(indxqual_expr));
-
-		if (lossy)
-			qpqual = lappend(qpqual, copyObject(indxqual_expr));
+		qpqual = set_difference(scan_clauses, makeList1(indxqual_or_expr));
 	}
 	else if (indxqual != NIL)
 	{
-
 		/*
 		 * Here, we can simply treat the first sublist as an independent
 		 * set of qual expressions, since there is no top-level OR
 		 * behavior.
 		 */
-		List	   *indxqual_list = lfirst(indxqual);
-
-		qpqual = set_difference(scan_clauses, indxqual_list);
-
-		if (lossy)
-			qpqual = nconc(qpqual, (List *) copyObject(indxqual_list));
+		qpqual = set_difference(scan_clauses, lfirst(indxqual));
 	}
 	else
 		qpqual = scan_clauses;
 
 	/*
 	 * The executor needs a copy with the indexkey on the left of each
-	 * clause and with index attr numbers substituted for table ones.
+	 * clause and with index attr numbers substituted for table ones. This
+	 * pass also looks for "lossy" operators.
 	 */
-	fixed_indxqual = fix_indxqual_references(indxqual, best_path);
+	fix_indxqual_references(indxqual, best_path,
+							&fixed_indxqual, &recheck_indxqual);
 
+	/*
+	 * If there were any "lossy" operators, need to add back the
+	 * appropriate qual clauses to the qpqual.	When there is just one
+	 * indexscan being performed (ie, we have simple AND semantics), we
+	 * can just add the lossy clauses themselves to qpqual.  If we have
+	 * OR-of-ANDs, we'd better add the entire original indexqual to make
+	 * sure that the semantics are correct.
+	 */
+	if (recheck_indxqual != NIL)
+	{
+		if (indxqual_or_expr)
+		{
+			/* Better do a deep copy of the original scanclauses */
+			qpqual = lappend(qpqual, copyObject(indxqual_or_expr));
+		}
+		else
+		{
+			/* Subroutine already copied quals, so just append to list */
+			Assert(length(recheck_indxqual) == 1);
+			qpqual = nconc(qpqual, (List *) lfirst(recheck_indxqual));
+		}
+	}
+
+	/* Finally ready to build the plan node */
 	scan_plan = make_indexscan(tlist,
 							   qpqual,
 							   baserelid,
-							   best_path->indexid,
+							   indexids,
 							   fixed_indxqual,
 							   indxqual,
 							   best_path->indexscandir);
@@ -574,7 +589,6 @@ create_nestloop_plan(NestPath *best_path,
 
 	if (IsA(inner_plan, IndexScan))
 	{
-
 		/*
 		 * An index is being used to reduce the number of tuples scanned
 		 * in the inner relation.  If there are join clauses being used
@@ -642,7 +656,6 @@ create_nestloop_plan(NestPath *best_path,
 	}
 	else if (IsA_Join(inner_plan))
 	{
-
 		/*
 		 * Materialize the inner join for speed reasons.
 		 *
@@ -679,7 +692,8 @@ create_nestloop_plan(NestPath *best_path,
 }
 
 static MergeJoin *
-create_mergejoin_plan(MergePath *best_path,
+create_mergejoin_plan(Query *root,
+					  MergePath *best_path,
 					  List *tlist,
 					  List *joinclauses,
 					  List *otherclauses,
@@ -726,13 +740,15 @@ create_mergejoin_plan(MergePath *best_path,
 	 */
 	if (best_path->outersortkeys)
 		outer_plan = (Plan *)
-			make_sort_from_pathkeys(outer_tlist,
+			make_sort_from_pathkeys(root,
+									outer_tlist,
 									outer_plan,
 									best_path->outersortkeys);
 
 	if (best_path->innersortkeys)
 		inner_plan = (Plan *)
-			make_sort_from_pathkeys(inner_tlist,
+			make_sort_from_pathkeys(root,
+									inner_tlist,
 									inner_plan,
 									best_path->innersortkeys);
 
@@ -869,9 +885,9 @@ create_hashjoin_plan(HashPath *best_path,
 /*
  * fix_indxqual_references
  *	  Adjust indexqual clauses to the form the executor's indexqual
- *	  machinery needs.
+ *	  machinery needs, and check for recheckable (lossy) index conditions.
  *
- * We have three tasks here:
+ * We have four tasks here:
  *	* Index keys must be represented by Var nodes with varattno set to the
  *	  index's attribute number, not the attribute number in the original rel.
  *	* indxpath.c may have selected an index that is binary-compatible with
@@ -880,62 +896,56 @@ create_hashjoin_plan(HashPath *best_path,
  *	  equivalent operator that the index will recognize.
  *	* If the index key is on the right, commute the clause to put it on the
  *	  left.  (Someday the executor might not need this, but for now it does.)
+ *	* If the indexable operator is marked 'amopreqcheck' in pg_amop, then
+ *	  the index is "lossy" for this operator: it may return more tuples than
+ *	  actually satisfy the operator condition.	For each such operator, we
+ *	  must add (the original form of) the indexqual clause to the "qpquals"
+ *	  of the indexscan node, where the operator will be re-evaluated to
+ *	  ensure it passes.
  *
  * This code used to be entirely bogus for multi-index scans.  Now it keeps
  * track of which index applies to each subgroup of index qual clauses...
  *
- * Returns a modified copy of the indexqual list --- the original is not
- * changed.  Note also that the copy shares no substructure with the
- * original; this is needed in case there is a subplan in it (we need
+ * Both the input list and the output lists have the form of lists of sublists
+ * of qual clauses --- the top-level list has one entry for each indexscan
+ * to be performed.  The semantics are OR-of-ANDs.
+ *
+ * fixed_indexquals receives a modified copy of the indexqual list --- the
+ * original is not changed.  Note also that the copy shares no substructure
+ * with the original; this is needed in case there is a subplan in it (we need
  * two separate copies of the subplan tree, or things will go awry).
+ *
+ * recheck_indexquals similarly receives a full copy of whichever clauses
+ * need rechecking.
  */
-
-static List *
-fix_indxqual_references(List *indexquals, IndexPath *index_path)
+static void
+fix_indxqual_references(List *indexquals, IndexPath *index_path,
+					  List **fixed_indexquals, List **recheck_indexquals)
 {
 	List	   *fixed_quals = NIL;
+	List	   *recheck_quals = NIL;
 	int			baserelid = lfirsti(index_path->path.parent->relids);
-	List	   *indexids = index_path->indexid;
+	List	   *ixinfo = index_path->indexinfo;
 	List	   *i;
 
 	foreach(i, indexquals)
 	{
 		List	   *indexqual = lfirst(i);
-		Oid			indexid = lfirsti(indexids);
-		HeapTuple	indexTuple;
-		Oid			relam;
-		Form_pg_index index;
+		IndexOptInfo *index = (IndexOptInfo *) lfirst(ixinfo);
+		List	   *fixed_qual;
+		List	   *recheck_qual;
 
-		/* Get the relam from the index's pg_class entry */
-		indexTuple = SearchSysCache(RELOID,
-									ObjectIdGetDatum(indexid),
-									0, 0, 0);
-		if (!HeapTupleIsValid(indexTuple))
-			elog(ERROR, "fix_indxqual_references: index %u not found in pg_class",
-				 indexid);
-		relam = ((Form_pg_class) GETSTRUCT(indexTuple))->relam;
-		ReleaseSysCache(indexTuple);
+		fix_indxqual_sublist(indexqual, baserelid, index,
+							 &fixed_qual, &recheck_qual);
+		fixed_quals = lappend(fixed_quals, fixed_qual);
+		if (recheck_qual != NIL)
+			recheck_quals = lappend(recheck_quals, recheck_qual);
 
-		/* Need the index's pg_index entry for other stuff */
-		indexTuple = SearchSysCache(INDEXRELID,
-									ObjectIdGetDatum(indexid),
-									0, 0, 0);
-		if (!HeapTupleIsValid(indexTuple))
-			elog(ERROR, "fix_indxqual_references: index %u not found in pg_index",
-				 indexid);
-		index = (Form_pg_index) GETSTRUCT(indexTuple);
-
-		fixed_quals = lappend(fixed_quals,
-							  fix_indxqual_sublist(indexqual,
-												   baserelid,
-												   relam,
-												   index));
-
-		ReleaseSysCache(indexTuple);
-
-		indexids = lnext(indexids);
+		ixinfo = lnext(ixinfo);
 	}
-	return fixed_quals;
+
+	*fixed_indexquals = fixed_quals;
+	*recheck_indexquals = recheck_quals;
 }
 
 /*
@@ -943,38 +953,32 @@ fix_indxqual_references(List *indexquals, IndexPath *index_path)
  *
  * For each qual clause, commute if needed to put the indexkey operand on the
  * left, and then fix its varattno.  (We do not need to change the other side
- * of the clause.)	Also change the operator if necessary.
+ * of the clause.)	Also change the operator if necessary, and check for
+ * lossy index behavior.
+ *
+ * Returns two lists: the list of fixed indexquals, and the list (usually
+ * empty) of original clauses that must be rechecked as qpquals because
+ * the index is lossy for this operator type.
  */
-static List *
-fix_indxqual_sublist(List *indexqual, int baserelid, Oid relam,
-					 Form_pg_index index)
+static void
+fix_indxqual_sublist(List *indexqual, int baserelid, IndexOptInfo *index,
+					 List **fixed_quals, List **recheck_quals)
 {
 	List	   *fixed_qual = NIL;
+	List	   *recheck_qual = NIL;
 	List	   *i;
 
 	foreach(i, indexqual)
 	{
 		Expr	   *clause = (Expr *) lfirst(i);
-		int			relid;
-		AttrNumber	attno;
-		Datum		constval;
-		int			flag;
 		Expr	   *newclause;
+		List	   *leftvarnos;
 		Oid			opclass,
 					newopno;
 
-		if (!is_opclause((Node *) clause) ||
-			length(clause->args) != 2)
+		if (!is_opclause((Node *) clause) || length(clause->args) != 2)
 			elog(ERROR, "fix_indxqual_sublist: indexqual clause is not binary opclause");
 
-		/*
-		 * Which side is the indexkey on?
-		 *
-		 * get_relattval sets flag&SEL_RIGHT if the indexkey is on the LEFT.
-		 */
-		get_relattval((Node *) clause, baserelid,
-					  &relid, &attno, &constval, &flag);
-
 		/*
 		 * Make a copy that will become the fixed clause.
 		 *
@@ -984,9 +988,15 @@ fix_indxqual_sublist(List *indexqual, int baserelid, Oid relam,
 		 */
 		newclause = (Expr *) copyObject((Node *) clause);
 
-		/* If the indexkey is on the right, commute the clause. */
-		if ((flag & SEL_RIGHT) == 0)
+		/*
+		 * Check to see if the indexkey is on the right; if so, commute
+		 * the clause.	The indexkey should be the side that refers to
+		 * (only) the base relation.
+		 */
+		leftvarnos = pull_varnos((Node *) lfirst(newclause->args));
+		if (length(leftvarnos) != 1 || lfirsti(leftvarnos) != baserelid)
 			CommuteClause(newclause);
+		freeList(leftvarnos);
 
 		/*
 		 * Now, determine which index attribute this is, change the
@@ -1002,21 +1012,30 @@ fix_indxqual_sublist(List *indexqual, int baserelid, Oid relam,
 		 * is merely binary-compatible with the index.	This shouldn't
 		 * fail, since indxpath.c found it before...
 		 */
-		newopno = indexable_operator(newclause, opclass, relam, true);
+		newopno = indexable_operator(newclause, opclass, true);
 		if (newopno == InvalidOid)
 			elog(ERROR, "fix_indxqual_sublist: failed to find substitute op");
 		((Oper *) newclause->oper)->opno = newopno;
 
 		fixed_qual = lappend(fixed_qual, newclause);
+
+		/*
+		 * Finally, check to see if index is lossy for this operator. If
+		 * so, add (a copy of) original form of clause to recheck list.
+		 */
+		if (op_requires_recheck(newopno, opclass))
+			recheck_qual = lappend(recheck_qual,
+								   copyObject((Node *) clause));
 	}
-	return fixed_qual;
+
+	*fixed_quals = fixed_qual;
+	*recheck_quals = recheck_qual;
 }
 
 static Node *
-fix_indxqual_operand(Node *node, int baserelid, Form_pg_index index,
+fix_indxqual_operand(Node *node, int baserelid, IndexOptInfo *index,
 					 Oid *opclass)
 {
-
 	/*
 	 * Remove any binary-compatible relabeling of the indexkey
 	 */
@@ -1033,27 +1052,29 @@ fix_indxqual_operand(Node *node, int baserelid, Form_pg_index index,
 	if (IsA(node, Var))
 	{
 		/* If it's a var, find which index key position it occupies */
+		Assert(index->indproc == InvalidOid);
+
 		if (((Var *) node)->varno == baserelid)
 		{
 			int			varatt = ((Var *) node)->varattno;
 			int			pos;
 
-			for (pos = 0; pos < INDEX_MAX_KEYS; pos++)
+			for (pos = 0; pos < index->nkeys; pos++)
 			{
-				if (index->indkey[pos] == varatt)
+				if (index->indexkeys[pos] == varatt)
 				{
 					Node	   *newnode = copyObject(node);
 
 					((Var *) newnode)->varattno = pos + 1;
 					/* return the correct opclass, too */
-					*opclass = index->indclass[pos];
+					*opclass = index->classlist[pos];
 					return newnode;
 				}
 			}
 		}
 
 		/*
-		 * Oops, this Var isn't the indexkey!
+		 * Oops, this Var isn't an indexkey!
 		 */
 		elog(ERROR, "fix_indxqual_operand: var is not index attribute");
 	}
@@ -1063,11 +1084,11 @@ fix_indxqual_operand(Node *node, int baserelid, Form_pg_index index,
 	 * Since we currently only support single-column functional indexes,
 	 * the returned varattno must be 1.
 	 */
+	Assert(index->indproc != InvalidOid);
+	Assert(is_funcclause(node));	/* not a very thorough check, but easy */
 
-	Assert(is_funcclause(node));/* not a very thorough check, but easy */
-
-	/* indclass[0] is the only class of a functional index */
-	*opclass = index->indclass[0];
+	/* classlist[0] is the only class of a functional index */
+	*opclass = index->classlist[0];
 
 	return (Node *) makeVar(baserelid, 1, exprType(node), -1, 0);
 }
@@ -1095,7 +1116,6 @@ switch_outer(List *clauses)
 		Assert(op && IsA(op, Var));
 		if (var_is_outer(op))
 		{
-
 			/*
 			 * Duplicate just enough of the structure to allow commuting
 			 * the clause without changing the original list.  Could use
@@ -1404,14 +1424,15 @@ make_mergejoin(List *tlist,
  * each key number from 1 to keycount), or the executor will get confused!
  */
 Sort *
-make_sort(List *tlist, Plan *lefttree, int keycount)
+make_sort(Query *root, List *tlist, Plan *lefttree, int keycount)
 {
 	Sort	   *node = makeNode(Sort);
 	Plan	   *plan = &node->plan;
 	Path		sort_path;		/* dummy for result of cost_sort */
 
 	copy_plan_costsize(plan, lefttree); /* only care about copying size */
-	cost_sort(&sort_path, NIL, lefttree->plan_rows, lefttree->plan_width);
+	cost_sort(&sort_path, root, NIL,
+			  lefttree->plan_rows, lefttree->plan_width);
 	plan->startup_cost = sort_path.startup_cost + lefttree->total_cost;
 	plan->total_cost = sort_path.total_cost + lefttree->total_cost;
 	plan->state = (EState *) NULL;
@@ -1436,7 +1457,8 @@ make_sort(List *tlist, Plan *lefttree, int keycount)
  * of resdom nodes in the sort plan's target list.
  */
 Sort *
-make_sort_from_pathkeys(List *tlist, Plan *lefttree, List *pathkeys)
+make_sort_from_pathkeys(Query *root, List *tlist,
+						Plan *lefttree, List *pathkeys)
 {
 	List	   *sort_tlist;
 	List	   *i;
@@ -1484,18 +1506,18 @@ make_sort_from_pathkeys(List *tlist, Plan *lefttree, List *pathkeys)
 		 */
 		if (resdom->reskey == 0)
 		{
-			/* OK, mark it as a sort key and set the sort operator regproc */
+			/* OK, mark it as a sort key and set the sort operator */
 			resdom->reskey = ++numsortkeys;
-			resdom->reskeyop = get_opcode(pathkey->sortop);
+			resdom->reskeyop = pathkey->sortop;
 		}
 	}
 
 	Assert(numsortkeys > 0);
 
-	return make_sort(sort_tlist, lefttree, numsortkeys);
+	return make_sort(root, sort_tlist, lefttree, numsortkeys);
 }
 
-Material   *
+Material *
 make_material(List *tlist, Plan *lefttree)
 {
 	Material   *node = makeNode(Material);
@@ -1801,7 +1823,16 @@ make_result(List *tlist,
 #ifdef NOT_USED
 	tlist = generate_fjoin(tlist);
 #endif
-	copy_plan_costsize(plan, subplan);
+	if (subplan)
+		copy_plan_costsize(plan, subplan);
+	else
+	{
+		plan->startup_cost = 0;
+		plan->total_cost = cpu_tuple_cost;
+		plan->plan_rows = 1;	/* wrong if we have a set-valued function? */
+		plan->plan_width = 0;	/* XXX try to be smarter? */
+	}
+
 	plan->state = (EState *) NULL;
 	plan->targetlist = tlist;
 	plan->qual = NIL;
