@@ -3,11 +3,11 @@
  * subselect.c
  *	  Planning routines for subselects and parameters.
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/optimizer/plan/subselect.c,v 1.36 2000/04/14 00:19:16 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/optimizer/plan/subselect.c,v 1.50 2001/11/30 19:24:15 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -16,22 +16,21 @@
 #include "catalog/pg_operator.h"
 #include "catalog/pg_type.h"
 #include "nodes/makefuncs.h"
-#include "nodes/nodeFuncs.h"
 #include "optimizer/clauses.h"
 #include "optimizer/cost.h"
 #include "optimizer/planmain.h"
 #include "optimizer/planner.h"
 #include "optimizer/subselect.h"
 #include "parser/parse_expr.h"
-#include "parser/parse_node.h"
 #include "parser/parse_oper.h"
-#include "utils/lsyscache.h"
+#include "utils/syscache.h"
 
 
 Index		PlannerQueryLevel;	/* level of current query */
 List	   *PlannerInitPlan;	/* init subplans for current query */
 List	   *PlannerParamVar;	/* to get Var from Param->paramid */
-int			PlannerPlanId;		/* to assign unique ID to subquery plans */
+
+int			PlannerPlanId = 0;	/* to assign unique ID to subquery plans */
 
 /*--------------------
  * PlannerParamVar is a list of Var nodes, wherein the n'th entry
@@ -83,12 +82,12 @@ replace_var(Var *var)
 
 	/*
 	 * If there's already a PlannerParamVar entry for this same Var, just
-	 * use it.	NOTE: in situations involving UNION or inheritance, it is
-	 * possible for the same varno/varlevel to refer to different RTEs in
-	 * different parts of the parsetree, so that different fields might
-	 * end up sharing the same Param number.  As long as we check the
-	 * vartype as well, I believe that this sort of aliasing will cause no
-	 * trouble. The correct field should get stored into the Param slot at
+	 * use it.	NOTE: in sufficiently complex querytrees, it is possible
+	 * for the same varno/varlevel to refer to different RTEs in different
+	 * parts of the parsetree, so that different fields might end up
+	 * sharing the same Param number.  As long as we check the vartype as
+	 * well, I believe that this sort of aliasing will cause no trouble.
+	 * The correct field should get stored into the Param slot at
 	 * execution in each part of the tree.
 	 */
 	i = 0;
@@ -126,23 +125,30 @@ make_subplan(SubLink *slink)
 {
 	SubPlan    *node = makeNode(SubPlan);
 	Query	   *subquery = (Query *) (slink->subselect);
+	Oid			result_type = exprType((Node *) slink);
 	double		tuple_fraction;
 	Plan	   *plan;
 	List	   *lst;
 	Node	   *result;
-	List	   *saved_ip = PlannerInitPlan;
 
-	PlannerInitPlan = NULL;
-
-	PlannerQueryLevel++;		/* we become child */
-
-	/* Check to see if this node was already processed; if so we have
-	 * trouble.  Someday should change tree representation so that we can
-	 * cope with multiple links to the same subquery, but for now...
+	/*
+	 * Check to see if this node was already processed; if so we have
+	 * trouble.  We check to see if the linked-to Query appears to have
+	 * been planned already, too.
 	 */
 	if (subquery == NULL)
+		elog(ERROR, "make_subplan: invalid expression structure (SubLink already processed?)");
+	if (subquery->base_rel_list != NIL)
 		elog(ERROR, "make_subplan: invalid expression structure (subquery already processed?)");
 
+	/*
+	 * Copy the source Query node.	This is a quick and dirty kluge to
+	 * resolve the fact that the parser can generate trees with multiple
+	 * links to the same sub-Query node, but the planner wants to scribble
+	 * on the Query. Try to clean this up when we do querytree redesign...
+	 */
+	subquery = (Query *) copyObject(subquery);
+
 	/*
 	 * For an EXISTS subplan, tell lower-level planner to expect that only
 	 * the first tuple will be retrieved.  For ALL and ANY subplans, we
@@ -172,45 +178,31 @@ make_subplan(SubLink *slink)
 	else
 		tuple_fraction = -1.0;	/* default behavior */
 
+	/*
+	 * Generate the plan for the subquery.
+	 */
 	node->plan = plan = subquery_planner(subquery, tuple_fraction);
 
-	/*
-	 * Assign subPlan, extParam and locParam to plan nodes. At the moment,
-	 * SS_finalize_plan doesn't handle initPlan-s and so we assign them to
-	 * the topmost plan node and take care about its extParam too.
-	 */
-	(void) SS_finalize_plan(plan);
-	plan->initPlan = PlannerInitPlan;
+	node->plan_id = PlannerPlanId++;	/* Assign unique ID to this
+										 * SubPlan */
 
-	/* Create extParam list as union of InitPlan-s' lists */
-	foreach(lst, PlannerInitPlan)
-	{
-		List	   *lp;
-
-		foreach(lp, ((SubPlan *) lfirst(lst))->plan->extParam)
-		{
-			if (!intMember(lfirsti(lp), plan->extParam))
-				plan->extParam = lappendi(plan->extParam, lfirsti(lp));
-		}
-	}
-
-	/* and now we are parent again */
-	PlannerInitPlan = saved_ip;
-	PlannerQueryLevel--;
-
-	node->plan_id = PlannerPlanId++;
 	node->rtable = subquery->rtable;
 	node->sublink = slink;
+
 	slink->subselect = NULL;	/* cool ?! see error check above! */
 
-	/* make parParam list of params coming from current query level */
+	/*
+	 * Make parParam list of params that current query level will pass to
+	 * this child plan.
+	 */
 	foreach(lst, plan->extParam)
 	{
-		Var		   *var = nth(lfirsti(lst), PlannerParamVar);
+		int			paramid = lfirsti(lst);
+		Var		   *var = nth(paramid, PlannerParamVar);
 
 		/* note varlevelsup is absolute level number */
 		if (var->varlevelsup == PlannerQueryLevel)
-			node->parParam = lappendi(node->parParam, lfirsti(lst));
+			node->parParam = lappendi(node->parParam, paramid);
 	}
 
 	/*
@@ -281,8 +273,11 @@ make_subplan(SubLink *slink)
 			pfree(var);			/* var is only needed for new_param */
 
 			Assert(IsA(oper, Oper));
-			tup = get_operator_tuple(oper->opno);
-			Assert(HeapTupleIsValid(tup));
+			tup = SearchSysCache(OPEROID,
+								 ObjectIdGetDatum(oper->opno),
+								 0, 0, 0);
+			if (!HeapTupleIsValid(tup))
+				elog(ERROR, "cache lookup failed for operator %u", oper->opno);
 			opform = (Form_pg_operator) GETSTRUCT(tup);
 
 			/*
@@ -293,6 +288,8 @@ make_subplan(SubLink *slink)
 								exprType(lefthand), opform->oprleft);
 			right = make_operand("", (Node *) prm,
 								 prm->paramtype, opform->oprright);
+			ReleaseSysCache(tup);
+
 			newoper = lappend(newoper,
 							  make_opclause(oper,
 											(Var *) left,
@@ -327,6 +324,12 @@ make_subplan(SubLink *slink)
 		 * is anything more complicated than a plain sequential scan, and
 		 * we do it even for seqscan if the qual appears selective enough
 		 * to eliminate many tuples.
+		 *
+		 * XXX It's pretty ugly to be inserting a MATERIAL node at this
+		 * point.  Since subquery_planner has already run SS_finalize_plan
+		 * on the subplan tree, we have to kluge up parameter lists for
+		 * the MATERIAL node.  Possibly this could be fixed by postponing
+		 * SS_finalize_plan processing until setrefs.c is run.
 		 */
 		if (node->parParam == NIL)
 		{
@@ -365,17 +368,20 @@ make_subplan(SubLink *slink)
 			}
 			if (use_material)
 			{
-				plan = (Plan *) make_noname(plan->targetlist,
-											NIL,
-											plan);
-				node->plan = plan;
+				Plan   *matplan;
+
+				matplan = (Plan *) make_material(plan->targetlist, plan);
+				/* kluge --- see comments above */
+				matplan->extParam = listCopy(plan->extParam);
+				matplan->locParam = listCopy(plan->locParam);
+				node->plan = plan = matplan;
 			}
 		}
 
 		/*
 		 * Make expression of SUBPLAN type
 		 */
-		expr->typeOid = BOOLOID;/* bogus, but we don't really care */
+		expr->typeOid = result_type;
 		expr->opType = SUBPLAN_EXPR;
 		expr->oper = (Node *) node;
 
@@ -413,15 +419,14 @@ make_subplan(SubLink *slink)
 			Node	   *left,
 					   *right;
 
-			/*
-			 * XXX really ought to fill in constlen and constbyval
-			 * correctly, but right now ExecEvalExpr won't look at them...
-			 */
-			con = makeConst(te->resdom->restype, 0, 0, true, 0, 0, 0);
+			con = makeNullConst(te->resdom->restype);
 
 			Assert(IsA(oper, Oper));
-			tup = get_operator_tuple(oper->opno);
-			Assert(HeapTupleIsValid(tup));
+			tup = SearchSysCache(OPEROID,
+								 ObjectIdGetDatum(oper->opno),
+								 0, 0, 0);
+			if (!HeapTupleIsValid(tup))
+				elog(ERROR, "cache lookup failed for operator %u", oper->opno);
 			opform = (Form_pg_operator) GETSTRUCT(tup);
 
 			/*
@@ -432,6 +437,8 @@ make_subplan(SubLink *slink)
 								exprType(lefthand), opform->oprleft);
 			right = make_operand("", (Node *) con,
 								 con->consttype, opform->oprright);
+			ReleaseSysCache(tup);
+
 			newoper = lappend(newoper,
 							  make_opclause(oper,
 											(Var *) left,
@@ -446,19 +453,6 @@ make_subplan(SubLink *slink)
 	return result;
 }
 
-/* this oughta be merged with LispUnioni */
-
-static List *
-set_unioni(List *l1, List *l2)
-{
-	if (l1 == NULL)
-		return l2;
-	if (l2 == NULL)
-		return l1;
-
-	return nconc(l1, set_differencei(l2, l1));
-}
-
 /*
  * finalize_primnode: build lists of subplans and params appearing
  * in the given expression tree.  NOTE: items are added to lists passed in,
@@ -631,6 +625,19 @@ SS_finalize_plan(Plan *plan)
 								 SS_finalize_plan((Plan *) lfirst(lst)));
 			break;
 
+		case T_SubqueryScan:
+
+			/*
+			 * In a SubqueryScan, SS_finalize_plan has already been run on
+			 * the subplan by the inner invocation of subquery_planner, so
+			 * there's no need to do it again.  Instead, just pull out the
+			 * subplan's extParams list, which represents the params it
+			 * needs from my level and higher levels.
+			 */
+			results.paramids = set_unioni(results.paramids,
+							 ((SubqueryScan *) plan)->subplan->extParam);
+			break;
+
 		case T_IndexScan:
 			finalize_primnode((Node *) ((IndexScan *) plan)->indxqual,
 							  &results);
@@ -642,18 +649,27 @@ SS_finalize_plan(Plan *plan)
 			 */
 			break;
 
+		case T_NestLoop:
+			finalize_primnode((Node *) ((Join *) plan)->joinqual,
+							  &results);
+			break;
+
 		case T_MergeJoin:
+			finalize_primnode((Node *) ((Join *) plan)->joinqual,
+							  &results);
 			finalize_primnode((Node *) ((MergeJoin *) plan)->mergeclauses,
 							  &results);
 			break;
 
 		case T_HashJoin:
+			finalize_primnode((Node *) ((Join *) plan)->joinqual,
+							  &results);
 			finalize_primnode((Node *) ((HashJoin *) plan)->hashclauses,
 							  &results);
 			break;
 
 		case T_Hash:
-			finalize_primnode((Node *) ((Hash *) plan)->hashkey,
+			finalize_primnode(((Hash *) plan)->hashkey,
 							  &results);
 			break;
 
@@ -664,10 +680,11 @@ SS_finalize_plan(Plan *plan)
 
 		case T_Agg:
 		case T_SeqScan:
-		case T_NestLoop:
 		case T_Material:
 		case T_Sort:
 		case T_Unique:
+		case T_SetOp:
+		case T_Limit:
 		case T_Group:
 			break;
 
@@ -686,17 +703,18 @@ SS_finalize_plan(Plan *plan)
 
 	foreach(lst, results.paramids)
 	{
-		Var		   *var = nth(lfirsti(lst), PlannerParamVar);
+		int			paramid = lfirsti(lst);
+		Var		   *var = nth(paramid, PlannerParamVar);
 
 		/* note varlevelsup is absolute level number */
 		if (var->varlevelsup < PlannerQueryLevel)
-			extParam = lappendi(extParam, lfirsti(lst));
+			extParam = lappendi(extParam, paramid);
 		else if (var->varlevelsup > PlannerQueryLevel)
 			elog(ERROR, "SS_finalize_plan: plan shouldn't reference subplan's variable");
 		else
 		{
 			Assert(var->varno == 0 && var->varattno == 0);
-			locParam = lappendi(locParam, lfirsti(lst));
+			locParam = lappendi(locParam, paramid);
 		}
 	}
 
