@@ -1,5 +1,6 @@
 /*-
- * Copyright (c) 1990 The Regents of the University of California.
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1990 The Regents of the University of California.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,8 +34,7 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)dl.c	5.4 (Berkeley) 2/23/91";
-
-#endif	 /* LIBC_SCCS and not lint */
+#endif   /* LIBC_SCCS and not lint */
 
 #include <sys/types.h>
 #include <nlist.h>
@@ -44,6 +44,7 @@ static char sccsid[] = "@(#)dl.c	5.4 (Berkeley) 2/23/91";
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "postgres.h"
 #include "dynloader.h"
 
 static char error_message[BUFSIZ];
@@ -61,7 +62,7 @@ BSD44_derived_dlerror(void)
 void *
 BSD44_derived_dlopen(const char *file, int num)
 {
-#if defined(__mips__) || (defined(__NetBSD__) && defined(vax))
+#if defined(__mips__)
 	sprintf(error_message, "dlopen (%s) not supported", file);
 	return NULL;
 #else
@@ -76,14 +77,15 @@ BSD44_derived_dlopen(const char *file, int num)
 void *
 BSD44_derived_dlsym(void *handle, const char *name)
 {
-#if defined(__mips__) || (defined(__NetBSD__) && defined(vax))
+#if defined(__mips__)
 	sprintf(error_message, "dlsym (%s) failed", name);
 	return NULL;
 #else
 	void	   *vp;
-	char		buf[BUFSIZ];
 
 #ifndef __ELF__
+	char		buf[BUFSIZ];
+
 	if (*name != '_')
 	{
 		sprintf(buf, "_%s", name);
@@ -99,7 +101,7 @@ BSD44_derived_dlsym(void *handle, const char *name)
 void
 BSD44_derived_dlclose(void *handle)
 {
-#if defined(__mips__) || (defined(__NetBSD__) && defined(vax))
+#if defined(__mips__)
 #else
 	dlclose(handle);
 #endif
