@@ -11,23 +11,25 @@
  *	  pool. rd_islocal is reset at the end of a transaction (commit/abort).
  *	  This is useful for queries like SELECT INTO TABLE and create index.
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994-5, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/buffer/localbuf.c,v 1.30 2000/04/12 17:15:34 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/storage/buffer/localbuf.c,v 1.41 2001/05/12 19:58:27 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
+#include "postgres.h"
+
 #include <sys/types.h>
 #include <sys/file.h>
 #include <math.h>
 #include <signal.h>
 
-#include "postgres.h"
-
 #include "executor/execdebug.h"
+#include "storage/buf_internals.h"
+#include "storage/bufmgr.h"
 #include "storage/smgr.h"
 #include "utils/relcache.h"
 
@@ -35,6 +37,7 @@ extern long int LocalBufferFlushCount;
 
 int			NLocBuffer = 64;
 BufferDesc *LocalBufferDescriptors = NULL;
+Block	   *LocalBufferBlockPointers = NULL;
 long	   *LocalRefCount = NULL;
 
 static int	nextFreeLocalBuf = 0;
@@ -51,16 +54,11 @@ LocalBufferAlloc(Relation reln, BlockNumber blockNum, bool *foundPtr)
 	int			i;
 	BufferDesc *bufHdr = (BufferDesc *) NULL;
 
-	if (blockNum == P_NEW)
-	{
-		blockNum = reln->rd_nblocks;
-		reln->rd_nblocks++;
-	}
-
 	/* a low tech search for now -- not optimized for scans */
 	for (i = 0; i < NLocBuffer; i++)
 	{
-		if (LocalBufferDescriptors[i].tag.relId.relId == RelationGetRelid(reln) &&
+		if (LocalBufferDescriptors[i].tag.rnode.relNode ==
+			reln->rd_node.relNode &&
 			LocalBufferDescriptors[i].tag.blockNum == blockNum)
 		{
 
@@ -100,9 +98,9 @@ LocalBufferAlloc(Relation reln, BlockNumber blockNum, bool *foundPtr)
 	 * transaction to touch it doesn't need its contents but has not
 	 * flushed it).  if that's the case, write it out before reusing it!
 	 */
-	if (bufHdr->flags & BM_DIRTY)
+	if (bufHdr->flags & BM_DIRTY || bufHdr->cntxDirty)
 	{
-		Relation	bufrel = RelationIdCacheGetRelation(bufHdr->tag.relId.relId);
+		Relation	bufrel = RelationNodeCacheGetRelation(bufHdr->tag.rnode);
 
 		Assert(bufrel != NULL);
 
@@ -120,20 +118,36 @@ LocalBufferAlloc(Relation reln, BlockNumber blockNum, bool *foundPtr)
 
 	/*
 	 * it's all ours now.
+	 *
+	 * We need not in tblNode currently but will in future I think, when
+	 * we'll give up rel->rd_fd to fmgr cache.
 	 */
-	bufHdr->tag.relId.relId = RelationGetRelid(reln);
+	bufHdr->tag.rnode = reln->rd_node;
 	bufHdr->tag.blockNum = blockNum;
 	bufHdr->flags &= ~BM_DIRTY;
+	bufHdr->cntxDirty = false;
 
 	/*
-	 * lazy memory allocation. (see MAKE_PTR for why we need to do
-	 * MAKE_OFFSET.)
+	 * lazy memory allocation: allocate space on first use of a buffer.
 	 */
 	if (bufHdr->data == (SHMEM_OFFSET) 0)
 	{
 		char	   *data = (char *) malloc(BLCKSZ);
 
+		if (data == NULL)
+			elog(FATAL, "Out of memory in LocalBufferAlloc");
+
+		/*
+		 * This is a bit of a hack: bufHdr->data needs to be a shmem
+		 * offset for consistency with the shared-buffer case, so make it
+		 * one even though it's not really a valid shmem offset.
+		 */
 		bufHdr->data = MAKE_OFFSET(data);
+
+		/*
+		 * Set pointer for use by BufferGetBlock() macro.
+		 */
+		LocalBufferBlockPointers[-(bufHdr->buf_id + 2)] = (Block) data;
 	}
 
 	*foundPtr = FALSE;
@@ -167,49 +181,10 @@ WriteLocalBuffer(Buffer buffer, bool release)
 	return true;
 }
 
-/*
- * FlushLocalBuffer -
- *	  flushes a local buffer
- */
-int
-FlushLocalBuffer(Buffer buffer, bool release)
-{
-	int			bufid;
-	Relation	bufrel;
-	BufferDesc *bufHdr;
-
-	Assert(BufferIsLocal(buffer));
-
-#ifdef LBDEBUG
-	fprintf(stderr, "LB FLUSH %d\n", buffer);
-#endif
-
-	bufid = -(buffer + 1);
-	bufHdr = &LocalBufferDescriptors[bufid];
-	bufHdr->flags &= ~BM_DIRTY;
-	bufrel = RelationIdCacheGetRelation(bufHdr->tag.relId.relId);
-
-	Assert(bufrel != NULL);
-	smgrflush(DEFAULT_SMGR, bufrel, bufHdr->tag.blockNum,
-			  (char *) MAKE_PTR(bufHdr->data));
-	LocalBufferFlushCount++;
-
-	/* drop relcache refcount incremented by RelationIdCacheGetRelation */
-	RelationDecrementReferenceCount(bufrel);
-
-	if (release)
-	{
-		Assert(LocalRefCount[bufid] > 0);
-		LocalRefCount[bufid]--;
-	}
-
-	return true;
-}
-
 /*
  * InitLocalBuffer -
  *	  init the local buffer cache. Since most queries (esp. multi-user ones)
- *	  don't involve local buffers, we delay allocating memory for actual the
+ *	  don't involve local buffers, we delay allocating actual memory for the
  *	  buffer until we need it.
  */
 void
@@ -221,8 +196,9 @@ InitLocalBuffer(void)
 	 * these aren't going away. I'm not gonna use palloc.
 	 */
 	LocalBufferDescriptors =
-		(BufferDesc *) malloc(sizeof(BufferDesc) * NLocBuffer);
-	MemSet(LocalBufferDescriptors, 0, sizeof(BufferDesc) * NLocBuffer);
+		(BufferDesc *) calloc(NLocBuffer, sizeof(BufferDesc));
+	LocalBufferBlockPointers = (Block *) calloc(NLocBuffer, sizeof(Block));
+	LocalRefCount = (long *) calloc(NLocBuffer, sizeof(long));
 	nextFreeLocalBuf = 0;
 
 	for (i = 0; i < NLocBuffer; i++)
@@ -237,16 +213,17 @@ InitLocalBuffer(void)
 		 */
 		buf->buf_id = -i - 2;
 	}
-
-	LocalRefCount = (long *) malloc(sizeof(long) * NLocBuffer);
-	MemSet(LocalRefCount, 0, sizeof(long) * NLocBuffer);
 }
 
 /*
- * LocalBufferSync -
- *	  flush all dirty buffers in the local buffer cache. Since the buffer
- *	  cache is only used for keeping relations visible during a transaction,
- *	  we will not need these buffers again.
+ * LocalBufferSync
+ *
+ * Flush all dirty buffers in the local buffer cache at commit time.
+ * Since the buffer cache is only used for keeping relations visible
+ * during a transaction, we will not need these buffers again.
+ *
+ * Note that we have to *flush* local buffers because of them are not
+ * visible to checkpoint makers. But we can skip XLOG flush check.
  */
 void
 LocalBufferSync(void)
@@ -258,24 +235,25 @@ LocalBufferSync(void)
 		BufferDesc *buf = &LocalBufferDescriptors[i];
 		Relation	bufrel;
 
-		if (buf->flags & BM_DIRTY)
+		if (buf->flags & BM_DIRTY || buf->cntxDirty)
 		{
 #ifdef LBDEBUG
 			fprintf(stderr, "LB SYNC %d\n", -i - 1);
 #endif
-			bufrel = RelationIdCacheGetRelation(buf->tag.relId.relId);
+			bufrel = RelationNodeCacheGetRelation(buf->tag.rnode);
 
 			Assert(bufrel != NULL);
 
 			smgrwrite(DEFAULT_SMGR, bufrel, buf->tag.blockNum,
 					  (char *) MAKE_PTR(buf->data));
+			smgrmarkdirty(DEFAULT_SMGR, bufrel, buf->tag.blockNum);
 			LocalBufferFlushCount++;
 
 			/* drop relcache refcount from RelationIdCacheGetRelation */
 			RelationDecrementReferenceCount(bufrel);
 
-			buf->tag.relId.relId = InvalidOid;
 			buf->flags &= ~BM_DIRTY;
+			buf->cntxDirty = false;
 		}
 	}
 
@@ -292,9 +270,9 @@ ResetLocalBufferPool(void)
 	{
 		BufferDesc *buf = &LocalBufferDescriptors[i];
 
-		buf->tag.relId.relId = InvalidOid;
+		buf->tag.rnode.relNode = InvalidOid;
 		buf->flags &= ~BM_DIRTY;
-		buf->buf_id = -i - 2;
+		buf->cntxDirty = false;
 	}
 
 	MemSet(LocalRefCount, 0, sizeof(long) * NLocBuffer);
