@@ -3,11 +3,11 @@
  * fd.c
  *	  Virtual file descriptor code.
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/storage/file/fd.c,v 1.56.2.1 2000/09/23 23:31:24 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/storage/file/fd.c,v 1.88 2002/02/10 22:56:31 tgl Exp $
  *
  * NOTES:
  *
@@ -38,18 +38,25 @@
  *-------------------------------------------------------------------------
  */
 
+#include "postgres.h"
+
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#include <dirent.h>
 #include <errno.h>
 #include <unistd.h>
 #include <fcntl.h>
 
-#include "postgres.h"
 #include "miscadmin.h"
 #include "storage/fd.h"
-#include "storage/ipc.h"
+
+
+/* Filename components for OpenTemporaryFile */
+#define PG_TEMP_FILES_DIR "pgsql_tmp"
+#define PG_TEMP_FILE_PREFIX "pgsql_tmp"
+
 
 /*
  * Problem: Postgres does a system(ld...) to do dynamic loading.
@@ -62,7 +69,7 @@
  *
  * (Even though most dynamic loaders now use dlopen(3) or the
  * equivalent, the OS must still open several files to perform the
- * dynamic loading.  Keep this here.)
+ * dynamic loading.  And stdin/stdout/stderr count too.  Keep this here.)
  */
 #ifndef RESERVE_FOR_LD
 #define RESERVE_FOR_LD	10
@@ -80,6 +87,14 @@
 #define FD_MINFREE 10
 #endif
 
+/*
+ * A number of platforms return values for sysconf(_SC_OPEN_MAX) that are
+ * far beyond what they can really support.  This GUC parameter limits what
+ * we will believe.
+ */
+int			max_files_per_process = 1000;
+
+
 /* Debugging.... */
 
 #ifdef FDDEBUG
@@ -95,22 +110,24 @@
 
 #define FileIsNotOpen(file) (VfdCache[file].fd == VFD_CLOSED)
 
+#define FileUnknownPos (-1L)
+
 typedef struct vfd
 {
 	signed short fd;			/* current FD, or VFD_CLOSED if none */
 	unsigned short fdstate;		/* bitflags for VFD's state */
 
 /* these are the assigned bits in fdstate: */
-#define FD_DIRTY		(1 << 0)/* written to, but not yet fsync'd */
-#define FD_TEMPORARY	(1 << 1)/* should be unlinked when closed */
+#define FD_DIRTY		(1 << 0)	/* written to, but not yet fsync'd */
+#define FD_TEMPORARY	(1 << 1)	/* should be unlinked when closed */
 
 	File		nextFree;		/* link to next free VFD, if in freelist */
-	File		lruMoreRecently;/* doubly linked recency-of-use list */
+	File		lruMoreRecently;	/* doubly linked recency-of-use list */
 	File		lruLessRecently;
 	long		seekPos;		/* current logical file position */
 	char	   *fileName;		/* name of file, or NULL for unused VFD */
 	/* NB: fileName is malloc'd, and must be free'd when closing the VFD */
-	int			fileFlags;		/* open(2) flags for opening the file */
+	int			fileFlags;		/* open(2) flags for (re)opening the file */
 	int			fileMode;		/* mode to pass to open(2) */
 } Vfd;
 
@@ -187,16 +204,81 @@ static void FreeVfd(File file);
 
 static int	FileAccess(File file);
 static File fileNameOpenFile(FileName fileName, int fileFlags, int fileMode);
-static char *filepath(char *filename);
+static char *filepath(const char *filename);
 static long pg_nofile(void);
 
 /*
- * pg_fsync --- same as fsync except does nothing if -F switch was given
+ * pg_fsync --- same as fsync except does nothing if enableFsync is off
  */
 int
 pg_fsync(int fd)
 {
-	return disableFsync ? 0 : fsync(fd);
+	if (enableFsync)
+		return fsync(fd);
+	else
+		return 0;
+}
+
+/*
+ * pg_fdatasync --- same as fdatasync except does nothing if enableFsync is off
+ *
+ * Not all platforms have fdatasync; treat as fsync if not available.
+ */
+int
+pg_fdatasync(int fd)
+{
+	if (enableFsync)
+	{
+#ifdef HAVE_FDATASYNC
+		return fdatasync(fd);
+#else
+		return fsync(fd);
+#endif
+	}
+	else
+		return 0;
+}
+
+/*
+ * BasicOpenFile --- same as open(2) except can free other FDs if needed
+ *
+ * This is exported for use by places that really want a plain kernel FD,
+ * but need to be proof against running out of FDs.  Once an FD has been
+ * successfully returned, it is the caller's responsibility to ensure that
+ * it will not be leaked on elog()!  Most users should *not* call this
+ * routine directly, but instead use the VFD abstraction level, which
+ * provides protection against descriptor leaks as well as management of
+ * files that need to be open for more than a short period of time.
+ *
+ * Ideally this should be the *only* direct call of open() in the backend.
+ * In practice, the postmaster calls open() directly, and there are some
+ * direct open() calls done early in backend startup.  Those are OK since
+ * this module wouldn't have any open files to close at that point anyway.
+ */
+int
+BasicOpenFile(FileName fileName, int fileFlags, int fileMode)
+{
+	int			fd;
+
+tryAgain:
+	fd = open(fileName, fileFlags, fileMode);
+
+	if (fd >= 0)
+		return fd;				/* success! */
+
+	if (errno == EMFILE || errno == ENFILE)
+	{
+		int			save_errno = errno;
+
+		DO_DB(elog(DEBUG, "BasicOpenFile: not enough descs, retry, er= %d",
+				   errno));
+		errno = 0;
+		if (ReleaseLruFile())
+			goto tryAgain;
+		errno = save_errno;
+	}
+
+	return -1;					/* failure */
 }
 
 /*
@@ -207,24 +289,46 @@ pg_nofile(void)
 {
 	static long no_files = 0;
 
+	/* need do this calculation only once */
 	if (no_files == 0)
 	{
-		/* need do this calculation only once */
-#ifndef HAVE_SYSCONF
+		/*
+		 * Ask the system what its files-per-process limit is.
+		 */
+#ifdef HAVE_SYSCONF
+		no_files = sysconf(_SC_OPEN_MAX);
+		if (no_files <= 0)
+		{
+#ifdef NOFILE
+			no_files = (long) NOFILE;
+#else
+			no_files = (long) max_files_per_process;
+#endif
+			elog(DEBUG, "pg_nofile: sysconf(_SC_OPEN_MAX) failed; using %ld",
+				 no_files);
+		}
+#else							/* !HAVE_SYSCONF */
+#ifdef NOFILE
 		no_files = (long) NOFILE;
 #else
-		no_files = sysconf(_SC_OPEN_MAX);
-		if (no_files == -1)
-		{
-			elog(DEBUG, "pg_nofile: Unable to get _SC_OPEN_MAX using sysconf(); using %d", NOFILE);
-			no_files = (long) NOFILE;
-		}
+		no_files = (long) max_files_per_process;
 #endif
+#endif   /* HAVE_SYSCONF */
 
+		/*
+		 * Some platforms return hopelessly optimistic values.	Apply a
+		 * configurable upper limit.
+		 */
+		if (no_files > (long) max_files_per_process)
+			no_files = (long) max_files_per_process;
+
+		/*
+		 * Make sure we have enough to get by after reserving some for LD.
+		 */
 		if ((no_files - RESERVE_FOR_LD) < FD_MINFREE)
-			elog(FATAL, "pg_nofile: insufficient File Descriptors in postmaster to start backend (%ld).\n"
-				 "                   O/S allows %ld, Postmaster reserves %d, We need %d (MIN) after that.",
-				 no_files - RESERVE_FOR_LD, no_files, RESERVE_FOR_LD, FD_MINFREE);
+			elog(FATAL, "pg_nofile: insufficient file descriptors available to start backend.\n"
+				 "\tSystem allows %ld, we need at least %d.",
+				 no_files, RESERVE_FOR_LD + FD_MINFREE);
 
 		no_files -= RESERVE_FOR_LD;
 	}
@@ -235,7 +339,7 @@ pg_nofile(void)
 #if defined(FDDEBUG)
 
 static void
-_dump_lru()
+_dump_lru(void)
 {
 	int			mru = VfdCache[0].lruLessRecently;
 	Vfd		   *vfdP = &VfdCache[mru];
@@ -251,8 +355,7 @@ _dump_lru()
 	sprintf(buf + strlen(buf), "LEAST");
 	elog(DEBUG, buf);
 }
-
-#endif	 /* FDDEBUG */
+#endif   /* FDDEBUG */
 
 static void
 Delete(File file)
@@ -277,7 +380,6 @@ static void
 LruDelete(File file)
 {
 	Vfd		   *vfdP;
-	int			returnValue;
 
 	Assert(file != 0);
 
@@ -291,19 +393,21 @@ LruDelete(File file)
 
 	/* save the seek position */
 	vfdP->seekPos = (long) lseek(vfdP->fd, 0L, SEEK_CUR);
-	Assert(vfdP->seekPos != -1);
+	Assert(vfdP->seekPos != -1L);
 
 	/* if we have written to the file, sync it before closing */
 	if (vfdP->fdstate & FD_DIRTY)
 	{
-		returnValue = pg_fsync(vfdP->fd);
-		Assert(returnValue != -1);
+		if (pg_fsync(vfdP->fd))
+			elog(DEBUG, "LruDelete: failed to fsync %s: %m",
+				 vfdP->fileName);
 		vfdP->fdstate &= ~FD_DIRTY;
 	}
 
 	/* close the file */
-	returnValue = close(vfdP->fd);
-	Assert(returnValue != -1);
+	if (close(vfdP->fd))
+		elog(DEBUG, "LruDelete: failed to close %s: %m",
+			 vfdP->fileName);
 
 	--nfile;
 	vfdP->fd = VFD_CLOSED;
@@ -334,7 +438,6 @@ static int
 LruInsert(File file)
 {
 	Vfd		   *vfdP;
-	int			returnValue;
 
 	Assert(file != 0);
 
@@ -345,10 +448,9 @@ LruInsert(File file)
 
 	if (FileIsNotOpen(file))
 	{
-
 		while (nfile + numAllocatedFiles >= pg_nofile())
 		{
-			if (! ReleaseLruFile())
+			if (!ReleaseLruFile())
 				break;
 		}
 
@@ -357,22 +459,11 @@ LruInsert(File file)
 		 * to overall system file table being full.  So, be prepared to
 		 * release another FD if necessary...
 		 */
-tryAgain:
-		vfdP->fd = open(vfdP->fileName, vfdP->fileFlags, vfdP->fileMode);
-		if (vfdP->fd < 0 && (errno == EMFILE || errno == ENFILE))
-		{
-			int		save_errno = errno;
-
-			errno = 0;
-			if (ReleaseLruFile())
-				goto tryAgain;
-			errno = save_errno;
-		}
-
+		vfdP->fd = BasicOpenFile(vfdP->fileName, vfdP->fileFlags,
+								 vfdP->fileMode);
 		if (vfdP->fd < 0)
 		{
-			DO_DB(elog(DEBUG, "RE_OPEN FAILED: %d",
-					   errno));
+			DO_DB(elog(DEBUG, "RE_OPEN FAILED: %d", errno));
 			return vfdP->fd;
 		}
 		else
@@ -384,8 +475,10 @@ tryAgain:
 		/* seek to the right position */
 		if (vfdP->seekPos != 0L)
 		{
-			returnValue = lseek(vfdP->fd, vfdP->seekPos, SEEK_SET);
-			Assert(returnValue != -1);
+			long		returnValue;
+
+			returnValue = (long) lseek(vfdP->fd, vfdP->seekPos, SEEK_SET);
+			Assert(returnValue != -1L);
 		}
 	}
 
@@ -399,7 +492,7 @@ tryAgain:
 }
 
 static bool
-ReleaseLruFile()
+ReleaseLruFile(void)
 {
 	DO_DB(elog(DEBUG, "ReleaseLruFile. Opened %d", nfile));
 
@@ -416,24 +509,8 @@ ReleaseLruFile()
 	return false;				/* no files available to free */
 }
 
-/*
- * Force one kernel file descriptor to be released (temporarily).
- */
-bool
-ReleaseDataFile()
-{
-	DO_DB(elog(DEBUG, "ReleaseDataFile. Opened %d", nfile));
-
-	if (nfile <= 0)
-		return (false);
-	Assert(VfdCache[0].lruMoreRecently != 0);
-	LruDelete(VfdCache[0].lruMoreRecently);
-
-	return (true);
-}
-
 static File
-AllocateVfd()
+AllocateVfd(void)
 {
 	Index		i;
 	File		file;
@@ -444,7 +521,8 @@ AllocateVfd()
 	{
 		/* initialize header entry first time through */
 		VfdCache = (Vfd *) malloc(sizeof(Vfd));
-		Assert(VfdCache != NULL);
+		if (VfdCache == NULL)
+			elog(FATAL, "AllocateVfd: no room for VFD array");
 		MemSet((char *) &(VfdCache[0]), 0, sizeof(Vfd));
 		VfdCache->fd = VFD_CLOSED;
 
@@ -454,29 +532,34 @@ AllocateVfd()
 		 * register proc-exit call to ensure temp files are dropped at
 		 * exit
 		 */
-		on_proc_exit(AtEOXact_Files, NULL);
+		on_proc_exit(AtEOXact_Files, 0);
 	}
 
 	if (VfdCache[0].nextFree == 0)
 	{
-
 		/*
 		 * The free list is empty so it is time to increase the size of
 		 * the array.  We choose to double it each time this happens.
 		 * However, there's not much point in starting *real* small.
 		 */
 		Size		newCacheSize = SizeVfdCache * 2;
+		Vfd		   *newVfdCache;
 
 		if (newCacheSize < 32)
 			newCacheSize = 32;
 
-		VfdCache = (Vfd *) realloc(VfdCache, sizeof(Vfd) * newCacheSize);
-		Assert(VfdCache != NULL);
+		/*
+		 * Be careful not to clobber VfdCache ptr if realloc fails; we
+		 * will need it during proc_exit cleanup!
+		 */
+		newVfdCache = (Vfd *) realloc(VfdCache, sizeof(Vfd) * newCacheSize);
+		if (newVfdCache == NULL)
+			elog(FATAL, "AllocateVfd: no room to enlarge VFD array");
+		VfdCache = newVfdCache;
 
 		/*
 		 * Initialize the new entries and link them into the free list.
 		 */
-
 		for (i = SizeVfdCache; i < newCacheSize; i++)
 		{
 			MemSet((char *) &(VfdCache[i]), 0, sizeof(Vfd));
@@ -489,7 +572,6 @@ AllocateVfd()
 		/*
 		 * Record the new size
 		 */
-
 		SizeVfdCache = newCacheSize;
 	}
 
@@ -513,6 +595,7 @@ FreeVfd(File file)
 		free(vfdP->fileName);
 		vfdP->fileName = NULL;
 	}
+	vfdP->fdstate = 0x0;
 
 	vfdP->nextFree = VfdCache[0].nextFree;
 	VfdCache[0].nextFree = file;
@@ -521,29 +604,26 @@ FreeVfd(File file)
 /* filepath()
  * Convert given pathname to absolute.
  *
+ * Result is a palloc'd string.
+ *
  * (Generally, this isn't actually necessary, considering that we
  * should be cd'd into the database directory.  Presently it is only
  * necessary to do it in "bootstrap" mode.	Maybe we should change
  * bootstrap mode to do the cd, and save a few cycles/bytes here.)
  */
 static char *
-filepath(char *filename)
+filepath(const char *filename)
 {
 	char	   *buf;
-	int			len;
 
 	/* Not an absolute path name? Then fill in with database path... */
-	if (*filename != SEP_CHAR)
+	if (*filename != '/')
 	{
-		len = strlen(DatabasePath) + strlen(filename) + 2;
-		buf = (char *) palloc(len);
-		sprintf(buf, "%s%c%s", DatabasePath, SEP_CHAR, filename);
+		buf = (char *) palloc(strlen(DatabasePath) + strlen(filename) + 2);
+		sprintf(buf, "%s/%s", DatabasePath, filename);
 	}
 	else
-	{
-		buf = (char *) palloc(strlen(filename) + 1);
-		strcpy(buf, filename);
-	}
+		buf = pstrdup(filename);
 
 #ifdef FILEDEBUG
 	printf("filepath: path is %s\n", buf);
@@ -574,7 +654,6 @@ FileAccess(File file)
 	}
 	else if (VfdCache[0].lruLessRecently != file)
 	{
-
 		/*
 		 * We now know that the file is open and that it is not the last
 		 * one accessed, so we need to move it to the head of the Lru
@@ -599,7 +678,6 @@ FileInvalidate(File file)
 	if (!FileIsNotOpen(file))
 		LruDelete(file);
 }
-
 #endif
 
 static File
@@ -621,23 +699,11 @@ fileNameOpenFile(FileName fileName,
 
 	while (nfile + numAllocatedFiles >= pg_nofile())
 	{
-		if (! ReleaseLruFile())
+		if (!ReleaseLruFile())
 			break;
 	}
 
-tryAgain:
-	vfdP->fd = open(fileName, fileFlags, fileMode);
-	if (vfdP->fd < 0 && (errno == EMFILE || errno == ENFILE))
-	{
-		int		save_errno = errno;
-
-		DO_DB(elog(DEBUG, "fileNameOpenFile: not enough descs, retry, er= %d",
-				   errno));
-		errno = 0;
-		if (ReleaseLruFile())
-			goto tryAgain;
-		errno = save_errno;
-	}
+	vfdP->fd = BasicOpenFile(fileName, fileFlags, fileMode);
 
 	if (vfdP->fd < 0)
 	{
@@ -650,13 +716,25 @@ tryAgain:
 
 	Insert(file);
 
-	vfdP->fileName = malloc(strlen(fileName) + 1);
+	vfdP->fileName = (char *) malloc(strlen(fileName) + 1);
+	if (vfdP->fileName == NULL)
+		elog(FATAL, "fileNameOpenFile: no room to save VFD filename");
 	strcpy(vfdP->fileName, fileName);
 
-	vfdP->fileFlags = fileFlags & ~(O_TRUNC | O_EXCL);
+	/* Saved flags are adjusted to be OK for re-opening file */
+	vfdP->fileFlags = fileFlags & ~(O_CREAT | O_TRUNC | O_EXCL);
 	vfdP->fileMode = fileMode;
+
 	vfdP->seekPos = 0;
-	vfdP->fdstate = 0x0;
+
+	/*
+	 * Have to fsync file on commit. Alternative way - log file creation
+	 * and fsync log before actual file creation.
+	 */
+	if (fileFlags & O_CREAT)
+		vfdP->fdstate = FD_DIRTY;
+	else
+		vfdP->fdstate = 0x0;
 
 	return file;
 }
@@ -695,27 +773,46 @@ PathNameOpenFile(FileName fileName, int fileFlags, int fileMode)
 File
 OpenTemporaryFile(void)
 {
-	char		tempfilename[64];
+	char		tempfilepath[128];
 	File		file;
 
 	/*
 	 * Generate a tempfile name that's unique within the current
-	 * transaction
+	 * transaction and database instance.
 	 */
-	snprintf(tempfilename, sizeof(tempfilename),
-			 "pg_sorttemp%d.%ld", MyProcPid, tempFileCounter++);
-
-	/* Open the file */
-#ifndef __CYGWIN32__
-	file = FileNameOpenFile(tempfilename,
-							O_RDWR | O_CREAT | O_TRUNC, 0600);
-#else
-	file = FileNameOpenFile(tempfilename,
-							O_RDWR | O_CREAT | O_TRUNC | O_BINARY, 0600);
-#endif
+	snprintf(tempfilepath, sizeof(tempfilepath),
+			 "%s/%s%d.%ld", PG_TEMP_FILES_DIR, PG_TEMP_FILE_PREFIX,
+			 MyProcPid, tempFileCounter++);
 
+	/*
+	 * Open the file.  Note: we don't use O_EXCL, in case there is an
+	 * orphaned temp file that can be reused.
+	 */
+	file = FileNameOpenFile(tempfilepath,
+							O_RDWR | O_CREAT | O_TRUNC | PG_BINARY,
+							0600);
 	if (file <= 0)
-		elog(ERROR, "Failed to create temporary file %s", tempfilename);
+	{
+		char	   *dirpath;
+
+		/*
+		 * We might need to create the pg_tempfiles subdirectory, if no
+		 * one has yet done so.
+		 *
+		 * Don't check for error from mkdir; it could fail if someone else
+		 * just did the same thing.  If it doesn't work then we'll bomb
+		 * out on the second create attempt, instead.
+		 */
+		dirpath = filepath(PG_TEMP_FILES_DIR);
+		mkdir(dirpath, S_IRWXU);
+		pfree(dirpath);
+
+		file = FileNameOpenFile(tempfilepath,
+								O_RDWR | O_CREAT | O_TRUNC | PG_BINARY,
+								0600);
+		if (file <= 0)
+			elog(ERROR, "Failed to create temporary file %s", tempfilepath);
+	}
 
 	/* Mark it for deletion at close or EOXact */
 	VfdCache[file].fdstate |= FD_TEMPORARY;
@@ -729,40 +826,49 @@ OpenTemporaryFile(void)
 void
 FileClose(File file)
 {
-	int			returnValue;
+	Vfd		   *vfdP;
 
 	Assert(FileIsValid(file));
 
 	DO_DB(elog(DEBUG, "FileClose: %d (%s)",
 			   file, VfdCache[file].fileName));
 
+	vfdP = &VfdCache[file];
+
 	if (!FileIsNotOpen(file))
 	{
-
 		/* remove the file from the lru ring */
 		Delete(file);
 
 		/* if we did any writes, sync the file before closing */
-		if (VfdCache[file].fdstate & FD_DIRTY)
+		if (vfdP->fdstate & FD_DIRTY)
 		{
-			returnValue = pg_fsync(VfdCache[file].fd);
-			Assert(returnValue != -1);
-			VfdCache[file].fdstate &= ~FD_DIRTY;
+			if (pg_fsync(vfdP->fd))
+				elog(DEBUG, "FileClose: failed to fsync %s: %m",
+					 vfdP->fileName);
+			vfdP->fdstate &= ~FD_DIRTY;
 		}
 
 		/* close the file */
-		returnValue = close(VfdCache[file].fd);
-		Assert(returnValue != -1);
+		if (close(vfdP->fd))
+			elog(DEBUG, "FileClose: failed to close %s: %m",
+				 vfdP->fileName);
 
 		--nfile;
-		VfdCache[file].fd = VFD_CLOSED;
+		vfdP->fd = VFD_CLOSED;
 	}
 
 	/*
 	 * Delete the file if it was temporary
 	 */
-	if (VfdCache[file].fdstate & FD_TEMPORARY)
-		unlink(VfdCache[file].fileName);
+	if (vfdP->fdstate & FD_TEMPORARY)
+	{
+		/* reset flag so that die() interrupt won't cause problems */
+		vfdP->fdstate &= ~FD_TEMPORARY;
+		if (unlink(vfdP->fileName))
+			elog(DEBUG, "FileClose: failed to unlink %s: %m",
+				 vfdP->fileName);
+	}
 
 	/*
 	 * Return the Vfd slot to the free list
@@ -794,13 +900,16 @@ FileRead(File file, char *buffer, int amount)
 
 	Assert(FileIsValid(file));
 
-	DO_DB(elog(DEBUG, "FileRead: %d (%s) %d %p",
-			   file, VfdCache[file].fileName, amount, buffer));
+	DO_DB(elog(DEBUG, "FileRead: %d (%s) %ld %d %p",
+			   file, VfdCache[file].fileName,
+			   VfdCache[file].seekPos, amount, buffer));
 
 	FileAccess(file);
 	returnCode = read(VfdCache[file].fd, buffer, amount);
 	if (returnCode > 0)
 		VfdCache[file].seekPos += returnCode;
+	else
+		VfdCache[file].seekPos = FileUnknownPos;
 
 	return returnCode;
 }
@@ -812,16 +921,23 @@ FileWrite(File file, char *buffer, int amount)
 
 	Assert(FileIsValid(file));
 
-	DO_DB(elog(DEBUG, "FileWrite: %d (%s) %d %p",
-			   file, VfdCache[file].fileName, amount, buffer));
+	DO_DB(elog(DEBUG, "FileWrite: %d (%s) %ld %d %p",
+			   file, VfdCache[file].fileName,
+			   VfdCache[file].seekPos, amount, buffer));
 
 	FileAccess(file);
+
+	errno = 0;
 	returnCode = write(VfdCache[file].fd, buffer, amount);
+
+	/* if write didn't set errno, assume problem is no disk space */
+	if (returnCode != amount && errno == 0)
+		errno = ENOSPC;
+
 	if (returnCode > 0)
 		VfdCache[file].seekPos += returnCode;
-
-	/* mark the file as needing fsync */
-	VfdCache[file].fdstate |= FD_DIRTY;
+	else
+		VfdCache[file].seekPos = FileUnknownPos;
 
 	return returnCode;
 }
@@ -831,14 +947,17 @@ FileSeek(File file, long offset, int whence)
 {
 	Assert(FileIsValid(file));
 
-	DO_DB(elog(DEBUG, "FileSeek: %d (%s) %ld %d",
-			   file, VfdCache[file].fileName, offset, whence));
+	DO_DB(elog(DEBUG, "FileSeek: %d (%s) %ld %ld %d",
+			   file, VfdCache[file].fileName,
+			   VfdCache[file].seekPos, offset, whence));
 
 	if (FileIsNotOpen(file))
 	{
 		switch (whence)
 		{
 			case SEEK_SET:
+				if (offset < 0)
+					elog(ERROR, "FileSeek: invalid offset: %ld", offset);
 				VfdCache[file].seekPos = offset;
 				break;
 			case SEEK_CUR:
@@ -854,7 +973,27 @@ FileSeek(File file, long offset, int whence)
 		}
 	}
 	else
-		VfdCache[file].seekPos = lseek(VfdCache[file].fd, offset, whence);
+	{
+		switch (whence)
+		{
+			case SEEK_SET:
+				if (offset < 0)
+					elog(ERROR, "FileSeek: invalid offset: %ld", offset);
+				if (VfdCache[file].seekPos != offset)
+					VfdCache[file].seekPos = lseek(VfdCache[file].fd, offset, whence);
+				break;
+			case SEEK_CUR:
+				if (offset != 0 || VfdCache[file].seekPos == FileUnknownPos)
+					VfdCache[file].seekPos = lseek(VfdCache[file].fd, offset, whence);
+				break;
+			case SEEK_END:
+				VfdCache[file].seekPos = lseek(VfdCache[file].fd, offset, whence);
+				break;
+			default:
+				elog(ERROR, "FileSeek: invalid whence: %d", whence);
+				break;
+		}
+	}
 	return VfdCache[file].seekPos;
 }
 
@@ -870,7 +1009,6 @@ FileTell(File file)
 			   file, VfdCache[file].fileName));
 	return VfdCache[file].seekPos;
 }
-
 #endif
 
 int
@@ -930,7 +1068,7 @@ FileSync(File file)
 		/* Need not sync if file is not dirty. */
 		returnCode = 0;
 	}
-	else if (disableFsync)
+	else if (!enableFsync)
 	{
 		/* Don't force the file open if pg_fsync isn't gonna sync it. */
 		returnCode = 0;
@@ -938,7 +1076,6 @@ FileSync(File file)
 	}
 	else
 	{
-
 		/*
 		 * We don't use FileAccess() because we don't want to force the
 		 * file to the front of the LRU ring; we aren't expecting to
@@ -1001,6 +1138,8 @@ FileMarkDirty(File file)
  * fd.c will automatically close all files opened with AllocateFile at
  * transaction commit or abort; this prevents FD leakage if a routine
  * that calls AllocateFile is terminated prematurely by elog(ERROR).
+ *
+ * Ideally this should be the *only* direct call of fopen() in the backend.
  */
 
 FILE *
@@ -1008,29 +1147,32 @@ AllocateFile(char *name, char *mode)
 {
 	FILE	   *file;
 
-	DO_DB(elog(DEBUG, "AllocateFile: Allocated %d.", numAllocatedFiles));
+	DO_DB(elog(DEBUG, "AllocateFile: Allocated %d", numAllocatedFiles));
 
 	if (numAllocatedFiles >= MAX_ALLOCATED_FILES)
 		elog(ERROR, "AllocateFile: too many private FDs demanded");
 
 TryAgain:
-	if ((file = fopen(name, mode)) == NULL)
+	if ((file = fopen(name, mode)) != NULL)
 	{
-		if (errno == EMFILE || errno == ENFILE)
-		{
-			int		save_errno = errno;
-
-			DO_DB(elog(DEBUG, "AllocateFile: not enough descs, retry, er= %d",
-					   errno));
-			errno = 0;
-			if (ReleaseLruFile())
-				goto TryAgain;
-			errno = save_errno;
-		}
+		allocatedFiles[numAllocatedFiles] = file;
+		numAllocatedFiles++;
+		return file;
 	}
-	else
-		allocatedFiles[numAllocatedFiles++] = file;
-	return file;
+
+	if (errno == EMFILE || errno == ENFILE)
+	{
+		int			save_errno = errno;
+
+		DO_DB(elog(DEBUG, "AllocateFile: not enough descs, retry, er= %d",
+				   errno));
+		errno = 0;
+		if (ReleaseLruFile())
+			goto TryAgain;
+		errno = save_errno;
+	}
+
+	return NULL;
 }
 
 void
@@ -1038,14 +1180,15 @@ FreeFile(FILE *file)
 {
 	int			i;
 
-	DO_DB(elog(DEBUG, "FreeFile: Allocated %d.", numAllocatedFiles));
+	DO_DB(elog(DEBUG, "FreeFile: Allocated %d", numAllocatedFiles));
 
 	/* Remove file from list of allocated files, if it's present */
 	for (i = numAllocatedFiles; --i >= 0;)
 	{
 		if (allocatedFiles[i] == file)
 		{
-			allocatedFiles[i] = allocatedFiles[--numAllocatedFiles];
+			numAllocatedFiles--;
+			allocatedFiles[i] = allocatedFiles[numAllocatedFiles];
 			break;
 		}
 	}
@@ -1063,7 +1206,7 @@ FreeFile(FILE *file)
  * change in the logical state of the VFDs.
  */
 void
-closeAllVfds()
+closeAllVfds(void)
 {
 	Index		i;
 
@@ -1119,3 +1262,74 @@ AtEOXact_Files(void)
 	 */
 	tempFileCounter = 0;
 }
+
+
+/*
+ * Remove old temporary files
+ *
+ * This should be called during postmaster startup.  It will forcibly
+ * remove any leftover files created by OpenTemporaryFile.
+ */
+void
+RemovePgTempFiles(void)
+{
+	char		db_path[MAXPGPATH];
+	char		temp_path[MAXPGPATH];
+	char		rm_path[MAXPGPATH];
+	DIR		   *db_dir;
+	DIR		   *temp_dir;
+	struct dirent *db_de;
+	struct dirent *temp_de;
+
+	/*
+	 * Cycle through pg_tempfiles for all databases and remove old temp
+	 * files.
+	 */
+	snprintf(db_path, sizeof(db_path), "%s/base", DataDir);
+	if ((db_dir = opendir(db_path)) != NULL)
+	{
+		while ((db_de = readdir(db_dir)) != NULL)
+		{
+			if (strcmp(db_de->d_name, ".") == 0 ||
+				strcmp(db_de->d_name, "..") == 0)
+				continue;
+
+			snprintf(temp_path, sizeof(temp_path),
+					 "%s/%s/%s",
+					 db_path, db_de->d_name,
+					 PG_TEMP_FILES_DIR);
+			if ((temp_dir = opendir(temp_path)) != NULL)
+			{
+				while ((temp_de = readdir(temp_dir)) != NULL)
+				{
+					if (strcmp(temp_de->d_name, ".") == 0 ||
+						strcmp(temp_de->d_name, "..") == 0)
+						continue;
+
+					snprintf(rm_path, sizeof(temp_path),
+							 "%s/%s/%s/%s",
+							 db_path, db_de->d_name,
+							 PG_TEMP_FILES_DIR,
+							 temp_de->d_name);
+
+					if (strncmp(temp_de->d_name,
+								PG_TEMP_FILE_PREFIX,
+								strlen(PG_TEMP_FILE_PREFIX)) == 0)
+						unlink(rm_path);
+					else
+					{
+						/*
+						 * would prefer to use elog here, but it's not up
+						 * and running during postmaster startup...
+						 */
+						fprintf(stderr,
+								"Unexpected file found in temporary-files directory: %s\n",
+								rm_path);
+					}
+				}
+				closedir(temp_dir);
+			}
+		}
+		closedir(db_dir);
+	}
+}
