@@ -3,72 +3,86 @@
  * lmgr.c
  *	  POSTGRES lock manager code
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/lmgr.c,v 1.39 2000/04/12 17:15:38 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/storage/lmgr/lmgr.c,v 1.52 2001/10/25 05:49:42 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
-/* #define LOCKDEBUGALL 1 */
-/* #define LOCKDEBUG	1 */
-
-#ifdef	LOCKDEBUGALL
-#define LOCKDEBUG		1
-#endif	 /* LOCKDEBUGALL */
-
 
 #include "postgres.h"
 
 #include "access/transam.h"
+#include "access/xact.h"
 #include "catalog/catalog.h"
 #include "miscadmin.h"
+#include "storage/lmgr.h"
 #include "utils/inval.h"
 
 
 static LOCKMASK LockConflicts[] = {
-	(int) NULL,
+	0,
 
-/* AccessShareLock */
+	/* AccessShareLock */
 	(1 << AccessExclusiveLock),
 
-/* RowShareLock */
+	/* RowShareLock */
 	(1 << ExclusiveLock) | (1 << AccessExclusiveLock),
 
-/* RowExclusiveLock */
-	(1 << ExclusiveLock) | (1 << ShareRowExclusiveLock) | (1 << ShareLock) |
-	(1 << AccessExclusiveLock),
+	/* RowExclusiveLock */
+	(1 << ShareLock) | (1 << ShareRowExclusiveLock) |
+	(1 << ExclusiveLock) | (1 << AccessExclusiveLock),
 
-/* ShareLock */
-	(1 << ExclusiveLock) | (1 << ShareRowExclusiveLock) |
-	(1 << RowExclusiveLock) | (1 << AccessExclusiveLock),
+	/* ShareUpdateExclusiveLock */
+	(1 << ShareUpdateExclusiveLock) |
+	(1 << ShareLock) | (1 << ShareRowExclusiveLock) |
+	(1 << ExclusiveLock) | (1 << AccessExclusiveLock),
 
-/* ShareRowExclusiveLock */
-	(1 << ExclusiveLock) | (1 << ShareRowExclusiveLock) |
-	(1 << ShareLock) | (1 << RowExclusiveLock) | (1 << AccessExclusiveLock),
+	/* ShareLock */
+	(1 << RowExclusiveLock) | (1 << ShareUpdateExclusiveLock) |
+	(1 << ShareRowExclusiveLock) |
+	(1 << ExclusiveLock) | (1 << AccessExclusiveLock),
 
-/* ExclusiveLock */
-	(1 << ExclusiveLock) | (1 << ShareRowExclusiveLock) | (1 << ShareLock) |
-	(1 << RowExclusiveLock) | (1 << RowShareLock) | (1 << AccessExclusiveLock),
+	/* ShareRowExclusiveLock */
+	(1 << RowExclusiveLock) | (1 << ShareUpdateExclusiveLock) |
+	(1 << ShareLock) | (1 << ShareRowExclusiveLock) |
+	(1 << ExclusiveLock) | (1 << AccessExclusiveLock),
 
-/* AccessExclusiveLock */
-	(1 << ExclusiveLock) | (1 << ShareRowExclusiveLock) | (1 << ShareLock) |
-	(1 << RowExclusiveLock) | (1 << RowShareLock) | (1 << AccessExclusiveLock) |
-	(1 << AccessShareLock),
+	/* ExclusiveLock */
+	(1 << RowShareLock) |
+	(1 << RowExclusiveLock) | (1 << ShareUpdateExclusiveLock) |
+	(1 << ShareLock) | (1 << ShareRowExclusiveLock) |
+	(1 << ExclusiveLock) | (1 << AccessExclusiveLock),
+
+	/* AccessExclusiveLock */
+	(1 << AccessShareLock) | (1 << RowShareLock) |
+	(1 << RowExclusiveLock) | (1 << ShareUpdateExclusiveLock) |
+	(1 << ShareLock) | (1 << ShareRowExclusiveLock) |
+	(1 << ExclusiveLock) | (1 << AccessExclusiveLock)
 
 };
 
 static int	LockPrios[] = {
-	(int) NULL,
+	0,
+	/* AccessShareLock */
 	1,
+	/* RowShareLock */
 	2,
+	/* RowExclusiveLock */
 	3,
+	/* ShareUpdateExclusiveLock */
 	4,
+	/* ShareLock */
 	5,
+	/* ShareRowExclusiveLock */
 	6,
-	7
+	/* ExclusiveLock */
+	7,
+	/* AccessExclusiveLock */
+	8
 };
 
 LOCKMETHOD	LockTableId = (LOCKMETHOD) NULL;
@@ -78,16 +92,17 @@ LOCKMETHOD	LongTermTableId = (LOCKMETHOD) NULL;
  * Create the lock table described by LockConflicts and LockPrios.
  */
 LOCKMETHOD
-InitLockTable()
+InitLockTable(int maxBackends)
 {
 	int			lockmethod;
 
 	lockmethod = LockMethodTableInit("LockTable",
-							LockConflicts, LockPrios, MAX_LOCKMODES - 1);
+									 LockConflicts, LockPrios,
+									 MAX_LOCKMODES - 1, maxBackends);
 	LockTableId = lockmethod;
 
 	if (!(LockTableId))
-		elog(ERROR, "InitLockTable: couldnt initialize lock table");
+		elog(ERROR, "InitLockTable: couldn't initialize lock table");
 
 #ifdef USER_LOCKS
 
@@ -96,10 +111,7 @@ InitLockTable()
 	 */
 	LongTermTableId = LockMethodTableRename(LockTableId);
 	if (!(LongTermTableId))
-	{
-		elog(ERROR,
-			 "InitLockTable: couldn't rename long-term lock table");
-	}
+		elog(ERROR, "InitLockTable: couldn't rename long-term lock table");
 #endif
 
 	return LockTableId;
@@ -114,16 +126,12 @@ InitLockTable()
 void
 RelationInitLockInfo(Relation relation)
 {
-	char	   *relname;
-
 	Assert(RelationIsValid(relation));
 	Assert(OidIsValid(RelationGetRelid(relation)));
 
-	relname = (char *) RelationGetPhysicalRelationName(relation);
-
 	relation->rd_lockInfo.lockRelId.relId = RelationGetRelid(relation);
 
-	if (IsSharedSystemRelationName(relname))
+	if (relation->rd_rel->relisshared)
 		relation->rd_lockInfo.lockRelId.dbId = InvalidOid;
 	else
 		relation->rd_lockInfo.lockRelId.dbId = MyDatabaseId;
@@ -137,15 +145,13 @@ LockRelation(Relation relation, LOCKMODE lockmode)
 {
 	LOCKTAG		tag;
 
-	if (LockingDisabled())
-		return;
-
 	MemSet(&tag, 0, sizeof(tag));
 	tag.relId = relation->rd_lockInfo.lockRelId.relId;
 	tag.dbId = relation->rd_lockInfo.lockRelId.dbId;
 	tag.objId.blkno = InvalidBlockNumber;
 
-	if (!LockAcquire(LockTableId, &tag, lockmode))
+	if (!LockAcquire(LockTableId, &tag, GetCurrentTransactionId(),
+					 lockmode, false))
 		elog(ERROR, "LockRelation: LockAcquire failed");
 
 	/*
@@ -155,10 +161,46 @@ LockRelation(Relation relation, LOCKMODE lockmode)
 	 * rebuild it and not just delete it.
 	 */
 	RelationIncrementReferenceCount(relation);
-	DiscardInvalid();
+	AcceptInvalidationMessages();
 	RelationDecrementReferenceCount(relation);
 }
 
+/*
+ *		ConditionalLockRelation
+ *
+ * As above, but only lock if we can get the lock without blocking.
+ * Returns TRUE iff the lock was acquired.
+ *
+ * NOTE: we do not currently need conditional versions of the other
+ * LockXXX routines in this file, but they could easily be added if needed.
+ */
+bool
+ConditionalLockRelation(Relation relation, LOCKMODE lockmode)
+{
+	LOCKTAG		tag;
+
+	MemSet(&tag, 0, sizeof(tag));
+	tag.relId = relation->rd_lockInfo.lockRelId.relId;
+	tag.dbId = relation->rd_lockInfo.lockRelId.dbId;
+	tag.objId.blkno = InvalidBlockNumber;
+
+	if (!LockAcquire(LockTableId, &tag, GetCurrentTransactionId(),
+					 lockmode, true))
+		return false;
+
+	/*
+	 * Check to see if the relcache entry has been invalidated while we
+	 * were waiting to lock it.  If so, rebuild it, or elog() trying.
+	 * Increment the refcount to ensure that RelationFlushRelation will
+	 * rebuild it and not just delete it.
+	 */
+	RelationIncrementReferenceCount(relation);
+	AcceptInvalidationMessages();
+	RelationDecrementReferenceCount(relation);
+
+	return true;
+}
+
 /*
  *		UnlockRelation
  */
@@ -167,34 +209,76 @@ UnlockRelation(Relation relation, LOCKMODE lockmode)
 {
 	LOCKTAG		tag;
 
-	if (LockingDisabled())
-		return;
-
 	MemSet(&tag, 0, sizeof(tag));
 	tag.relId = relation->rd_lockInfo.lockRelId.relId;
 	tag.dbId = relation->rd_lockInfo.lockRelId.dbId;
 	tag.objId.blkno = InvalidBlockNumber;
 
-	LockRelease(LockTableId, &tag, lockmode);
+	LockRelease(LockTableId, &tag, GetCurrentTransactionId(), lockmode);
+}
+
+/*
+ *		LockRelationForSession
+ *
+ * This routine grabs a session-level lock on the target relation.	The
+ * session lock persists across transaction boundaries.  It will be removed
+ * when UnlockRelationForSession() is called, or if an elog(ERROR) occurs,
+ * or if the backend exits.
+ *
+ * Note that one should also grab a transaction-level lock on the rel
+ * in any transaction that actually uses the rel, to ensure that the
+ * relcache entry is up to date.
+ */
+void
+LockRelationForSession(LockRelId *relid, LOCKMODE lockmode)
+{
+	LOCKTAG		tag;
+
+	MemSet(&tag, 0, sizeof(tag));
+	tag.relId = relid->relId;
+	tag.dbId = relid->dbId;
+	tag.objId.blkno = InvalidBlockNumber;
+
+	if (!LockAcquire(LockTableId, &tag, InvalidTransactionId,
+					 lockmode, false))
+		elog(ERROR, "LockRelationForSession: LockAcquire failed");
+}
+
+/*
+ *		UnlockRelationForSession
+ */
+void
+UnlockRelationForSession(LockRelId *relid, LOCKMODE lockmode)
+{
+	LOCKTAG		tag;
+
+	MemSet(&tag, 0, sizeof(tag));
+	tag.relId = relid->relId;
+	tag.dbId = relid->dbId;
+	tag.objId.blkno = InvalidBlockNumber;
+
+	LockRelease(LockTableId, &tag, InvalidTransactionId, lockmode);
 }
 
 /*
  *		LockPage
+ *
+ * Obtain a page-level lock.  This is currently used by some index access
+ * methods to lock index pages.  For heap relations, it is used only with
+ * blkno == 0 to signify locking the relation for extension.
  */
 void
 LockPage(Relation relation, BlockNumber blkno, LOCKMODE lockmode)
 {
 	LOCKTAG		tag;
 
-	if (LockingDisabled())
-		return;
-
 	MemSet(&tag, 0, sizeof(tag));
 	tag.relId = relation->rd_lockInfo.lockRelId.relId;
 	tag.dbId = relation->rd_lockInfo.lockRelId.dbId;
 	tag.objId.blkno = blkno;
 
-	if (!LockAcquire(LockTableId, &tag, lockmode))
+	if (!LockAcquire(LockTableId, &tag, GetCurrentTransactionId(),
+					 lockmode, false))
 		elog(ERROR, "LockPage: LockAcquire failed");
 }
 
@@ -206,71 +290,66 @@ UnlockPage(Relation relation, BlockNumber blkno, LOCKMODE lockmode)
 {
 	LOCKTAG		tag;
 
-	if (LockingDisabled())
-		return;
-
 	MemSet(&tag, 0, sizeof(tag));
 	tag.relId = relation->rd_lockInfo.lockRelId.relId;
 	tag.dbId = relation->rd_lockInfo.lockRelId.dbId;
 	tag.objId.blkno = blkno;
 
-	LockRelease(LockTableId, &tag, lockmode);
+	LockRelease(LockTableId, &tag, GetCurrentTransactionId(), lockmode);
 }
 
+/*
+ *		XactLockTableInsert
+ *
+ * Insert a lock showing that the given transaction ID is running ---
+ * this is done during xact startup.  The lock can then be used to wait
+ * for the transaction to finish.
+ *
+ * We need no corresponding unlock function, since the lock will always
+ * be released implicitly at transaction commit/abort, never any other way.
+ */
 void
 XactLockTableInsert(TransactionId xid)
 {
 	LOCKTAG		tag;
 
-	if (LockingDisabled())
-		return;
-
 	MemSet(&tag, 0, sizeof(tag));
 	tag.relId = XactLockTableId;
-	tag.dbId = InvalidOid;
+	tag.dbId = InvalidOid;		/* xids are globally unique */
 	tag.objId.xid = xid;
 
-	if (!LockAcquire(LockTableId, &tag, ExclusiveLock))
+	if (!LockAcquire(LockTableId, &tag, xid,
+					 ExclusiveLock, false))
 		elog(ERROR, "XactLockTableInsert: LockAcquire failed");
 }
 
-void
-XactLockTableDelete(TransactionId xid)
-{
-	LOCKTAG		tag;
-
-	if (LockingDisabled())
-		return;
-
-	MemSet(&tag, 0, sizeof(tag));
-	tag.relId = XactLockTableId;
-	tag.dbId = InvalidOid;
-	tag.objId.xid = xid;
-
-	LockRelease(LockTableId, &tag, ExclusiveLock);
-}
-
+/*
+ *		XactLockTableWait
+ *
+ * Wait for the specified transaction to commit or abort.
+ */
 void
 XactLockTableWait(TransactionId xid)
 {
 	LOCKTAG		tag;
+	TransactionId myxid = GetCurrentTransactionId();
 
-	if (LockingDisabled())
-		return;
+	Assert(!TransactionIdEquals(xid, myxid));
 
 	MemSet(&tag, 0, sizeof(tag));
 	tag.relId = XactLockTableId;
 	tag.dbId = InvalidOid;
 	tag.objId.xid = xid;
 
-	if (!LockAcquire(LockTableId, &tag, ShareLock))
+	if (!LockAcquire(LockTableId, &tag, myxid,
+					 ShareLock, false))
 		elog(ERROR, "XactLockTableWait: LockAcquire failed");
 
-	LockRelease(LockTableId, &tag, ShareLock);
+	LockRelease(LockTableId, &tag, myxid, ShareLock);
 
 	/*
 	 * Transaction was committed/aborted/crashed - we have to update
-	 * pg_log if transaction is still marked as running.
+	 * pg_clog if transaction is still marked as running.
 	 */
 	if (!TransactionIdDidCommit(xid) && !TransactionIdDidAbort(xid))
 		TransactionIdAbort(xid);
