@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/utils/adt/arrayfuncs.c,v 1.70 2001/03/22 03:59:49 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/utils/adt/arrayfuncs.c,v 1.72.2.1 2002/03/17 20:05:59 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -46,19 +46,12 @@
  * Local definitions
  * ----------
  */
-#ifndef MIN
-#define MIN(a,b) (((a)<(b)) ? (a) : (b))
-#endif
-#ifndef MAX
-#define MAX(a,b) (((a)>(b)) ? (a) : (b))
-#endif
-
 #define ASSGN	 "="
 
 #define RETURN_NULL(type)  do { *isNull = true; return (type) 0; } while (0)
 
 
-static int	ArrayCount(char *str, int *dim, int typdelim);
+static int	ArrayCount(char *str, int *dim, char typdelim);
 static Datum *ReadArrayStr(char *arrayStr, int nitems, int ndim, int *dim,
 			 FmgrInfo *inputproc, Oid typelem, int32 typmod,
 			 char typdelim, int typlen, bool typbyval,
@@ -252,7 +245,7 @@ array_in(PG_FUNCTION_ARGS)
  *-----------------------------------------------------------------------------
  */
 static int
-ArrayCount(char *str, int *dim, int typdelim)
+ArrayCount(char *str, int *dim, char typdelim)
 {
 	int			nest_level = 0,
 				i;
@@ -260,7 +253,7 @@ ArrayCount(char *str, int *dim, int typdelim)
 				temp[MAXDIM];
 	bool		scanning_string = false;
 	bool		eoArray = false;
-	char	   *q;
+	char	   *ptr;
 
 	for (i = 0; i < MAXDIM; ++i)
 		temp[i] = dim[i] = 0;
@@ -268,65 +261,68 @@ ArrayCount(char *str, int *dim, int typdelim)
 	if (strncmp(str, "{}", 2) == 0)
 		return 0;
 
-	q = str;
-	while (eoArray != true)
+	ptr = str;
+	while (!eoArray)
 	{
-		bool		done = false;
+		bool		itemdone = false;
 
-		while (!done)
+		while (!itemdone)
 		{
-			switch (*q)
+			switch (*ptr)
 			{
-				case '\\':
-					/* skip escaped characters (\ and ") inside strings */
-					if (scanning_string && *(q + 1))
-						q++;
-					break;
 				case '\0':
-
-					/*
-					 * Signal a premature end of the string.  DZ -
-					 * 2-9-1996
-					 */
+					/* Signal a premature end of the string */
 					elog(ERROR, "malformed array constant: %s", str);
 					break;
+				case '\\':
+					/* skip the escaped character */
+					if (*(ptr + 1))
+						ptr++;
+					else
+						elog(ERROR, "malformed array constant: %s", str);
+					break;
 				case '\"':
 					scanning_string = !scanning_string;
 					break;
 				case '{':
 					if (!scanning_string)
 					{
+						if (nest_level >= MAXDIM)
+							elog(ERROR, "array_in: illformed array constant");
 						temp[nest_level] = 0;
 						nest_level++;
+						if (ndim < nest_level)
+							ndim = nest_level;
 					}
 					break;
 				case '}':
 					if (!scanning_string)
 					{
-						if (!ndim)
-							ndim = nest_level;
-						nest_level--;
-						if (nest_level)
-							temp[nest_level - 1]++;
 						if (nest_level == 0)
-							eoArray = done = true;
+							elog(ERROR, "array_in: illformed array constant");
+						nest_level--;
+						if (nest_level == 0)
+							eoArray = itemdone = true;
+						else
+						{
+							/*
+							 * We don't set itemdone here; see comments in
+							 * ReadArrayStr
+							 */
+							temp[nest_level - 1]++;
+						}
 					}
 					break;
 				default:
-					if (!ndim)
-						ndim = nest_level;
-					if (*q == typdelim && !scanning_string)
-						done = true;
+					if (*ptr == typdelim && !scanning_string)
+						itemdone = true;
 					break;
 			}
-			if (!done)
-				q++;
+			if (!itemdone)
+				ptr++;
 		}
 		temp[ndim - 1]++;
-		q++;
-		if (!eoArray)
-			while (isspace((unsigned char) *q))
-				q++;
+		ptr++;
 	}
 	for (i = 0; i < ndim; ++i)
 		dim[i] = temp[i];
@@ -346,6 +342,7 @@ ArrayCount(char *str, int *dim, int typdelim)
  *	 If element type is pass-by-ref, the Datums point to palloc'd values.
  *	 *nbytes is set to the amount of data space needed for the array,
  *	 including alignment padding but not including array header overhead.
+ *	 CAUTION: the contents of "arrayStr" may be modified!
  *---------------------------------------------------------------------------
  */
 static Datum *
@@ -365,103 +362,119 @@ ReadArrayStr(char *arrayStr,
 	int			i,
 				nest_level = 0;
 	Datum	   *values;
-	char	   *p,
-			   *q,
-			   *r;
+	char	   *ptr;
 	bool		scanning_string = false;
+	bool		eoArray = false;
 	int			indx[MAXDIM],
 				prod[MAXDIM];
-	bool		eoArray = false;
 
 	mda_get_prod(ndim, dim, prod);
 	values = (Datum *) palloc(nitems * sizeof(Datum));
 	MemSet(values, 0, nitems * sizeof(Datum));
 	MemSet(indx, 0, sizeof(indx));
-	q = p = arrayStr;
 
 	/* read array enclosed within {} */
+	ptr = arrayStr;
 	while (!eoArray)
 	{
-		bool		done = false;
+		bool		itemdone = false;
 		int			i = -1;
+		char	   *itemstart;
 
-		while (!done)
+		/* skip leading whitespace */
+		while (isspace((unsigned char) *ptr))
+			ptr++;
+		itemstart = ptr;
+
+		while (!itemdone)
 		{
-			switch (*q)
+			switch (*ptr)
 			{
+				case '\0':
+					/* Signal a premature end of the string */
+					elog(ERROR, "malformed array constant: %s", arrayStr);
+					break;
 				case '\\':
+				{
+					char   *cptr;
+
 					/* Crunch the string on top of the backslash. */
-					for (r = q; *r != '\0'; r++)
-						*r = *(r + 1);
+					for (cptr = ptr; *cptr != '\0'; cptr++)
+						*cptr = *(cptr + 1);
+					if (*ptr == '\0')
+						elog(ERROR, "malformed array constant: %s", arrayStr);
 					break;
+				}
 				case '\"':
-					if (!scanning_string)
-					{
-						while (p != q)
-							p++;
-						p++;	/* get p past first doublequote */
-					}
-					else
-						*q = '\0';
+				{
+					char   *cptr;
+
 					scanning_string = !scanning_string;
+					/* Crunch the string on top of the quote. */
+					for (cptr = ptr; *cptr != '\0'; cptr++)
+						*cptr = *(cptr + 1);
+					/* Back up to not miss following character. */
+					ptr--;
 					break;
+				}
 				case '{':
 					if (!scanning_string)
 					{
-						p++;
-						nest_level++;
-						if (nest_level > ndim)
+						if (nest_level >= ndim)
 							elog(ERROR, "array_in: illformed array constant");
+						nest_level++;
 						indx[nest_level - 1] = 0;
-						indx[ndim - 1] = 0;
+						/* skip leading whitespace */
+						while (isspace((unsigned char) *(ptr+1)))
+							ptr++;
+						itemstart = ptr+1;
 					}
 					break;
 				case '}':
 					if (!scanning_string)
 					{
+						if (nest_level == 0)
+							elog(ERROR, "array_in: illformed array constant");
 						if (i == -1)
 							i = ArrayGetOffset0(ndim, indx, prod);
+						indx[nest_level - 1] = 0;
 						nest_level--;
 						if (nest_level == 0)
-							eoArray = done = true;
+							eoArray = itemdone = true;
 						else
 						{
-							*q = '\0';
+							/*
+							 * tricky coding: terminate item value string at
+							 * first '}', but don't process it till we see
+							 * a typdelim char or end of array.  This handles
+							 * case where several '}'s appear successively
+							 * in a multidimensional array.
+							 */
+							*ptr = '\0';
 							indx[nest_level - 1]++;
 						}
 					}
 					break;
 				default:
-					if (*q == typdelim && !scanning_string)
+					if (*ptr == typdelim && !scanning_string)
 					{
 						if (i == -1)
 							i = ArrayGetOffset0(ndim, indx, prod);
-						done = true;
+						itemdone = true;
 						indx[ndim - 1]++;
 					}
 					break;
 			}
-			if (!done)
-				q++;
+			if (!itemdone)
+				ptr++;
 		}
-		*q = '\0';
-		if (i >= nitems)
+		*ptr++ = '\0';
+		if (i < 0 || i >= nitems)
 			elog(ERROR, "array_in: illformed array constant");
 		values[i] = FunctionCall3(inputproc,
-								  CStringGetDatum(p),
+								  CStringGetDatum(itemstart),
 								  ObjectIdGetDatum(typelem),
 								  Int32GetDatum(typmod));
-		p = ++q;
-
-		/*
-		 * if not at the end of the array skip white space
-		 */
-		if (!eoArray)
-			while (isspace((unsigned char) *q))
-			{
-				p++;
-				q++;
-			}
 	}
 
 	/*
@@ -564,16 +577,13 @@ array_out(PG_FUNCTION_ARGS)
 	char	   *p,
 			   *tmp,
 			   *retval,
-			  **values,
-				delim[2];
+			  **values;
+	bool	   *needquotes;
 	int			nitems,
 				overall_length,
 				i,
 				j,
 				k,
-#ifndef TCL_ARRAYS
-				l,
-#endif
 				indx[MAXDIM];
 	int			ndim,
 			   *dim;
@@ -581,26 +591,29 @@ array_out(PG_FUNCTION_ARGS)
 	system_cache_lookup(element_type, false, &typlen, &typbyval,
 						&typdelim, &typelem, &typoutput, &typalign);
 	fmgr_info(typoutput, &outputproc);
-	sprintf(delim, "%c", typdelim);
 	ndim = ARR_NDIM(v);
 	dim = ARR_DIMS(v);
 	nitems = ArrayGetNItems(ndim, dim);
 
 	if (nitems == 0)
 	{
-		retval = (char *) palloc(3);
-		retval[0] = '{';
-		retval[1] = '}';
-		retval[2] = '\0';
+		retval = pstrdup("{}");
 		PG_RETURN_CSTRING(retval);
 	}
 
+	/*
+	 * Convert all values to string form, count total space needed
+	 * (including any overhead such as escaping backslashes),
+	 * and detect whether each item needs double quotes.
+	 */
+	values = (char **) palloc(nitems * sizeof(char *));
+	needquotes = (bool *) palloc(nitems * sizeof(bool));
 	p = ARR_DATA_PTR(v);
 	overall_length = 1;			/* [TRH] don't forget to count \0 at end. */
-	values = (char **) palloc(nitems * sizeof(char *));
 	for (i = 0; i < nitems; i++)
 	{
 		Datum		itemvalue;
+		bool		nq;
 
 		itemvalue = fetch_att(p, typbyval, typlen);
 		values[i] = DatumGetCString(FunctionCall3(&outputproc,
@@ -612,20 +625,32 @@ array_out(PG_FUNCTION_ARGS)
 		else
 			p += INTALIGN(*(int32 *) p);
 
-		/*
-		 * For the pair of double quotes
-		 */
-		if (!typbyval)
-			overall_length += 2;
-
+		/* count data plus backslashes; detect chars needing quotes */
+		nq = (values[i][0] == '\0');	/* force quotes for empty string */
 		for (tmp = values[i]; *tmp; tmp++)
 		{
+			char	ch = *tmp;
+
 			overall_length += 1;
+			if (ch == '"' || ch == '\\')
+			{
+				nq = true;
 #ifndef TCL_ARRAYS
-			if (*tmp == '"')
 				overall_length += 1;
 #endif
+			}
+			else if (ch == '{' || ch == '}' || ch == typdelim ||
+					 isspace((unsigned char) ch))
+				nq = true;
 		}
+
+		needquotes[i] = nq;
+
+		/* Count the pair of double quotes, if needed */
+		if (nq)
+			overall_length += 2;
+
+		/* and the comma */
 		overall_length += 1;
 	}
 
@@ -634,41 +659,41 @@ array_out(PG_FUNCTION_ARGS)
 	 */
 	for (i = j = 0, k = 1; i < ndim; k *= dim[i++], j += k);
 
-	p = (char *) palloc(overall_length + 2 * j);
-	retval = p;
+	retval = (char *) palloc(overall_length + 2 * j);
+	p = retval;
 
-	strcpy(p, "{");
+#define APPENDSTR(str)	(strcpy(p, (str)), p += strlen(p))
+#define APPENDCHAR(ch)	(*p++ = (ch), *p = '\0')
+
+	APPENDCHAR('{');
 	for (i = 0; i < ndim; indx[i++] = 0);
 	j = 0;
 	k = 0;
 	do
 	{
 		for (i = j; i < ndim - 1; i++)
-			strcat(p, "{");
+			APPENDCHAR('{');
 
-		/*
-		 * Surround anything that is not passed by value in double quotes.
-		 * See above for more details.
-		 */
-		if (!typbyval)
+		if (needquotes[k])
 		{
-			strcat(p, "\"");
+			APPENDCHAR('"');
 #ifndef TCL_ARRAYS
-			l = strlen(p);
 			for (tmp = values[k]; *tmp; tmp++)
 			{
-				if (*tmp == '"')
-					p[l++] = '\\';
-				p[l++] = *tmp;
+				char	ch = *tmp;
+
+				if (ch == '"' || ch == '\\')
+					*p++ = '\\';
+				*p++ = ch;
 			}
-			p[l] = '\0';
+			*p = '\0';
 #else
-			strcat(p, values[k]);
+			APPENDSTR(values[k]);
 #endif
-			strcat(p, "\"");
+			APPENDCHAR('"');
 		}
 		else
-			strcat(p, values[k]);
+			APPENDSTR(values[k]);
 		pfree(values[k++]);
 
 		for (i = ndim - 1; i >= 0; i--)
@@ -676,16 +701,21 @@ array_out(PG_FUNCTION_ARGS)
 			indx[i] = (indx[i] + 1) % dim[i];
 			if (indx[i])
 			{
-				strcat(p, delim);
+				APPENDCHAR(typdelim);
 				break;
 			}
 			else
-				strcat(p, "}");
+				APPENDCHAR('}');
 		}
 		j = i;
 	} while (j != -1);
 
+#undef APPENDSTR
+#undef APPENDCHAR
+
 	pfree(values);
+	pfree(needquotes);
+
 	PG_RETURN_CSTRING(retval);
 }
 
@@ -764,7 +794,6 @@ array_ref(ArrayType *array,
 
 	if (arraylen > 0)
 	{
-
 		/*
 		 * fixed-length arrays -- these are assumed to be 1-d, 0-based
 		 */
@@ -802,6 +831,7 @@ array_ref(ArrayType *array,
 
 	retptr = array_seek(arraydataptr, elmlen, offset);
 
+	*isNull = false;
 	return ArrayCast(retptr, elmbyval, elmlen);
 }
 
@@ -815,7 +845,7 @@ array_ref(ArrayType *array,
  * lowerIndx[] and upperIndx[].  These are generally just temporaries.
  *-----------------------------------------------------------------------------
  */
-ArrayType  *
+ArrayType *
 array_get_slice(ArrayType *array,
 				int nSubscripts,
 				int *upperIndx,
@@ -828,7 +858,8 @@ array_get_slice(ArrayType *array,
 	int			i,
 				ndim,
 			   *dim,
-			   *lb;
+			   *lb,
+			   *newlb;
 	int			fixedDim[1],
 				fixedLb[1];
 	char	   *arraydataptr;
@@ -841,7 +872,6 @@ array_get_slice(ArrayType *array,
 
 	if (arraylen > 0)
 	{
-
 		/*
 		 * fixed-length arrays -- currently, cannot slice these because
 		 * parser labels output as being of the fixed-length array type!
@@ -901,7 +931,14 @@ array_get_slice(ArrayType *array,
 	newarray->ndim = ndim;
 	newarray->flags = 0;
 	memcpy(ARR_DIMS(newarray), span, ndim * sizeof(int));
-	memcpy(ARR_LBOUND(newarray), lowerIndx, ndim * sizeof(int));
+	/*
+	 * Lower bounds of the new array are set to 1.  Formerly (before 7.3)
+	 * we copied the given lowerIndx values ... but that seems confusing.
+	 */
+	newlb = ARR_LBOUND(newarray);
+	for (i = 0; i < ndim; i++)
+		newlb[i] = 1;
+
 	array_extract_slice(ndim, dim, lb, arraydataptr, elmlen,
 						lowerIndx, upperIndx, ARR_DATA_PTR(newarray));
 
@@ -926,7 +963,7 @@ array_get_slice(ArrayType *array,
  * with NULL, which will probably not make him happy.
  *-----------------------------------------------------------------------------
  */
-ArrayType  *
+ArrayType *
 array_set(ArrayType *array,
 		  int nSubscripts,
 		  int *indx,
@@ -958,7 +995,6 @@ array_set(ArrayType *array,
 
 	if (arraylen > 0)
 	{
-
 		/*
 		 * fixed-length arrays -- these are assumed to be 1-d, 0-based. We
 		 * cannot extend them, either.
@@ -1090,7 +1126,7 @@ array_set(ArrayType *array,
  * with NULL, which will probably not make him happy.
  *----------------------------------------------------------------------------
  */
-ArrayType  *
+ArrayType *
 array_set_slice(ArrayType *array,
 				int nSubscripts,
 				int *upperIndx,
@@ -1123,7 +1159,6 @@ array_set_slice(ArrayType *array,
 
 	if (arraylen > 0)
 	{
-
 		/*
 		 * fixed-length arrays -- not got round to doing this...
 		 */
@@ -1192,7 +1227,6 @@ array_set_slice(ArrayType *array,
 	olddatasize = ARR_SIZE(array) - overheadlen;
 	if (ndim > 1)
 	{
-
 		/*
 		 * here we do not need to cope with extension of the array; it
 		 * would be a lot more complicated if we had to do so...
@@ -1203,15 +1237,14 @@ array_set_slice(ArrayType *array,
 	}
 	else
 	{
-
 		/*
 		 * here we must allow for possibility of slice larger than orig
 		 * array
 		 */
 		int			oldlb = ARR_LBOUND(array)[0];
 		int			oldub = oldlb + ARR_DIMS(array)[0] - 1;
-		int			slicelb = MAX(oldlb, lowerIndx[0]);
-		int			sliceub = MIN(oldub, upperIndx[0]);
+		int			slicelb = Max(oldlb, lowerIndx[0]);
+		int			sliceub = Min(oldub, upperIndx[0]);
 		char	   *oldarraydata = ARR_DATA_PTR(array);
 
 		lenbefore = array_nelems_size(oldarraydata,
@@ -1237,7 +1270,6 @@ array_set_slice(ArrayType *array,
 
 	if (ndim > 1)
 	{
-
 		/*
 		 * here we do not need to cope with extension of the array; it
 		 * would be a lot more complicated if we had to do so...
@@ -1401,7 +1433,7 @@ array_map(FunctionCallInfo fcinfo, Oid inpType, Oid retType)
  * NULL element values are not supported.
  *----------
  */
-ArrayType  *
+ArrayType *
 construct_array(Datum *elems, int nelems,
 				bool elmbyval, int elmlen, char elmalign)
 {
