@@ -8,16 +8,16 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/utils/adt/float.c,v 1.71 2001/05/03 19:00:36 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/utils/adt/float.c,v 1.78 2001/12/11 02:02:12 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
 /*----------
  * OLD COMMENTS
  *		Basic float4 ops:
- *		 float4in, float4out, float4abs, float4um
+ *		 float4in, float4out, float4abs, float4um, float4up
  *		Basic float8 ops:
- *		 float8in, float8out, float8abs, float8um
+ *		 float8in, float8out, float8abs, float8um, float8up
  *		Arithmetic operators:
  *		 float4pl, float4mi, float4mul, float4div
  *		 float8pl, float8mi, float8mul, float8div
@@ -71,7 +71,6 @@
 
 #ifndef atof
 extern double atof(const char *p);
-
 #endif
 
 #ifndef HAVE_CBRT
@@ -81,9 +80,8 @@ static double cbrt(double x);
 #else
 #if !defined(nextstep)
 extern double cbrt(double x);
-
 #endif
-#endif	 /* HAVE_CBRT */
+#endif   /* HAVE_CBRT */
 
 #ifndef HAVE_RINT
 #define rint my_rint
@@ -91,10 +89,8 @@ static double rint(double x);
 
 #else
 extern double rint(double x);
-
-#endif	 /* HAVE_RINT */
-
-#endif	 /* NeXT check */
+#endif   /* HAVE_RINT */
+#endif   /* NeXT check */
 
 
 static void CheckFloat4Val(double val);
@@ -140,7 +136,6 @@ static void CheckFloat8Val(double val);
 static void
 CheckFloat4Val(double val)
 {
-
 	/*
 	 * defining unsafe floats's will make float4 and float8 ops faster at
 	 * the cost of safety, of course!
@@ -153,7 +148,7 @@ CheckFloat4Val(double val)
 	if (val != 0.0 && fabs(val) < FLOAT4_MIN)
 		elog(ERROR, "Bad float4 input format -- underflow");
 	return;
-#endif	 /* UNSAFE_FLOATS */
+#endif   /* UNSAFE_FLOATS */
 }
 
 /*
@@ -165,7 +160,6 @@ CheckFloat4Val(double val)
 static void
 CheckFloat8Val(double val)
 {
-
 	/*
 	 * defining unsafe floats's will make float4 and float8 ops faster at
 	 * the cost of safety, of course!
@@ -178,7 +172,7 @@ CheckFloat8Val(double val)
 	if (val != 0.0 && fabs(val) < FLOAT8_MIN)
 		elog(ERROR, "Bad float8 input format -- underflow");
 	return;
-#endif	 /* UNSAFE_FLOATS */
+#endif   /* UNSAFE_FLOATS */
 }
 
 /*
@@ -199,8 +193,15 @@ float4in(PG_FUNCTION_ARGS)
 	val = strtod(num, &endptr);
 	if (*endptr != '\0')
 	{
-		/* Shouldn't we accept "NaN" or "Infinity" for float4? */
-		elog(ERROR, "Bad float4 input format '%s'", num);
+		/*
+		 * XXX we should accept "Infinity" and "-Infinity" too, but what
+		 * are the correct values to assign?  HUGE_VAL will provoke an
+		 * error from CheckFloat4Val.
+		 */
+		if (strcasecmp(num, "NaN") == 0)
+			val = NAN;
+		else
+			elog(ERROR, "Bad float4 input format '%s'", num);
 	}
 	else
 	{
@@ -226,11 +227,15 @@ float4out(PG_FUNCTION_ARGS)
 {
 	float4		num = PG_GETARG_FLOAT4(0);
 	char	   *ascii = (char *) palloc(MAXFLOATWIDTH + 1);
+	int			infflag;
 
 	if (isnan(num))
 		PG_RETURN_CSTRING(strcpy(ascii, "NaN"));
-	if (isinf(num))
+	infflag = isinf(num);
+	if (infflag > 0)
 		PG_RETURN_CSTRING(strcpy(ascii, "Infinity"));
+	if (infflag < 0)
+		PG_RETURN_CSTRING(strcpy(ascii, "-Infinity"));
 
 	sprintf(ascii, "%.*g", FLT_DIG, num);
 	PG_RETURN_CSTRING(ascii);
@@ -258,6 +263,8 @@ float8in(PG_FUNCTION_ARGS)
 			val = NAN;
 		else if (strcasecmp(num, "Infinity") == 0)
 			val = HUGE_VAL;
+		else if (strcasecmp(num, "-Infinity") == 0)
+			val = -HUGE_VAL;
 		else
 			elog(ERROR, "Bad float8 input format '%s'", num);
 	}
@@ -282,11 +289,15 @@ float8out(PG_FUNCTION_ARGS)
 {
 	float8		num = PG_GETARG_FLOAT8(0);
 	char	   *ascii = (char *) palloc(MAXDOUBLEWIDTH + 1);
+	int			infflag;
 
 	if (isnan(num))
 		PG_RETURN_CSTRING(strcpy(ascii, "NaN"));
-	if (isinf(num))
+	infflag = isinf(num);
+	if (infflag > 0)
 		PG_RETURN_CSTRING(strcpy(ascii, "Infinity"));
+	if (infflag < 0)
+		PG_RETURN_CSTRING(strcpy(ascii, "-Infinity"));
 
 	sprintf(ascii, "%.*g", DBL_DIG, num);
 	PG_RETURN_CSTRING(ascii);
@@ -323,6 +334,14 @@ float4um(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT4((float4) -arg1);
 }
 
+Datum
+float4up(PG_FUNCTION_ARGS)
+{
+	float4		arg = PG_GETARG_FLOAT4(0);
+
+	PG_RETURN_FLOAT4(arg);
+}
+
 Datum
 float4larger(PG_FUNCTION_ARGS)
 {
@@ -382,6 +401,14 @@ float8um(PG_FUNCTION_ARGS)
 	PG_RETURN_FLOAT8(result);
 }
 
+Datum
+float8up(PG_FUNCTION_ARGS)
+{
+	float8		arg = PG_GETARG_FLOAT8(0);
+
+	PG_RETURN_FLOAT8(arg);
+}
+
 Datum
 float8larger(PG_FUNCTION_ARGS)
 {
@@ -547,9 +574,9 @@ static int
 float4_cmp_internal(float4 a, float4 b)
 {
 	/*
-	 * We consider all NANs to be equal and larger than any non-NAN.
-	 * This is somewhat arbitrary; the important thing is to have a
-	 * consistent sort order.
+	 * We consider all NANs to be equal and larger than any non-NAN. This
+	 * is somewhat arbitrary; the important thing is to have a consistent
+	 * sort order.
 	 */
 	if (isnan(a))
 	{
@@ -566,10 +593,10 @@ float4_cmp_internal(float4 a, float4 b)
 	{
 		if (a > b)
 			return 1;
-		else if (a == b)
-			return 0;
-		else
+		else if (a < b)
 			return -1;
+		else
+			return 0;
 	}
 }
 
@@ -643,9 +670,9 @@ static int
 float8_cmp_internal(float8 a, float8 b)
 {
 	/*
-	 * We consider all NANs to be equal and larger than any non-NAN.
-	 * This is somewhat arbitrary; the important thing is to have a
-	 * consistent sort order.
+	 * We consider all NANs to be equal and larger than any non-NAN. This
+	 * is somewhat arbitrary; the important thing is to have a consistent
+	 * sort order.
 	 */
 	if (isnan(a))
 	{
@@ -662,10 +689,10 @@ float8_cmp_internal(float8 a, float8 b)
 	{
 		if (a > b)
 			return 1;
-		else if (a == b)
-			return 0;
-		else
+		else if (a < b)
 			return -1;
+		else
+			return 0;
 	}
 }
 
@@ -1448,7 +1475,6 @@ setseed(PG_FUNCTION_ARGS)
 static float8 *
 check_float8_array(ArrayType *transarray, const char *caller)
 {
-
 	/*
 	 * We expect the input to be a 3-element float array; verify that. We
 	 * don't need to use deconstruct_array() since the array data is just
@@ -1554,7 +1580,8 @@ float8_variance(PG_FUNCTION_ARGS)
 	float8	   *transvalues;
 	float8		N,
 				sumX,
-				sumX2;
+				sumX2,
+				numerator;
 
 	transvalues = check_float8_array(transarray, "float8_variance");
 	N = transvalues[0];
@@ -1568,7 +1595,13 @@ float8_variance(PG_FUNCTION_ARGS)
 	if (N <= 1.0)
 		PG_RETURN_FLOAT8(0.0);
 
-	PG_RETURN_FLOAT8((N * sumX2 - sumX * sumX) / (N * (N - 1.0)));
+	numerator = N * sumX2 - sumX * sumX;
+
+	/* Watch out for roundoff error producing a negative numerator */
+	if (numerator <= 0.0)
+		PG_RETURN_FLOAT8(0.0);
+
+	PG_RETURN_FLOAT8(numerator / (N * (N - 1.0)));
 }
 
 Datum
@@ -1578,7 +1611,8 @@ float8_stddev(PG_FUNCTION_ARGS)
 	float8	   *transvalues;
 	float8		N,
 				sumX,
-				sumX2;
+				sumX2,
+				numerator;
 
 	transvalues = check_float8_array(transarray, "float8_stddev");
 	N = transvalues[0];
@@ -1592,7 +1626,13 @@ float8_stddev(PG_FUNCTION_ARGS)
 	if (N <= 1.0)
 		PG_RETURN_FLOAT8(0.0);
 
-	PG_RETURN_FLOAT8(sqrt((N * sumX2 - sumX * sumX) / (N * (N - 1.0))));
+	numerator = N * sumX2 - sumX * sumX;
+
+	/* Watch out for roundoff error producing a negative numerator */
+	if (numerator <= 0.0)
+		PG_RETURN_FLOAT8(0.0);
+
+	PG_RETURN_FLOAT8(sqrt(numerator / (N * (N - 1.0))));
 }
 
 
@@ -1871,7 +1911,7 @@ float84ge(PG_FUNCTION_ARGS)
 static const double one = 1.0,
 			TWO52[2] = {
 	4.50359962737049600000e+15, /* 0x43300000, 0x00000000 */
-	-4.50359962737049600000e+15,/* 0xC3300000, 0x00000000 */
+	-4.50359962737049600000e+15,	/* 0xC3300000, 0x00000000 */
 };
 
 static double
@@ -1943,8 +1983,7 @@ rint(double x)
 	w = TWO52[sx] + x;
 	return w - TWO52[sx];
 }
-
-#endif	 /* !HAVE_RINT */
+#endif   /* !HAVE_RINT */
 
 #ifndef HAVE_CBRT
 
@@ -1957,4 +1996,4 @@ cbrt(double x)
 	return isneg ? -tmpres : tmpres;
 }
 
-#endif	 /* !HAVE_CBRT */
+#endif   /* !HAVE_CBRT */
