@@ -5,7 +5,7 @@
  *
  *	1998 Jan Wieck
  *
- * $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/utils/adt/numeric.c,v 1.39 2001/03/22 06:16:17 momjian Exp $
+ * $Header: /cvsroot/pgsql/src/backend/utils/adt/numeric.c,v 1.49 2001/12/11 02:02:12 tgl Exp $
  *
  * ----------
  */
@@ -153,11 +153,13 @@ static Numeric make_result(NumericVar *var);
 
 static void apply_typmod(NumericVar *var, int32 typmod);
 
+static int	cmp_numerics(Numeric num1, Numeric num2);
 static int	cmp_var(NumericVar *var1, NumericVar *var2);
 static void add_var(NumericVar *var1, NumericVar *var2, NumericVar *result);
 static void sub_var(NumericVar *var1, NumericVar *var2, NumericVar *result);
 static void mul_var(NumericVar *var1, NumericVar *var2, NumericVar *result);
 static void div_var(NumericVar *var1, NumericVar *var2, NumericVar *result);
+static int select_div_scale(NumericVar *var1, NumericVar *var2);
 static void mod_var(NumericVar *var1, NumericVar *var2, NumericVar *result);
 static void ceil_var(NumericVar *var, NumericVar *result);
 static void floor_var(NumericVar *var, NumericVar *result);
@@ -195,7 +197,6 @@ numeric_in(PG_FUNCTION_ARGS)
 
 #ifdef NOT_USED
 	Oid			typelem = PG_GETARG_OID(1);
-
 #endif
 	int32		typmod = PG_GETARG_INT32(2);
 	NumericVar	value;
@@ -404,6 +405,19 @@ numeric_uminus(PG_FUNCTION_ARGS)
 }
 
 
+Datum
+numeric_uplus(PG_FUNCTION_ARGS)
+{
+	Numeric		num = PG_GETARG_NUMERIC(0);
+	Numeric		res;
+
+	res = (Numeric) palloc(num->varlen);
+	memcpy(res, num, num->varlen);
+
+	PG_RETURN_NUMERIC(res);
+}
+
+
 Datum
 numeric_sign(PG_FUNCTION_ARGS)
 {
@@ -428,7 +442,6 @@ numeric_sign(PG_FUNCTION_ARGS)
 		set_var_from_var(&const_zero, &result);
 	else
 	{
-
 		/*
 		 * And if there are some, we return a copy of ONE with the sign of
 		 * our argument
@@ -484,7 +497,6 @@ numeric_round(PG_FUNCTION_ARGS)
 
 	if (i < arg.ndigits)
 	{
-
 		/*
 		 * If i = 0, the value loses all digits, but could round up if its
 		 * first digit is more than 4.	If i < 0 the result must be 0.
@@ -506,7 +518,7 @@ numeric_round(PG_FUNCTION_ARGS)
 
 			if (i < 0)
 			{
-				Assert(i == -1);/* better not have added more than 1 digit */
+				Assert(i == -1);	/* better not have added more than 1 digit */
 				Assert(arg.digits > arg.buf);
 				arg.digits--;
 				arg.ndigits++;
@@ -664,8 +676,126 @@ numeric_cmp(PG_FUNCTION_ARGS)
 	Numeric		num2 = PG_GETARG_NUMERIC(1);
 	int			result;
 
-	if (NUMERIC_IS_NAN(num1) || NUMERIC_IS_NAN(num2))
-		result = 0;
+	result = cmp_numerics(num1, num2);
+
+	PG_FREE_IF_COPY(num1, 0);
+	PG_FREE_IF_COPY(num2, 1);
+
+	PG_RETURN_INT32(result);
+}
+
+
+Datum
+numeric_eq(PG_FUNCTION_ARGS)
+{
+	Numeric		num1 = PG_GETARG_NUMERIC(0);
+	Numeric		num2 = PG_GETARG_NUMERIC(1);
+	bool		result;
+
+	result = cmp_numerics(num1, num2) == 0;
+
+	PG_FREE_IF_COPY(num1, 0);
+	PG_FREE_IF_COPY(num2, 1);
+
+	PG_RETURN_BOOL(result);
+}
+
+Datum
+numeric_ne(PG_FUNCTION_ARGS)
+{
+	Numeric		num1 = PG_GETARG_NUMERIC(0);
+	Numeric		num2 = PG_GETARG_NUMERIC(1);
+	bool		result;
+
+	result = cmp_numerics(num1, num2) != 0;
+
+	PG_FREE_IF_COPY(num1, 0);
+	PG_FREE_IF_COPY(num2, 1);
+
+	PG_RETURN_BOOL(result);
+}
+
+Datum
+numeric_gt(PG_FUNCTION_ARGS)
+{
+	Numeric		num1 = PG_GETARG_NUMERIC(0);
+	Numeric		num2 = PG_GETARG_NUMERIC(1);
+	bool		result;
+
+	result = cmp_numerics(num1, num2) > 0;
+
+	PG_FREE_IF_COPY(num1, 0);
+	PG_FREE_IF_COPY(num2, 1);
+
+	PG_RETURN_BOOL(result);
+}
+
+Datum
+numeric_ge(PG_FUNCTION_ARGS)
+{
+	Numeric		num1 = PG_GETARG_NUMERIC(0);
+	Numeric		num2 = PG_GETARG_NUMERIC(1);
+	bool		result;
+
+	result = cmp_numerics(num1, num2) >= 0;
+
+	PG_FREE_IF_COPY(num1, 0);
+	PG_FREE_IF_COPY(num2, 1);
+
+	PG_RETURN_BOOL(result);
+}
+
+Datum
+numeric_lt(PG_FUNCTION_ARGS)
+{
+	Numeric		num1 = PG_GETARG_NUMERIC(0);
+	Numeric		num2 = PG_GETARG_NUMERIC(1);
+	bool		result;
+
+	result = cmp_numerics(num1, num2) < 0;
+
+	PG_FREE_IF_COPY(num1, 0);
+	PG_FREE_IF_COPY(num2, 1);
+
+	PG_RETURN_BOOL(result);
+}
+
+Datum
+numeric_le(PG_FUNCTION_ARGS)
+{
+	Numeric		num1 = PG_GETARG_NUMERIC(0);
+	Numeric		num2 = PG_GETARG_NUMERIC(1);
+	bool		result;
+
+	result = cmp_numerics(num1, num2) <= 0;
+
+	PG_FREE_IF_COPY(num1, 0);
+	PG_FREE_IF_COPY(num2, 1);
+
+	PG_RETURN_BOOL(result);
+}
+
+static int
+cmp_numerics(Numeric num1, Numeric num2)
+{
+	int			result;
+
+	/*
+	 * We consider all NANs to be equal and larger than any non-NAN. This
+	 * is somewhat arbitrary; the important thing is to have a consistent
+	 * sort order.
+	 */
+	if (NUMERIC_IS_NAN(num1))
+	{
+		if (NUMERIC_IS_NAN(num2))
+			result = 0;			/* NAN = NAN */
+		else
+			result = 1;			/* NAN > non-NAN */
+	}
+	else if (NUMERIC_IS_NAN(num2))
+	{
+		result = -1;			/* non-NAN < NAN */
+	}
 	else
 	{
 		NumericVar	arg1;
@@ -683,203 +813,7 @@ numeric_cmp(PG_FUNCTION_ARGS)
 		free_var(&arg2);
 	}
 
-	PG_FREE_IF_COPY(num1, 0);
-	PG_FREE_IF_COPY(num2, 1);
-
-	PG_RETURN_INT32(result);
-}
-
-
-Datum
-numeric_eq(PG_FUNCTION_ARGS)
-{
-	Numeric		num1 = PG_GETARG_NUMERIC(0);
-	Numeric		num2 = PG_GETARG_NUMERIC(1);
-	bool		result;
-
-	if (NUMERIC_IS_NAN(num1) || NUMERIC_IS_NAN(num2))
-		result = false;
-	else
-	{
-		NumericVar	arg1;
-		NumericVar	arg2;
-
-		init_var(&arg1);
-		init_var(&arg2);
-
-		set_var_from_num(num1, &arg1);
-		set_var_from_num(num2, &arg2);
-
-		result = cmp_var(&arg1, &arg2) == 0;
-
-		free_var(&arg1);
-		free_var(&arg2);
-	}
-
-	PG_FREE_IF_COPY(num1, 0);
-	PG_FREE_IF_COPY(num2, 1);
-
-	PG_RETURN_BOOL(result);
-}
-
-Datum
-numeric_ne(PG_FUNCTION_ARGS)
-{
-	Numeric		num1 = PG_GETARG_NUMERIC(0);
-	Numeric		num2 = PG_GETARG_NUMERIC(1);
-	bool		result;
-
-	if (NUMERIC_IS_NAN(num1) || NUMERIC_IS_NAN(num2))
-		result = false;
-	else
-	{
-		NumericVar	arg1;
-		NumericVar	arg2;
-
-		init_var(&arg1);
-		init_var(&arg2);
-
-		set_var_from_num(num1, &arg1);
-		set_var_from_num(num2, &arg2);
-
-		result = cmp_var(&arg1, &arg2) != 0;
-
-		free_var(&arg1);
-		free_var(&arg2);
-	}
-
-	PG_FREE_IF_COPY(num1, 0);
-	PG_FREE_IF_COPY(num2, 1);
-
-	PG_RETURN_BOOL(result);
-}
-
-Datum
-numeric_gt(PG_FUNCTION_ARGS)
-{
-	Numeric		num1 = PG_GETARG_NUMERIC(0);
-	Numeric		num2 = PG_GETARG_NUMERIC(1);
-	bool		result;
-
-	if (NUMERIC_IS_NAN(num1) || NUMERIC_IS_NAN(num2))
-		result = false;
-	else
-	{
-		NumericVar	arg1;
-		NumericVar	arg2;
-
-		init_var(&arg1);
-		init_var(&arg2);
-
-		set_var_from_num(num1, &arg1);
-		set_var_from_num(num2, &arg2);
-
-		result = cmp_var(&arg1, &arg2) > 0;
-
-		free_var(&arg1);
-		free_var(&arg2);
-	}
-
-	PG_FREE_IF_COPY(num1, 0);
-	PG_FREE_IF_COPY(num2, 1);
-
-	PG_RETURN_BOOL(result);
-}
-
-Datum
-numeric_ge(PG_FUNCTION_ARGS)
-{
-	Numeric		num1 = PG_GETARG_NUMERIC(0);
-	Numeric		num2 = PG_GETARG_NUMERIC(1);
-	bool		result;
-
-	if (NUMERIC_IS_NAN(num1) || NUMERIC_IS_NAN(num2))
-		result = false;
-	else
-	{
-		NumericVar	arg1;
-		NumericVar	arg2;
-
-		init_var(&arg1);
-		init_var(&arg2);
-
-		set_var_from_num(num1, &arg1);
-		set_var_from_num(num2, &arg2);
-
-		result = cmp_var(&arg1, &arg2) >= 0;
-
-		free_var(&arg1);
-		free_var(&arg2);
-	}
-
-	PG_FREE_IF_COPY(num1, 0);
-	PG_FREE_IF_COPY(num2, 1);
-
-	PG_RETURN_BOOL(result);
-}
-
-Datum
-numeric_lt(PG_FUNCTION_ARGS)
-{
-	Numeric		num1 = PG_GETARG_NUMERIC(0);
-	Numeric		num2 = PG_GETARG_NUMERIC(1);
-	bool		result;
-
-	if (NUMERIC_IS_NAN(num1) || NUMERIC_IS_NAN(num2))
-		result = false;
-	else
-	{
-		NumericVar	arg1;
-		NumericVar	arg2;
-
-		init_var(&arg1);
-		init_var(&arg2);
-
-		set_var_from_num(num1, &arg1);
-		set_var_from_num(num2, &arg2);
-
-		result = cmp_var(&arg1, &arg2) < 0;
-
-		free_var(&arg1);
-		free_var(&arg2);
-	}
-
-	PG_FREE_IF_COPY(num1, 0);
-	PG_FREE_IF_COPY(num2, 1);
-
-	PG_RETURN_BOOL(result);
-}
-
-Datum
-numeric_le(PG_FUNCTION_ARGS)
-{
-	Numeric		num1 = PG_GETARG_NUMERIC(0);
-	Numeric		num2 = PG_GETARG_NUMERIC(1);
-	bool		result;
-
-	if (NUMERIC_IS_NAN(num1) || NUMERIC_IS_NAN(num2))
-		result = false;
-	else
-	{
-		NumericVar	arg1;
-		NumericVar	arg2;
-
-		init_var(&arg1);
-		init_var(&arg2);
-
-		set_var_from_num(num1, &arg1);
-		set_var_from_num(num2, &arg2);
-
-		result = cmp_var(&arg1, &arg2) <= 0;
-
-		free_var(&arg1);
-		free_var(&arg2);
-	}
-
-	PG_FREE_IF_COPY(num1, 0);
-	PG_FREE_IF_COPY(num2, 1);
-
-	PG_RETURN_BOOL(result);
+	return result;
 }
 
 
@@ -1066,28 +1000,7 @@ numeric_div(PG_FUNCTION_ARGS)
 	set_var_from_num(num1, &arg1);
 	set_var_from_num(num2, &arg2);
 
-	/* ----------
-	 * The result scale of a division isn't specified in any
-	 * SQL standard. For Postgres it is the following (where
-	 * SR, DR are the result- and display-scales of the returned
-	 * value, S1, D1, S2 and D2 are the scales of the two arguments,
-	 * The minimum and maximum scales are compile time options from
-	 * numeric.h):
-	 *
-	 *	DR = MIN(MAX(D1 + D2, MIN_DISPLAY_SCALE), MAX_DISPLAY_SCALE)
-	 *	SR = MIN(MAX(MAX(S1 + S2, MIN_RESULT_SCALE), DR + 4), MAX_RESULT_SCALE)
-	 *
-	 * By default, any result is computed with a minimum of 34 digits
-	 * after the decimal point or at least with 4 digits more than
-	 * displayed.
-	 * ----------
-	 */
-	res_dscale = MAX(arg1.dscale + arg2.dscale, NUMERIC_MIN_DISPLAY_SCALE);
-	res_dscale = MIN(res_dscale, NUMERIC_MAX_DISPLAY_SCALE);
-	global_rscale = MAX(arg1.rscale + arg2.rscale,
-						NUMERIC_MIN_RESULT_SCALE);
-	global_rscale = MAX(global_rscale, res_dscale + 4);
-	global_rscale = MIN(global_rscale, NUMERIC_MAX_RESULT_SCALE);
+	res_dscale = select_div_scale(&arg1, &arg2);
 
 	/*
 	 * Do the divide, set the display scale and return the result
@@ -1134,7 +1047,6 @@ numeric_mod(PG_FUNCTION_ARGS)
 
 	mod_var(&arg1, &arg2, &result);
 
-	result.dscale = result.rscale;
 	res = make_result(&result);
 
 	free_var(&result);
@@ -1664,6 +1576,7 @@ numeric_int2(PG_FUNCTION_ARGS)
 	char	   *str;
 	Datum		result;
 
+	/* XXX would it be better to return NULL? */
 	if (NUMERIC_IS_NAN(num))
 		elog(ERROR, "Cannot convert NaN to int2");
 
@@ -1729,6 +1642,35 @@ numeric_float8(PG_FUNCTION_ARGS)
 }
 
 
+/* Convert numeric to float8; if out of range, return +/- HUGE_VAL */
+Datum
+numeric_float8_no_overflow(PG_FUNCTION_ARGS)
+{
+	Numeric		num = PG_GETARG_NUMERIC(0);
+	char	   *tmp;
+	double		val;
+	char	   *endptr;
+
+	if (NUMERIC_IS_NAN(num))
+		PG_RETURN_FLOAT8(NAN);
+
+	tmp = DatumGetCString(DirectFunctionCall1(numeric_out,
+											  NumericGetDatum(num)));
+
+	/* unlike float8in, we ignore ERANGE from strtod */
+	val = strtod(tmp, &endptr);
+	if (*endptr != '\0')
+	{
+		/* shouldn't happen ... */
+		elog(ERROR, "Bad float8 input format '%s'", tmp);
+	}
+
+	pfree(tmp);
+
+	PG_RETURN_FLOAT8(val);
+}
+
+
 Datum
 float4_numeric(PG_FUNCTION_ARGS)
 {
@@ -1836,7 +1778,11 @@ numeric_accum(PG_FUNCTION_ARGS)
 
 /*
  * Integer data types all use Numeric accumulators to share code and
- * avoid risk of overflow.
+ * avoid risk of overflow.	For int2 and int4 inputs, Numeric accumulation
+ * is overkill for the N and sum(X) values, but definitely not overkill
+ * for the sum(X*X) value.	Hence, we use int2_accum and int4_accum only
+ * for stddev/variance --- there are faster special-purpose accumulator
+ * routines for SUM and AVG of these datatypes.
  */
 
 Datum
@@ -1918,6 +1864,7 @@ numeric_variance(PG_FUNCTION_ARGS)
 				vsumX,
 				vsumX2,
 				vNminus1;
+	int			div_dscale;
 
 	/* We assume the input is array of numeric */
 	deconstruct_array(transarray,
@@ -1958,10 +1905,21 @@ numeric_variance(PG_FUNCTION_ARGS)
 	mul_var(&vsumX, &vsumX, &vsumX);	/* now vsumX contains sumX * sumX */
 	mul_var(&vN, &vsumX2, &vsumX2);		/* now vsumX2 contains N * sumX2 */
 	sub_var(&vsumX2, &vsumX, &vsumX2);	/* N * sumX2 - sumX * sumX */
-	mul_var(&vN, &vNminus1, &vNminus1); /* N * (N - 1) */
-	div_var(&vsumX2, &vNminus1, &vsumX);		/* variance */
 
-	res = make_result(&vsumX);
+	if (cmp_var(&vsumX2, &const_zero) <= 0)
+	{
+		/* Watch out for roundoff error producing a negative numerator */
+		res = make_result(&const_zero);
+	}
+	else
+	{
+		mul_var(&vN, &vNminus1, &vNminus1);	/* N * (N - 1) */
+		div_dscale = select_div_scale(&vsumX2, &vNminus1);
+		div_var(&vsumX2, &vNminus1, &vsumX);	/* variance */
+		vsumX.dscale = div_dscale;
+
+		res = make_result(&vsumX);
+	}
 
 	free_var(&vN);
 	free_var(&vNminus1);
@@ -1985,6 +1943,7 @@ numeric_stddev(PG_FUNCTION_ARGS)
 				vsumX,
 				vsumX2,
 				vNminus1;
+	int			div_dscale;
 
 	/* We assume the input is array of numeric */
 	deconstruct_array(transarray,
@@ -2025,11 +1984,22 @@ numeric_stddev(PG_FUNCTION_ARGS)
 	mul_var(&vsumX, &vsumX, &vsumX);	/* now vsumX contains sumX * sumX */
 	mul_var(&vN, &vsumX2, &vsumX2);		/* now vsumX2 contains N * sumX2 */
 	sub_var(&vsumX2, &vsumX, &vsumX2);	/* N * sumX2 - sumX * sumX */
-	mul_var(&vN, &vNminus1, &vNminus1); /* N * (N - 1) */
-	div_var(&vsumX2, &vNminus1, &vsumX);		/* variance */
-	sqrt_var(&vsumX, &vsumX);	/* stddev */
 
-	res = make_result(&vsumX);
+	if (cmp_var(&vsumX2, &const_zero) <= 0)
+	{
+		/* Watch out for roundoff error producing a negative numerator */
+		res = make_result(&const_zero);
+	}
+	else
+	{
+		mul_var(&vN, &vNminus1, &vNminus1); /* N * (N - 1) */
+		div_dscale = select_div_scale(&vsumX2, &vNminus1);
+		div_var(&vsumX2, &vNminus1, &vsumX);	/* variance */
+		vsumX.dscale = div_dscale;
+		sqrt_var(&vsumX, &vsumX);		/* stddev */
+
+		res = make_result(&vsumX);
+	}
 
 	free_var(&vN);
 	free_var(&vNminus1);
@@ -2043,20 +2013,25 @@ numeric_stddev(PG_FUNCTION_ARGS)
 /*
  * SUM transition functions for integer datatypes.
  *
- * We use a Numeric accumulator to avoid overflow.	Because SQL92 defines
- * the SUM() of no values to be NULL, not zero, the initial condition of
- * the transition data value needs to be NULL.	This means we can't rely
- * on ExecAgg to automatically insert the first non-null data value into
- * the transition data: it doesn't know how to do the type conversion.
- * The upshot is that these routines have to be marked non-strict and
- * handle substitution of the first non-null input themselves.
+ * To avoid overflow, we use accumulators wider than the input datatype.
+ * A Numeric accumulator is needed for int8 input; for int4 and int2
+ * inputs, we use int8 accumulators which should be sufficient for practical
+ * purposes.  (The latter two therefore don't really belong in this file,
+ * but we keep them here anyway.)
+ *
+ * Because SQL92 defines the SUM() of no values to be NULL, not zero,
+ * the initial condition of the transition data value needs to be NULL. This
+ * means we can't rely on ExecAgg to automatically insert the first non-null
+ * data value into the transition data: it doesn't know how to do the type
+ * conversion.	The upshot is that these routines have to be marked non-strict
+ * and handle substitution of the first non-null input themselves.
  */
 
 Datum
 int2_sum(PG_FUNCTION_ARGS)
 {
-	Numeric		oldsum;
-	Datum		newval;
+	int64		oldsum;
+	int64		newval;
 
 	if (PG_ARGISNULL(0))
 	{
@@ -2064,28 +2039,27 @@ int2_sum(PG_FUNCTION_ARGS)
 		if (PG_ARGISNULL(1))
 			PG_RETURN_NULL();	/* still no non-null */
 		/* This is the first non-null input. */
-		newval = DirectFunctionCall1(int2_numeric, PG_GETARG_DATUM(1));
-		PG_RETURN_DATUM(newval);
+		newval = (int64) PG_GETARG_INT16(1);
+		PG_RETURN_INT64(newval);
 	}
 
-	oldsum = PG_GETARG_NUMERIC(0);
+	oldsum = PG_GETARG_INT64(0);
 
 	/* Leave sum unchanged if new input is null. */
 	if (PG_ARGISNULL(1))
-		PG_RETURN_NUMERIC(oldsum);
+		PG_RETURN_INT64(oldsum);
 
 	/* OK to do the addition. */
-	newval = DirectFunctionCall1(int2_numeric, PG_GETARG_DATUM(1));
+	newval = oldsum + (int64) PG_GETARG_INT16(1);
 
-	PG_RETURN_DATUM(DirectFunctionCall2(numeric_add,
-										NumericGetDatum(oldsum), newval));
+	PG_RETURN_INT64(newval);
 }
 
 Datum
 int4_sum(PG_FUNCTION_ARGS)
 {
-	Numeric		oldsum;
-	Datum		newval;
+	int64		oldsum;
+	int64		newval;
 
 	if (PG_ARGISNULL(0))
 	{
@@ -2093,21 +2067,20 @@ int4_sum(PG_FUNCTION_ARGS)
 		if (PG_ARGISNULL(1))
 			PG_RETURN_NULL();	/* still no non-null */
 		/* This is the first non-null input. */
-		newval = DirectFunctionCall1(int4_numeric, PG_GETARG_DATUM(1));
-		PG_RETURN_DATUM(newval);
+		newval = (int64) PG_GETARG_INT32(1);
+		PG_RETURN_INT64(newval);
 	}
 
-	oldsum = PG_GETARG_NUMERIC(0);
+	oldsum = PG_GETARG_INT64(0);
 
 	/* Leave sum unchanged if new input is null. */
 	if (PG_ARGISNULL(1))
-		PG_RETURN_NUMERIC(oldsum);
+		PG_RETURN_INT64(oldsum);
 
 	/* OK to do the addition. */
-	newval = DirectFunctionCall1(int4_numeric, PG_GETARG_DATUM(1));
+	newval = oldsum + (int64) PG_GETARG_INT32(1);
 
-	PG_RETURN_DATUM(DirectFunctionCall2(numeric_add,
-										NumericGetDatum(oldsum), newval));
+	PG_RETURN_INT64(newval);
 }
 
 Datum
@@ -2140,6 +2113,90 @@ int8_sum(PG_FUNCTION_ARGS)
 }
 
 
+/*
+ * Routines for avg(int2) and avg(int4).  The transition datatype
+ * is a two-element int8 array, holding count and sum.
+ */
+
+typedef struct Int8TransTypeData
+{
+#ifndef INT64_IS_BUSTED
+	int64		count;
+	int64		sum;
+#else
+	/* "int64" isn't really 64 bits, so fake up properly-aligned fields */
+	int32		count;
+	int32		pad1;
+	int32		sum;
+	int32		pad2;
+#endif
+} Int8TransTypeData;
+
+Datum
+int2_avg_accum(PG_FUNCTION_ARGS)
+{
+	ArrayType  *transarray = PG_GETARG_ARRAYTYPE_P_COPY(0);
+	int16		newval = PG_GETARG_INT16(1);
+	Int8TransTypeData *transdata;
+
+	/*
+	 * We copied the input array, so it's okay to scribble on it directly.
+	 */
+	if (ARR_SIZE(transarray) != ARR_OVERHEAD(1) + sizeof(Int8TransTypeData))
+		elog(ERROR, "int2_avg_accum: expected 2-element int8 array");
+	transdata = (Int8TransTypeData *) ARR_DATA_PTR(transarray);
+
+	transdata->count++;
+	transdata->sum += newval;
+
+	PG_RETURN_ARRAYTYPE_P(transarray);
+}
+
+Datum
+int4_avg_accum(PG_FUNCTION_ARGS)
+{
+	ArrayType  *transarray = PG_GETARG_ARRAYTYPE_P_COPY(0);
+	int32		newval = PG_GETARG_INT32(1);
+	Int8TransTypeData *transdata;
+
+	/*
+	 * We copied the input array, so it's okay to scribble on it directly.
+	 */
+	if (ARR_SIZE(transarray) != ARR_OVERHEAD(1) + sizeof(Int8TransTypeData))
+		elog(ERROR, "int4_avg_accum: expected 2-element int8 array");
+	transdata = (Int8TransTypeData *) ARR_DATA_PTR(transarray);
+
+	transdata->count++;
+	transdata->sum += newval;
+
+	PG_RETURN_ARRAYTYPE_P(transarray);
+}
+
+Datum
+int8_avg(PG_FUNCTION_ARGS)
+{
+	ArrayType  *transarray = PG_GETARG_ARRAYTYPE_P(0);
+	Int8TransTypeData *transdata;
+	Datum		countd,
+				sumd;
+
+	if (ARR_SIZE(transarray) != ARR_OVERHEAD(1) + sizeof(Int8TransTypeData))
+		elog(ERROR, "int8_avg: expected 2-element int8 array");
+	transdata = (Int8TransTypeData *) ARR_DATA_PTR(transarray);
+
+	/* SQL92 defines AVG of no values to be NULL */
+	if (transdata->count == 0)
+		PG_RETURN_NULL();
+
+	countd = DirectFunctionCall1(int8_numeric,
+								 Int64GetDatumFast(transdata->count));
+	sumd = DirectFunctionCall1(int8_numeric,
+							   Int64GetDatumFast(transdata->sum));
+
+	PG_RETURN_DATUM(DirectFunctionCall2(numeric_div, sumd, countd));
+}
+
+
 /* ----------------------------------------------------------------------
  *
  * Local functions follow
@@ -2215,8 +2272,7 @@ dump_var(char *str, NumericVar *var)
 
 	printf("\n");
 }
-
-#endif	 /* NUMERIC_DEBUG */
+#endif   /* NUMERIC_DEBUG */
 
 
 /* ----------
@@ -2741,7 +2797,6 @@ cmp_var(NumericVar *var1, NumericVar *var2)
 static void
 add_var(NumericVar *var1, NumericVar *var2, NumericVar *result)
 {
-
 	/*
 	 * Decide on the signs of the two variables what to do
 	 */
@@ -2749,7 +2804,6 @@ add_var(NumericVar *var1, NumericVar *var2, NumericVar *result)
 	{
 		if (var2->sign == NUMERIC_POS)
 		{
-
 			/*
 			 * Both are positive result = +(ABS(var1) + ABS(var2))
 			 */
@@ -2758,7 +2812,6 @@ add_var(NumericVar *var1, NumericVar *var2, NumericVar *result)
 		}
 		else
 		{
-
 			/*
 			 * var1 is positive, var2 is negative Must compare absolute
 			 * values
@@ -2865,7 +2918,6 @@ add_var(NumericVar *var1, NumericVar *var2, NumericVar *result)
 static void
 sub_var(NumericVar *var1, NumericVar *var2, NumericVar *result)
 {
-
 	/*
 	 * Decide on the signs of the two variables what to do
 	 */
@@ -3270,6 +3322,50 @@ div_var(NumericVar *var1, NumericVar *var2, NumericVar *result)
 }
 
 
+/*
+ * Default scale selection for division
+ *
+ * Returns the appropriate display scale for the division result,
+ * and sets global_rscale to the result scale to use during div_var.
+ *
+ * Note that this must be called before div_var.
+ */
+static int
+select_div_scale(NumericVar *var1, NumericVar *var2)
+{
+	int			res_dscale;
+	int			res_rscale;
+
+	/* ----------
+	 * The result scale of a division isn't specified in any
+	 * SQL standard. For Postgres it is the following (where
+	 * SR, DR are the result- and display-scales of the returned
+	 * value, S1, D1, S2 and D2 are the scales of the two arguments,
+	 * The minimum and maximum scales are compile time options from
+	 * numeric.h):
+	 *
+	 *	DR = MIN(MAX(D1 + D2, MIN_DISPLAY_SCALE), MAX_DISPLAY_SCALE)
+	 *	SR = MIN(MAX(MAX(S1 + S2, DR + 4), MIN_RESULT_SCALE), MAX_RESULT_SCALE)
+	 *
+	 * By default, any result is computed with a minimum of 34 digits
+	 * after the decimal point or at least with 4 digits more than
+	 * displayed.
+	 * ----------
+	 */
+	res_dscale = var1->dscale + var2->dscale;
+	res_dscale = MAX(res_dscale, NUMERIC_MIN_DISPLAY_SCALE);
+	res_dscale = MIN(res_dscale, NUMERIC_MAX_DISPLAY_SCALE);
+
+	res_rscale = var1->rscale + var2->rscale;
+	res_rscale = MAX(res_rscale, res_dscale + 4);
+	res_rscale = MAX(res_rscale, NUMERIC_MIN_RESULT_SCALE);
+	res_rscale = MIN(res_rscale, NUMERIC_MAX_RESULT_SCALE);
+	global_rscale = res_rscale;
+
+	return res_dscale;
+}
+
+
 /* ----------
  * mod_var() -
  *
@@ -3281,29 +3377,37 @@ mod_var(NumericVar *var1, NumericVar *var2, NumericVar *result)
 {
 	NumericVar	tmp;
 	int			save_global_rscale;
+	int			div_dscale;
 
 	init_var(&tmp);
 
 	/* ---------
 	 * We do this using the equation
 	 *		mod(x,y) = x - trunc(x/y)*y
-	 * We fiddle a bit with global_rscale to control result precision.
+	 * We set global_rscale the same way numeric_div and numeric_mul do
+	 * to get the right answer from the equation.  The final result,
+	 * however, need not be displayed to more precision than the inputs.
 	 * ----------
 	 */
 	save_global_rscale = global_rscale;
-	global_rscale = var2->rscale + 2;
+
+	div_dscale = select_div_scale(var1, var2);
 
 	div_var(var1, var2, &tmp);
 
+	tmp.dscale = div_dscale;
+
 	/* do trunc() by forgetting digits to the right of the decimal point */
 	tmp.ndigits = MAX(0, MIN(tmp.ndigits, tmp.weight + 1));
-	tmp.rscale = var2->rscale;
 
-	global_rscale = var2->rscale;
+	global_rscale = var2->rscale + tmp.rscale;
+
 	mul_var(var2, &tmp, &tmp);
 
 	sub_var(var1, &tmp, result);
 
+	result->dscale = MAX(var1->dscale, var2->dscale);
+
 	global_rscale = save_global_rscale;
 	free_var(&tmp);
 }
