@@ -18,7 +18,7 @@
  * Portions Copyright (c) 2000-2001, PostgreSQL Global Development Group
  * Copyright 1999 Jan Wieck
  *
- * $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/utils/adt/ri_triggers.c,v 1.23.2.2 2001/05/31 17:33:03 tgl Exp $
+ * $Header: /cvsroot/pgsql/src/backend/utils/adt/ri_triggers.c,v 1.31 2001/11/12 06:09:09 momjian Exp $
  *
  * ----------
  */
@@ -43,16 +43,6 @@
  * Local definitions
  * ----------
  */
-#define RI_CONSTRAINT_NAME_ARGNO		0
-#define RI_FK_RELNAME_ARGNO				1
-#define RI_PK_RELNAME_ARGNO				2
-#define RI_MATCH_TYPE_ARGNO				3
-#define RI_FIRST_ATTNAME_ARGNO			4
-
-#define RI_MAX_NUMKEYS					16
-#define RI_MAX_ARGUMENTS		(RI_FIRST_ATTNAME_ARGNO + (RI_MAX_NUMKEYS * 2))
-#define RI_KEYPAIR_FK_IDX				0
-#define RI_KEYPAIR_PK_IDX				1
 
 #define RI_INIT_QUERYHASHSIZE			128
 #define RI_INIT_OPREQHASHSIZE			128
@@ -242,10 +232,10 @@ RI_FKey_check(PG_FUNCTION_ARGS)
 
 			/* ---------
 			 * The query string built is
-			 *	SELECT oid FROM ONLY <pktable>
+			 *	SELECT 1 FROM ONLY <pktable>
 			 * ----------
 			 */
-			sprintf(querystr, "SELECT oid FROM ONLY \"%s\" FOR UPDATE OF \"%s\"",
+			sprintf(querystr, "SELECT 1 FROM ONLY \"%s\" FOR UPDATE OF \"%s\"",
 					tgargs[RI_PK_RELNAME_ARGNO],
 					tgargs[RI_PK_RELNAME_ARGNO]);
 
@@ -386,14 +376,14 @@ RI_FKey_check(PG_FUNCTION_ARGS)
 
 		/* ----------
 		 * The query string built is
-		 *	SELECT oid FROM ONLY <pktable> WHERE pkatt1 = $1 [AND ...]
+		 *	SELECT 1 FROM ONLY <pktable> WHERE pkatt1 = $1 [AND ...]
 		 * The type id's for the $ parameters are those of the
 		 * corresponding FK attributes. Thus, SPI_prepare could
 		 * eventually fail if the parser cannot identify some way
 		 * how to compare these two types by '='.
 		 * ----------
 		 */
-		sprintf(querystr, "SELECT oid FROM ONLY \"%s\"",
+		sprintf(querystr, "SELECT 1 FROM ONLY \"%s\"",
 				tgargs[RI_PK_RELNAME_ARGNO]);
 		querysep = "WHERE";
 		for (i = 0; i < qkey.nkeypairs; i++)
@@ -423,7 +413,6 @@ RI_FKey_check(PG_FUNCTION_ARGS)
 	 */
 	for (i = 0; i < qkey.nkeypairs; i++)
 	{
-
 		/*
 		 * We can implement MATCH PARTIAL by excluding this column from
 		 * the query if it is null.  Simple!  Unfortunately, the
@@ -620,14 +609,14 @@ RI_FKey_noaction_del(PG_FUNCTION_ARGS)
 
 				/* ----------
 				 * The query string built is
-				 *	SELECT oid FROM ONLY <fktable> WHERE fkatt1 = $1 [AND ...]
+				 *	SELECT 1 FROM ONLY <fktable> WHERE fkatt1 = $1 [AND ...]
 				 * The type id's for the $ parameters are those of the
 				 * corresponding PK attributes. Thus, SPI_prepare could
 				 * eventually fail if the parser cannot identify some way
 				 * how to compare these two types by '='.
 				 * ----------
 				 */
-				sprintf(querystr, "SELECT oid FROM ONLY \"%s\"",
+				sprintf(querystr, "SELECT 1 FROM ONLY \"%s\"",
 						tgargs[RI_FK_RELNAME_ARGNO]);
 				querysep = "WHERE";
 				for (i = 0; i < qkey.nkeypairs; i++)
@@ -834,14 +823,14 @@ RI_FKey_noaction_upd(PG_FUNCTION_ARGS)
 
 				/* ----------
 				 * The query string built is
-				 *	SELECT oid FROM ONLY <fktable> WHERE fkatt1 = $1 [AND ...]
+				 *	SELECT 1 FROM ONLY <fktable> WHERE fkatt1 = $1 [AND ...]
 				 * The type id's for the $ parameters are those of the
 				 * corresponding PK attributes. Thus, SPI_prepare could
 				 * eventually fail if the parser cannot identify some way
 				 * how to compare these two types by '='.
 				 * ----------
 				 */
-				sprintf(querystr, "SELECT oid FROM ONLY \"%s\"",
+				sprintf(querystr, "SELECT 1 FROM ONLY \"%s\"",
 						tgargs[RI_FK_RELNAME_ARGNO]);
 				querysep = "WHERE";
 				for (i = 0; i < qkey.nkeypairs; i++)
@@ -1461,14 +1450,14 @@ RI_FKey_restrict_del(PG_FUNCTION_ARGS)
 
 				/* ----------
 				 * The query string built is
-				 *	SELECT oid FROM ONLY <fktable> WHERE fkatt1 = $1 [AND ...]
+				 *	SELECT 1 FROM ONLY <fktable> WHERE fkatt1 = $1 [AND ...]
 				 * The type id's for the $ parameters are those of the
 				 * corresponding PK attributes. Thus, SPI_prepare could
 				 * eventually fail if the parser cannot identify some way
 				 * how to compare these two types by '='.
 				 * ----------
 				 */
-				sprintf(querystr, "SELECT oid FROM ONLY \"%s\"",
+				sprintf(querystr, "SELECT 1 FROM ONLY \"%s\"",
 						tgargs[RI_FK_RELNAME_ARGNO]);
 				querysep = "WHERE";
 				for (i = 0; i < qkey.nkeypairs; i++)
@@ -1681,14 +1670,14 @@ RI_FKey_restrict_upd(PG_FUNCTION_ARGS)
 
 				/* ----------
 				 * The query string built is
-				 *	SELECT oid FROM ONLY <fktable> WHERE fkatt1 = $1 [AND ...]
+				 *	SELECT 1 FROM ONLY <fktable> WHERE fkatt1 = $1 [AND ...]
 				 * The type id's for the $ parameters are those of the
 				 * corresponding PK attributes. Thus, SPI_prepare could
 				 * eventually fail if the parser cannot identify some way
 				 * how to compare these two types by '='.
 				 * ----------
 				 */
-				sprintf(querystr, "SELECT oid FROM ONLY \"%s\"",
+				sprintf(querystr, "SELECT 1 FROM ONLY \"%s\"",
 						tgargs[RI_FK_RELNAME_ARGNO]);
 				querysep = "WHERE";
 				for (i = 0; i < qkey.nkeypairs; i++)
@@ -2140,7 +2129,6 @@ RI_FKey_setnull_upd(PG_FUNCTION_ARGS)
 				qualsep = "WHERE";
 				for (i = 0; i < qkey.nkeypairs; i++)
 				{
-
 					/*
 					 * MATCH <unspecified> - only change columns
 					 * corresponding to changed columns in pk_rel's key
@@ -2402,7 +2390,6 @@ RI_FKey_setdefault_del(PG_FUNCTION_ARGS)
 					defval = NULL;
 				for (i = 0; i < qkey.nkeypairs && defval != NULL; i++)
 				{
-
 					/*
 					 * For each key attribute lookup the tuple constructor
 					 * for a corresponding default value
@@ -2412,7 +2399,6 @@ RI_FKey_setdefault_del(PG_FUNCTION_ARGS)
 						if (defval[j].adnum ==
 							qkey.keypair[i][RI_KEYPAIR_FK_IDX])
 						{
-
 							/*
 							 * That's the one - push the expression from
 							 * defval.adbin into the plan's targetlist
@@ -2629,7 +2615,6 @@ RI_FKey_setdefault_upd(PG_FUNCTION_ARGS)
 				qualsep = "WHERE";
 				for (i = 0; i < qkey.nkeypairs; i++)
 				{
-
 					/*
 					 * MATCH <unspecified> - only change columns
 					 * corresponding to changed columns in pk_rel's key
@@ -2669,7 +2654,6 @@ RI_FKey_setdefault_upd(PG_FUNCTION_ARGS)
 					defval = NULL;
 				for (i = 0; i < qkey.nkeypairs && defval != NULL; i++)
 				{
-
 					/*
 					 * MATCH <unspecified> - only change columns
 					 * corresponding to changed columns in pk_rel's key.
@@ -2680,7 +2664,6 @@ RI_FKey_setdefault_upd(PG_FUNCTION_ARGS)
 						!ri_OneKeyEqual(pk_rel, i, old_row,
 									  new_row, &qkey, RI_KEYPAIR_PK_IDX))
 					{
-
 						/*
 						 * For each key attribute lookup the tuple
 						 * constructor for a corresponding default value
@@ -2690,7 +2673,6 @@ RI_FKey_setdefault_upd(PG_FUNCTION_ARGS)
 							if (defval[j].adnum ==
 								qkey.keypair[i][RI_KEYPAIR_FK_IDX])
 							{
-
 								/*
 								 * That's the one - push the expression
 								 * from defval.adbin into the plan's
@@ -2807,7 +2789,6 @@ RI_FKey_keyequal_upd(TriggerData *trigdata)
 
 	switch (ri_DetermineMatchType(tgargs[RI_MATCH_TYPE_ARGNO]))
 	{
-
 			/*
 			 * MATCH <UNSPECIFIED>
 			 */
@@ -2988,15 +2969,16 @@ ri_InitHashTables(void)
 
 	memset(&ctl, 0, sizeof(ctl));
 	ctl.keysize = sizeof(RI_QueryKey);
-	ctl.datasize = sizeof(void *);
-	ri_query_cache = hash_create(RI_INIT_QUERYHASHSIZE, &ctl, HASH_ELEM);
-
-	memset(&ctl, 0, sizeof(ctl));
-	ctl.keysize = sizeof(Oid);
-	ctl.datasize = sizeof(Oid) + sizeof(FmgrInfo);
+	ctl.entrysize = sizeof(RI_QueryHashEntry);
 	ctl.hash = tag_hash;
-	ri_opreq_cache = hash_create(RI_INIT_OPREQHASHSIZE, &ctl,
-								 HASH_ELEM | HASH_FUNCTION);
+	ri_query_cache = hash_create("RI query cache", RI_INIT_QUERYHASHSIZE,
+								 &ctl, HASH_ELEM | HASH_FUNCTION);
+
+	ctl.keysize = sizeof(Oid);
+	ctl.entrysize = sizeof(RI_OpreqHashEntry);
+	ctl.hash = tag_hash;
+	ri_opreq_cache = hash_create("RI OpReq cache", RI_INIT_OPREQHASHSIZE,
+								 &ctl, HASH_ELEM | HASH_FUNCTION);
 }
 
 
@@ -3011,7 +2993,6 @@ static void *
 ri_FetchPreparedPlan(RI_QueryKey *key)
 {
 	RI_QueryHashEntry *entry;
-	bool		found;
 
 	/*
 	 * On the first call initialize the hashtable
@@ -3023,10 +3004,9 @@ ri_FetchPreparedPlan(RI_QueryKey *key)
 	 * Lookup for the key
 	 */
 	entry = (RI_QueryHashEntry *) hash_search(ri_query_cache,
-										(char *) key, HASH_FIND, &found);
+											  (void *) key,
+											  HASH_FIND, NULL);
 	if (entry == NULL)
-		elog(FATAL, "error in RI plan cache");
-	if (!found)
 		return NULL;
 	return entry->plan;
 }
@@ -3054,9 +3034,10 @@ ri_HashPreparedPlan(RI_QueryKey *key, void *plan)
 	 * Add the new plan.
 	 */
 	entry = (RI_QueryHashEntry *) hash_search(ri_query_cache,
-									   (char *) key, HASH_ENTER, &found);
+											  (void *) key,
+											  HASH_ENTER, &found);
 	if (entry == NULL)
-		elog(FATAL, "can't insert into RI plan cache");
+		elog(ERROR, "out of memory for RI plan cache");
 	entry->plan = plan;
 }
 
@@ -3079,7 +3060,6 @@ ri_KeysEqual(Relation rel, HeapTuple oldtup, HeapTuple newtup,
 
 	for (i = 0; i < key->nkeypairs; i++)
 	{
-
 		/*
 		 * Get one attributes oldvalue. If it is NULL - they're not equal.
 		 */
@@ -3129,7 +3109,6 @@ ri_AllKeysUnequal(Relation rel, HeapTuple oldtup, HeapTuple newtup,
 	keys_unequal = true;
 	for (i = 0; keys_unequal && i < key->nkeypairs; i++)
 	{
-
 		/*
 		 * Get one attributes oldvalue. If it is NULL - they're not equal.
 		 */
@@ -3224,27 +3203,25 @@ ri_AttributesEqual(Oid typeid, Datum oldvalue, Datum newvalue)
 	/*
 	 * On the first call initialize the hashtable
 	 */
-	if (!ri_query_cache)
+	if (!ri_opreq_cache)
 		ri_InitHashTables();
 
 	/*
 	 * Try to find the '=' operator for this type in our cache
 	 */
 	entry = (RI_OpreqHashEntry *) hash_search(ri_opreq_cache,
-									(char *) &typeid, HASH_FIND, &found);
-	if (entry == NULL)
-		elog(FATAL, "error in RI operator cache");
+											  (void *) &typeid,
+											  HASH_FIND, NULL);
 
 	/*
 	 * If not found, lookup the OPERNAME system cache for it to get the
 	 * func OID, then do the function manager lookup, and remember that
 	 * info.
 	 */
-	if (!found)
+	if (!entry)
 	{
 		HeapTuple	opr_tup;
 		Oid			opr_proc;
-		MemoryContext	oldcontext;
 		FmgrInfo	finfo;
 
 		opr_tup = SearchSysCache(OPERNAME,
@@ -3254,28 +3231,25 @@ ri_AttributesEqual(Oid typeid, Datum oldvalue, Datum newvalue)
 								 CharGetDatum('b'));
 		if (!HeapTupleIsValid(opr_tup))
 			elog(ERROR,
-				 "ri_AttributesEqual(): cannot find '=' operator for type %u",
+			"ri_AttributesEqual(): cannot find '=' operator for type %u",
 				 typeid);
 		opr_proc = ((Form_pg_operator) GETSTRUCT(opr_tup))->oprcode;
 		ReleaseSysCache(opr_tup);
 
 		/*
 		 * Since fmgr_info could fail, call it *before* creating the
-		 * hashtable entry --- otherwise we could elog leaving an incomplete
-		 * entry in the hashtable.  Also, because this will be a permanent
-		 * table entry, we must make sure any subsidiary structures of the
-		 * fmgr record are kept in TopMemoryContext.
+		 * hashtable entry --- otherwise we could elog leaving an
+		 * incomplete entry in the hashtable.  Also, because this will be
+		 * a permanent table entry, we must make sure any subsidiary
+		 * structures of the fmgr record are kept in TopMemoryContext.
 		 */
-		oldcontext = MemoryContextSwitchTo(TopMemoryContext);
-		fmgr_info(opr_proc, &finfo);
-		MemoryContextSwitchTo(oldcontext);
+		fmgr_info_cxt(opr_proc, &finfo, TopMemoryContext);
 
 		entry = (RI_OpreqHashEntry *) hash_search(ri_opreq_cache,
-												  (char *) &typeid,
-												  HASH_ENTER,
-												  &found);
+												  (void *) &typeid,
+												  HASH_ENTER, &found);
 		if (entry == NULL)
-			elog(FATAL, "can't insert into RI operator cache");
+			elog(ERROR, "out of memory for RI operator cache");
 
 		entry->typeid = typeid;
 		memcpy(&(entry->oprfmgrinfo), &finfo, sizeof(FmgrInfo));
