@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/utils/adt/varlena.c,v 1.70 2001/05/03 19:00:36 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/utils/adt/varlena.c,v 1.78 2001/11/19 19:15:07 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -53,14 +53,22 @@ byteain(PG_FUNCTION_ARGS)
 
 	for (byte = 0, tp = inputText; *tp != '\0'; byte++)
 	{
-		if (*tp++ == '\\')
+		if (tp[0] != '\\')
+			tp++;
+		else if ((tp[0] == '\\') &&
+				 (tp[1] >= '0' && tp[1] <= '3') &&
+				 (tp[2] >= '0' && tp[2] <= '7') &&
+				 (tp[3] >= '0' && tp[3] <= '7'))
+			tp += 4;
+		else if ((tp[0] == '\\') &&
+				 (tp[1] == '\\'))
+			tp += 2;
+		else
 		{
-			if (*tp == '\\')
-				tp++;
-			else if (!isdigit((unsigned char) *tp++) ||
-					 !isdigit((unsigned char) *tp++) ||
-					 !isdigit((unsigned char) *tp++))
-				elog(ERROR, "Bad input string for type bytea");
+			/*
+			 * one backslash, not followed by 0 or ### valid octal
+			 */
+			elog(ERROR, "Bad input string for type bytea");
 		}
 	}
 
@@ -72,15 +80,33 @@ byteain(PG_FUNCTION_ARGS)
 	rp = result->vl_dat;
 	while (*tp != '\0')
 	{
-		if (*tp != '\\' || *++tp == '\\')
+		if (tp[0] != '\\')
 			*rp++ = *tp++;
+		else if ((tp[0] == '\\') &&
+				 (tp[1] >= '0' && tp[1] <= '3') &&
+				 (tp[2] >= '0' && tp[2] <= '7') &&
+				 (tp[3] >= '0' && tp[3] <= '7'))
+		{
+			byte = VAL(tp[1]);
+			byte <<= 3;
+			byte += VAL(tp[2]);
+			byte <<= 3;
+			*rp++ = byte + VAL(tp[3]);
+			tp += 4;
+		}
+		else if ((tp[0] == '\\') &&
+				 (tp[1] == '\\'))
+		{
+			*rp++ = '\\';
+			tp += 2;
+		}
 		else
 		{
-			byte = VAL(*tp++);
-			byte <<= 3;
-			byte += VAL(*tp++);
-			byte <<= 3;
-			*rp++ = byte + VAL(*tp++);
+			/*
+			 * We should never get here. The first pass should not allow
+			 * it.
+			 */
+			elog(ERROR, "Bad input string for type bytea");
 		}
 	}
 
@@ -155,7 +181,17 @@ textin(PG_FUNCTION_ARGS)
 	text	   *result;
 	int			len;
 
+#ifdef MULTIBYTE
+	char	   *ermsg;
+#endif
+
 	len = strlen(inputText) + VARHDRSZ;
+
+#ifdef MULTIBYTE
+	if ((ermsg = pg_verifymbstr(inputText, len - VARHDRSZ)))
+		elog(ERROR, "%s", ermsg);
+#endif
+
 	result = (text *) palloc(len);
 	VARATT_SIZEP(result) = len;
 
@@ -204,22 +240,13 @@ textlen(PG_FUNCTION_ARGS)
 	text	   *t = PG_GETARG_TEXT_P(0);
 
 #ifdef MULTIBYTE
-	unsigned char *s;
-	int			len,
-				l,
-				wl;
+	/* optimization for single byte encoding */
+	if (pg_database_encoding_max_length() <= 1)
+		PG_RETURN_INT32(VARSIZE(t) - VARHDRSZ);
 
-	len = 0;
-	s = VARDATA(t);
-	l = VARSIZE(t) - VARHDRSZ;
-	while (l > 0)
-	{
-		wl = pg_mblen(s);
-		l -= wl;
-		s += wl;
-		len++;
-	}
-	PG_RETURN_INT32(len);
+	PG_RETURN_INT32(
+		pg_mbstrlen_with_len(VARDATA(t), VARSIZE(t) - VARHDRSZ)
+		);
 #else
 	PG_RETURN_INT32(VARSIZE(t) - VARHDRSZ);
 #endif
@@ -229,20 +256,13 @@ textlen(PG_FUNCTION_ARGS)
  * textoctetlen -
  *	  returns the physical length of a text*
  *	   (which is less than the VARSIZE of the text*)
- *
- * XXX is it actually appropriate to return the compressed length
- * when the value is compressed?  It's not at all clear to me that
- * this is what SQL92 has in mind ...
  */
 Datum
 textoctetlen(PG_FUNCTION_ARGS)
 {
-	struct varattrib *t = (struct varattrib *) PG_GETARG_RAW_VARLENA_P(0);
+	text    *arg = PG_GETARG_TEXT_P(0);
 
-	if (!VARATT_IS_EXTERNAL(t))
-		PG_RETURN_INT32(VARATT_SIZE(t) - VARHDRSZ);
-
-	PG_RETURN_INT32(t->va_content.va_external.va_extsize);
+	PG_RETURN_INT32(VARSIZE(arg) - VARHDRSZ);
 }
 
 /*
@@ -301,7 +321,7 @@ textcat(PG_FUNCTION_ARGS)
  *	- string length
  *
  * If the starting position is zero or less, then return from the start of the string
- *	adjusting the length to be consistant with the "negative start" per SQL92.
+ *	adjusting the length to be consistent with the "negative start" per SQL92.
  * If the length is less than zero, return the remaining string.
  *
  * Note that the arguments operate on octet length,
@@ -325,7 +345,6 @@ text_substr(PG_FUNCTION_ARGS)
 #ifdef MULTIBYTE
 	int			i;
 	char	   *p;
-
 #endif
 
 	len = VARSIZE(string) - VARHDRSZ;
@@ -399,7 +418,6 @@ textpos(PG_FUNCTION_ARGS)
 #ifdef MULTIBYTE
 	pg_wchar   *ps1,
 			   *ps2;
-
 #endif
 
 	if (VARSIZE(t2) <= VARHDRSZ)
@@ -662,6 +680,147 @@ byteaoctetlen(PG_FUNCTION_ARGS)
 	PG_RETURN_INT32(VARSIZE(v) - VARHDRSZ);
 }
 
+/*
+ * byteacat -
+ *	  takes two bytea* and returns a bytea* that is the concatenation of
+ *	  the two.
+ *
+ * Cloned from textcat and modified as required.
+ */
+Datum
+byteacat(PG_FUNCTION_ARGS)
+{
+	bytea	   *t1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *t2 = PG_GETARG_BYTEA_P(1);
+	int			len1,
+				len2,
+				len;
+	bytea	   *result;
+	char	   *ptr;
+
+	len1 = (VARSIZE(t1) - VARHDRSZ);
+	if (len1 < 0)
+		len1 = 0;
+
+	len2 = (VARSIZE(t2) - VARHDRSZ);
+	if (len2 < 0)
+		len2 = 0;
+
+	len = len1 + len2 + VARHDRSZ;
+	result = (bytea *) palloc(len);
+
+	/* Set size of result string... */
+	VARATT_SIZEP(result) = len;
+
+	/* Fill data field of result string... */
+	ptr = VARDATA(result);
+	if (len1 > 0)
+		memcpy(ptr, VARDATA(t1), len1);
+	if (len2 > 0)
+		memcpy(ptr + len1, VARDATA(t2), len2);
+
+	PG_RETURN_BYTEA_P(result);
+}
+
+/*
+ * bytea_substr()
+ * Return a substring starting at the specified position.
+ * Cloned from text_substr and modified as required.
+ *
+ * Input:
+ *	- string
+ *	- starting position (is one-based)
+ *	- string length
+ *
+ * If the starting position is zero or less, then return from the start of the string
+ * adjusting the length to be consistent with the "negative start" per SQL92.
+ * If the length is less than zero, return the remaining string.
+ *
+ */
+Datum
+bytea_substr(PG_FUNCTION_ARGS)
+{
+	bytea	   *string = PG_GETARG_BYTEA_P(0);
+	int32		m = PG_GETARG_INT32(1);
+	int32		n = PG_GETARG_INT32(2);
+	bytea	   *ret;
+	int			len;
+
+	len = VARSIZE(string) - VARHDRSZ;
+
+	/* starting position after the end of the string? */
+	if (m > len)
+	{
+		m = 1;
+		n = 0;
+	}
+
+	/*
+	 * starting position before the start of the string? then offset into
+	 * the string per SQL92 spec...
+	 */
+	else if (m < 1)
+	{
+		n += (m - 1);
+		m = 1;
+	}
+
+	/* m will now become a zero-based starting position */
+	m--;
+	if (((m + n) > len) || (n < 0))
+		n = (len - m);
+
+	ret = (bytea *) palloc(VARHDRSZ + n);
+	VARATT_SIZEP(ret) = VARHDRSZ + n;
+
+	memcpy(VARDATA(ret), VARDATA(string) + m, n);
+
+	PG_RETURN_BYTEA_P(ret);
+}
+
+/*
+ * byteapos -
+ *	  Return the position of the specified substring.
+ *	  Implements the SQL92 POSITION() function.
+ * Cloned from textpos and modified as required.
+ */
+Datum
+byteapos(PG_FUNCTION_ARGS)
+{
+	bytea	   *t1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *t2 = PG_GETARG_BYTEA_P(1);
+	int			pos;
+	int			px,
+				p;
+	int			len1,
+				len2;
+	char	   *p1,
+			   *p2;
+
+	if (VARSIZE(t2) <= VARHDRSZ)
+		PG_RETURN_INT32(1);		/* result for empty pattern */
+
+	len1 = (VARSIZE(t1) - VARHDRSZ);
+	len2 = (VARSIZE(t2) - VARHDRSZ);
+
+	p1 = VARDATA(t1);
+	p2 = VARDATA(t2);
+
+	pos = 0;
+	px = (len1 - len2);
+	for (p = 0; p <= px; p++)
+	{
+		if ((*p2 == *p1) && (memcmp(p1, p2, len2) == 0))
+		{
+			pos = p + 1;
+			break;
+		};
+		p1++;
+	};
+
+	PG_RETURN_INT32(pos);
+}
+
 /*-------------------------------------------------------------
  * byteaGetByte
  *
@@ -875,3 +1034,162 @@ name_text(PG_FUNCTION_ARGS)
 
 	PG_RETURN_TEXT_P(result);
 }
+
+
+/*****************************************************************************
+ *	Comparison Functions used for bytea
+ *
+ * Note: btree indexes need these routines not to leak memory; therefore,
+ * be careful to free working copies of toasted datums.  Most places don't
+ * need to be so careful.
+ *****************************************************************************/
+
+Datum
+byteaeq(PG_FUNCTION_ARGS)
+{
+	bytea	   *arg1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *arg2 = PG_GETARG_BYTEA_P(1);
+	int			len1,
+				len2;
+	bool		result;
+
+	len1 = VARSIZE(arg1) - VARHDRSZ;
+	len2 = VARSIZE(arg2) - VARHDRSZ;
+
+	/* fast path for different-length inputs */
+	if (len1 != len2)
+		result = false;
+	else
+		result = (memcmp(VARDATA(arg1), VARDATA(arg2), len1) == 0);
+
+	PG_FREE_IF_COPY(arg1, 0);
+	PG_FREE_IF_COPY(arg2, 1);
+
+	PG_RETURN_BOOL(result);
+}
+
+Datum
+byteane(PG_FUNCTION_ARGS)
+{
+	bytea	   *arg1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *arg2 = PG_GETARG_BYTEA_P(1);
+	int			len1,
+				len2;
+	bool		result;
+
+	len1 = VARSIZE(arg1) - VARHDRSZ;
+	len2 = VARSIZE(arg2) - VARHDRSZ;
+
+	/* fast path for different-length inputs */
+	if (len1 != len2)
+		result = true;
+	else
+		result = (memcmp(VARDATA(arg1), VARDATA(arg2), len1) != 0);
+
+	PG_FREE_IF_COPY(arg1, 0);
+	PG_FREE_IF_COPY(arg2, 1);
+
+	PG_RETURN_BOOL(result);
+}
+
+Datum
+bytealt(PG_FUNCTION_ARGS)
+{
+	bytea	   *arg1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *arg2 = PG_GETARG_BYTEA_P(1);
+	int			len1,
+				len2;
+	int			cmp;
+
+	len1 = VARSIZE(arg1) - VARHDRSZ;
+	len2 = VARSIZE(arg2) - VARHDRSZ;
+
+	cmp = memcmp(VARDATA(arg1), VARDATA(arg2), Min(len1, len2));
+
+	PG_FREE_IF_COPY(arg1, 0);
+	PG_FREE_IF_COPY(arg2, 1);
+
+	PG_RETURN_BOOL((cmp < 0) || ((cmp == 0) && (len1 < len2)));
+}
+
+Datum
+byteale(PG_FUNCTION_ARGS)
+{
+	bytea	   *arg1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *arg2 = PG_GETARG_BYTEA_P(1);
+	int			len1,
+				len2;
+	int			cmp;
+
+	len1 = VARSIZE(arg1) - VARHDRSZ;
+	len2 = VARSIZE(arg2) - VARHDRSZ;
+
+	cmp = memcmp(VARDATA(arg1), VARDATA(arg2), Min(len1, len2));
+
+	PG_FREE_IF_COPY(arg1, 0);
+	PG_FREE_IF_COPY(arg2, 1);
+
+	PG_RETURN_BOOL((cmp < 0) || ((cmp == 0) && (len1 <= len2)));
+}
+
+Datum
+byteagt(PG_FUNCTION_ARGS)
+{
+	bytea	   *arg1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *arg2 = PG_GETARG_BYTEA_P(1);
+	int			len1,
+				len2;
+	int			cmp;
+
+	len1 = VARSIZE(arg1) - VARHDRSZ;
+	len2 = VARSIZE(arg2) - VARHDRSZ;
+
+	cmp = memcmp(VARDATA(arg1), VARDATA(arg2), Min(len1, len2));
+
+	PG_FREE_IF_COPY(arg1, 0);
+	PG_FREE_IF_COPY(arg2, 1);
+
+	PG_RETURN_BOOL((cmp > 0) || ((cmp == 0) && (len1 > len2)));
+}
+
+Datum
+byteage(PG_FUNCTION_ARGS)
+{
+	bytea	   *arg1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *arg2 = PG_GETARG_BYTEA_P(1);
+	int			len1,
+				len2;
+	int			cmp;
+
+	len1 = VARSIZE(arg1) - VARHDRSZ;
+	len2 = VARSIZE(arg2) - VARHDRSZ;
+
+	cmp = memcmp(VARDATA(arg1), VARDATA(arg2), Min(len1, len2));
+
+	PG_FREE_IF_COPY(arg1, 0);
+	PG_FREE_IF_COPY(arg2, 1);
+
+	PG_RETURN_BOOL((cmp > 0) || ((cmp == 0) && (len1 >= len2)));
+}
+
+Datum
+byteacmp(PG_FUNCTION_ARGS)
+{
+	bytea	   *arg1 = PG_GETARG_BYTEA_P(0);
+	bytea	   *arg2 = PG_GETARG_BYTEA_P(1);
+	int			len1,
+				len2;
+	int			cmp;
+
+	len1 = VARSIZE(arg1) - VARHDRSZ;
+	len2 = VARSIZE(arg2) - VARHDRSZ;
+
+	cmp = memcmp(VARDATA(arg1), VARDATA(arg2), Min(len1, len2));
+	if ((cmp == 0) && (len1 != len2))
+		cmp = (len1 < len2) ? -1 : 1;
+
+	PG_FREE_IF_COPY(arg1, 0);
+	PG_FREE_IF_COPY(arg2, 1);
+
+	PG_RETURN_INT32(cmp);
+}
