@@ -78,7 +78,7 @@
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/backend/utils/sort/tuplesort.c,v 1.15 2001/03/23 04:49:55 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/utils/sort/tuplesort.c,v 1.22 2002/01/06 00:37:44 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -87,9 +87,15 @@
 
 #include "access/heapam.h"
 #include "access/nbtree.h"
+#include "catalog/catname.h"
+#include "catalog/pg_amop.h"
+#include "catalog/pg_amproc.h"
+#include "catalog/pg_operator.h"
 #include "miscadmin.h"
+#include "utils/fmgroids.h"
 #include "utils/logtape.h"
 #include "utils/lsyscache.h"
+#include "utils/syscache.h"
 #include "utils/tuplesort.h"
 
 /*
@@ -129,10 +135,9 @@ struct Tuplesortstate
 	 * kind of tuple we are sorting from the routines that don't need to
 	 * know it. They are set up by the tuplesort_begin_xxx routines.
 	 *
-	 * Function to compare two tuples; result is per qsort() convention,
-	 * ie:
+	 * Function to compare two tuples; result is per qsort() convention, ie:
 	 *
-	 * 	<0, 0, >0 according as a<b, a=b, a>b.
+	 * <0, 0, >0 according as a<b, a=b, a>b.
 	 */
 	int			(*comparetup) (Tuplesortstate *state, const void *a, const void *b);
 
@@ -263,6 +268,7 @@ struct Tuplesortstate
 	TupleDesc	tupDesc;
 	int			nKeys;
 	ScanKey		scanKeys;
+	SortFunctionKind *sortFnKinds;
 
 	/*
 	 * These variables are specific to the IndexTuple case; they are set
@@ -279,6 +285,7 @@ struct Tuplesortstate
 	Oid			datumType;
 	Oid			sortOperator;
 	FmgrInfo	sortOpFn;		/* cached lookup data for sortOperator */
+	SortFunctionKind sortFnKind;
 	/* we need typelen and byval in order to know how to copy the Datums. */
 	int			datumTypeLen;
 	bool		datumTypeByVal;
@@ -458,14 +465,14 @@ tuplesort_begin_common(bool randomAccess)
 
 Tuplesortstate *
 tuplesort_begin_heap(TupleDesc tupDesc,
-					 int nkeys, ScanKey keys,
+					 int nkeys,
+					 Oid *sortOperators, AttrNumber *attNums,
 					 bool randomAccess)
 {
 	Tuplesortstate *state = tuplesort_begin_common(randomAccess);
+	int			i;
 
-	AssertArg(nkeys >= 1);
-	AssertArg(keys[0].sk_attno != 0);
-	AssertArg(keys[0].sk_procedure != 0);
+	AssertArg(nkeys > 0);
 
 	state->comparetup = comparetup_heap;
 	state->copytup = copytup_heap;
@@ -475,7 +482,29 @@ tuplesort_begin_heap(TupleDesc tupDesc,
 
 	state->tupDesc = tupDesc;
 	state->nKeys = nkeys;
-	state->scanKeys = keys;
+	state->scanKeys = (ScanKey) palloc(nkeys * sizeof(ScanKeyData));
+	MemSet(state->scanKeys, 0, nkeys * sizeof(ScanKeyData));
+	state->sortFnKinds = (SortFunctionKind *)
+		palloc(nkeys * sizeof(SortFunctionKind));
+	MemSet(state->sortFnKinds, 0, nkeys * sizeof(SortFunctionKind));
+
+	for (i = 0; i < nkeys; i++)
+	{
+		RegProcedure sortFunction;
+
+		AssertArg(sortOperators[i] != 0);
+		AssertArg(attNums[i] != 0);
+
+		/* select a function that implements the sort operator */
+		SelectSortFunction(sortOperators[i], &sortFunction,
+						   &state->sortFnKinds[i]);
+
+		ScanKeyEntryInitialize(&state->scanKeys[i],
+							   0x0,
+							   attNums[i],
+							   sortFunction,
+							   (Datum) 0);
+	}
 
 	return state;
 }
@@ -507,6 +536,7 @@ tuplesort_begin_datum(Oid datumType,
 					  bool randomAccess)
 {
 	Tuplesortstate *state = tuplesort_begin_common(randomAccess);
+	RegProcedure sortFunction;
 	int16		typlen;
 	bool		typbyval;
 
@@ -518,8 +548,12 @@ tuplesort_begin_datum(Oid datumType,
 
 	state->datumType = datumType;
 	state->sortOperator = sortOperator;
-	/* lookup the function that implements the sort operator */
-	fmgr_info(get_opcode(sortOperator), &state->sortOpFn);
+
+	/* select a function that implements the sort operator */
+	SelectSortFunction(sortOperator, &sortFunction, &state->sortFnKind);
+	/* and look up the function */
+	fmgr_info(sortFunction, &state->sortOpFn);
+
 	/* lookup necessary attributes of the datum type */
 	get_typlenbyval(datumType, &typlen, &typbyval);
 	state->datumTypeLen = typlen;
@@ -548,6 +582,16 @@ tuplesort_end(Tuplesortstate *state)
 	}
 	if (state->memtupindex)
 		pfree(state->memtupindex);
+
+	/*
+	 * this stuff might better belong in a variant-specific shutdown
+	 * routine
+	 */
+	if (state->scanKeys)
+		pfree(state->scanKeys);
+	if (state->sortFnKinds)
+		pfree(state->sortFnKinds);
+
 	pfree(state);
 }
 
@@ -559,7 +603,6 @@ tuplesort_end(Tuplesortstate *state)
 void
 tuplesort_puttuple(Tuplesortstate *state, void *tuple)
 {
-
 	/*
 	 * Copy the given tuple into memory we control, and decrease availMem.
 	 * Then call the code shared with the Datum case.
@@ -618,7 +661,7 @@ puttuple_common(Tuplesortstate *state, void *tuple)
 {
 	switch (state->status)
 	{
-			case TSS_INITIAL:
+		case TSS_INITIAL:
 
 			/*
 			 * Save the copied tuple into the unsorted array.
@@ -690,7 +733,7 @@ tuplesort_performsort(Tuplesortstate *state)
 {
 	switch (state->status)
 	{
-			case TSS_INITIAL:
+		case TSS_INITIAL:
 
 			/*
 			 * We were able to accumulate all the tuples within the
@@ -801,7 +844,6 @@ tuplesort_gettuple(Tuplesortstate *state, bool forward,
 			 */
 			if (state->eof_reached)
 			{
-
 				/*
 				 * Seek position is pointing just past the zero tuplen at
 				 * the end of file; back up to fetch last tuple's ending
@@ -816,7 +858,6 @@ tuplesort_gettuple(Tuplesortstate *state, bool forward,
 			}
 			else
 			{
-
 				/*
 				 * Back up and fetch previously-returned tuple's ending
 				 * length word.  If seek fails, assume we are at start of
@@ -835,7 +876,6 @@ tuplesort_gettuple(Tuplesortstate *state, bool forward,
 										  state->result_tape,
 									  tuplen + 2 * sizeof(unsigned int)))
 				{
-
 					/*
 					 * If that fails, presumably the prev tuple is the
 					 * first in the file.  Back up so that it becomes next
@@ -886,7 +926,6 @@ tuplesort_gettuple(Tuplesortstate *state, bool forward,
 				tuplesort_heap_siftup(state, false);
 				if ((tupIndex = state->mergenext[srcTape]) == 0)
 				{
-
 					/*
 					 * out of preloaded data on this tape, try to read
 					 * more
@@ -1194,6 +1233,7 @@ mergeonerun(Tuplesortstate *state)
 	 */
 	while (state->memtupcount > 0)
 	{
+		CHECK_FOR_INTERRUPTS();
 		/* write the tuple to destTape */
 		priorAvail = state->availMem;
 		srcTape = state->memtupindex[0];
@@ -1415,7 +1455,6 @@ dumptuples(Tuplesortstate *state, bool alltuples)
 	while (alltuples ||
 		   (LACKMEM(state) && state->memtupcount > 1))
 	{
-
 		/*
 		 * Dump the heap's frontmost entry, and sift up to remove it from
 		 * the heap.
@@ -1693,38 +1732,24 @@ comparetup_heap(Tuplesortstate *state, const void *a, const void *b)
 	{
 		ScanKey		scanKey = state->scanKeys + nkey;
 		AttrNumber	attno = scanKey->sk_attno;
-		Datum		lattr,
-					rattr;
+		Datum		datum1,
+					datum2;
 		bool		isnull1,
 					isnull2;
+		int32		compare;
 
-		lattr = heap_getattr(ltup, attno, tupDesc, &isnull1);
-		rattr = heap_getattr(rtup, attno, tupDesc, &isnull2);
+		datum1 = heap_getattr(ltup, attno, tupDesc, &isnull1);
+		datum2 = heap_getattr(rtup, attno, tupDesc, &isnull2);
 
-		if (isnull1)
+		compare = ApplySortFunction(&scanKey->sk_func,
+									state->sortFnKinds[nkey],
+									datum1, isnull1, datum2, isnull2);
+		if (compare != 0)
 		{
-			if (!isnull2)
-				return 1;		/* NULL sorts after non-NULL */
-		}
-		else if (isnull2)
-			return -1;
-		else if (scanKey->sk_flags & SK_COMMUTE)
-		{
-			if (DatumGetBool(FunctionCall2(&scanKey->sk_func,
-										   rattr, lattr)))
-				return -1;		/* a < b after commute */
-			if (DatumGetBool(FunctionCall2(&scanKey->sk_func,
-										   lattr, rattr)))
-				return 1;		/* a > b after commute */
-		}
-		else
-		{
-			if (DatumGetBool(FunctionCall2(&scanKey->sk_func,
-										   lattr, rattr)))
-				return -1;		/* a < b */
-			if (DatumGetBool(FunctionCall2(&scanKey->sk_func,
-										   rattr, lattr)))
-				return 1;		/* a > b */
+			/* dead code? SK_COMMUTE can't actually be set here, can it? */
+			if (scanKey->sk_flags & SK_COMMUTE)
+				compare = -compare;
+			return compare;
 		}
 	}
 
@@ -1807,7 +1832,6 @@ tuplesize_heap(Tuplesortstate *state, void *tup)
 static int
 comparetup_index(Tuplesortstate *state, const void *a, const void *b)
 {
-
 	/*
 	 * This is almost the same as _bt_tuplecompare(), but we need to keep
 	 * track of whether any null fields are present.
@@ -1826,39 +1850,28 @@ comparetup_index(Tuplesortstate *state, const void *a, const void *b)
 	for (i = 1; i <= keysz; i++)
 	{
 		ScanKey		entry = &scankey[i - 1];
-		Datum		attrDatum1,
-					attrDatum2;
-		bool		isFirstNull,
-					isSecondNull;
+		Datum		datum1,
+					datum2;
+		bool		isnull1,
+					isnull2;
 		int32		compare;
 
-		attrDatum1 = index_getattr(tuple1, i, tupDes, &isFirstNull);
-		attrDatum2 = index_getattr(tuple2, i, tupDes, &isSecondNull);
+		datum1 = index_getattr(tuple1, i, tupDes, &isnull1);
+		datum2 = index_getattr(tuple2, i, tupDes, &isnull2);
 
 		/* see comments about NULLs handling in btbuild */
-		if (isFirstNull)		/* attr in tuple1 is NULL */
-		{
-			if (isSecondNull)	/* attr in tuple2 is NULL too */
-			{
-				compare = 0;
-				equal_hasnull = true;
-			}
-			else
-				compare = 1;	/* NULL ">" not-NULL */
-		}
-		else if (isSecondNull)	/* attr in tuple1 is NOT_NULL and */
-		{						/* attr in tuple2 is NULL */
-			compare = -1;		/* not-NULL "<" NULL */
-		}
-		else
-		{
-			compare = DatumGetInt32(FunctionCall2(&entry->sk_func,
-												attrDatum1, attrDatum2));
-		}
+
+		/* the comparison function is always of CMP type */
+		compare = ApplySortFunction(&entry->sk_func, SORTFUNC_CMP,
+									datum1, isnull1, datum2, isnull2);
 
 		if (compare != 0)
 			return (int) compare;		/* done when we find unequal
 										 * attributes */
+
+		/* they are equal, so we only need to examine one null flag */
+		if (isnull1)
+			equal_hasnull = true;
 	}
 
 	/*
@@ -1869,8 +1882,13 @@ comparetup_index(Tuplesortstate *state, const void *a, const void *b)
 	 * equal they *must* get compared at some stage of the sort ---
 	 * otherwise the sort algorithm wouldn't have checked whether one must
 	 * appear before the other.
+	 *
+	 * Some rather brain-dead implementations of qsort will sometimes
+	 * call the comparison routine to compare a value to itself.  (At this
+	 * writing only QNX 4 is known to do such silly things.)  Don't raise
+	 * a bogus error in that case.
 	 */
-	if (state->enforceUnique && !equal_hasnull)
+	if (state->enforceUnique && !equal_hasnull && tuple1 != tuple2)
 		elog(ERROR, "Cannot create unique index. Table contains non-unique values");
 
 	return 0;
@@ -1946,24 +1964,9 @@ comparetup_datum(Tuplesortstate *state, const void *a, const void *b)
 	DatumTuple *ltup = (DatumTuple *) a;
 	DatumTuple *rtup = (DatumTuple *) b;
 
-	if (ltup->isNull)
-	{
-		if (!rtup->isNull)
-			return 1;			/* NULL sorts after non-NULL */
-		return 0;
-	}
-	else if (rtup->isNull)
-		return -1;
-	else
-	{
-		if (DatumGetBool(FunctionCall2(&state->sortOpFn,
-									   ltup->val, rtup->val)))
-			return -1;			/* a < b */
-		if (DatumGetBool(FunctionCall2(&state->sortOpFn,
-									   rtup->val, ltup->val)))
-			return 1;			/* a > b */
-		return 0;
-	}
+	return ApplySortFunction(&state->sortOpFn, state->sortFnKind,
+							 ltup->val, ltup->isNull,
+							 rtup->val, rtup->isNull);
 }
 
 static void *
@@ -2032,3 +2035,182 @@ tuplesize_datum(Tuplesortstate *state, void *tup)
 		return (unsigned int) tuplelen;
 	}
 }
+
+
+/*
+ * This routine selects an appropriate sorting function to implement
+ * a sort operator as efficiently as possible.	The straightforward
+ * method is to use the operator's implementation proc --- ie, "<"
+ * comparison.	However, that way often requires two calls of the function
+ * per comparison.	If we can find a btree three-way comparator function
+ * associated with the operator, we can use it to do the comparisons
+ * more efficiently.  We also support the possibility that the operator
+ * is ">" (descending sort), in which case we have to reverse the output
+ * of the btree comparator.
+ *
+ * Possibly this should live somewhere else (backend/catalog/, maybe?).
+ */
+void
+SelectSortFunction(Oid sortOperator,
+				   RegProcedure *sortFunction,
+				   SortFunctionKind *kind)
+{
+	Relation	relation;
+	HeapScanDesc scan;
+	ScanKeyData skey[1];
+	HeapTuple	tuple;
+	Form_pg_operator optup;
+	Oid			opclass = InvalidOid;
+
+	/*
+	 * Scan pg_amop to see if the target operator is registered as the "<"
+	 * or ">" operator of any btree opclass.  It's possible that it might
+	 * be registered both ways (eg, if someone were to build a "reverse
+	 * sort" opclass for some reason); prefer the "<" case if so. If the
+	 * operator is registered the same way in multiple opclasses, assume
+	 * we can use the associated comparator function from any one.
+	 */
+	ScanKeyEntryInitialize(&skey[0], 0x0,
+						   Anum_pg_amop_amopopr,
+						   F_OIDEQ,
+						   ObjectIdGetDatum(sortOperator));
+
+	relation = heap_openr(AccessMethodOperatorRelationName, AccessShareLock);
+	scan = heap_beginscan(relation, false, SnapshotNow, 1, skey);
+
+	while (HeapTupleIsValid(tuple = heap_getnext(scan, 0)))
+	{
+		Form_pg_amop aform = (Form_pg_amop) GETSTRUCT(tuple);
+
+		if (!opclass_is_btree(aform->amopclaid))
+			continue;
+		if (aform->amopstrategy == BTLessStrategyNumber)
+		{
+			opclass = aform->amopclaid;
+			*kind = SORTFUNC_CMP;
+			break;				/* done looking */
+		}
+		else if (aform->amopstrategy == BTGreaterStrategyNumber)
+		{
+			opclass = aform->amopclaid;
+			*kind = SORTFUNC_REVCMP;
+			/* keep scanning in hopes of finding a BTLess entry */
+		}
+	}
+
+	heap_endscan(scan);
+	heap_close(relation, AccessShareLock);
+
+	if (OidIsValid(opclass))
+	{
+		/* Found a suitable opclass, get its comparator support function */
+		tuple = SearchSysCache(AMPROCNUM,
+							   ObjectIdGetDatum(opclass),
+							   Int16GetDatum(BTORDER_PROC),
+							   0, 0);
+		if (HeapTupleIsValid(tuple))
+		{
+			Form_pg_amproc aform = (Form_pg_amproc) GETSTRUCT(tuple);
+
+			*sortFunction = aform->amproc;
+			ReleaseSysCache(tuple);
+			Assert(RegProcedureIsValid(*sortFunction));
+			return;
+		}
+	}
+
+	/*
+	 * Can't find a comparator, so use the operator as-is.  Decide whether
+	 * it is forward or reverse sort by looking at its name (grotty, but
+	 * this only matters for deciding which end NULLs should get sorted
+	 * to).
+	 */
+	tuple = SearchSysCache(OPEROID,
+						   ObjectIdGetDatum(sortOperator),
+						   0, 0, 0);
+	if (!HeapTupleIsValid(tuple))
+		elog(ERROR, "SelectSortFunction: cache lookup failed for operator %u",
+			 sortOperator);
+	optup = (Form_pg_operator) GETSTRUCT(tuple);
+	if (strcmp(NameStr(optup->oprname), ">") == 0)
+		*kind = SORTFUNC_REVLT;
+	else
+		*kind = SORTFUNC_LT;
+	*sortFunction = optup->oprcode;
+	ReleaseSysCache(tuple);
+
+	Assert(RegProcedureIsValid(*sortFunction));
+}
+
+/*
+ * Apply a sort function (by now converted to fmgr lookup form)
+ * and return a 3-way comparison result.  This takes care of handling
+ * NULLs and sort ordering direction properly.
+ */
+int32
+ApplySortFunction(FmgrInfo *sortFunction, SortFunctionKind kind,
+				  Datum datum1, bool isNull1,
+				  Datum datum2, bool isNull2)
+{
+	switch (kind)
+	{
+		case SORTFUNC_LT:
+			if (isNull1)
+			{
+				if (isNull2)
+					return 0;
+				return 1;		/* NULL sorts after non-NULL */
+			}
+			if (isNull2)
+				return -1;
+			if (DatumGetBool(FunctionCall2(sortFunction, datum1, datum2)))
+				return -1;		/* a < b */
+			if (DatumGetBool(FunctionCall2(sortFunction, datum2, datum1)))
+				return 1;		/* a > b */
+			return 0;
+
+		case SORTFUNC_REVLT:
+			/* We reverse the ordering of NULLs, but not the operator */
+			if (isNull1)
+			{
+				if (isNull2)
+					return 0;
+				return -1;		/* NULL sorts before non-NULL */
+			}
+			if (isNull2)
+				return 1;
+			if (DatumGetBool(FunctionCall2(sortFunction, datum1, datum2)))
+				return -1;		/* a < b */
+			if (DatumGetBool(FunctionCall2(sortFunction, datum2, datum1)))
+				return 1;		/* a > b */
+			return 0;
+
+		case SORTFUNC_CMP:
+			if (isNull1)
+			{
+				if (isNull2)
+					return 0;
+				return 1;		/* NULL sorts after non-NULL */
+			}
+			if (isNull2)
+				return -1;
+			return DatumGetInt32(FunctionCall2(sortFunction,
+											   datum1, datum2));
+
+		case SORTFUNC_REVCMP:
+			if (isNull1)
+			{
+				if (isNull2)
+					return 0;
+				return -1;		/* NULL sorts before non-NULL */
+			}
+			if (isNull2)
+				return 1;
+			return -DatumGetInt32(FunctionCall2(sortFunction,
+												datum1, datum2));
+
+		default:
+			elog(ERROR, "Invalid SortFunctionKind %d", (int) kind);
+			return 0;			/* can't get here, but keep compiler quiet */
+	}
+}
