@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $Header: /home/projects/pgsql/cvsroot/pgsql/src/bin/ipcclean/ipcclean.sh,v 1.10 2001/02/10 10:27:28 petere Exp $
+# $Header: /cvsroot/pgsql/src/bin/ipcclean/ipcclean.sh,v 1.12 2001/09/30 22:17:50 momjian Exp $
 #
 
 CMDNAME=`basename $0`
@@ -33,7 +33,17 @@ EffectiveUser=`id -n -u 2>/dev/null || whoami 2>/dev/null`
 # List of platform-specific hacks
 # Feel free to add yours here.
 #-----------------------------------
-
+#
+# This is QNX 4.25
+#
+if [ `uname` = 'QNX' ]; then
+    if ps -eA  | grep -s '[p]ostmaster' >/dev/null 2>&1 ; then
+        echo "$CMDNAME: You still have a postmaster running." 1>&2
+        exit 1
+    fi
+    rm -f /dev/shmem/PgS*
+    exit $?
+fi
 #
 # This is based on RedHat 5.2.
 #
@@ -64,14 +74,14 @@ if [ `uname` = 'Linux' ]; then
         # (This check is conceptually phony, but it's
         # useful anyway in practice.)
         ps hj $ipcs_cpid $ipcs_lpid >/dev/null 2>&1
-        if [ $? -eq 0 ]; then
+        if [ "$?" -eq 0 ]; then
             echo "skipped; process still exists (pid $ipcs_cpid or $ipcs_lpid)."
 	    continue
 	fi
 
         # try remove
         ipcrm shm $ipcs_shmid
-        if [ $? -eq 0 ]; then
+        if [ "$?" -eq 0 ]; then
             did_anything=t
         else
             exit
@@ -83,7 +93,7 @@ if [ `uname` = 'Linux' ]; then
         echo -n "Semaphore $val ... "
         # try remove
         ipcrm sem $val
-        if [ $? -eq 0 ]; then
+        if [ "$?" -eq 0 ]; then
             did_anything=t
         else
             exit
