@@ -5,9 +5,10 @@
  *	  along with the relation's initial contents.
  *
  *
- * Copyright (c) 1994, Regents of the University of California
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: pg_database.h,v 1.8 1999/02/13 23:21:08 momjian Exp $
+ * $Id: pg_database.h,v 1.21 2001/11/05 17:46:32 momjian Exp $
  *
  * NOTES
  *	  the genbki.sh script reads this file and generates .bki
@@ -32,9 +33,14 @@
  */
 CATALOG(pg_database) BOOTSTRAP
 {
-	NameData	datname;
-	int4		datdba;
-	int4		encoding;
+	NameData	datname;		/* database name */
+	int4		datdba;			/* sysid of owner */
+	int4		encoding;		/* multibyte encoding, if any */
+	bool		datistemplate;	/* allowed as CREATE DATABASE template? */
+	bool		datallowconn;	/* new connections allowed? */
+	Oid			datlastsysoid;	/* highest OID to consider a system OID */
+	TransactionId datvacuumxid; /* all XIDs before this are vacuumed */
+	TransactionId datfrozenxid; /* all XIDs before this are frozen */
 	text		datpath;		/* VARIABLE LENGTH FIELD */
 } FormData_pg_database;
 
@@ -49,9 +55,28 @@ typedef FormData_pg_database *Form_pg_database;
  *		compiler constants for pg_database
  * ----------------
  */
-#define Natts_pg_database				4
+#define Natts_pg_database				9
 #define Anum_pg_database_datname		1
 #define Anum_pg_database_datdba			2
 #define Anum_pg_database_encoding		3
-#define Anum_pg_database_datpath		4
-#endif	 /* PG_DATABASE_H */
+#define Anum_pg_database_datistemplate	4
+#define Anum_pg_database_datallowconn	5
+#define Anum_pg_database_datlastsysoid	6
+#define Anum_pg_database_datvacuumxid	7
+#define Anum_pg_database_datfrozenxid	8
+#define Anum_pg_database_datpath		9
+
+DATA(insert OID = 1 (  template1 PGUID ENCODING t t 0 0 0 "" ));
+DESCR("Default template database");
+
+#define TemplateDbOid			1
+
+/* Just to mark OID as used for unused_oid script -:) */
+#define DATAMARKOID(x)
+
+DATAMARKOID(= 2)
+#define RecoveryDb	2
+
+#undef DATAMARKOID
+
+#endif   /* PG_DATABASE_H */
