@@ -4,17 +4,16 @@
  *
  *
  *
- * Copyright (c) 1994, Regents of the University of California
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: nodeUnique.h,v 1.8 1999/02/13 23:21:29 momjian Exp $
+ * $Id: nodeUnique.h,v 1.14 2001/11/05 17:46:33 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #ifndef NODEUNIQUE_H
 #define NODEUNIQUE_H
 
-#include "executor/tuptable.h"
-#include "nodes/execnodes.h"
 #include "nodes/plannodes.h"
 
 extern TupleTableSlot *ExecUnique(Unique *node);
@@ -23,4 +22,4 @@ extern int	ExecCountSlotsUnique(Unique *node);
 extern void ExecEndUnique(Unique *node);
 extern void ExecReScanUnique(Unique *node, ExprContext *exprCtxt, Plan *parent);
 
-#endif	 /* NODEUNIQUE_H */
+#endif   /* NODEUNIQUE_H */
