@@ -2,6 +2,7 @@
  *
  * spi.h
  *
+ * $Id: spi.h,v 1.33 2001/11/08 20:37:52 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -41,23 +42,24 @@
 
 typedef struct
 {
+	MemoryContext tuptabcxt;	/* memory context of result table */
 	uint32		alloced;		/* # of alloced vals */
 	uint32		free;			/* # of free vals */
 	TupleDesc	tupdesc;		/* tuple descriptor */
 	HeapTuple  *vals;			/* tuples */
 } SPITupleTable;
 
-#define SPI_ERROR_CONNECT		-1
-#define SPI_ERROR_COPY			-2
-#define SPI_ERROR_OPUNKNOWN		-3
-#define SPI_ERROR_UNCONNECTED	-4
-#define SPI_ERROR_CURSOR		-5
-#define SPI_ERROR_ARGUMENT		-6
-#define SPI_ERROR_PARAM			-7
-#define SPI_ERROR_TRANSACTION	-8
-#define SPI_ERROR_NOATTRIBUTE	-9
-#define SPI_ERROR_NOOUTFUNC		-10
-#define SPI_ERROR_TYPUNKNOWN	-11
+#define SPI_ERROR_CONNECT		(-1)
+#define SPI_ERROR_COPY			(-2)
+#define SPI_ERROR_OPUNKNOWN		(-3)
+#define SPI_ERROR_UNCONNECTED	(-4)
+#define SPI_ERROR_CURSOR		(-5)
+#define SPI_ERROR_ARGUMENT		(-6)
+#define SPI_ERROR_PARAM			(-7)
+#define SPI_ERROR_TRANSACTION	(-8)
+#define SPI_ERROR_NOATTRIBUTE	(-9)
+#define SPI_ERROR_NOOUTFUNC		(-10)
+#define SPI_ERROR_TYPUNKNOWN	(-11)
 
 #define SPI_OK_CONNECT			1
 #define SPI_OK_FINISH			2
@@ -83,8 +85,12 @@ extern int	SPI_exec(char *src, int tcount);
 extern int	SPI_execp(void *plan, Datum *values, char *Nulls, int tcount);
 extern void *SPI_prepare(char *src, int nargs, Oid *argtypes);
 extern void *SPI_saveplan(void *plan);
+extern int	SPI_freeplan(void *plan);
 
 extern HeapTuple SPI_copytuple(HeapTuple tuple);
+extern TupleDesc SPI_copytupledesc(TupleDesc tupdesc);
+extern TupleTableSlot *SPI_copytupleintoslot(HeapTuple tuple,
+					  TupleDesc tupdesc);
 extern HeapTuple SPI_modifytuple(Relation rel, HeapTuple tuple, int natts,
 				int *attnum, Datum *Values, char *Nulls);
 extern int	SPI_fnumber(TupleDesc tupdesc, char *fname);
@@ -98,7 +104,15 @@ extern void *SPI_palloc(Size size);
 extern void *SPI_repalloc(void *pointer, Size size);
 extern void SPI_pfree(void *pointer);
 extern void SPI_freetuple(HeapTuple pointer);
+extern void SPI_freetuptable(SPITupleTable *tuptable);
+
+extern Portal SPI_cursor_open(char *name, void *plan,
+				Datum *Values, char *Nulls);
+extern Portal SPI_cursor_find(char *name);
+extern void SPI_cursor_fetch(Portal portal, bool forward, int count);
+extern void SPI_cursor_move(Portal portal, bool forward, int count);
+extern void SPI_cursor_close(Portal portal);
 
 extern void AtEOXact_SPI(void);
 
-#endif	 /* SPI_H */
+#endif   /* SPI_H */
