@@ -1,11 +1,12 @@
 /*-------------------------------------------------------------------------
  *
- * geqo_pool.h--
+ * geqo_pool.h
  *	  pool representation in optimizer/geqo
  *
- * Copyright (c) 1994, Regents of the University of California
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: geqo_pool.h,v 1.4 1997/09/08 21:53:15 momjian Exp $
+ * $Id: geqo_pool.h,v 1.15 2001/11/05 17:46:34 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -22,11 +23,14 @@
 #ifndef GEQO_POOL_H
 #define GEQO_POOL_H
 
+#include "optimizer/geqo_gene.h"
+#include "nodes/parsenodes.h"
 
 extern Pool *alloc_pool(int pool_size, int string_length);
 extern void free_pool(Pool *pool);
 
-extern void random_init_pool(Query *root, Pool *pool, int strt, int stop);
+extern void random_init_pool(Query *root, List *initial_rels,
+				 Pool *pool, int strt, int stop);
 extern Chromosome *alloc_chromo(int string_length);
 extern void free_chromo(Chromosome *chromo);
 
@@ -34,4 +38,4 @@ extern void spread_chromo(Chromosome *chromo, Pool *pool);
 
 extern void sort_pool(Pool *pool);
 
-#endif							/* GEQO_POOL_H */
+#endif   /* GEQO_POOL_H */
