@@ -37,22 +37,8 @@
  *		@(#)regex2.h	8.4 (Berkeley) 3/20/94
  */
 
-/*
- * First, the stuff that ends up in the outside-world include file
-*/
-/*
- typedef off_t regoff_t;
- typedef struct {
-		int re_magic;
-		size_t re_nsub;			// number of parenthesized subexpressions
-		const char *re_endp;	// end pointer for REG_PEND
-		struct re_guts *re_g;	// none of your business :-)
- } regex_t;
- typedef struct {
-		regoff_t rm_so;			// start of match
-		regoff_t rm_eo;			// end of match
- } regmatch_t;
-*/
+#include <limits.h>
+
 /*
  * internals of regex_t
  */
@@ -80,8 +66,8 @@
 typedef unsigned long sop;		/* strip operator */
 typedef long sopno;
 
-#define OPRMASK 0xf8000000
-#define OPDMASK 0x07ffffff
+#define OPRMASK ((sop) 0xf8000000)
+#define OPDMASK ((sop) 0x07ffffff)
 #define OPSHIFT ((unsigned)27)
 #define OP(n)	((n)&OPRMASK)
 #define OPND(n) ((n)&OPDMASK)
@@ -203,8 +189,9 @@ struct re_guts
 #endif
 
 #ifdef MULTIBYTE
-#define ISWORD(c)	((c >= 0 && c <= UCHAR_MAX) && \
-			 (isalnum(c) || (c) == '_'))
+#define ISWORD(c)	(((c) >= 0 && (c) <= UCHAR_MAX) && \
+			 (isalnum((unsigned char) (c)) || (c) == '_'))
 #else
-#define ISWORD(c)	(isalnum(c) || (c) == '_')
+#define ISWORD(c)	(isalnum((unsigned char) (c)) || (c) == '_')
+
 #endif
