@@ -1,17 +1,21 @@
+/* $Header: /cvsroot/pgsql/src/interfaces/ecpg/lib/prepare.c,v 1.12 2001/12/23 12:17:41 meskes Exp $ */
+
+#include "postgres_fe.h"
+
 #include <ctype.h>
 
-#include <ecpgtype.h>
-#include <ecpglib.h>
-#include <ecpgerrno.h>
+#include "ecpgtype.h"
+#include "ecpglib.h"
+#include "ecpgerrno.h"
 #include "extern.h"
-#include <sqlca.h>
+#include "sqlca.h"
 
 static struct prepared_statement
 {
 	char	   *name;
 	struct statement *stmt;
 	struct prepared_statement *next;
-}		   *prep_stmts = NULL;
+}	*prep_stmts = NULL;
 
 static bool
 isvarchar(unsigned char c)
@@ -65,28 +69,28 @@ ECPGprepare(int lineno, char *name, char *variable)
 			return false;
 	}
 
-	this = (struct prepared_statement *) ecpg_alloc(sizeof(struct prepared_statement), lineno);
+	this = (struct prepared_statement *) ECPGalloc(sizeof(struct prepared_statement), lineno);
 	if (!this)
 		return false;
 
-	stmt = (struct statement *) ecpg_alloc(sizeof(struct statement), lineno);
+	stmt = (struct statement *) ECPGalloc(sizeof(struct statement), lineno);
 	if (!stmt)
 	{
-		free(this);
+		ECPGfree(this);
 		return false;
 	}
 
 	/* create statement */
 	stmt->lineno = lineno;
 	stmt->connection = NULL;
-	stmt->command = ecpg_strdup(variable, lineno);
+	stmt->command = ECPGstrdup(variable, lineno);
 	stmt->inlist = stmt->outlist = NULL;
 
 	/* if we have C variables in our statment replace them with '?' */
 	replace_variables(stmt->command);
 
 	/* add prepared statement to our list */
-	this->name = ecpg_strdup(name, lineno);
+	this->name = ECPGstrdup(name, lineno);
 	this->stmt = stmt;
 
 	if (prep_stmts == NULL)
@@ -110,18 +114,17 @@ ECPGdeallocate(int lineno, char *name)
 	if (this)
 	{
 		/* okay, free all the resources */
-		free(this->name);
-		free(this->stmt->command);
-		free(this->stmt);
+		ECPGfree(this->name);
+		ECPGfree(this->stmt->command);
+		ECPGfree(this->stmt);
 		if (prev != NULL)
 			prev->next = this->next;
 		else
 			prep_stmts = this->next;
 
-		free(this);
+		ECPGfree(this);
 		return true;
 	}
-	ECPGlog("deallocate_prepare: invalid statement name %s\n", name);
 	ECPGraise(lineno, ECPG_INVALID_STMT, name);
 	return false;
 }
