@@ -10,11 +10,11 @@
  *   NOTES
  *      Currently under construction.
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
- *  $Id: pgdatabase.h,v 1.8 2000/04/22 22:39:15 tgl Exp $
+ *  $Id: pgdatabase.h,v 1.12 2001/07/11 22:12:43 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -34,38 +34,42 @@
 // This is the basic database access class.  Its interface should 
 // be used only after a query has been sent to the backend and
 // results are being received.
-class PgDatabase : public PgConnection {
+class DLLIMPORT PgDatabase : public PgConnection {
 public:
   // connect to the database with conninfo
-  PgDatabase(const char* conninfo) : PgConnection(conninfo) {}
+  explicit PgDatabase(const char* conninfo) : PgConnection(conninfo) {}
 
   ~PgDatabase() {}				// close connection and clean up
+
+  typedef int size_type;
   
   // query result access
-  int Tuples();
-  int CmdTuples(); 
+  size_type Tuples() const;
+  size_type CmdTuples() const; 
   int Fields();
-  const char* FieldName(int field_num);
-  int FieldNum(const char* field_name);
-  Oid FieldType(int field_num);
-  Oid FieldType(const char* field_name);
-  short FieldSize(int field_num);
-  short FieldSize(const char* field_name);
-  const char* GetValue(int tup_num, int field_num);
-  const char* GetValue(int tup_num, const char* field_name);
-  int GetIsNull(int tup_num, int field_num);
-  int GetIsNull(int tup_num, const char* field_name);
-  int GetLength(int tup_num, int field_num);
-  int GetLength(int tup_num, const char* field_name);
-  void DisplayTuples(FILE *out = 0, int fillAlign = 1, 
-	const char* fieldSep = "|",int printHeader = 1, int quiet = 0) ;
-  void PrintTuples(FILE *out = 0, int printAttName = 1, 
-	int terseOutput = 0, int width = 0) ;
+  const char* FieldName(int field_num) const;
+  int FieldNum(const char* field_name) const;
+  Oid FieldType(int field_num) const;
+  Oid FieldType(const char* field_name) const;
+  int FieldSize(int field_num) const;
+  int FieldSize(const char* field_name) const;
+  const char* GetValue(size_type tup_num, int field_num) const;
+  const char* GetValue(size_type tup_num, const char* field_name) const;
+  bool GetIsNull(size_type tup_num, int field_num) const;
+  bool GetIsNull(size_type tup_num, const char* field_name) const;
+  int GetLength(size_type tup_num, int field_num) const;
+  int GetLength(size_type tup_num, const char* field_name) const;
+
+  // OBSOLESCENT (use PQprint()):
+  void DisplayTuples(FILE *out=0, bool fillAlign=true, 
+	const char* fieldSep="|", bool printHeader=true, bool quiet=false) const;
+  void PrintTuples(FILE *out=0, bool printAttName=true, 
+	bool terseOutput=false, bool fillAlign=false) const;
 
   // copy command related access
-  int GetLine(char* string, int length);
-  void PutLine(const char* string);
-  const char* OidStatus();
+  int GetLine(char str[], int length);
+  void PutLine(const char str[]);
+  const char* OidStatus() const;
   int EndCopy();
     
 protected:
