@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /home/projects/pgsql/cvsroot/pgsql/src/interfaces/libpq/fe-connect.c,v 1.164 2001/03/31 23:14:37 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/interfaces/libpq/fe-connect.c,v 1.181 2001/11/11 02:09:05 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -59,13 +59,11 @@ inet_aton(const char *cp, struct in_addr * inp)
 	inp->s_addr = a;
 	return 1;
 }
-
 #endif
 
 
 #ifdef USE_SSL
 static SSL_CTX *SSL_context = NULL;
-
 #endif
 
 #define NOTIFYLIST_INITIAL_SIZE 10
@@ -99,7 +97,6 @@ static SSL_CTX *SSL_context = NULL;
  * ----------
  */
 static const PQconninfoOption PQconninfoOptions[] = {
-
 	/*
 	 * "authtype" is no longer used, so mark it "don't show".  We keep it
 	 * in the array so as not to reject conninfo strings from old apps
@@ -150,7 +147,7 @@ static const struct EnvironmentOptions
 {
 	const char *envName,
 			   *pgName;
-}			EnvironmentOptions[] =
+}	EnvironmentOptions[] =
 
 {
 	/* common user-interface settings */
@@ -189,9 +186,12 @@ static char *conninfo_getval(PQconninfoOption *connOptions,
 static void defaultNoticeProcessor(void *arg, const char *message);
 static int parseServiceInfo(PQconninfoOption *options,
 				 PQExpBuffer errorMessage);
+#ifdef USE_SSL
+static const char *SSLerrmessage(void);
+#endif
 
 
-/* ----------------
+/*
  *		Connecting to a Database
  *
  * There are now four different ways a user of this API can connect to the
@@ -208,11 +208,9 @@ static int parseServiceInfo(PQconninfoOption *options,
  *
  * Internally, the static functions connectDBStart, connectDBComplete
  * are part of the connection procedure.
- *
- * ----------------
  */
 
-/* ----------------
+/*
  *		PQconnectdb
  *
  * establishes a connection to a postgres backend through the postmaster
@@ -233,8 +231,6 @@ static int parseServiceInfo(PQconninfoOption *options,
  *
  * You should call PQfinish (if conn is not NULL) regardless of whether this
  * call succeeded.
- *
- * ----------------
  */
 PGconn *
 PQconnectdb(const char *conninfo)
@@ -247,7 +243,7 @@ PQconnectdb(const char *conninfo)
 	return conn;
 }
 
-/* ----------------
+/*
  *		PQconnectStart
  *
  * Begins the establishment of a connection to a postgres backend through the
@@ -265,8 +261,6 @@ PQconnectdb(const char *conninfo)
  * this is necessary.
  *
  * See PQconnectPoll for more info.
- *
- * ----------------
  */
 PGconn *
 PQconnectStart(const char *conninfo)
@@ -346,7 +340,7 @@ PQconnectStart(const char *conninfo)
 	return conn;
 }
 
-/* ----------------
+/*
  *		PQconndefaults
  *
  * Parse an empty string like PQconnectdb() would do and return the
@@ -360,7 +354,6 @@ PQconnectStart(const char *conninfo)
  * versions, the returned array was static, but that's not thread-safe.)
  * Pre-7.0 applications that use this function will see a small memory leak
  * until they are updated to call PQconninfoFree.
- * ----------------
  */
 PQconninfoOption *
 PQconndefaults(void)
@@ -411,7 +404,6 @@ PQconndefaults(void)
  * the database name to lower case if it is not surrounded by double quotes.
  * Otherwise, strip the double quotes but leave the reset of the string intact.
  * - thomas 1997-11-08
- *
  * ----------------
  */
 PGconn *
@@ -488,7 +480,7 @@ PQsetdbLogin(const char *pghost, const char *pgport, const char *pgoptions,
 	{
 		error = TRUE;
 		printfPQExpBuffer(&conn->errorMessage,
-						  "FATAL: PQsetdbLogin(): Unable to determine a Postgres username!\n");
+						  libpq_gettext("could not determine the PostgreSQL user name to use\n"));
 	}
 
 	if (pwd)
@@ -584,7 +576,7 @@ update_db_info(PGconn *conn)
 
 			/*-------
 			 * new style:
-			 * 	<tcp|unix>:postgresql://server[:port|:/unixsocket/path:]
+			 *	<tcp|unix>:postgresql://server[:port|:/unixsocket/path:]
 			 *	[/db name][?options]
 			 *-------
 			 */
@@ -609,7 +601,6 @@ update_db_info(PGconn *conn)
 			}
 			else
 			{
-
 				/*
 				 * Why do we default only this value from the environment
 				 * again?
@@ -684,8 +675,7 @@ update_db_info(PGconn *conn)
 
 	return 0;
 }
-
-#endif	 /* NOT_USED */
+#endif   /* NOT_USED */
 
 
 /* ----------
@@ -697,21 +687,21 @@ update_db_info(PGconn *conn)
 static int
 connectMakeNonblocking(PGconn *conn)
 {
-#ifdef WIN32
+#if defined(WIN32) || defined(__BEOS__)
 	int			on = 1;
+#endif
 
+#if defined(WIN32)
 	if (ioctlsocket(conn->sock, FIONBIO, &on) != 0)
 #elif defined(__BEOS__)
-		int			on = 1;
-
-	if (ioctl(conn->sock, FIONBIO, &on) != 0)
+		if (ioctl(conn->sock, FIONBIO, &on) != 0)
 #else
-	if			(fcntl(conn->sock, F_SETFL, O_NONBLOCK) < 0)
+	if (fcntl(conn->sock, F_SETFL, O_NONBLOCK) < 0)
 #endif
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-			  "connectMakeNonblocking -- fcntl() failed: errno=%d\n%s\n",
-						  errno, strerror(errno));
+		libpq_gettext("could not set socket to non-blocking mode: %s\n"),
+						  SOCK_STRERROR(SOCK_ERRNO));
 		return 0;
 	}
 
@@ -734,11 +724,8 @@ connectNoDelay(PGconn *conn)
 				   sizeof(on)) < 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				 "connectNoDelay() -- setsockopt failed: errno=%d\n%s\n",
-						  errno, strerror(errno));
-#ifdef WIN32
-		printf("Winsock error: %i\n", WSAGetLastError());
-#endif
+		libpq_gettext("could not set socket to TCP no delay mode: %s\n"),
+						  SOCK_STRERROR(SOCK_ERRNO));
 		return 0;
 	}
 
@@ -752,26 +739,30 @@ connectNoDelay(PGconn *conn)
  * ----------
  */
 static void
-connectFailureMessage(PGconn *conn, const char *caller, int errorno)
+connectFailureMessage(PGconn *conn, int errorno)
 {
-#ifdef HAVE_UNIX_SOCKETS
 	if (conn->raddr.sa.sa_family == AF_UNIX)
 		printfPQExpBuffer(&conn->errorMessage,
-						  "%s -- connect() failed: %s\n"
-						  "\tIs the postmaster running locally\n"
-					"\tand accepting connections on Unix socket '%s'?\n",
-						  caller,
-						  strerror(errorno),
+						  libpq_gettext(
+									  "could not connect to server: %s\n"
+						"\tIs the server running locally and accepting\n"
+						  "\tconnections on Unix domain socket \"%s\"?\n"
+										),
+						  SOCK_STRERROR(errorno),
 						  conn->raddr.un.sun_path);
 	else
-#endif
 		printfPQExpBuffer(&conn->errorMessage,
-						  "%s -- connect() failed: %s\n"
-						"\tIs the postmaster running (with -i) at '%s'\n"
-					  "\tand accepting connections on TCP/IP port %s?\n",
-						  caller,
-						  strerror(errorno),
-						  conn->pghost ? conn->pghost : "localhost",
+						  libpq_gettext(
+									  "could not connect to server: %s\n"
+					 "\tIs the server running on host %s and accepting\n"
+									 "\tTCP/IP connections on port %s?\n"
+										),
+						  SOCK_STRERROR(errorno),
+						  conn->pghost
+						  ? conn->pghost
+						  : (conn->pghostaddr
+							 ? conn->pghostaddr
+							 : "???"),
 						  conn->pgport);
 }
 
@@ -792,7 +783,6 @@ connectDBStart(PGconn *conn)
 #ifdef USE_SSL
 	StartupPacket np;			/* Used to negotiate SSL connection */
 	char		SSLok;
-
 #endif
 
 	if (!conn)
@@ -827,8 +817,8 @@ connectDBStart(PGconn *conn)
 		if (!inet_aton(conn->pghostaddr, &addr))
 		{
 			printfPQExpBuffer(&conn->errorMessage,
-							  "connectDBStart() -- "
-						 "invalid host address: %s\n", conn->pghostaddr);
+							  libpq_gettext("invalid host address: %s\n"),
+							  conn->pghostaddr);
 			goto connect_errReturn;
 		}
 
@@ -846,7 +836,7 @@ connectDBStart(PGconn *conn)
 		if ((hp == NULL) || (hp->h_addrtype != AF_INET))
 		{
 			printfPQExpBuffer(&conn->errorMessage,
-						   "connectDBStart() --  unknown hostname: %s\n",
+							  libpq_gettext("unknown host name: %s\n"),
 							  conn->pghost);
 			goto connect_errReturn;
 		}
@@ -858,7 +848,7 @@ connectDBStart(PGconn *conn)
 	}
 	else
 	{
-		/* pghostaddr and pghost are NULL, so use UDP */
+		/* pghostaddr and pghost are NULL, so use Unix domain socket */
 		family = AF_UNIX;
 	}
 
@@ -888,9 +878,8 @@ connectDBStart(PGconn *conn)
 	if ((conn->sock = socket(family, SOCK_STREAM, 0)) < 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  "connectDBStart() -- "
-						  "socket() failed: errno=%d\n%s\n",
-						  errno, strerror(errno));
+						  libpq_gettext("could not create socket: %s\n"),
+						  SOCK_STRERROR(SOCK_ERRNO));
 		goto connect_errReturn;
 	}
 
@@ -906,17 +895,7 @@ connectDBStart(PGconn *conn)
 			goto connect_errReturn;
 	}
 
-	/*
-	 * Since I have no idea whether this is a valid thing to do under
-	 * Windows before a connection is made, and since I have no way of
-	 * testing it, I leave the code looking as below.  When someone
-	 * decides that they want non-blocking connections under Windows, they
-	 * can define WIN32_NON_BLOCKING_CONNECTIONS before compilation.  If
-	 * it works, then this code can be cleaned up.
-	 *
-	 * Ewan Mellor <eem21@cam.ac.uk>.
-	 */
-#if ((!defined(WIN32) && !defined(__CYGWIN__)) || defined(WIN32_NON_BLOCKING_CONNECTIONS)) && !defined(USE_SSL)
+#if !defined(USE_SSL)
 	if (connectMakeNonblocking(conn) == 0)
 		goto connect_errReturn;
 #endif
@@ -926,21 +905,14 @@ connectDBStart(PGconn *conn)
 	 * now, but it is possible that:
 	 *	 1. Older systems will still block on connect, despite the
 	 *		non-blocking flag. (Anyone know if this is true?)
-	 *	 2. We are running under Windows, and aren't even trying
-	 *		to be non-blocking (see above).
-	 *	 3. We are using SSL.
-	 * Thus, we have make arrangements for all eventualities.
+	 *	 2. We are using SSL.
+	 * Thus, we have to make arrangements for all eventualities.
 	 * ----------
 	 */
 	if (connect(conn->sock, &conn->raddr.sa, conn->raddr_len) < 0)
 	{
-#ifndef WIN32
-		if (errno == EINPROGRESS || errno == 0)
-#else
-		if (WSAGetLastError() == WSAEINPROGRESS)
-#endif
+		if (SOCK_ERRNO == EINPROGRESS || SOCK_ERRNO == EWOULDBLOCK || SOCK_ERRNO == 0)
 		{
-
 			/*
 			 * This is fine - we're in non-blocking mode, and the
 			 * connection is in progress.
@@ -950,7 +922,7 @@ connectDBStart(PGconn *conn)
 		else
 		{
 			/* Something's gone wrong */
-			connectFailureMessage(conn, "connectDBStart()", errno);
+			connectFailureMessage(conn, SOCK_ERRNO);
 			goto connect_errReturn;
 		}
 	}
@@ -969,15 +941,16 @@ connectDBStart(PGconn *conn)
 		if (pqPacketSend(conn, (char *) &np, sizeof(StartupPacket)) != STATUS_OK)
 		{
 			printfPQExpBuffer(&conn->errorMessage,
-							  "connectDB() -- couldn't send SSL negotiation packet: errno=%d\n%s\n",
-							  errno, strerror(errno));
+			libpq_gettext("could not send SSL negotiation packet: %s\n"),
+							  SOCK_STRERROR(SOCK_ERRNO));
 			goto connect_errReturn;
 		}
 		/* Now receive the postmasters response */
 		if (recv(conn->sock, &SSLok, 1, 0) != 1)
 		{
-			printfPQExpBuffer(&conn->errorMessage, "PQconnectDB() -- couldn't read postmaster response: errno=%d\n%s\n",
-							  errno, strerror(errno));
+			printfPQExpBuffer(&conn->errorMessage,
+							  libpq_gettext("could not receive server response to SSL negotiation packet: %s\n"),
+							  SOCK_STRERROR(SOCK_ERRNO));
 			goto connect_errReturn;
 		}
 		if (SSLok == 'S')
@@ -990,8 +963,8 @@ connectDBStart(PGconn *conn)
 				if (!SSL_context)
 				{
 					printfPQExpBuffer(&conn->errorMessage,
-					  "connectDB() -- couldn't create SSL context: %s\n",
-							   ERR_reason_error_string(ERR_get_error()));
+					 libpq_gettext("could not create SSL context: %s\n"),
+									  SSLerrmessage());
 					goto connect_errReturn;
 				}
 			}
@@ -1000,8 +973,8 @@ connectDBStart(PGconn *conn)
 				SSL_connect(conn->ssl) <= 0)
 			{
 				printfPQExpBuffer(&conn->errorMessage,
-				"connectDB() -- couldn't establish SSL connection: %s\n",
-							   ERR_reason_error_string(ERR_get_error()));
+				libpq_gettext("could not establish SSL connection: %s\n"),
+								  SSLerrmessage());
 				goto connect_errReturn;
 			}
 			/* SSL connection finished. Continue to send startup packet */
@@ -1011,14 +984,20 @@ connectDBStart(PGconn *conn)
 			/* Received error - probably protocol mismatch */
 			if (conn->Pfdebug)
 				fprintf(conn->Pfdebug, "Postmaster reports error, attempting fallback to pre-7.0.\n");
+#ifdef WIN32
+			closesocket(conn->sock);
+#else
 			close(conn->sock);
+#endif
+			conn->sock = -1;
 			conn->allow_ssl_try = FALSE;
 			return connectDBStart(conn);
 		}
 		else if (SSLok != 'N')
 		{
 			printfPQExpBuffer(&conn->errorMessage,
-							  "Received invalid negotiation response.\n");
+							  libpq_gettext("received invalid response to SSL negotiation: %c\n"),
+							  SSLok);
 			goto connect_errReturn;
 		}
 	}
@@ -1026,7 +1005,7 @@ connectDBStart(PGconn *conn)
 	{
 		/* Require SSL, but server does not support/want it */
 		printfPQExpBuffer(&conn->errorMessage,
-				 "Server does not support SSL when SSL was required.\n");
+						  libpq_gettext("server does not support SSL, but SSL was required\n"));
 		goto connect_errReturn;
 	}
 #endif
@@ -1036,7 +1015,7 @@ connectDBStart(PGconn *conn)
 	 * This makes the connection non-blocking, for all those cases which
 	 * forced us not to do it above.
 	 */
-#if ((defined(WIN32) || defined(__CYGWIN__)) && !defined(WIN32_NON_BLOCKING_CONNECTIONS)) || defined(USE_SSL)
+#if defined(USE_SSL)
 	if (connectMakeNonblocking(conn) == 0)
 		goto connect_errReturn;
 #endif
@@ -1059,13 +1038,12 @@ connect_errReturn:
 }
 
 
-/* ----------------
+/*
  *		connectDBComplete
  *
  * Block and complete a connection.
  *
  * Returns 1 on success, 0 on failure.
- * ----------------
  */
 static int
 connectDBComplete(PGconn *conn)
@@ -1077,7 +1055,6 @@ connectDBComplete(PGconn *conn)
 
 	for (;;)
 	{
-
 		/*
 		 * Wait, if necessary.	Note that the initial state (just after
 		 * PQconnectStart) is to wait for the socket to select for
@@ -1158,7 +1135,6 @@ PQconnectPoll(PGconn *conn)
 	/* Get the new data */
 	switch (conn->status)
 	{
-
 			/*
 			 * We really shouldn't have been polled in these two cases,
 			 * but we can handle it.
@@ -1194,8 +1170,10 @@ PQconnectPoll(PGconn *conn)
 
 		default:
 			printfPQExpBuffer(&conn->errorMessage,
-						 "PQconnectPoll() -- unknown connection state - "
-						  "probably indicative of memory corruption!\n");
+							  libpq_gettext(
+											"invalid connection state, "
+							 "probably indicative of memory corruption\n"
+											));
 			goto error_return;
 	}
 
@@ -1207,14 +1185,7 @@ keep_going:						/* We will come back to here until there
 		case CONNECTION_STARTED:
 			{
 				ACCEPT_TYPE_ARG3 laddrlen;
-
-#ifndef WIN32
 				int			optval;
-
-#else
-				char		optval;
-
-#endif
 				ACCEPT_TYPE_ARG3 optlen = sizeof(optval);
 
 				/*
@@ -1231,20 +1202,18 @@ keep_going:						/* We will come back to here until there
 							   (char *) &optval, &optlen) == -1)
 				{
 					printfPQExpBuffer(&conn->errorMessage,
-							   "PQconnectPoll() -- getsockopt() failed: "
-									  "errno=%d\n%s\n",
-									  errno, strerror(errno));
+									  libpq_gettext("could not get socket error status: %s\n"),
+									  SOCK_STRERROR(SOCK_ERRNO));
 					goto error_return;
 				}
 				else if (optval != 0)
 				{
-
 					/*
 					 * When using a nonblocking connect, we will typically
 					 * see connect failures at this point, so provide a
 					 * friendly error message.
 					 */
-					connectFailureMessage(conn, "PQconnectPoll()", optval);
+					connectFailureMessage(conn, optval);
 					goto error_return;
 				}
 
@@ -1253,9 +1222,8 @@ keep_going:						/* We will come back to here until there
 				if (getsockname(conn->sock, &conn->laddr.sa, &laddrlen) < 0)
 				{
 					printfPQExpBuffer(&conn->errorMessage,
-							  "PQconnectPoll() -- getsockname() failed: "
-									  "errno=%d\n%s\n",
-									  errno, strerror(errno));
+									  libpq_gettext("could not get client address from socket: %s\n"),
+									  SOCK_STRERROR(SOCK_ERRNO));
 					goto error_return;
 				}
 
@@ -1293,10 +1261,8 @@ keep_going:						/* We will come back to here until there
 								 sizeof(StartupPacket)) != STATUS_OK)
 				{
 					printfPQExpBuffer(&conn->errorMessage,
-									  "PQconnectPoll() --  "
-									  "couldn't send startup packet: "
-									  "errno=%d\n%s\n",
-									  errno, strerror(errno));
+					libpq_gettext("could not send startup packet: %s\n"),
+									  SOCK_STRERROR(SOCK_ERRNO));
 					goto error_return;
 				}
 
@@ -1350,8 +1316,11 @@ keep_going:						/* We will come back to here until there
 				if (beresp != 'R')
 				{
 					printfPQExpBuffer(&conn->errorMessage,
-									  "PQconnectPoll() -- expected "
-									  "authentication request\n");
+									  libpq_gettext(
+								  "expected authentication request from "
+											  "server, but received %c\n"
+													),
+									  beresp);
 					goto error_return;
 				}
 
@@ -1363,9 +1332,19 @@ keep_going:						/* We will come back to here until there
 				}
 
 				/* Get the password salt if there is one. */
+				if (areq == AUTH_REQ_MD5)
+				{
+					if (pqGetnchar(conn->md5Salt,
+								   sizeof(conn->md5Salt), conn))
+					{
+						/* We'll come back when there are more data */
+						return PGRES_POLLING_READING;
+					}
+				}
 				if (areq == AUTH_REQ_CRYPT)
 				{
-					if (pqGetnchar(conn->salt, sizeof(conn->salt), conn))
+					if (pqGetnchar(conn->cryptSalt,
+								   sizeof(conn->cryptSalt), conn))
 					{
 						/* We'll come back when there are more data */
 						return PGRES_POLLING_READING;
@@ -1425,7 +1404,6 @@ keep_going:						/* We will come back to here until there
 
 		case CONNECTION_AUTH_OK:
 			{
-
 				/*
 				 * Now we expect to hear from the backend. A ReadyForQuery
 				 * message indicates that startup is successful, but we
@@ -1451,8 +1429,7 @@ keep_going:						/* We will come back to here until there
 				{
 					if (res->resultStatus != PGRES_FATAL_ERROR)
 						printfPQExpBuffer(&conn->errorMessage,
-								 "PQconnectPoll() -- unexpected message "
-										  "during startup\n");
+										  libpq_gettext("unexpected message from server during startup\n"));
 
 					/*
 					 * if the resultStatus is FATAL, then
@@ -1510,8 +1487,11 @@ keep_going:						/* We will come back to here until there
 
 		default:
 			printfPQExpBuffer(&conn->errorMessage,
-						 "PQconnectPoll() -- unknown connection state - "
-						  "probably indicative of memory corruption!\n");
+							  libpq_gettext(
+										  "invalid connection state %c, "
+							 "probably indicative of memory corruption\n"
+											),
+							  conn->status);
 			goto error_return;
 	}
 
@@ -1530,13 +1510,11 @@ error_return:
 }
 
 
-/* ----------------
+/*
  *		PQsetenvStart
  *
  * Starts the process of passing the values of a standard set of environment
  * variables to the backend.
- *
- * ----------------
  */
 static bool
 PQsetenvStart(PGconn *conn)
@@ -1557,13 +1535,11 @@ PQsetenvStart(PGconn *conn)
 	return true;
 }
 
-/* ----------------
+/*
  *		PQsetenvPoll
  *
  * Polls the process of passing the values of a standard set of environment
  * variables to the backend.
- *
- * ----------------
  */
 static PostgresPollingStatusType
 PQsetenvPoll(PGconn *conn)
@@ -1572,7 +1548,6 @@ PQsetenvPoll(PGconn *conn)
 
 #ifdef MULTIBYTE
 	static const char envname[] = "PGCLIENTENCODING";
-
 #endif
 
 	if (conn == NULL || conn->status == CONNECTION_BAD)
@@ -1611,8 +1586,11 @@ PQsetenvPoll(PGconn *conn)
 
 		default:
 			printfPQExpBuffer(&conn->errorMessage,
-							  "PQsetenvPoll() -- unknown state - "
-						  "probably indicative of memory corruption!\n");
+							  libpq_gettext(
+											"invalid setenv state %c, "
+							 "probably indicative of memory corruption\n"
+											),
+							  conn->setenv_state);
 			goto error_return;
 	}
 
@@ -1630,7 +1608,6 @@ keep_going:						/* We will come back to here until there
 				env = getenv(envname);
 				if (!env || *env == '\0')
 				{
-
 					/*
 					 * query server encoding if PGCLIENTENCODING is not
 					 * specified
@@ -1649,8 +1626,9 @@ keep_going:						/* We will come back to here until there
 
 					if (encoding < 0)
 					{
-						strcpy(conn->errorMessage.data,
-						"PGCLIENTENCODING has no valid encoding name.\n");
+						printfPQExpBuffer(&conn->errorMessage,
+										  libpq_gettext("invalid encoding name in PGCLIENTENCODING: %s\n"),
+										  env);
 						goto error_return;
 					}
 					conn->client_encoding = encoding;
@@ -1678,7 +1656,7 @@ keep_going:						/* We will come back to here until there
 					/* set client encoding in pg_conn struct */
 					encoding = PQgetvalue(res, 0, 0);
 					if (!encoding)		/* this should not happen */
-						conn->client_encoding = SQL_ASCII;
+						conn->client_encoding = PG_SQL_ASCII;
 					else
 						conn->client_encoding = pg_char_to_encoding(encoding);
 					PQclear(res);
@@ -1774,8 +1752,9 @@ keep_going:						/* We will come back to here until there
 
 		default:
 			printfPQExpBuffer(&conn->errorMessage,
-							  "PQsetenvPoll() -- unknown state - "
-						  "probably indicative of memory corruption!\n");
+							  libpq_gettext("invalid state %c, "
+						   "probably indicative of memory corruption\n"),
+							  conn->setenv_state);
 			goto error_return;
 	}
 
@@ -1789,7 +1768,7 @@ error_return:
 
 #ifdef NOT_USED
 
-/* ----------------
+/*
  *		PQsetenv
  *
  * Passes the values of a standard set of environment variables to the
@@ -1800,7 +1779,6 @@ error_return:
  * This function used to be exported for no particularly good reason.
  * Since it's no longer used by libpq itself, let's try #ifdef'ing it out
  * and see if anyone complains.
- * ----------------
  */
 static bool
 PQsetenv(PGconn *conn)
@@ -1812,7 +1790,6 @@ PQsetenv(PGconn *conn)
 
 	for (;;)
 	{
-
 		/*
 		 * Wait, if necessary.	Note that the initial state (just after
 		 * PQsetenvStart) is to wait for the socket to select for writing.
@@ -1853,8 +1830,7 @@ PQsetenv(PGconn *conn)
 		flag = PQsetenvPoll(conn);
 	}
 }
-
-#endif	 /* NOT_USED */
+#endif   /* NOT_USED */
 
 
 /*
@@ -1928,11 +1904,13 @@ freePGconn(PGconn *conn)
 		SSL_free(conn->ssl);
 #endif
 	if (conn->sock >= 0)
+	{
 #ifdef WIN32
 		closesocket(conn->sock);
 #else
 		close(conn->sock);
 #endif
+	}
 	if (conn->pghost)
 		free(conn->pghost);
 	if (conn->pghostaddr)
@@ -1972,17 +1950,20 @@ freePGconn(PGconn *conn)
 static void
 closePGconn(PGconn *conn)
 {
-	if (conn->sock >= 0)
+	/*
+	 * Note that the protocol doesn't allow us to send Terminate messages
+	 * during the startup phase.
+	 */
+	if (conn->sock >= 0 && conn->status == CONNECTION_OK)
 	{
-
 		/*
 		 * Try to send "close connection" message to backend. Ignore any
 		 * error. Note: this routine used to go to substantial lengths to
 		 * avoid getting SIGPIPE'd if the connection were already closed.
 		 * Now we rely on pqFlush to avoid the signal.
 		 */
-		(void) pqPuts("X", conn);
-		(void) pqFlush(conn);
+		pqPutc('X', conn);
+		pqFlush(conn);
 	}
 
 	/*
@@ -1996,11 +1977,13 @@ closePGconn(PGconn *conn)
 	 * Close the connection, reset all transient state, flush I/O buffers.
 	 */
 	if (conn->sock >= 0)
+	{
 #ifdef WIN32
 		closesocket(conn->sock);
 #else
 		close(conn->sock);
 #endif
+	}
 	conn->sock = -1;
 	conn->status = CONNECTION_BAD;		/* Well, not really _bad_ - just
 										 * absent */
@@ -2110,7 +2093,7 @@ PQresetPoll(PGconn *conn)
 int
 PQrequestCancel(PGconn *conn)
 {
-	int			save_errno = errno;
+	int			save_errno = SOCK_ERRNO;
 	int			tmpsock = -1;
 	struct
 	{
@@ -2127,7 +2110,11 @@ PQrequestCancel(PGconn *conn)
 		strcpy(conn->errorMessage.data,
 			   "PQrequestCancel() -- connection is not open\n");
 		conn->errorMessage.len = strlen(conn->errorMessage.data);
+#ifdef WIN32
+		WSASetLastError(save_errno);
+#else
 		errno = save_errno;
+#endif
 		return FALSE;
 	}
 
@@ -2169,26 +2156,28 @@ PQrequestCancel(PGconn *conn)
 	/* Sent it, done */
 #ifdef WIN32
 	closesocket(tmpsock);
+	WSASetLastError(save_errno);
 #else
 	close(tmpsock);
+	errno = save_errno;
 #endif
 
-	errno = save_errno;
 	return TRUE;
 
 cancel_errReturn:
-	strcat(conn->errorMessage.data, strerror(errno));
+	strcat(conn->errorMessage.data, SOCK_STRERROR(SOCK_ERRNO));
 	strcat(conn->errorMessage.data, "\n");
 	conn->errorMessage.len = strlen(conn->errorMessage.data);
 	if (tmpsock >= 0)
 	{
 #ifdef WIN32
 		closesocket(tmpsock);
+		WSASetLastError(save_errno);
 #else
 		close(tmpsock);
+		errno = save_errno;
 #endif
 	}
-	errno = save_errno;
 	return FALSE;
 }
 
@@ -2305,7 +2294,6 @@ parseServiceInfo(PQconninfoOption *options, PQExpBuffer errorMessage)
 			{
 				if (group_found)
 				{
-
 					/*
 					 * Finally, we are in the right group and can parse
 					 * the line
@@ -2358,13 +2346,12 @@ parseServiceInfo(PQconninfoOption *options, PQExpBuffer errorMessage)
 }
 
 
-/* ----------------
+/*
  * Conninfo parser routine
  *
  * If successful, a malloc'd PQconninfoOption array is returned.
  * If not successful, NULL is returned and an error message is
  * left in errorMessage.
- * ----------------
  */
 static PQconninfoOption *
 conninfo_parse(const char *conninfo, PQExpBuffer errorMessage)
@@ -2384,7 +2371,7 @@ conninfo_parse(const char *conninfo, PQExpBuffer errorMessage)
 	if (options == NULL)
 	{
 		printfPQExpBuffer(errorMessage,
-		"FATAL: cannot allocate memory for copy of PQconninfoOptions\n");
+						  libpq_gettext("out of memory\n"));
 		return NULL;
 	}
 	memcpy(options, PQconninfoOptions, sizeof(PQconninfoOptions));
@@ -2393,7 +2380,7 @@ conninfo_parse(const char *conninfo, PQExpBuffer errorMessage)
 	if ((buf = strdup(conninfo)) == NULL)
 	{
 		printfPQExpBuffer(errorMessage,
-		  "FATAL: cannot allocate memory for copy of conninfo string\n");
+						  libpq_gettext("out of memory\n"));
 		PQconninfoFree(options);
 		return NULL;
 	}
@@ -2432,7 +2419,7 @@ conninfo_parse(const char *conninfo, PQExpBuffer errorMessage)
 		if (*cp != '=')
 		{
 			printfPQExpBuffer(errorMessage,
-						   "ERROR: Missing '=' after '%s' in conninfo\n",
+							  libpq_gettext("missing \"=\" after \"%s\" in connection info string\n"),
 							  pname);
 			PQconninfoFree(options);
 			free(buf);
@@ -2481,7 +2468,7 @@ conninfo_parse(const char *conninfo, PQExpBuffer errorMessage)
 				if (*cp == '\0')
 				{
 					printfPQExpBuffer(errorMessage,
-									  "ERROR: PQconnectdb() - unterminated quoted string in conninfo\n");
+									  libpq_gettext("unterminated quoted string in connection info string\n"));
 					PQconninfoFree(options);
 					free(buf);
 					return NULL;
@@ -2515,7 +2502,7 @@ conninfo_parse(const char *conninfo, PQExpBuffer errorMessage)
 		if (option->keyword == NULL)
 		{
 			printfPQExpBuffer(errorMessage,
-							  "ERROR: Unknown conninfo option '%s'\n",
+					 libpq_gettext("invalid connection option \"%s\"\n"),
 							  pname);
 			PQconninfoFree(options);
 			free(buf);
@@ -2631,6 +2618,36 @@ PQconninfoFree(PQconninfoOption *connOptions)
 	free(connOptions);
 }
 
+
+#ifdef USE_SSL
+
+/*
+ * Obtain reason string for last SSL error
+ *
+ * Some caution is needed here since ERR_reason_error_string will
+ * return NULL if it doesn't recognize the error code.  We don't
+ * want to return NULL ever.
+ */
+static const char *
+SSLerrmessage(void)
+{
+	unsigned long	errcode;
+	const char	   *errreason;
+	static char		errbuf[32];
+
+	errcode = ERR_get_error();
+	if (errcode == 0)
+		return "No SSL error reported";
+	errreason = ERR_reason_error_string(errcode);
+	if (errreason != NULL)
+		return errreason;
+	snprintf(errbuf, sizeof(errbuf), "SSL error code %lu", errcode);
+	return errbuf;
+}
+
+#endif /* USE_SSL */
+
+
 /* =========== accessor functions for PGconn ========= */
 char *
 PQdb(const PGconn *conn)
@@ -2700,7 +2717,7 @@ char *
 PQerrorMessage(const PGconn *conn)
 {
 	if (!conn)
-		return "PQerrorMessage: conn pointer is NULL\n";
+		return libpq_gettext("connection pointer is NULL\n");
 
 	return conn->errorMessage.data;
 }
@@ -2766,13 +2783,13 @@ PQsetClientEncoding(PGconn *conn, const char *encoding)
 	return (status);
 }
 
-#else
+#else							/* without multibytle support */
+
 int
 PQsetClientEncoding(PGconn *conn, const char *encoding)
 {
 	return -1;
 }
-
 #endif
 
 #ifdef USE_SSL
@@ -2783,7 +2800,6 @@ PQgetssl(PGconn *conn)
 		return NULL;
 	return conn->ssl;
 }
-
 #endif
 
 void
@@ -2840,73 +2856,3 @@ defaultNoticeProcessor(void *arg, const char *message)
 	/* Note: we expect the supplied string to end with a newline already. */
 	fprintf(stderr, "%s", message);
 }
-
-#ifdef MULTIBYTE
-/*
- * convert an encoding string to encoding symbol value.
- * case is ignored.
- * if there's no valid encoding, returns -1
- */
-
-typedef struct
-{
-	int			encoding;		/* encoding symbol value */
-	char	   *name;			/* encoding string */
-}			PQ_encoding_conv_tbl;
-
-static PQ_encoding_conv_tbl pq_conv_tbl[] = {
-	{SQL_ASCII, "SQL_ASCII"},
-	{EUC_JP, "EUC_JP"},
-	{EUC_CN, "EUC_CN"},
-	{EUC_KR, "EUC_KR"},
-	{EUC_TW, "EUC_TW"},
-	{UNICODE, "UNICODE"},
-	{MULE_INTERNAL, "MULE_INTERNAL"},
-	{LATIN1, "LATIN1"},
-	{LATIN2, "LATIN2"},
-	{LATIN3, "LATIN3"},
-	{LATIN4, "LATIN4"},
-	{LATIN5, "LATIN5"},
-	{KOI8, "KOI8"},
-	{WIN, "WIN"},
-	{ALT, "ALT"},
-	{SJIS, "SJIS"},
-	{BIG5, "BIG5"},
-	{WIN1250, "WIN1250"},
-	{-1, ""}
-};
-
-int
-pg_char_to_encoding(const char *s)
-{
-	PQ_encoding_conv_tbl *p = pq_conv_tbl;
-
-	if (!s)
-		return (-1);
-
-	for (; p->encoding >= 0; p++)
-	{
-		if (!strcasecmp(s, p->name))
-			break;
-	}
-	return (p->encoding);
-}
-
-/*
- * convert encoding symbol to encoding char.
- * if there's no valid encoding symbol, returns ""
- */
-const char *
-pg_encoding_to_char(int encoding)
-{
-	PQ_encoding_conv_tbl *p = pq_conv_tbl;
-
-	for (; p->encoding >= 0; p++)
-	{
-		if (p->encoding == encoding)
-			return (p->name);
-	}
-	return ("");
-}
-
-#endif
