@@ -3,22 +3,20 @@
  * fe-lobj.c
  *	  Front-end large object interface
  *
- * Portions Copyright (c) 1996-2000, PostgreSQL, Inc
+ * Portions Copyright (c) 1996-2001, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/interfaces/libpq/fe-lobj.c,v 1.29 2000/04/12 17:17:15 momjian Exp $
+ *	  $Header: /cvsroot/pgsql/src/interfaces/libpq/fe-lobj.c,v 1.40 2001/10/25 05:50:13 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
+#include "postgres_fe.h"
 
 #include <fcntl.h>
 #include <sys/stat.h>
-
-#include "postgres.h"
-#include "libpq-fe.h"
-#include "libpq-int.h"
+#include <errno.h>
 
 #ifdef WIN32
 #include "win32.h"
@@ -27,13 +25,16 @@
 #include <unistd.h>
 #endif
 
+#include "libpq-fe.h"
+#include "libpq-int.h"
 #include "libpq/libpq-fs.h"		/* must come after sys/stat.h */
 
 
-#define LO_BUFSIZE		  1024
+#define LO_BUFSIZE		  8192
 
 static int	lo_initialize(PGconn *conn);
 
+
 /*
  * lo_open
  *	  opens an existing large object
@@ -123,7 +124,7 @@ lo_close(PGconn *conn, int fd)
  * lo_read
  *	  read len bytes of the large object into buf
  *
- * returns the length of bytes read.
+ * returns the number of bytes read, or -1 on failure.
  * the CALLER must have allocated enough space to hold the result returned
  */
 
@@ -166,6 +167,7 @@ lo_read(PGconn *conn, int fd, char *buf, size_t len)
  * lo_write
  *	  write len bytes of buf into the large object fd
  *
+ * returns the number of bytes written, or -1 on failure.
  */
 int
 lo_write(PGconn *conn, int fd, char *buf, size_t len)
@@ -273,7 +275,7 @@ lo_creat(PGconn *conn, int mode)
 	if (conn->lobjfuncs == (PGlobjfuncs *) NULL)
 	{
 		if (lo_initialize(conn) < 0)
-			return -1;
+			return InvalidOid;
 	}
 
 	argv[0].isint = 1;
@@ -373,9 +375,9 @@ lo_unlink(PGconn *conn, Oid lobjId)
 /*
  * lo_import -
  *	  imports a file as an (inversion) large object.
- *		returns the oid of that object upon success,
- * returns InvalidOid upon failure
  *
+ * returns the oid of that object upon success,
+ * returns InvalidOid upon failure
  */
 
 Oid
@@ -391,16 +393,12 @@ lo_import(PGconn *conn, const char *filename)
 	/*
 	 * open the file to be read in
 	 */
-#ifndef __CYGWIN32__
-	fd = open(filename, O_RDONLY, 0666);
-#else
-	fd = open(filename, O_RDONLY | O_BINARY, 0666);
-#endif
+	fd = open(filename, O_RDONLY | PG_BINARY, 0666);
 	if (fd < 0)
 	{							/* error */
 		printfPQExpBuffer(&conn->errorMessage,
-						  "lo_import: can't open unix file\"%s\"\n",
-						  filename);
+					   libpq_gettext("could not open file \"%s\": %s\n"),
+						  filename, strerror(errno));
 		return InvalidOid;
 	}
 
@@ -411,8 +409,9 @@ lo_import(PGconn *conn, const char *filename)
 	if (lobjOid == InvalidOid)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  "lo_import: can't create inv object for \"%s\"",
+		libpq_gettext("could not create large object for file \"%s\"\n"),
 						  filename);
+		(void) close(fd);
 		return InvalidOid;
 	}
 
@@ -420,8 +419,9 @@ lo_import(PGconn *conn, const char *filename)
 	if (lobj == -1)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  "lo_import: could not open inv object oid %u",
+					   libpq_gettext("could not open large object %u\n"),
 						  lobjOid);
+		(void) close(fd);
 		return InvalidOid;
 	}
 
@@ -434,8 +434,10 @@ lo_import(PGconn *conn, const char *filename)
 		if (tmp < nbytes)
 		{
 			printfPQExpBuffer(&conn->errorMessage,
-							  "lo_import: error while reading \"%s\"",
+					  libpq_gettext("error while reading file \"%s\"\n"),
 							  filename);
+			(void) close(fd);
+			(void) lo_close(conn, lobj);
 			return InvalidOid;
 		}
 	}
@@ -461,30 +463,27 @@ lo_export(PGconn *conn, Oid lobjId, const char *filename)
 	int			lobj;
 
 	/*
-	 * create an inversion "object"
+	 * open the large object.
 	 */
 	lobj = lo_open(conn, lobjId, INV_READ);
 	if (lobj == -1)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  "lo_export: can't open inv object %u", lobjId);
+			  libpq_gettext("could not open large object %u\n"), lobjId);
 		return -1;
 	}
 
 	/*
-	 * open the file to be written to
+	 * create the file to be written to
 	 */
-#ifndef __CYGWIN32__
-	fd = open(filename, O_CREAT | O_WRONLY | O_TRUNC, 0666);
-#else
-	fd = open(filename, O_CREAT | O_WRONLY | O_TRUNC | O_BINARY, 0666);
-#endif
+	fd = open(filename, O_CREAT | O_WRONLY | O_TRUNC | PG_BINARY, 0666);
 	if (fd < 0)
 	{							/* error */
 		printfPQExpBuffer(&conn->errorMessage,
-						  "lo_export: can't open unix file\"%s\"",
-						  filename);
-		return 0;
+					   libpq_gettext("could not open file \"%s\": %s\n"),
+						  filename, strerror(errno));
+		(void) lo_close(conn, lobj);
+		return -1;
 	}
 
 	/*
@@ -496,8 +495,10 @@ lo_export(PGconn *conn, Oid lobjId, const char *filename)
 		if (tmp < nbytes)
 		{
 			printfPQExpBuffer(&conn->errorMessage,
-							  "lo_export: error while writing \"%s\"",
+				   libpq_gettext("error while writing to file \"%s\"\n"),
 							  filename);
+			(void) lo_close(conn, lobj);
+			(void) close(fd);
 			return -1;
 		}
 	}
@@ -509,13 +510,12 @@ lo_export(PGconn *conn, Oid lobjId, const char *filename)
 }
 
 
-/* ----------------
+/*
  * lo_initialize
  *
  * Initialize the large object interface for an existing connection.
  * We ask the backend about the functions OID's in pg_proc for all
  * functions that are required for large object operations.
- * ----------------
  */
 static int
 lo_initialize(PGconn *conn)
@@ -526,25 +526,23 @@ lo_initialize(PGconn *conn)
 	const char *fname;
 	Oid			foid;
 
-	/* ----------------
+	/*
 	 * Allocate the structure to hold the functions OID's
-	 * ----------------
 	 */
 	lobjfuncs = (PGlobjfuncs *) malloc(sizeof(PGlobjfuncs));
 	if (lobjfuncs == (PGlobjfuncs *) NULL)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-						  "FATAL: malloc() failed in lo_initialize()\n");
+						  libpq_gettext("out of memory\n"));
 		return -1;
 	}
 	MemSet((char *) lobjfuncs, 0, sizeof(PGlobjfuncs));
 
-	/* ----------------
+	/*
 	 * Execute the query to get all the functions at once
-	 * ----------------
 	 */
 	res = PQexec(conn, "select proname, oid from pg_proc	\
-    		where proname = 'lo_open'	\
+			where proname = 'lo_open'	\
 		   or proname = 'lo_close'	\
 		   or proname = 'lo_creat'	\
 		   or proname = 'lo_unlink'	\
@@ -563,13 +561,12 @@ lo_initialize(PGconn *conn)
 		free(lobjfuncs);
 		PQclear(res);
 		printfPQExpBuffer(&conn->errorMessage,
-				"ERROR: SELECT didn't return data in lo_initialize()\n");
+						  libpq_gettext("query to initialize large object functions did not return data\n"));
 		return -1;
 	}
 
-	/* ----------------
+	/*
 	 * Examine the result and put the OID's into the struct
-	 * ----------------
 	 */
 	for (n = 0; n < PQntuples(res); n++)
 	{
@@ -595,71 +592,69 @@ lo_initialize(PGconn *conn)
 
 	PQclear(res);
 
-	/* ----------------
-	 * Finally check that we really got all large object
-	 * interface functions.
-	 * ----------------
+	/*
+	 * Finally check that we really got all large object interface
+	 * functions.
 	 */
 	if (lobjfuncs->fn_lo_open == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				   "ERROR: Cannot determine OID for function lo_open\n");
+			libpq_gettext("cannot determine OID of function lo_open\n"));
 		free(lobjfuncs);
 		return -1;
 	}
 	if (lobjfuncs->fn_lo_close == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				  "ERROR: Cannot determine OID for function lo_close\n");
+		   libpq_gettext("cannot determine OID of function lo_close\n"));
 		free(lobjfuncs);
 		return -1;
 	}
 	if (lobjfuncs->fn_lo_creat == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				  "ERROR: Cannot determine OID for function lo_creat\n");
+		   libpq_gettext("cannot determine OID of function lo_creat\n"));
 		free(lobjfuncs);
 		return -1;
 	}
 	if (lobjfuncs->fn_lo_unlink == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				 "ERROR: Cannot determine OID for function lo_unlink\n");
+		  libpq_gettext("cannot determine OID of function lo_unlink\n"));
 		free(lobjfuncs);
 		return -1;
 	}
 	if (lobjfuncs->fn_lo_lseek == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				  "ERROR: Cannot determine OID for function lo_lseek\n");
+		   libpq_gettext("cannot determine OID of function lo_lseek\n"));
 		free(lobjfuncs);
 		return -1;
 	}
 	if (lobjfuncs->fn_lo_tell == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				   "ERROR: Cannot determine OID for function lo_tell\n");
+			libpq_gettext("cannot determine OID of function lo_tell\n"));
 		free(lobjfuncs);
 		return -1;
 	}
 	if (lobjfuncs->fn_lo_read == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-					"ERROR: Cannot determine OID for function loread\n");
+			 libpq_gettext("cannot determine OID of function loread\n"));
 		free(lobjfuncs);
 		return -1;
 	}
 	if (lobjfuncs->fn_lo_write == 0)
 	{
 		printfPQExpBuffer(&conn->errorMessage,
-				   "ERROR: Cannot determine OID for function lowrite\n");
+			libpq_gettext("cannot determine OID of function lowrite\n"));
 		free(lobjfuncs);
 		return -1;
 	}
 
-	/* ----------------
+	/*
 	 * Put the structure into the connection control
-	 * ----------------
 	 */
 	conn->lobjfuncs = lobjfuncs;
 	return 0;
