@@ -1,6 +1,7 @@
-#ifndef Y_TAB_H
-# define Y_TAB_H
+#ifndef BISON_Y_TAB_H
+# define BISON_Y_TAB_H
 
+#ifndef PLPGSQL_YYSTYPE
 typedef union {
 		int32					ival;
 		char					*str;
@@ -43,7 +44,9 @@ typedef union {
 		PLpgSQL_stmts			*stmts;
 		PLpgSQL_stmt_block		*program;
 		PLpgSQL_nsitem			*nsitem;
-} PLPGSQL_YYSTYPE;
+} plpgsql_yystype;
+# define PLPGSQL_YYSTYPE plpgsql_yystype
+#endif
 # define	K_ALIAS	257
 # define	K_ASSIGN	258
 # define	K_BEGIN	259
@@ -108,4 +111,4 @@ typedef union {
 
 extern PLPGSQL_YYSTYPE plpgsql_yylval;
 
-#endif /* not Y_TAB_H */
+#endif /* not BISON_Y_TAB_H */
